using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;

namespace Ato.EN.IntegrationServices.CodeGenerationTRT
{

    public partial class TRT2023Validator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// The report parameter
        /// </summary>
        private TRT2023 report;

        /// <summary>
        /// The response parameter
        /// </summary>
        public List<ProcessMessageDocument> response {get; private set;}

        /// <summary>
        /// Initializes a new instance of the <see cref="TRT2023Validator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public TRT2023Validator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
            this.response = new List<ProcessMessageDocument>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="TRT2023Validator" /> class.
        /// </summary>
        /// <param name="report">Report variable </param>                
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public TRT2023Validator(TRT2023 reportIn, int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY"):this(maxParameterNameLength,maxParameterValueLength,emptyParameterValue)
        {                   
            this.ConsumedReport = reportIn;                    
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }

        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString().ToLower() : string.Empty;
        }

        private static string GetValueOrEmptyDate(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-dd") : string.Empty;
        }

        private static string GetValueOrEmptyDate(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-dd");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(string val)
        {
            DateTime dateTimeOut;
            if (DateTime.TryParse(val, out dateTimeOut))
            {
                return dateTimeOut.ToString("yyyy-MM-ddTHH:mm:ssZ");
            }
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmptyDateTime(DateTime? val)
        {
            return (val.HasValue) ? val.Value.ToString("yyyy-MM-ddTHH:mm:ssZ") : string.Empty;
        }

        private static string GetValueOrEmpty(string val)
        {
            return !string.IsNullOrWhiteSpace(val) ? val : string.Empty;
        }

        private static string GetValueOrEmpty(decimal? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(int? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

         /// <summary>
         /// Get string value between [first] a and [last] b.
         /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

         /// <summary>
         /// Get string value after [first] a.
         /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

         /// <summary>
         /// Get string value after [last] a.
         /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
            return 0;
            else
            return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return (from x in nodes select x).Distinct().Count() != 1;
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(IEnumerable<DateTime?> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
           if (nodes == null)
                return false;

            nodes = nodes.Where(x => x != null).ToArray();
            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }

        public DateTime Date(DateTime? datetime)
        {                   
            return datetime.GetValueOrDefault().Date;
        }

				//The logic in After function expects "---" for day and "--" for month. 
				//Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
					return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }

        private static DateTime? AddMonthsToDate(DateTime? dateTime, int months)
        {
            return dateTime == null ? null : (DateTime?)dateTime.Value.AddMonths(months);
        }

        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response = true;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length == 14)
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 3), out numeric))
                        response = false;

                }
                else if (usi.Length == 9)
                {

                    if (Regex.IsMatch(usi, @"^([a-zA-Z]{3}\d{4}[a-zA-Z]{2})$"))
                        response = false;
                }
            }
            return response;
        }

        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
             return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
             decimal numberValue;
             decimal.TryParse(value, out numberValue);
             return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags="")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':                        
                        options = options | RegexOptions.IgnoreCase;                       
                        break;
                    case 'm':                        
                        options = options | RegexOptions.Multiline;                       
                        break;
                    case 's':                        
                            options = options | RegexOptions.Singleline;                       
                        break;
                    case 'n':                        
                            options = options | RegexOptions.ExplicitCapture;                       
                        break;
                    case 'x':                       
                            options = options | RegexOptions.IgnorePatternWhitespace;                       
                        break;
                }
            }

            return options;
        }

        private static string Substring(object field, int start, int length)
        {
            return field.ToString().Substring(start, length);
        }
        
        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public TRT2023 ConsumedReport { get {return report;} private set {report = value;}}

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string,ProcessMessageDocument>();				

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(TRT2023 reportIn)
        {
            ProcessMessageDocument processMessage;
            ProcessMessageParameter parameter;
            bool assertion;

            this.ConsumedReport = reportIn;

            // ------------------------------------------------------------------------------
            // Validations are now done in code.
            // This version supports full validation of a report - for those rules that have successfully parsed and could be generated.
            // Generated Validations include:
            //   - Validation logic in C#.
            //   - Production of errors in EBMS, SBR and SWS format.
            // 
            // The generation at this stage does not support the following - and has to be completed manually (later versions of the generation will do this for you)
            //   - Business Rules that did not parse.  The ESR team will help support you where this happens as an effort is being made to rectify these
            //   - You will get TODO tasks for those that the parser could not cope with
            // ------------------------------------------------------------------------------
                VRATOGEN000209();
                VRATOGEN000227();
                VRATOGEN000406_1();
                VRATOGEN000406_2();
                VRATOGEN000406_3();
                VRATOGEN000408_1();
                VRATOGEN000408_2();
                VRATOGEN000408_3();
                VRATOGEN000458_1();
                VRATOGEN000459_1();
                VRATOGEN001023();
                VRATOGEN001023_1();
                VRATOGEN001023_2();
                VRATOGEN001023_3();
                VRATOGEN001023_4();
                VRATOGEN001023_5();
                VRATOGEN001023_6();
                VRATOGEN001023_7();
                VRATOGEN001023_8();
                VRATOGEN001023_9();
                VRATOGEN001023_10();
                VRATOGEN001023_11();
                VRATOGEN001023_12();
                VRATOGEN001023_13();
                VRATOGEN001023_14();
                VRATOGEN001023_15();
                VRATOGEN001023_16();
                VRATOGEN001023_17();
                VRATOGEN001023_18();
                VRATOGEN001023_19();
                VRATOGEN001023_20();
                VRATOGEN010028();
                VRATOGEN010029();
                VRATOGEN010030();
                VRATOGEN408010();
                VRATOGEN408010_1();
                VRATOGEN408010_2();
                VRATOGEN410002_1();
                VRATOGEN410002_2();
                VRATOGEN410002_3();
                VRATOGEN410008_1();
                VRATOGEN410008_2();
                VRATOGEN410008_3();
                VRATOGEN410013_1();
                VRATOGEN410013_2();
                VRATOGEN410013_3();
                VRATOGEN410038();
                VRATOGEN410038_1();
                VRATOGEN410038_3();
                VRATOGEN410038_4();
                VRATOGEN410039();
                VRATOGEN410039_1();
                VRATOGEN410039_3();
                VRATOGEN410039_4();
                VRATOGEN410063_1();
                VRATOGEN410131_1();
                VRATOGEN410191_1();
                VRATOGEN410191_2();
                VRATOGEN410191_3();
                VRATOGEN410194_1();
                VRATOGEN410194_2();
                VRATOGEN410194_3();
                VRATOGEN410195_1();
                VRATOGEN410195_2();
                VRATOGEN410195_3();
                VRATOGEN410200_1();
                VRATOGEN410201_1();
                VRATOGEN410203_1();
                VRATOGEN410204_1();
                VRATOGEN410205_1();
                VRATOGEN410205_2();
                VRATOGEN410205_3();
                VRATOGEN410208_1();
                VRATOGEN410209_1();
                VRATOGEN410211_1();
                VRATOGEN410211_2();
                VRATOGEN410211_3();
                VRATOGEN410212_1();
                VRATOGEN410212_2();
                VRATOGEN410212_3();
                VRATOGEN410218_1();
                VRATOGEN428016();
                VRATOGEN428230_1();
                VRATOGEN428230_2();
                VRATOGEN428230_3();
                VRATOGEN428247();
                VRATOGEN428253();
                VRATOGEN428254_1();
                VRATOGEN428254_2();
                VRATOGEN428254_3();
                VRATOGEN428255_1();
                VRATOGEN428255_2();
                VRATOGEN428255_3();
                VRATOGEN428256_1();
                VRATOGEN428256_2();
                VRATOGEN428256_3();
                VRATOGEN428258();
                VRATOGEN428258_1();
                VRATOGEN428258_3();
                VRATOGEN428258_4();
                VRATOGEN428259();
                VRATOGEN428259_1();
                VRATOGEN428259_3();
                VRATOGEN428259_4();
                VRATOGEN428260();
                VRATOGEN428260_1();
                VRATOGEN428260_3();
                VRATOGEN428260_4();
                VRATOGEN428261();
                VRATOGEN428261_1();
                VRATOGEN428261_3();
                VRATOGEN428261_4();
                VRATOGEN428262_1();
                VRATOGEN428263_1();
                VRATOGEN428264_1();
                VRATOGEN428265_1();
                VRATOGEN428266_1();
                VRATOGEN428267_1();
                VRATOGEN430206();
                VRATOGEN430246_1();
                VRATOGEN430246_2();
                VRATOGEN430246_3();
                VRATOGEN430252();
                VRATOGEN430252_1();
                VRATOGEN430252_2();
                VRATOGEN430255();
                VRATOGEN430255_1();
                VRATOGEN430258();
                VRATOGEN430259();
                VRATOGEN430266();
                VRATOGEN430267();
                VRATOGEN430269();
                VRATOGEN430270();
                VRATOGEN430273();
                VRATOGEN430275();
                VRATOGEN430279();
                VRATOGEN430280();
                VRATOGEN430302();
                VRATOGEN432037();
                VRATOGEN432040();
                VRATOGEN432041();
                VRATOGEN432267();
                VRATOGEN434147_1();
                VRATOGEN434147_2();
                VRATOGEN434147_3();
                VRATOGEN434223();
                VRATOGEN434223_1();
                VRATOGEN438010();
                VRATOGEN438011();
                VRATOGEN438012();
                VRATOGEN438013();
                VRATOGEN438014();
                VRATOGEN438015();
                VRATOGEN438016();
                VRATOGEN438018();
                VRATOGEN438018_1();
                VRATOGEN438019();
                VRATOGEN438021();
                VRATOGEN438029();
                VRATOGEN438029_1();
                VRATOGEN438029_2();
                VRATOGEN438029_3();
                VRATOGEN438029_4();
                VRATOGEN438029_5();
                VRATOGEN438029_6();
                VRATOGEN438029_7();
                VRATOGEN438029_8();
                VRATOGEN438029_9();
                VRATOGEN438029_10();
                VRATOGEN438029_11();
                VRATOGEN438029_12();
                VRATOGEN438029_13();
                VRATOGEN438029_14();
                VRATOGEN438029_15();
                VRATOGEN438029_16();
                VRATOGEN438029_17();
                VRATOGEN438029_18();
                VRATOGEN438029_19();
                VRATOGEN438029_20();
                VRATOGEN438029_21();
                VRATOGEN438030();
                VRATOGEN438035();
                VRATOGEN438035_1();
                VRATOGEN438035_2();
                VRATOGEN438051();
                VRATOGEN438052();
                VRATOGEN438053();
                VRATOGEN438053_1();
                VRATOGEN438053_2();
                VRATOGEN438053_3();
                VRATOGEN438053_4();
                VRATOGEN438053_5();
                VRATOGEN438053_6();
                VRATOGEN438053_7();
                VRATOGEN438053_8();
                VRATOGEN438053_9();
                VRATOGEN438053_10();
                VRATOGEN438053_11();
                VRATOGEN438053_12();
                VRATOGEN438053_13();
                VRATOGEN438053_14();
                VRATOGEN438053_15();
                VRATOGEN438053_16();
                VRATOGEN438053_17();
                VRATOGEN438053_18();
                VRATOGEN438053_19();
                VRATOGEN438054();
                VRATOGEN438054_1();
                VRATOGEN438054_2();
                VRATOGEN438054_3();
                VRATOGEN438054_4();
                VRATOGEN438054_5();
                VRATOGEN438054_6();
                VRATOGEN438054_7();
                VRATOGEN438054_8();
                VRATOGEN438054_9();
                VRATOGEN438054_10();
                VRATOGEN438054_11();
                VRATOGEN438054_12();
                VRATOGEN438054_13();
                VRATOGEN438054_14();
                VRATOGEN438054_15();
                VRATOGEN438054_16();
                VRATOGEN438054_17();
                VRATOGEN438054_18();
                VRATOGEN438054_19();
                VRATOGEN438058();
                VRATOGEN438059();
                VRATOGEN438063();
                VRATOGEN438064();
                VRATOGEN438065();
                VRATOGEN438066();
                VRATOGEN438067();
                VRATOGEN438068();
                VRATOGEN438081();
                VRATOGEN438082();
                VRATOGEN500130();
                VRATOGEN500130_1();
                VRATOGEN500130_2();
                VRATOGEN500132();
                VRATOGEN500132_1();
                VRATOGEN500132_3();
                VRATOGEN500132_4();
                VRATOGEN500133_1();
                VRATOGEN500133_2();
                VRATOGEN500133_3();
                VRATOGEN500134_1();
                VRATOGEN500134_2();
                VRATOGEN500134_3();
                VRATOGEN500135_1();
                VRATOGEN500136();
                VRATOGEN500137();
                VRATOGEN500141_1();
                VRATOGEN500141_2();
                VRATOGEN500141_3();
                VRATOGEN500142_1();
                VRATOGEN500145();
                VRATOGEN500151_1();
                VRATOGEN500152_1();
                VRATOGEN500153_1();
                VRATOTRT432008();
                VRATOTRT432013();
                VRATOTRT432014();
                VRATOTRT432015();
                VRATOTRT432023();
                VRATOTRT432024();
                VRATOTRT432025();
                VRATOTRT432026();
                VRATOTRT432027();
                VRATOTRT432028();
                VRATOTRT432029();
                VRATOTRT432030();
                VRATOTRT432031();
                VRATOTRT432038();
                VRATOTRT432039();
                VRATOTRT432041();
                VRATOTRT432045();
                VRATOTRT432046();
                VRATOTRT432047();
                VRATOTRT432048();
                VRATOTRT432049();
                VRATOTRT432050();
                VRATOTRT432052();
                VRATOTRT432053();
                VRATOTRT432054();
                VRATOTRT432055();
                VRATOTRT432056();
                VRATOTRT432057();
                VRATOTRT432058();
                VRATOTRT432059();
                VRATOTRT432060();
                VRATOTRT432061();
                VRATOTRT432064();
                VRATOTRT432070();
                VRATOTRT432071();
                VRATOTRT432074();
                VRATOTRT432075();
                VRATOTRT432078();
                VRATOTRT432079();
                VRATOTRT432080();
                VRATOTRT432081();
                VRATOTRT432082();
                VRATOTRT432083();
                VRATOTRT432084();
                VRATOTRT432085();
                VRATOTRT432086();
                VRATOTRT432087();
                VRATOTRT432089();
                VRATOTRT432090();
                VRATOTRT432091();
                VRATOTRT432092();
                VRATOTRT432093();
                VRATOTRT432097();
                VRATOTRT432098();
                VRATOTRT432099();
                VRATOTRT432100();
                VRATOTRT432102();
                VRATOTRT432103();
                VRATOTRT432104();
                VRATOTRT432105();
                VRATOTRT432107();
                VRATOTRT432108();
                VRATOTRT432109();
                VRATOTRT432110();
                VRATOTRT432111();
                VRATOTRT432112();
                VRATOTRT432113();
                VRATOTRT432114();
                VRATOTRT432118();
                VRATOTRT432119();
                VRATOTRT432121();
                VRATOTRT432123();
                VRATOTRT432125();
                VRATOTRT432127();
                VRATOTRT432128();
                VRATOTRT432129();
                VRATOTRT432130();
                VRATOTRT432131();
                VRATOTRT432132();
                VRATOTRT432133();
                VRATOTRT432134();
                VRATOTRT432135();
                VRATOTRT432136();
                VRATOTRT432137();
                VRATOTRT432138();
                VRATOTRT432139();
                VRATOTRT432140();
                VRATOTRT432141();
                VRATOTRT432142();
                VRATOTRT432143();
                VRATOTRT432144();
                VRATOTRT432144_1();
                VRATOTRT432146();
                VRATOTRT432148();
                VRATOTRT432149();
                VRATOTRT432152();
                VRATOTRT432159();
                VRATOTRT432159_1();
                VRATOTRT432160();
                VRATOTRT432160_1();
                VRATOTRT432163();
                VRATOTRT432167();
                VRATOTRT432168();
                VRATOTRT432169();
                VRATOTRT432179();
                VRATOTRT432180();
                VRATOTRT432181();
                VRATOTRT432188();
                VRATOTRT432191_1();
                VRATOTRT432191_2();
                VRATOTRT432192();
                VRATOTRT432193();
                VRATOTRT432194();
                VRATOTRT432196();
                VRATOTRT432199();
                VRATOTRT432202();
                VRATOTRT432203();
                VRATOTRT432204();
                VRATOTRT432205();
                VRATOTRT432206();
                VRATOTRT432207();
                VRATOTRT432209();
                VRATOTRT432211();
                VRATOTRT432227();
                VRATOTRT432228();
                VRATOTRT432229();
                VRATOTRT432230();
                VRATOTRT432231();
                VRATOTRT432232();
                VRATOTRT432234();
                VRATOTRT432235();
                VRATOTRT432236();
                VRATOTRT432237();
                VRATOTRT432238();
                VRATOTRT432239();
                VRATOTRT432240();
                VRATOTRT432241();
                VRATOTRT432242();
                VRATOTRT432243();
                VRATOTRT432244();
                VRATOTRT432245();
                VRATOTRT432246();
                VRATOTRT432247();
                VRATOTRT432248();
                VRATOTRT432249();
                VRATOTRT432250();
                VRATOTRT432251();
                VRATOTRT432252();
                VRATOTRT432253();
                VRATOTRT432254();
                VRATOTRT432255();
                VRATOTRT432257();
                VRATOTRT432259();
                VRATOTRT432260();
                VRATOTRT432261();
                VRATOTRT432263();
                VRATOTRT432264();
                VRATOTRT432265();
                VRATOTRT432275();
                VRATOTRT432276();
                VRATOTRT432277();
                VRATOTRT432285();
                VRATOTRT432290();
                VRATOTRT432298();
                VRATOTRT432299_1();
                VRATOTRT432306_1();
                VRATOTRT432308_1();
                VRATOTRT432309_1();
                VRATOTRT432311();
                VRATOTRT432312_1();
                VRATOTRT432314();
                VRATOTRT432315();
                VRATOTRT432316_1();
                VRATOTRT432321();
                VRATOTRT432324();
                VRATOTRT432325();
                VRATOTRT432326();
                VRATOTRT432335();
                VRATOTRT432336();
                VRATOTRT432337();
                VRATOTRT432338();
                VRATOTRT432339();
                VRATOTRT432342();
                VRATOTRT432343();
                VRATOTRT432343_2();
                VRATOTRT432352();
                VRATOTRT432353();
                VRATOTRT432360();
                VRATOTRT432361();
                VRATOTRT432365();
                VRATOTRT432366();
                VRATOTRT432367();
                VRATOTRT432368();
                VRATOTRT432369();
                VRATOTRT432370();
                VRATOTRT432373();
                VRATOTRT432374();
                VRATOTRT432375();
                VRATOTRT432377();
                VRATOTRT432378();
                VRATOTRT432379();
                VRATOTRT432383();
                VRATOTRT432387();
                VRATOTRT432389();
                VRATOTRT432413();
                VRATOTRT432419();
                VRATOTRT432421();
                VRATOTRT432430();
                VRATOTRT432431();
                VRATOTRT432560();
                VRATOTRT432561();
                VRATOTRT432562();
                VRATOTRT432562_1();
                VRATOTRT432562_2();
                VRATOTRT432562_3();
                VRATOTRT432562_4();
                VRATOTRT432562_5();
                VRATOTRT432562_6();
                VRATOTRT432562_7();
                VRATOTRT432562_8();
                VRATOTRT432562_9();
                VRATOTRT432562_10();
                VRATOTRT432562_11();
                VRATOTRT432562_12();
                VRATOTRT432562_13();
                VRATOTRT432562_14();
                VRATOTRT432562_15();
                VRATOTRT432562_16();
                VRATOTRT432562_17();
                VRATOTRT432562_18();
                VRATOTRT432562_19();
                VRATOTRT432562_20();
                VRATOTRT432567();
                VRATOTRT432570();
                VRATOTRT432572();
                VRATOTRT432573();
                VRATOTRT432577();
                VRATOTRT432578();
                VRATOTRT432579();
                VRATOTRT432587_1();
                VRATOTRT432593();
                VRATOTRT432594();
                VRATOTRT432600();
                VRATOTRT432601();
                VRATOTRT432604();
                VRATOTRT432605();
                VRATOTRT432606();
                VRATOTRT432607();
                VRATOTRT432609();
                VRATOTRT432610();
                VRATOTRT432611();
                VRATOTRT432612();
                VRATOTRT432631();
                VRATOTRT432632();
                VRATOTRT432633();
                VRATOTRT432634();
                VRATOTRT432635();
                VRATOTRT432636();
                VRATOTRT432637();
                VRATOTRT432638();
                VRATOTRT432639();
                VRATOTRT432640_1();
                VRATOTRT432641_1();
                VRATOTRT432642();
                VRATOTRT432658();
                VRATOTRT432661();
                VRATOTRT432666();
                VRATOTRT432668();
                VRATOTRT432681();
                VRATOTRT432694();
                VRATOTRT432696();
                VRATOTRT432697();
                VRATOTRT432701();
                VRATOTRT432702_1();
                VRATOTRT432703();
                VRATOTRT432704();
                VRATOTRT432705();
                VRATOTRT432708();
                VRATOTRT432709();
                VRATOTRT432710();
                VRATOTRT432711();
                VRATOTRT432712();
                VRATOTRT432713();
                VRATOTRT432714();
                VRATOTRT432715();
                VRATOTRT432717();
                VRATOTRT432717_1();
                VRATOTRT432718();
                VRATOTRT432721();
                VRATOTRT432730();
                VRATOTRT432731();
                VRATOTRT432742();
                VRATOTRT432747();
                VRATOTRT432765();
                VRATOTRT432778();
                VRATOTRT432783();
                VRATOTRT432784();
                VRATOTRT432785();
                VRATOTRT432786();
                VRATOTRT432787();
                VRATOTRT432790();
                VRATOTRT432791();
                VRATOTRT432792();
                VRATOTRT432793();
                VRATOTRT432800();
                VRATOTRT432801();
                VRATOTRT432802();
                VRATOTRT432805();
                VRATOTRT432813();
                VRATOTRT432814();
                VRATOTRT432815();
                VRATOTRT432816();
                VRATOTRT432817();
                VRATOTRT432818();
                VRATOTRT432819();
                VRATOTRT432820();
                VRATOTRT432821();
                VRATOTRT432822();
                VRATOTRT432823();
                VRATOTRT432824();
                VRATOTRT432825();
                VRATOTRT432826();
                VRATOTRT432827();
                VRATOTRT432828();
                VRATOTRT432829();
                VRATOTRT432830();
                VRATOTRT432831();
                VRATOTRT432832();
                VRATOTRT432833();
                VRATOTRT432834();
                VRATOTRT432863();
                VRATOTRT432871();
                VRATOTRT432872();
                VRATOTRT432873();
                VRATOTRT432874();
                VRATOTRT432875();
                VRATOTRT432876();
                VRATOTRT432877();
                VRATOTRT432880();
                VRATOTRT432887();
                VRATOTRT432888();
                VRATOTRT432891();
                VRATOTRT432893();
                VRATOTRT432894();
                VRATOTRT432896();
                VRATOTRT432897();
                VRATOTRT432898();
                VRATOTRT432899();
                VRATOTRT432900();
                VRATOTRT432901();
                VRATOTRT432906();
                VRATOTRT432920();
                VRATOTRT432921();
                VRATOTRT432922();
                VRATOTRT432923();
                VRATOTRT432924();
                VRATOTRT432925();
                VRATOTRT432926();
                VRATOTRTW00001();
                VRATOTRTW00002();
                VRATOTRTW00003();
                VRATOTRTW00004();
                VRATOTRTW00005();
                VRATOTRTW00006();
                VRATOTRTW00007();
                VRATOTRTW00008();
                VRATOTRTW00009();
                VRATOTRTW00010();
                VRATOTRTW00011();
                VRATOTRTW00012();
                VRATOTRTW00013();
                VRATOTRTW00014();
                VRATOTRTW00015();
                VRATOTRTW00016();
                VRATOTRTW00019();

            #region Repeating report.StatementOfDistributionToBeneficiaryCollection  
            if (report.StatementOfDistributionToBeneficiaryCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary in report.StatementOfDistributionToBeneficiaryCollection)
                {
                    itemIndex1++;
                        VRATOGEN000406(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN000406_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN000408(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN000408_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN000458(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN000459(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410002(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410002_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410008(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410008_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410013(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410013_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410031(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410038_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410039_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410063(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410131(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410191(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410191_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410194(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410194_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410195(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410195_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410200(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410201(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410203(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410204(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410205(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410205_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410208(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410209(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410211(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410211_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410212(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410212_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN410218(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428230(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428230_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428254(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428254_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428255(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428255_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428256(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428256_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428258_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428259_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428260_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428261_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428262(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428263(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428264(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428265(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428266(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN428267(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN430246(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN430246_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN434147(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN434147_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500132_2(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500133(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500133_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500134(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500134_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500135(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500141(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500141_4(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500142(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500151(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500152(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOGEN500153(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432022(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432191(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432280(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432281(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432282(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432283(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432284(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432286(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432287(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432288(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432289(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432292(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432293(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432294(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432295(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432296(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432299(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432300(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432301(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432302(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432303(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432304(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432306(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432307(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432308(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432309(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432312(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432316(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432318(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432319(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432320(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432343_1(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432347(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432348(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432349(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432350(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432418(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432569(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432581(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432582(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432583(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432584(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432585(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432586(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432587(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432588(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432589(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432613(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432640(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432641(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432702(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432716(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432719(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432739(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432788(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432789(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432794(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432807(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432808(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432809(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432810(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432811(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432812(statementOfDistributionToBeneficiary, itemIndex1);
                        VRATOTRT432864(statementOfDistributionToBeneficiary, itemIndex1);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.IndividualNonBusinessWithholdingPaymentDetailsCollection  
            if (report.IndividualNonBusinessWithholdingPaymentDetailsCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails in report.IndividualNonBusinessWithholdingPaymentDetailsCollection)
                {
                    itemIndex1++;
                        VRATOGEN434223_2(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432152_1(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432153(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432154(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432155(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432156(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432157(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                        VRATOTRT432425(individualNonBusinessWithholdingPaymentDetails, itemIndex1);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.OtherAustralianIncomeCollection  
            if (report.OtherAustralianIncomeCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (TRT2023.OtherAustralianIncome otherAustralianIncome in report.OtherAustralianIncomeCollection)
                {
                    itemIndex1++;
                        VRATOTRT432164(otherAustralianIncome, itemIndex1);
                        VRATOTRT432166(otherAustralianIncome, itemIndex1);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating report.OtherDeductionsCollection  
            if (report.OtherDeductionsCollection != null)
            {    
                int itemIndex1 = 0;
                foreach (TRT2023.OtherDeductions otherDeductions in report.OtherDeductionsCollection)
                {
                    itemIndex1++;
                        VRATOTRT432177(otherDeductions, itemIndex1);
                        VRATOTRT432178(otherDeductions, itemIndex1);
                    }
                }
        
                #endregion // Foreach loop

            #region Repeating TRT448Collection  
            if (report.TRT448Collection != null)
            {
                int itemIndex1 = 0;
                foreach (string TRT448 in report.TRT448Collection)
                {
                    itemIndex1++;
                        VRATOTRT432693(TRT448, itemIndex1);
                    }
                }
        
                #endregion // Foreach loop

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Value = _emptyParameterValue;
                        }

                        if (currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }
				
			
    
      #region VR.ATO.GEN.000209
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000209()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000209
            Invalid context. The number of Reporting party contexts must equal 1
    
            Legacy Rule Format:
            IF COUNT(RP) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) <> 1
    
            Data Elements:
    
            ^Context = RP
            */
            assertion = (Count(report.RPCount) != 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430296", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party contexts must equal 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000209"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000227
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000227()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000227
            Invalid context. The number of Intermediary contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(INT) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = INT
            */
            assertion = (Count(report.INTCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.436280", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Intermediary contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000227"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000406
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000406(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT511 <> BLANK) AND (StartsWithSet(^TRT511,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^TRT511, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT511) != true && IsMatch(statementOfDistributionToBeneficiary.TRT511, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(statementOfDistributionToBeneficiary.TRT511, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000406
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000406_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT162 <> BLANK) AND (StartsWithSet(^TRT162,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^TRT162, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT162) != true && IsMatch(report.TRT162, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT162, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000406
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000406_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT9 <> BLANK) AND (StartsWithSet(^TRT9,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^TRT9, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT9) != true && IsMatch(report.TRT9, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT9, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000406
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000406_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000406
            Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT14 <> BLANK) AND (StartsWithSet(^TRT14,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^TRT14, '" C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT14) != true && IsMatch(report.TRT14, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT14, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000406
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000406_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000406
                    Address line 1 contains incorrect 'care of' reference. The only acceptable 'care of' reference is C/-
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line1.Text STARTSWITH SET("C/O ","C/ ","Care Of ","CO ")) OR (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS SET(" C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT205 <> BLANK) AND (StartsWithSet(^TRT205,'"C/O ","C/ ","Care Of ","CO "')) OR (ContainsSet(^TRT205, '" C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT205) != true && IsMatch(statementOfDistributionToBeneficiary.TRT205, @"^(C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(statementOfDistributionToBeneficiary.TRT205, @" C/O | C/ | Care Of ",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000406", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 contains incorrect 'care of' reference.  The only acceptable 'care of' reference is C/-",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000406"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000408
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000408(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT512 <> BLANK) AND (StartsWithSet(^TRT512,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^TRT512, '" C/- "," C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^TRT512 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT512) != true && (IsMatch(statementOfDistributionToBeneficiary.TRT512, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(statementOfDistributionToBeneficiary.TRT512, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT512", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT512) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000408
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000408_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT309 <> BLANK) AND (StartsWithSet(^TRT309,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^TRT309, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT309 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT309) != true && (IsMatch(report.TRT309, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT309, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT309", Value = GetValueOrEmpty(report.TRT309) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000408
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000408_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT306 <> BLANK) AND (StartsWithSet(^TRT306,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^TRT306, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT306 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT306) != true && (IsMatch(report.TRT306, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT306, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT306", Value = GetValueOrEmpty(report.TRT306) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000408
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000408_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000408
            Care of' is not valid for address line 2
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT312 <> BLANK) AND (StartsWithSet(^TRT312,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^TRT312, '" C/- "," C/O "," C/ "," Care Of "'))
    
            Data Elements:
    
            ^TRT312 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT312) != true && (IsMatch(report.TRT312, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(report.TRT312, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                    Description = @"Care of' is not valid for address line 2",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT312", Value = GetValueOrEmpty(report.TRT312) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000408
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000408_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000408
                    Care of' is not valid for address line 2
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Line2.Text STARTSWITH SET("C/- ","C/O ","C/ ","Care Of ","CO ") OR pyde.xx.xx:AddressDetails.Line2.Text CONTAINS SET(" C/- "," C/O "," C/ "," Care Of "))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT206 <> BLANK) AND (StartsWithSet(^TRT206,'"C/- ","C/O ","C/ ","Care Of ","CO "') OR ContainsSet(^TRT206, '" C/- "," C/O "," C/ "," Care Of "'))
            
                    Data Elements:
            
                    ^TRT206 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT206) != true && (IsMatch(statementOfDistributionToBeneficiary.TRT206, @"^(C/- |C/O |C/ |Care Of |CO )",RegexOptions.IgnoreCase) || IsMatch(statementOfDistributionToBeneficiary.TRT206, @" C/- | C/O | C/ | Care Of ",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000408", Severity = ProcessMessageSeverity.Error,
                            Description = @"Care of' is not valid for address line 2",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000408"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT206", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT206) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000458
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000458(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000458
                    Suffix is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT379 <> BLANK) AND (NotInDomain(^TRT379, 'SUFFIX CODES'))
            
                    Data Elements:
            
                    ^TRT379 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT379) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT379, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000458", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suffix is invalid",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT379", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT379) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000458
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000458_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000458
            Suffix is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.NameSuffix.Text <> SET(DOMAIN(SUFFIX CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT21 <> BLANK) AND (NotInDomain(^TRT21, 'SUFFIX CODES'))
    
            Data Elements:
    
            ^TRT21 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.NameSuffix.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT21) != true && !(IsMatch(report.TRT21, @"^(ESQ|II|III|IV|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000458", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suffix is invalid",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.NameSuffix.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000458"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT21", Value = GetValueOrEmpty(report.TRT21) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.000459
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000459(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.000459
                    Title is invalid
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT378 <> BLANK) AND (NotInDomain(^TRT378, 'TITLE CODES'))
            
                    Data Elements:
            
                    ^TRT378 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT378) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT378, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000459", Severity = ProcessMessageSeverity.Error,
                            Description = @"Title is invalid",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT378", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT378) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.000459
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN000459_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.000459
            Title is invalid
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.Title.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.Title.Text <> SET(DOMAIN(TITLE CODES)))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT19 <> BLANK) AND (NotInDomain(^TRT19, 'TITLE CODES'))
    
            Data Elements:
    
            ^TRT19 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.Title.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT19) != true && !(IsMatch(report.TRT19, @"^(2LT|AB|ABBOT|AC|ACM|ADM|AIR CDRE|ALDERMAN|AM|ARCHBISHOP|ARCHDEACON|ASSOC PROF|AVM|BARON|BARONESS|BISHOP|BR|BRIG|CANON|CAPT|CARDINAL|CDRE|CDT|CHAP|CMDR|CMM|COL|CONST|COUNT|COUNTESS|CPL|CPO|DAME|DEACON|DEACONESS|DEAN|DEPUTY SUPT|DR|DUCHESS|DUKE|EARL|EF|FLGOFF|FLT LT|FR|FSGT|GEN|GNR|GP CAPT|HON JUDGE|HON JUST|HON|HRH|INSP|JUDGE|JUST|LAC|LACW|LADY|LBDR|LCPL|LORD|LS|LT CMDR|LT COL|LT GEN|LT|MADAM|MAJ GEN|MAJ|MASTER|MATRON|MAYOR|MAYORESS|MIDN|MISS|MON|MOST REV|MR|MRS|MS|PASTOR|PATRIARCH|PLT OFF|PO|PRIOR|PROF|PTE|RABBI|RADM|RECTOR|REV|RF|RT HON|RT REV MON|RT REV|RTREV BISHOP|SBLT|SEN|SGT|SIR|SMN|SNR CONST|SQN LDR|SR|SSGT|SUPR|SWAMI|TF|VADM|VERY REV|VICAR|VISCOUNT|WG CDR|WO|WO1|WO2)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000459", Severity = ProcessMessageSeverity.Error,
                    Description = @"Title is invalid",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.Title.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.000459"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT19", Value = GetValueOrEmpty(report.TRT19) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Closing
    
            ^OtherTFN = RP:RP.Closing
    
            ^TFN = RP
            */
            assertion = (report.RPClosingExists != false && report.RPClosingIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.Closing(Instant)
    
            ^OtherTFN = RP.Closing(Instant)
    
            ^TFN = RP
            */
            assertion = (report.RPClosingInstantExists != false && report.RPClosingInstantIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
    
            ^OtherTFN = RP:RP.GeneralPool
    
            ^TFN = RP
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Investor
    
            ^OtherTFN = RP:RP.Investor
    
            ^TFN = RP
            */
            assertion = (report.RPInvestorExists != false && report.RPInvestorIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.Investor:RP.Investor.YPrevious
    
            ^OtherTFN = RP.Investor:RP.Investor.YPrevious
    
            ^TFN = RP
            */
            assertion = (report.RPInvestorYPreviousExists != false && report.RPInvestorYPreviousIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_5()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
    
            ^OtherTFN = RP:RP.JAUS
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_6()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
    
            ^OtherTFN = RP.JAUS:RP.JAUS.NonPrim
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_7()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
    
            ^OtherTFN = RP.JAUS:RP.JAUS.Prim
    
            ^TFN = RP
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_8()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
    
            ^OtherTFN = RP:RP.JFOREIGN
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_9()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^OtherTFN = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_10()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^OtherTFN = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^TFN = RP
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_11()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.LimitedPartners
    
            ^OtherTFN = RP:RP.LimitedPartners
    
            ^TFN = RP
            */
            assertion = (report.RPLimitedPartnersExists != false && report.RPLimitedPartnersIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_12()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^OtherTFN = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^TFN = RP
            */
            assertion = (report.RPLimitedPartnersYPreviousExists != false && report.RPLimitedPartnersYPreviousIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_13()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
    
            ^OtherTFN = RP:RP.LowCost
    
            ^TFN = RP
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_14()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
    
            ^OtherTFN = RP:RP.NonPrim
    
            ^TFN = RP
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_15()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Opening
    
            ^OtherTFN = RP:RP.Opening
    
            ^TFN = RP
            */
            assertion = (report.RPOpeningExists != false && report.RPOpeningIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_16()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP.Opening(Instant)
    
            ^OtherTFN = RP.Opening(Instant)
    
            ^TFN = RP
            */
            assertion = (report.RPOpeningInstantExists != false && report.RPOpeningInstantIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_17()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Prim
    
            ^OtherTFN = RP:RP.Prim
    
            ^TFN = RP
            */
            assertion = (report.RPPrimExists != false && report.RPPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_18()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Share.NonPrim
    
            ^OtherTFN = RP:RP.Share.NonPrim
    
            ^TFN = RP
            */
            assertion = (report.RPShareNonPrimExists != false && report.RPShareNonPrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_19()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.Share.Prim
    
            ^OtherTFN = RP:RP.Share.Prim
    
            ^TFN = RP
            */
            assertion = (report.RPSharePrimExists != false && report.RPSharePrimIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.001023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN001023_20()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.001023
            TFN in entity identifier of context does not match the TFN provided for the Reporting Party
    
            Legacy Rule Format:
            IF (entity.identifier.TFN WHERE CONTEXT(ALL)) <> RP:entity.identifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^OtherTFN <> ^TFN
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
    
            ^OtherTFN = RP:RP.TOFA
    
            ^TFN = RP
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAIdentifierTFN != report.RPIdentifierTFN);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001023", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unexpected TFN mismatch with Reporting Party",
                    LongDescription = @"TFN in entity identifier of context does not match the TFN provided for the Reporting Party",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.001023"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.010028
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN010028()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.010028
            If the return is an amendment, sequence number, type and reason must be present
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator = TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number = NULL OR pyin.xx.xx:Report.AmendmentType.Code = NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text = NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT414 = TRUE AND (^TRT417 = NULL OR ^TRT415 = BLANK OR ^TRT416 = BLANK)
    
            Data Elements:
    
            ^TRT414 = RP:Report.Amendment.Indicator
    
            ^TRT417 = RP:Report.AmendmentSequence.Number
    
            ^TRT415 = RP:Report.AmendmentType.Code
    
            ^TRT416 = RP:Report.AmendmentReason.Text
            */
            assertion = (report.TRT414 == true && (report.TRT417 == null || string.IsNullOrWhiteSpace(report.TRT415) == true || string.IsNullOrWhiteSpace(report.TRT416) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010028", Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, sequence number, type and reason must be present",
                    LongDescription = @"If the Amendment Indicator is 'yes' (true), then the Amendment Sequence Number, Amendment Type and Amendment Reason must be present and not blank.",
                    Location = "/xbrli:xbrl/tns:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.010028"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT414", Value = GetValueOrEmpty(report.TRT414) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT417", Value = report.TRT417.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT415", Value = GetValueOrEmpty(report.TRT415) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT416", Value = GetValueOrEmpty(report.TRT416) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.010029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN010029()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.010029
            If the return is an amendment, Amendment Indicator must be 'yes' (true)
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.Amendment.Indicator <> TRUE AND (pyin.xx.xx:Report.AmendmentSequence.Number <> NULL OR pyin.xx.xx:Report.AmendmentType.Code <> NULLORBLANK OR pyin.xx.xx:Report.AmendmentReason.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT414 <> TRUE AND (^TRT417 <> NULL OR ^TRT415 <> BLANK OR ^TRT416 <> BLANK)
    
            Data Elements:
    
            ^TRT414 = RP:Report.Amendment.Indicator
    
            ^TRT417 = RP:Report.AmendmentSequence.Number
    
            ^TRT415 = RP:Report.AmendmentType.Code
    
            ^TRT416 = RP:Report.AmendmentReason.Text
            */
            assertion = (report.TRT414 != true && (report.TRT417 != null || string.IsNullOrWhiteSpace(report.TRT415) != true || string.IsNullOrWhiteSpace(report.TRT416) != true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010029", Severity = ProcessMessageSeverity.Error,
                    Description = @"If the return is an amendment, Amendment Indicator must be 'yes' (true)",
                    LongDescription = @"If an Amendment Sequence Number, Amendment Type or Amendment Reason is present, then the Amendment Indicator must be 'yes' (true)",
                    Location = "/xbrli:xbrl/tns:Report.Amendment.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.010029"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT414", Value = GetValueOrEmpty(report.TRT414) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT417", Value = report.TRT417.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT415", Value = GetValueOrEmpty(report.TRT415) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT416", Value = GetValueOrEmpty(report.TRT416) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.010030
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN010030()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.010030
            Amendment sequence number must be a number between 1 and 9
    
            Legacy Rule Format:
            IF pyin.xx.xx:Report.AmendmentSequence.Number <> NULL AND pyin.xx.xx:Report.AmendmentSequence.Number <> SET(1-9)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT417 <> NULL AND (^TRT417 < 1 OR ^TRT417 > 9)
    
            Data Elements:
    
            ^TRT417 = RP:Report.AmendmentSequence.Number
            */
            assertion = (report.TRT417 != null && (report.TRT417.GetValueOrDefault() < 1 || report.TRT417.GetValueOrDefault() > 9));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010030", Severity = ProcessMessageSeverity.Error,
                    Description = @"Amendment sequence number must be a number between 1 and 9",
                    Location = "/xbrli:xbrl/tns:Report.AmendmentSequence.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.010030"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT417", Value = report.TRT417.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.408010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN408010()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT250, ' - ')
    
            Data Elements:
    
            ^TRT250 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (report.TRT250 == null ? false : report.TRT250.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010", Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT250", Value = report.TRT250 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.408010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN408010_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT396, ' - ')
    
            Data Elements:
    
            ^TRT396 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (report.TRT396 == null ? false : report.TRT396.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010", Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT396", Value = report.TRT396 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.408010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN408010_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.408010
            Name field cannot contain 'space hyphen space'.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT389, ' - ')
    
            Data Elements:
    
            ^TRT389 = RP:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (report.TRT389 == null ? false : report.TRT389.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.408010", Severity = ProcessMessageSeverity.Error,
                    Description = @"Name field cannot contain 'space hyphen space'.",
                    LongDescription = @"Name field cannot contain 'space hyphen space'.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.408010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT389", Value = report.TRT389 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410002(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^TRT511, 'C/-')) AND (^TRT512 = BLANK))
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT512 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((statementOfDistributionToBeneficiary.TRT511 == null ? false : statementOfDistributionToBeneficiary.TRT511.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT512) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT512", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT512) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410002_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^TRT162, 'C/-')) AND (^TRT309 = BLANK))
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT309 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.TRT162 == null ? false : report.TRT162.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.TRT309) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT309", Value = GetValueOrEmpty(report.TRT309) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410002_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^TRT9, 'C/-')) AND (^TRT306 = BLANK))
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT306 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.TRT9 == null ? false : report.TRT9.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.TRT306) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT306", Value = GetValueOrEmpty(report.TRT306) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410002_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410002
            Address Line 2 must be present if Address Line 1 contains 'C/-'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ((Contains(^TRT14, 'C/-')) AND (^TRT312 = BLANK))
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT312 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = ((report.TRT14 == null ? false : report.TRT14.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(report.TRT312) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT312", Value = GetValueOrEmpty(report.TRT312) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410002_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410002
                    Address Line 2 must be present if Address Line 1 contains 'C/-'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF ((pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "C/-") AND (pyde.xx.xx:AddressDetails.Line2.Text = NULLORBLANK))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ((Contains(^TRT205, 'C/-')) AND (^TRT206 = BLANK))
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT206 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((statementOfDistributionToBeneficiary.TRT205 == null ? false : statementOfDistributionToBeneficiary.TRT205.ToUpperInvariant().Contains(@"C/-")) && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT206) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410002", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must be present if Address Line 1 contains 'C/-'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410002"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT206", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT206) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410008(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^TRT513 <> BLANK
            
                    Data Elements:
            
                    ^TRT513 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT513) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT513", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT513) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410008_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT310 <> BLANK
    
            Data Elements:
    
            ^TRT310 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT310) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT310", Value = GetValueOrEmpty(report.TRT310) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410008_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT307 <> BLANK
    
            Data Elements:
    
            ^TRT307 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT307) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT307", Value = GetValueOrEmpty(report.TRT307) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410008_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410008
            Address Line 3 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT313 <> BLANK
    
            Data Elements:
    
            ^TRT313 = RP:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT313) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 3 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT313", Value = GetValueOrEmpty(report.TRT313) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410008_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410008
                    Address Line 3 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line3.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^TRT318 <> BLANK
            
                    Data Elements:
            
                    ^TRT318 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line3.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT318) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410008", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 3 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line3.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410008"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT318", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT318) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410013(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^TRT514 <> BLANK
            
                    Data Elements:
            
                    ^TRT514 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT514) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT514", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT514) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410013_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT311 <> BLANK
    
            Data Elements:
    
            ^TRT311 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT311) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT311", Value = GetValueOrEmpty(report.TRT311) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410013_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT308 <> BLANK
    
            Data Elements:
    
            ^TRT308 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT308) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT308", Value = GetValueOrEmpty(report.TRT308) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410013_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410013
            Address Line 4 must be left blank
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT314 <> BLANK
    
            Data Elements:
    
            ^TRT314 = RP:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT314) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 4 must be left blank",
                    LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT314", Value = GetValueOrEmpty(report.TRT314) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410013_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410013
                    Address Line 4 must be left blank
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line4.Text <> NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^TRT319 <> BLANK
            
                    Data Elements:
            
                    ^TRT319 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line4.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT319) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410013", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 4 must be left blank",
                            LongDescription = @"Lines 3 and 4 of the address are not used on this form",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line4.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410013"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT319", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT319) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410031
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410031(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410031
                    The Tax File Number has failed the Tax File Number algorithm check.
    
                    Legacy Rule Format:
                    IF (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier <> NULLORBLANK) AND (TFNALGORITHM (pyid.xx.xx:Identifiers.TaxFileNumber.Identifier) = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT211 <> BLANK) AND (FailsTFNAlgorithm(^TRT211))
            
                    Data Elements:
            
                    ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT211) != true && FailsTFNAlgorithm(statementOfDistributionToBeneficiary.TRT211));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410031", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN must be a valid TFN",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410031"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT211", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT211) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410038()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^TRT161, 'T/A ')) AND (EndsWithSet(^TRT161, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.TRT161, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.TRT161, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410038_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^TRT24, 'T/A ')) AND (EndsWithSet(^TRT24, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.TRT24, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.TRT24, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410038_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410038
                    Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (StartsWith(^TRT203, 'T/A ')) AND (EndsWithSet(^TRT203, '" Pship"," P'ship"," P/ship"'))
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(statementOfDistributionToBeneficiary.TRT203, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(statementOfDistributionToBeneficiary.TRT203, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410038_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^TRT6, 'T/A ')) AND (EndsWithSet(^TRT6, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (IsMatch(report.TRT6, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.TRT6, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410038_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410038
            Non-individual name field cannot contain 'Pship', 'P'ship or 'P/Ship'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text STARTSWITH "T/A ") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" Pship"," P'ship"," P/ship"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (StartsWith(^TRT8, 'T/A ')) AND (EndsWithSet(^TRT8, '" Pship"," P'ship"," P/ship"'))
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (IsMatch(report.TRT8, @"^(T/A )",RegexOptions.IgnoreCase) && IsMatch(report.TRT8, @"( Pship| P'ship| P/ship)$",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410038", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-individual name field cannot start with 'T/A' and end with 'Pship', 'P'ship or 'P/Ship'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410039()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT161  <> BLANK) AND (NotContainsSet(^TRT161, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT161) != true && !(IsMatch(report.TRT161, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410039_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT24  <> BLANK) AND (NotContainsSet(^TRT24, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT24) != true && !(IsMatch(report.TRT24, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410039_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410039
                    Organisation name must contain at least one alpha or numeric character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT203  <> BLANK) AND (NotContainsSet(^TRT203, '"a-z", "A-Z", "0-9"'))
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT203, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must contain at least one alpha or numeric character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410039_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT6  <> BLANK) AND (NotContainsSet(^TRT6, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT6) != true && !(IsMatch(report.TRT6, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410039_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410039
            Organisation name must contain at least one alpha or numeric character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK) AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  DOES NOT CONTAIN SET("a-z", "A-Z", "0-9"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT8  <> BLANK) AND (NotContainsSet(^TRT8, '"a-z", "A-Z", "0-9"'))
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT8) != true && !(IsMatch(report.TRT8, @"[a-z]|[A-Z]|[0-9]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410039", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must contain at least one alpha or numeric character",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410063
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410063(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410063
                    Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT201 <> BLANK) AND (NotContainsSet(^TRT201, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT201) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT201, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410063", Severity = ProcessMessageSeverity.Error,
                            Description = @"Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410063
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410063_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410063
            Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT22 <> BLANK) AND (NotContainsSet(^TRT22, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT22) != true && !(IsMatch(report.TRT22, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410063", Severity = ProcessMessageSeverity.Error,
                    Description = @"Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410063"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410131
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410131(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410131
                    Other Given Name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT202 <> BLANK) AND (NotContainsSet(^TRT202, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT202) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT202, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410131", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other Given Name must contain at least one alphabetical character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410131
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410131_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410131
            Other Given Name must contain at least one alphabetical character
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text DOES NOT CONTAIN SET("A-Z","a-z"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT23 <> BLANK) AND (NotContainsSet(^TRT23, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT23) != true && !(IsMatch(report.TRT23, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410131", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other Given Name must contain at least one alphabetical character",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410131"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410191(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT508 = TRUE) AND (^TRT519 = NULL)
            
                    Data Elements:
            
                    ^TRT519 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT508 == true && statementOfDistributionToBeneficiary.TRT519 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT519", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT519) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410191_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT316 = TRUE) AND (^TRT374 = NULL)
    
            Data Elements:
    
            ^TRT374 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT316 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT316 == true && report.TRT374 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT316", Value = GetValueOrEmpty(report.TRT316) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT374", Value = GetValueOrEmpty(report.TRT374) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410191_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT315 = TRUE) AND (^TRT367 = NULL)
    
            Data Elements:
    
            ^TRT367 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT315 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT315 == true && report.TRT367 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT315", Value = GetValueOrEmpty(report.TRT315) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT367", Value = GetValueOrEmpty(report.TRT367) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410191_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410191
            Country code must be present for overseas address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT317 = TRUE) AND (^TRT368 = NULL)
    
            Data Elements:
    
            ^TRT368 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT317 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT317 == true && report.TRT368 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                    Description = @"Country code must be present for overseas address",
                    LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT317", Value = GetValueOrEmpty(report.TRT317) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT368", Value = GetValueOrEmpty(report.TRT368) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410191_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410191
                    Country code must be present for overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) AND (pyde.xx.xx:AddressDetails.Country.Code = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT320 = TRUE) AND (^TRT382 = NULL)
            
                    Data Elements:
            
                    ^TRT382 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT320 == true && statementOfDistributionToBeneficiary.TRT382 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410191", Severity = ProcessMessageSeverity.Error,
                            Description = @"Country code must be present for overseas address",
                            LongDescription = @"When the overseas address indicator is 'TRUE', the corresponding Country Code must be supplied",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410191"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT382", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT382) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410194(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^TRT511) > 38
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistributionToBeneficiary.TRT511) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410194_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT162) > 38
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.TRT162) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410194_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT9) > 38
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.TRT9) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410194_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410194
            Address Line 1 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT14) > 38
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.TRT14) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410194_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410194
                    Address Line 1 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line1.Text) > 38
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^TRT205) > 38
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistributionToBeneficiary.TRT205) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410194", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 1 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410194"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410195
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410195(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^TRT512) > 38
            
                    Data Elements:
            
                    ^TRT512 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistributionToBeneficiary.TRT512) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT512", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT512) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410195
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410195_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT309) > 38
    
            Data Elements:
    
            ^TRT309 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.TRT309) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT309", Value = GetValueOrEmpty(report.TRT309) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410195
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410195_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT306) > 38
    
            Data Elements:
    
            ^TRT306 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.TRT306) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT306", Value = GetValueOrEmpty(report.TRT306) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410195
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410195_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410195
            Address Line 2 must not exceed 38 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT312) > 38
    
            Data Elements:
    
            ^TRT312 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (Length(report.TRT312) > 38);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 2 must not exceed 38 characters",
                    LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT312", Value = GetValueOrEmpty(report.TRT312) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410195
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410195_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410195
                    Address Line 2 must not exceed 38 characters
    
                    Legacy Rule Format:
                    IF LENGTH(pyde.xx.xx:AddressDetails.Line2.Text) > 38
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Length(^TRT206) > 38
            
                    Data Elements:
            
                    ^TRT206 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (Length(statementOfDistributionToBeneficiary.TRT206) > 38);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410195", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 2 must not exceed 38 characters",
                            LongDescription = @"Although SBR sets a limit of 50 characters for the Line 2 of an address, the maximum allowable for this form is 38",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410195"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT206", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT206) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410200
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410200(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410200
                    Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT200, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT200, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000424", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410200
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410200_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410200
            Family name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT20, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT20, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000424", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410200"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410201
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410201(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410201
                    Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT200, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT200, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000426", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410201
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410201_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410201
            Family name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT20, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT20, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000426", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410201"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410203
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410203(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410203
                    First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT201, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT201, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000437", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410203
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410203_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410203
            First name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT22, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT22, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000437", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410203"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410204
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410204(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410204
                    First name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT201, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT201, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000438", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410204
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410204_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410204
            First name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT22, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT22, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000438", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410204"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410205(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT511, '"AS ABOVE"'))
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT511, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410205_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT162, '"AS ABOVE"'))
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT162, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410205_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT9, '"AS ABOVE"'))
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT9, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410205_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410205
            Address Line 1 must not contain 'AS ABOVE'
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT14, '"AS ABOVE"'))
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.TRT14, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must not contain 'AS ABOVE'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410205_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410205
                    Address Line 1 must not contain 'AS ABOVE'
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text = FOUND("AS ABOVE"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT205, '"AS ABOVE"'))
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT205, @"(^(AS ABOVE)$)|(^(.*)( AS ABOVE )(.*)$)|(^(AS ABOVE )(.*)$)|(^(.*)( AS ABOVE)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410001", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must not contain 'AS ABOVE'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410205"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410208
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410208(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410208
                    Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT202, '"Exec for","Rep for","Trustee for"'))
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT202, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000449", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410208
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410208_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410208
            Other given name cannot contain "Exec for", "Rep for" or "Trustee for"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("Exec for","Rep for","Trustee for"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT23, '"Exec for","Rep for","Trustee for"'))
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT23, @"(^(Exec for|Rep for|Trustee for)$)|(^(.*)( Exec for | Rep for | Trustee for )(.*)$)|(^(Exec for |Rep for |Trustee for )(.*)$)|(^(.*)( Exec for| Rep for| Trustee for)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000449", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""Exec for"", ""Rep for"" or ""Trustee for""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410208"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410209
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410209(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410209
                    Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT202, '"MR","MRS","MISS","MS"'))
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT202, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000450", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410209
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410209_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410209
            Other given name cannot contain "MR", "MRS", "MISS" or "MS"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("MR","MRS","MISS","MS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT23, '"MR","MRS","MISS","MS"'))
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT23, @"(^(MR|MRS|MISS|MS)$)|(^(.*)( MR | MRS | MISS | MS )(.*)$)|(^(MR |MRS |MISS |MS )(.*)$)|(^(.*)( MR| MRS| MISS| MS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000450", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain ""MR"", ""MRS"", ""MISS"" or ""MS""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410209"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410211(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT519 = 'au') AND  (^TRT508 = TRUE)
            
                    Data Elements:
            
                    ^TRT519 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT519 == @"au" && statementOfDistributionToBeneficiary.TRT508 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT519", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT519) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410211_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT374 = 'au') AND  (^TRT316 = TRUE)
    
            Data Elements:
    
            ^TRT374 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT316 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT374 == @"au" && report.TRT316 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT374", Value = GetValueOrEmpty(report.TRT374) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT316", Value = GetValueOrEmpty(report.TRT316) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410211_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT367 = 'au') AND  (^TRT315 = TRUE)
    
            Data Elements:
    
            ^TRT367 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT315 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT367 == @"au" && report.TRT315 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT367", Value = GetValueOrEmpty(report.TRT367) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT315", Value = GetValueOrEmpty(report.TRT315) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410211_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410211
            Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT368 = 'au') AND  (^TRT317 = TRUE)
    
            Data Elements:
    
            ^TRT368 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT317 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT368 == @"au" && report.TRT317 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                    LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT368", Value = GetValueOrEmpty(report.TRT368) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT317", Value = GetValueOrEmpty(report.TRT317) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410211_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410211
                    Overseas address indicator must not be 'TRUE' if Country Code is 'au'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code = "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = TRUE) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT382 = 'au') AND  (^TRT320 = TRUE)
            
                    Data Elements:
            
                    ^TRT382 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT382 == @"au" && statementOfDistributionToBeneficiary.TRT320 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410211", Severity = ProcessMessageSeverity.Error,
                            Description = @"Overseas address indicator must not be 'TRUE' if Country Code is 'au'",
                            LongDescription = @"The overseas address indicator must be 'FALSE' if the Country Code is 'au' - representing Australia. If the address is an overseas address please supply the correct Country Code",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410211"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT382", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT382) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410212
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410212(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT519 <> BLANK) AND (^TRT519 <> 'au') AND  (^TRT508 = FALSE)
            
                    Data Elements:
            
                    ^TRT519 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT519) != true && statementOfDistributionToBeneficiary.TRT519 != @"au" && statementOfDistributionToBeneficiary.TRT508 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT519", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT519) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410212
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410212_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT374 <> BLANK) AND (^TRT374 <> 'au') AND  (^TRT316 = FALSE)
    
            Data Elements:
    
            ^TRT374 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT316 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT374) != true && report.TRT374 != @"au" && report.TRT316 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT374", Value = GetValueOrEmpty(report.TRT374) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT316", Value = GetValueOrEmpty(report.TRT316) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410212
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410212_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT367 <> BLANK) AND (^TRT367 <> 'au') AND  (^TRT315 = FALSE)
    
            Data Elements:
    
            ^TRT367 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT315 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT367) != true && report.TRT367 != @"au" && report.TRT315 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT367", Value = GetValueOrEmpty(report.TRT367) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT315", Value = GetValueOrEmpty(report.TRT315) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410212
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410212_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410212
            If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT368 <> BLANK) AND (^TRT368 <> 'au') AND  (^TRT317 = FALSE)
    
            Data Elements:
    
            ^TRT368 = RP:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT317 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT368) != true && report.TRT368 != @"au" && report.TRT317 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                    Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                    LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT368", Value = GetValueOrEmpty(report.TRT368) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT317", Value = GetValueOrEmpty(report.TRT317) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.410212
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410212_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410212
                    If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.Country.Code <> NULLORBLANK) AND (pyde.xx.xx:AddressDetails.Country.Code <> "au") AND  (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT382 <> BLANK) AND (^TRT382 <> 'au') AND  (^TRT320 = FALSE)
            
                    Data Elements:
            
                    ^TRT382 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT382) != true && statementOfDistributionToBeneficiary.TRT382 != @"au" && statementOfDistributionToBeneficiary.TRT320 == false);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410212", Severity = ProcessMessageSeverity.Error,
                            Description = @"If an overseas Country Code is used, the Overseas Address indicator must be 'TRUE'",
                            LongDescription = @"If the address is an Australian address, the overseas address indicator must be set to 'FALSE' and you do not need to supply a Country Code. If the address is an overseas address, the overseas address indicator must be set to 'TRUE' and you must supply a Country Code other than 'au'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Country.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410212"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT382", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT382) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410218
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410218(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.410218
                    FirstName must be entered if OtherGivenNames is entered
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
                       RETURN VALIDATION MESSAGE
                     ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND ((^TRT202 <> BLANK) AND (^TRT201 = BLANK))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PersonNameDetails = RP:StatementOfDistributionToBeneficiary:PersonNameDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPExists != false && (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT202) != true && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT201) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.410218", Severity = ProcessMessageSeverity.Error,
                            Description = @"FirstName must be entered if OtherGivenNames is entered",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.410218
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN410218_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.410218
            FirstName must be entered if OtherGivenNames is entered
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails) 
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text <> NULLORBLANK) AND (pyde.xx.xx:PersonNameDetails.GivenName.Text = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
             ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND ((^TRT23 <> BLANK) AND (^TRT22 = BLANK))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
    
            ^PersonNameDetails = RP:MailRecipient:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPExists != false && (string.IsNullOrWhiteSpace(report.TRT23) != true && string.IsNullOrWhiteSpace(report.TRT22) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410218", Severity = ProcessMessageSeverity.Error,
                    Description = @"FirstName must be entered if OtherGivenNames is entered",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.410218"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428016
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428016()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428016
            Tax File Number must pass the TFN algorithm check.
    
            Legacy Rule Format:
            IF (RP:entity.identifier.TFN <> NULLORBLANK) AND (TFNALGORITHM (RP:entity.identifier.TFN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TFN <> BLANK) AND (FailsTFNAlgorithm(^TFN))
    
            Data Elements:
    
            ^TFN = RP
            */
            assertion = (string.IsNullOrWhiteSpace(report.RPIdentifierTFN) != true && FailsTFNAlgorithm(report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.428016", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax File Number has failed the algorithm check",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428016"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428230(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT515, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT515, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428230_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT163, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^TRT163 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT163, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT163", Value = report.TRT163 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428230_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT10, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT10, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428230_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428230
            Suburb/Town/City cannot contain state code
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT15, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
    
            Data Elements:
    
            ^TRT15 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.TRT15, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain state code",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT15", Value = report.TRT15 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428230_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428230
                    Suburb/Town/City cannot contain state code
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text = FOUND("QLD","NSW","VIC","SA","WA","NT","ACT","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT207, '"QLD","NSW","VIC","SA","WA","NT","ACT","TAS"'))
            
                    Data Elements:
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT207, @"(^(QLD|NSW|VIC|SA|WA|NT|ACT|TAS)$)|(^(.*)( QLD | NSW | VIC | SA | WA | NT | ACT | TAS )(.*)$)|(^(QLD |NSW |VIC |SA |WA |NT |ACT |TAS )(.*)$)|(^(.*)( QLD| NSW| VIC| SA| WA| NT| ACT| TAS)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000411", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain state code",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428230"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428247
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428247()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428247
            The Tax Agent number has failed the Tax Agent number algorithm check
    
            Legacy Rule Format:
            IF ((pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) <> NULLORBLANK) AND (TANALGORITHM(pyid.xx.xx:Identifiers.TaxAgentNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT2 <> BLANK) AND (FailsTANAlgorithm(^TRT2))
    
            Data Elements:
    
            ^TRT2 = INT:Identifiers.TaxAgentNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT2) != true && FailsTANAlgorithm(report.TRT2));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.410009", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Tax Agent number has failed the Tax Agent number algorithm check",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428247"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT2", Value = GetValueOrEmpty(report.TRT2) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428253
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428253()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428253
            Invalid ABN
    
            Legacy Rule Format:
            IF (INT:entity.identifier.ABN <> NULLORBLANK) AND (ABNALGORITHM (INT:entity.identifier.ABN) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^ABN <> BLANK) AND (FailsABNAlgorithm(^ABN))
    
            Data Elements:
    
            ^ABN = INT
            */
            assertion = (string.IsNullOrWhiteSpace(report.INTIdentifierABN) != true && FailsABNAlgorithm(report.INTIdentifierABN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.200011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid ABN",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428253"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428254(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT511, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT511 == null ? false : statementOfDistributionToBeneficiary.TRT511.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428254_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT162, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT162 == null ? false : report.TRT162.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428254_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT9, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT9 == null ? false : report.TRT9.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428254_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428254
            Address line 1 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT14, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT14 == null ? false : report.TRT14.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 1 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428254_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428254
                    Address line 1 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line1.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT205, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT205 == null ? false : statementOfDistributionToBeneficiary.TRT205.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000405", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 1 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428254"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428255(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT512, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT512 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT512 == null ? false : statementOfDistributionToBeneficiary.TRT512.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT512", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT512) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428255_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT309, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT309 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT309 == null ? false : report.TRT309.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT309", Value = GetValueOrEmpty(report.TRT309) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428255_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT306, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT306 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT306 == null ? false : report.TRT306.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT306", Value = GetValueOrEmpty(report.TRT306) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428255_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428255
            Address line 2 cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT312, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT312 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT312 == null ? false : report.TRT312.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address line 2 cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT312", Value = GetValueOrEmpty(report.TRT312) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428255_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428255
                    Address line 2 cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.Line2.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT206, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT206 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT206 == null ? false : statementOfDistributionToBeneficiary.TRT206.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000407", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address line 2 cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line2.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428255"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT206", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT206) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428256
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428256(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT515, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT515 == null ? false : statementOfDistributionToBeneficiary.TRT515.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428256
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428256_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT163, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT163 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT163 == null ? false : report.TRT163.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT163", Value = report.TRT163 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428256
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428256_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT10, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT10 == null ? false : report.TRT10.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428256
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428256_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428256
            Suburb/Town/City cannot contain "UNKNOWN"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT15, 'UNKNOWN')
    
            Data Elements:
    
            ^TRT15 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT15 == null ? false : report.TRT15.ToUpperInvariant().Contains(@"UNKNOWN"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT15", Value = report.TRT15 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428256
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428256_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428256
                    Suburb/Town/City cannot contain "UNKNOWN"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS "UNKNOWN")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT207, 'UNKNOWN')
            
                    Data Elements:
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT207 == null ? false : statementOfDistributionToBeneficiary.TRT207.ToUpperInvariant().Contains(@"UNKNOWN"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000409", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain ""UNKNOWN""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428256"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428258()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT161, ' - ')
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT161 == null ? false : report.TRT161.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428258_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT24, ' - ')
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT24 == null ? false : report.TRT24.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428258_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428258
                    Non Individual Name cannot contain space hyphen space combinations
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT203, ' - ')
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT203 == null ? false : statementOfDistributionToBeneficiary.TRT203.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain space hyphen space combinations",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428258_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT6, ' - ')
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT6 == null ? false : report.TRT6.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428258_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428258
            Non Individual Name cannot contain space hyphen space combinations
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS  " - " )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT8, ' - ')
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (report.TRT8 == null ? false : report.TRT8.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain space hyphen space combinations",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428258"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428259()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT161, 'P/L')
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT161 == null ? false : report.TRT161.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428259_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT24, 'P/L')
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT24 == null ? false : report.TRT24.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428259_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428259
                    Non Individual Name cannot contain "P/L"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT203, 'P/L')
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT203 == null ? false : statementOfDistributionToBeneficiary.TRT203.ToUpperInvariant().Contains(@"P/L"));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain ""P/L""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428259_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT6, 'P/L')
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (report.TRT6 == null ? false : report.TRT6.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428259_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428259
            Non Individual Name cannot contain "P/L"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS "P/L")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT8, 'P/L')
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = (report.TRT8 == null ? false : report.TRT8.ToUpperInvariant().Contains(@"P/L"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000414", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain ""P/L""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428260()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT161, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT161, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428260_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT24, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT24, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428260_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428260
                    Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRT203, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT203, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428260_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT6, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT6, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428260_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428260
            Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT8, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.TRT8, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000416", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428261()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^TRT161, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT161, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428261_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^TRT24, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT24, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428261_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428261
                    Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (EndsWithSet(^TRT203, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT203, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428261_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^TRT6, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = IsMatch(report.TRT6, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428261_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428261
            Non Individual Name cannot end with "T/A", "T/A P'ship", "T/A Pship", "T/A P/Ship" or "T/A Partnership"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text ENDSWITH SET(" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (EndsWithSet(^TRT8, '" T/A"," T/A P'ship"," T/A Pship"," T/A P/Ship"," T/A Partnership"'))
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
            */
            assertion = IsMatch(report.TRT8, @"( T/A| T/A P'ship| T/A Pship| T/A P/Ship| T/A Partnership)$",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000417", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non Individual Name cannot end with ""T/A"", ""T/A P'ship"", ""T/A Pship"", ""T/A P/Ship"" or ""T/A Partnership""",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428262
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428262(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428262
                    Family Name cannot contain "space hyphen space"
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT200, ' - ')
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT200 == null ? false : statementOfDistributionToBeneficiary.TRT200.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000423", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name cannot contain ""space hyphen space""",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428262
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428262_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428262
            Family Name cannot contain "space hyphen space"
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT20, ' - ')
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (report.TRT20 == null ? false : report.TRT20.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000423", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name cannot contain ""space hyphen space""",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428262"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428263
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428263(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428263
                    Family name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRT200, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT200, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000427", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428263
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428263_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428263
            Family name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT20, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT20, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000427", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428263"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428264
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428264(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428264
                    First name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT201, ' - ')
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT201 == null ? false : statementOfDistributionToBeneficiary.TRT201.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000434", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428264
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428264_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428264
            First name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT22, ' - ')
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (report.TRT22 == null ? false : report.TRT22.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000434", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428264"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428265
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428265(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428265
                    First name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRT201, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT201, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000439", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428265
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428265_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428265
            First name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT22, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT22, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000439", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428265"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428266
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428266(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428266
                    Other given name cannot contain space hyphen space
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    Contains(^TRT202, ' - ')
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT202 == null ? false : statementOfDistributionToBeneficiary.TRT202.ToUpperInvariant().Contains(@" - "));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000446", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain space hyphen space",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428266
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428266_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428266
            Other given name cannot contain space hyphen space
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS  " - ")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Contains(^TRT23, ' - ')
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (report.TRT23 == null ? false : report.TRT23.ToUpperInvariant().Contains(@" - "));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000446", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain space hyphen space",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428266"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.428267
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428267(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.428267
                    Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (ContainsSet(^TRT202, '"--","''","  "'))
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT202, @"--|''|  ",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000451", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.428267
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN428267_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.428267
            Other given name cannot contain repeated hyphen, apostrophe, or space characters
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text CONTAINS SET("--","''","  ") )
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (ContainsSet(^TRT23, '"--","''","  "'))
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT23, @"--|''|  ",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000451", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain repeated hyphen, apostrophe, or space characters",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.428267"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430206
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430206()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430206
            Hours taken to prepare and complete form must be 1 to 999
    
            Legacy Rule Format:
            IF (pyin.xx.xx:Report.CompletionHours.Number <> NULL) AND (pyin.xx.xx:Report.CompletionHours.Number <> SET (1-999))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT247 <> NULL) AND ((^TRT247 < 1 OR ^TRT247 > 999))
    
            Data Elements:
    
            ^TRT247 = RP:Report.CompletionHours.Number
            */
            assertion = (report.TRT247 != null && (report.TRT247.GetValueOrDefault() < 1 || report.TRT247.GetValueOrDefault() > 999));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430206", Severity = ProcessMessageSeverity.Error,
                    Description = @"Hours taken to prepare and complete form must be 1 to 999",
                    Location = "/xbrli:xbrl/tns:Report.CompletionHours.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430206"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT247", Value = report.TRT247.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430246(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT508 = FALSE) AND (ContainsSet(^TRT515, '0-9'))
            
                    Data Elements:
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT508 == false && IsMatch(statementOfDistributionToBeneficiary.TRT515, @"[0-9]",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.430246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430246_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT316 = FALSE) AND (ContainsSet(^TRT163, '0-9'))
    
            Data Elements:
    
            ^TRT163 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT316 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT316 == false && IsMatch(report.TRT163, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT316", Value = GetValueOrEmpty(report.TRT316) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT163", Value = report.TRT163 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430246_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT315 = FALSE) AND (ContainsSet(^TRT10, '0-9'))
    
            Data Elements:
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT315 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (report.TRT315 == false && IsMatch(report.TRT10, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT315", Value = GetValueOrEmpty(report.TRT315) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430246_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430246
            Suburb/Town/City cannot contain numeric characters
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT317 = FALSE) AND (ContainsSet(^TRT15, '0-9'))
    
            Data Elements:
    
            ^TRT15 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT317 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT317 == false && IsMatch(report.TRT15, @"[0-9]",RegexOptions.IgnoreCase));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City cannot contain numeric characters",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT317", Value = GetValueOrEmpty(report.TRT317) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT15", Value = report.TRT15 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430246_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.430246
                    Suburb/Town/City cannot contain numeric characters
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF (pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE) AND (pyde.xx.xx:AddressDetails.LocalityName.Text CONTAINS NUMERICS SET(0-9))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT320 = FALSE) AND (ContainsSet(^TRT207, '0-9'))
            
                    Data Elements:
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT320 == false && IsMatch(statementOfDistributionToBeneficiary.TRT207, @"[0-9]",RegexOptions.IgnoreCase));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000410", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City cannot contain numeric characters",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430246"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.430252
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430252()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT250) > 200
    
            Data Elements:
    
            ^TRT250 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (Length(report.TRT250) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT250", Value = report.TRT250 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430252
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430252_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT396) > 200
    
            Data Elements:
    
            ^TRT396 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.TRT396) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT396", Value = report.TRT396 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430252
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430252_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430252
            Unstructured name must not be more than 200 characters
    
            Legacy Rule Format:
            IF LENGTH(pyde.xx.xx:PersonUnstructuredName.FullName.Text) > 200
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT389) > 200
    
            Data Elements:
    
            ^TRT389 = RP:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = (Length(report.TRT389) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430252", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not be more than 200 characters",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT389", Value = report.TRT389 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430255()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430255
            Signature date must not be later than today
    
            Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT393 > Today()
    
            Data Elements:
    
            ^TRT393 = INT:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            */
            assertion = (report.TRT393.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255", Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT393", Value = GetValueOrEmptyDate(report.TRT393) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430255_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430255
            Signature date must not be later than today
    
            Legacy Rule Format:
            IF pyin.xx.xx:Declaration.Signature.Date > DATE(TODAY)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT386 > Today()
    
            Data Elements:
    
            ^TRT386 = RP:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
            */
            assertion = (report.TRT386.GetValueOrDefault() > DateTime.Now.Date);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430255", Severity = ProcessMessageSeverity.Error,
                    Description = @"Signature date must not be later than today",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT386", Value = GetValueOrEmptyDate(report.TRT386) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430258
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430258()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430258
            Invalid context. The number of Reporting party: Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Prim)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Prim
            */
            assertion = (Count(report.RPPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430258", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430258"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430259()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430259
            Invalid context. The number of Reporting party: Non-Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.NonPrim)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (Count(report.RPNonPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430259", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Non-Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430259"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430266
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430266()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430266
            Invalid context. The number of Reporting party: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.JAus)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
            */
            assertion = (Count(report.RPJAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430266", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430266"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430267
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430267()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430267
            Invalid context. The number of Reporting party: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.JForeign)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (Count(report.RPJFOREIGNCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430267", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430267"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430269
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430269()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430269
            Invalid context. The number of Reporting party: Primary: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Prim.JAus)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (Count(report.RPJAUSPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430269", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Primary: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430269"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430270
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430270()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430270
            Invalid context. The number of Reporting party: Non-Primary: Australian contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.NonPrim.JAus)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (Count(report.RPJAUSNonPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430270", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Non-Primary: Australian contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430270"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430273
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430273()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430273
            Invalid context. The number of Reporting party: ListedCountry: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CFCLC.JForeign)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (Count(report.RPJFOREIGNCFCLCCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430273", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: ListedCountry: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430273"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430275
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430275()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430275
            Invalid context. The number of Reporting party: UnlistedCountry: Foreign contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.CFCUC.JForeign)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (Count(report.RPJFOREIGNCFCUCCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430275", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: UnlistedCountry: Foreign contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430275"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430279
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430279()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430279
            Invalid context. The number of Reporting Party Instant: Opening contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Opening(Instant))) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.Opening(Instant)
            */
            assertion = (Count(report.RPOpeningInstantCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430279", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Instant: Opening contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430279"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430280
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430280()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430280
            Invalid context. The number of Reporting Party Instant: Closing contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Closing(Instant))) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.Closing(Instant)
            */
            assertion = (Count(report.RPClosingInstantCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430280", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Instant: Closing contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430280"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.430302
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN430302()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.430302
            Invalid context. The number of Reporting party: TOFA contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.TOFA)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
            */
            assertion = (Count(report.RPTOFACount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430302", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: TOFA contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFALastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.430302"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.432037
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN432037()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.432037
            Bank account name must not contain invalid characters _ ! @ $ % [ ] ? = : ; " ,
    
            Legacy Rule Format:
            IF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text CONTAINS SET("_", "!", "@", "$", "%", "]", "[", "?", "=", ":", ";", """, ",")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRT40, '"_", "!", "@", "$", "%", "]", "[", "?", "=", ":", ";", """, ","')
    
            Data Elements:
    
            ^TRT40 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
            */
            assertion = IsMatch(report.TRT40, @"_|!|@|\$|%|\]|\[|\?|=|:|;|""|,",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432037", Severity = ProcessMessageSeverity.Error,
                    Description = @"Bank account name must not contain invalid characters _ ! @ $ % [ ] ? = : ; "" ,",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.432037"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT40", Value = report.TRT40 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.432040
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN432040()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.432040
            Only one EFT bank account may be present
    
            Legacy Rule Format:
            IF COUNT (TUPLE(financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount)) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^FinancialInstitutionAccount) > 1
    
            Data Elements:
    
            ^FinancialInstitutionAccount = RP:FinancialInstitutionAccount
            */
            assertion = (Count(report.FinancialInstitutionAccountCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432040", Severity = ProcessMessageSeverity.Error,
                    Description = @"Only one EFT bank account may be present",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount" + OccurrenceIndex(report.FinancialInstitutionAccountCollectionCount) + "/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.432040"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.432041
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN432041()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.432041
            Financial institution account name cannot be greater than 32 characters
    
            Legacy Rule Format:
            IF LENGTH(pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text) > 32
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT40) > 32
    
            Data Elements:
    
            ^TRT40 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
            */
            assertion = (Length(report.TRT40) > 32);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432041", Severity = ProcessMessageSeverity.Error,
                    Description = @"Bank account name must not be longer than 32 characters",
                    LongDescription = @"32 characters is the Banking Standard and set by the Banking Industry",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.432041"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT40", Value = report.TRT40 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.432267
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN432267()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.432267
            Financial institution name and branch name must not be present
    
            Legacy Rule Format:
            IF (pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionBranchName.Text) <> NULLORBLANK OR (pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionName.Text <> NULLORBLANK)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT406 <> BLANK OR (^TRT407 <> BLANK)
    
            Data Elements:
    
            ^TRT407 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionName.Text
    
            ^TRT406 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionBranchName.Text
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT406) != true || string.IsNullOrWhiteSpace(report.TRT407) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432267", Severity = ProcessMessageSeverity.Error,
                    Description = @"Financial institution name and branch name must not be present",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.432267"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT406", Value = GetValueOrEmpty(report.TRT406) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT407", Value = GetValueOrEmpty(report.TRT407) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434147
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434147(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT508 = FALSE AND (^TRT517 = NULL OR ^TRT516 = NULL))
            
                    Data Elements:
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT517 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT516 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPExists != false && (statementOfDistributionToBeneficiary.TRT508 == false && (statementOfDistributionToBeneficiary.TRT517 == null || statementOfDistributionToBeneficiary.TRT516 == null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT517", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT517) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT516", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT516) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.434147
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434147_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT316 = FALSE AND (^TRT164 = NULL OR ^TRT165 = NULL))
    
            Data Elements:
    
            ^TRT316 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT164 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^TRT165 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && (report.TRT316 == false && (report.TRT164 == null || report.TRT165 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT316", Value = GetValueOrEmpty(report.TRT316) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT164", Value = GetValueOrEmpty(report.TRT164) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT165", Value = GetValueOrEmpty(report.TRT165) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434147
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434147_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT315 = FALSE AND (^TRT11 = NULL OR ^TRT12 = NULL))
    
            Data Elements:
    
            ^TRT315 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT11 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT12 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && (report.TRT315 == false && (report.TRT11 == null || report.TRT12 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT315", Value = GetValueOrEmpty(report.TRT315) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT11", Value = GetValueOrEmpty(report.TRT11) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT12", Value = GetValueOrEmpty(report.TRT12) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434147
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434147_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.434147
            State and post code must be present for an Australian address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT317 = FALSE AND (^TRT16 = NULL OR ^TRT17 = NULL))
    
            Data Elements:
    
            ^TRT317 = RP:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT16 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT17 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && (report.TRT317 == false && (report.TRT16 == null || report.TRT17 == null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                    Description = @"State and post code must be present for an Australian address",
                    LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT317", Value = GetValueOrEmpty(report.TRT317) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT16", Value = GetValueOrEmpty(report.TRT16) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT17", Value = GetValueOrEmpty(report.TRT17) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434147
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434147_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.434147
                    State and post code must be present for an Australian address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.OverseasAddress.Indicator = FALSE AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code = NULL OR pyde.xx.xx:AddressDetails.Postcode.Text = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT320 = FALSE AND (^TRT208 = NULL OR ^TRT209 = NULL))
            
                    Data Elements:
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT208 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT209 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPExists != false && (statementOfDistributionToBeneficiary.TRT320 == false && (statementOfDistributionToBeneficiary.TRT208 == null || statementOfDistributionToBeneficiary.TRT209 == null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.434147", Severity = ProcessMessageSeverity.Error,
                            Description = @"State and post code must be present for an Australian address",
                            LongDescription = @"If the overseas indicator is no (false), then an Australian state code and post code must be present",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.OverseasAddress.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434147"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT208", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT208) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT209", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT209) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.434223
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434223()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.434223
            ABN must pass the ABN algorithm
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT25 <> BLANK) and (FailsABNAlgorithm(^TRT25))
    
            Data Elements:
    
            ^TRT25 = RP:MailRecipient:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT25) != true && FailsABNAlgorithm(report.TRT25));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477", Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT25", Value = GetValueOrEmpty(report.TRT25) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434223
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434223_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.434223
            ABN must pass the ABN algorithm
    
            Legacy Rule Format:
            IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT7 <> BLANK) and (FailsABNAlgorithm(^TRT7))
    
            Data Elements:
    
            ^TRT7 = RP:Identifiers.AustralianBusinessNumber.Identifier
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT7) != true && FailsABNAlgorithm(report.TRT7));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.000477", Severity = ProcessMessageSeverity.Error,
                    Description = @"ABN is invalid",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier[@contextRef='" + report.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT7", Value = GetValueOrEmpty(report.TRT7) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.434223
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN434223_2(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.434223
                    ABN must pass the ABN algorithm
    
                    Legacy Rule Format:
                    IF (pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) and (ABNALGORITHM(pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier) = FALSE)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT107 <> BLANK) and (FailsABNAlgorithm(^TRT107))
            
                    Data Elements:
            
                    ^TRT107 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Identifiers.AustralianBusinessNumber.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(individualNonBusinessWithholdingPaymentDetails.TRT107) != true && FailsABNAlgorithm(individualNonBusinessWithholdingPaymentDetails.TRT107));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.000477", Severity = ProcessMessageSeverity.Error,
                            Description = @"ABN is invalid",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:Identifiers.AustralianBusinessNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.434223"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT107", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT107) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.438010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438010()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438010
            If the income includes personal services income (PSI) then total amount of income and deductions, results test satisfied and business determination indicators must all be completed
    
            Legacy Rule Format:
            IF (RP:bafpr1.xx.xx:Income.PersonalServicesIncomeIncluded.Indicator = TRUE) AND ((RP:bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount = NULL) OR (RP:bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount = NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator = NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator = NULL))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT155 = TRUE) AND ((^TRT450 = NULL) OR (^TRT451 = NULL) OR (^TRT452 = NULL) OR (^TRT453 = NULL))
    
            Data Elements:
    
            ^TRT155 = RP:Income.PersonalServicesIncomeIncluded.Indicator
    
            ^TRT450 = RP:Income.PersonalServicesIncome.Total.Amount
    
            ^TRT451 = RP:Expense.PersonalServicesIncome.Deduction.Amount
    
            ^TRT452 = RP:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator
    
            ^TRT453 = RP:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator
            */
            assertion = (report.TRT155 == true && (report.TRT450 == null || report.TRT451 == null || report.TRT452 == null || report.TRT453 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438010", Severity = ProcessMessageSeverity.Error,
                    Description = @"Personal services income section completed incorrectly",
                    LongDescription = @"If the income includes personal services income (PSI) then total amount of income and deductions, results test satisfied and business determination indicators must all be completed",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT155", Value = GetValueOrEmpty(report.TRT155) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT450", Value = GetValueOrEmpty(report.TRT450) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT451", Value = GetValueOrEmpty(report.TRT451) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT452", Value = GetValueOrEmpty(report.TRT452) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT453", Value = GetValueOrEmpty(report.TRT453) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438011
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438011()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438011
            If the income does not include personal services income (PSI) then total amount of income and deductions,results test satisfied,business determination,unrelated clients test,employment test and business premises test indicators must not be completed
    
            Legacy Rule Format:
            IF (RP:bafpr1.xx.xx:Income.PersonalServicesIncomeIncluded.Indicator = FALSE) AND ((RP:bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> NULL) OR (RP:bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.UnrelatedClientsTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.EmploymentTestSatisfied.Indicator <> NULL) OR (RP:rvctc2.xx.xx:IncomeTax.PersonalServicesIncome.BusinessPremisesTestSatisfied.Indicator <> NULL))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT155 = FALSE) AND ((^TRT450 <> NULL) OR (^TRT451 <> NULL) OR (^TRT452 <> NULL) OR (^TRT453 <> NULL) OR (^TRT455 <> NULL) OR (^TRT456 <> NULL) OR (^TRT457 <> NULL))
    
            Data Elements:
    
            ^TRT155 = RP:Income.PersonalServicesIncomeIncluded.Indicator
    
            ^TRT450 = RP:Income.PersonalServicesIncome.Total.Amount
    
            ^TRT451 = RP:Expense.PersonalServicesIncome.Deduction.Amount
    
            ^TRT452 = RP:IncomeTax.PersonalServicesIncome.IndividualResultsTestSatisfied.Indicator
    
            ^TRT453 = RP:IncomeTax.PersonalServicesIncome.IndividualPersonalServiceBusinessDeterminationHeld.Indicator
    
            ^TRT455 = RP:IncomeTax.PersonalServicesIncome.UnrelatedClientsTestSatisfied.Indicator
    
            ^TRT456 = RP:IncomeTax.PersonalServicesIncome.EmploymentTestSatisfied.Indicator
    
            ^TRT457 = RP:IncomeTax.PersonalServicesIncome.BusinessPremisesTestSatisfied.Indicator
            */
            assertion = (report.TRT155 == false && (report.TRT450 != null || report.TRT451 != null || report.TRT452 != null || report.TRT453 != null || report.TRT455 != null || report.TRT456 != null || report.TRT457 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Personal services income section completed incorrectly",
                    LongDescription = @"If the income does not include personal services income (PSI) then total amount of income and deductions,results test satisfied,business determination,unrelated clients test,employment test and business premises test indicators must not be completed",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438011"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT155", Value = GetValueOrEmpty(report.TRT155) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT450", Value = GetValueOrEmpty(report.TRT450) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT451", Value = GetValueOrEmpty(report.TRT451) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT452", Value = GetValueOrEmpty(report.TRT452) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT453", Value = GetValueOrEmpty(report.TRT453) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT455", Value = GetValueOrEmpty(report.TRT455) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT456", Value = GetValueOrEmpty(report.TRT456) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT457", Value = GetValueOrEmpty(report.TRT457) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438012
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438012()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438012
            Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> NULL) AND (bafpr3.xx.xx:Income.PersonalServicesIncome.Total.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT450, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT450 = RP:Income.PersonalServicesIncome.Total.Amount
            */
            assertion = NotMonetary(report.TRT450, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438012"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT450", Value = GetValueOrEmpty(report.TRT450) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438013()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438013
            Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> NULL) AND (bafpr3.xx.xx:Expense.PersonalServicesIncome.Deduction.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT451, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT451 = RP:Expense.PersonalServicesIncome.Deduction.Amount
            */
            assertion = NotMonetary(report.TRT451, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.PersonalServicesIncome.Deduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT451", Value = GetValueOrEmpty(report.TRT451) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438014
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438014()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438014
            Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character
    
            Legacy Rule Format:
            IF (bafpo7.xx.xx:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount <> NULL) AND (bafpo7.xx.xx:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT460, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT460 = RP:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount
            */
            assertion = NotMonetary(report.TRT460, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.DepreciatingAssets.AdjustableValuesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438014"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT460", Value = GetValueOrEmpty(report.TRT460) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438015
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438015()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438015
            Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> NULL) AND (bafpr3.xx.xx:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT461, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT461 = RP:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount
            */
            assertion = NotMonetary(report.TRT461, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.DepreciatingAssets.IntangibleBalancingAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT461", Value = GetValueOrEmpty(report.TRT461) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438016
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438016()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438016
            Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT462, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT462 = RP:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount
            */
            assertion = NotMonetary(report.TRT462, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciatingAssets.IntangibleBalancingAdjustment.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438016"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT462", Value = GetValueOrEmpty(report.TRT462) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438018
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438018()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438018
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT189, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT189 = RP.LowCost:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = NotMonetary(report.TRT189, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciatingAssets.SmallBusinessEntity.Amount[@contextRef='" + report.RPLowCostId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438018"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT189", Value = GetValueOrEmpty(report.TRT189) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438018
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438018_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438018
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> NULL) AND (bafpr3.xx.xx:Expense.DepreciatingAssets.SmallBusinessEntity.Amount <> MONETARY(U,11,0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT190, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT190 = RP.GeneralPool:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = NotMonetary(report.TRT190, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciatingAssets.SmallBusinessEntity.Amount[@contextRef='" + report.RPGeneralPoolId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438018"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT190", Value = GetValueOrEmpty(report.TRT190) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438019
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438019()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438019
            If an exemption or rollover has been applied then the exemption or rollover code(s) must be supplied.
    
            Legacy Rule Format:
            IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator = TRUE) AND (COUNT(RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code <> NULL) = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT454 = TRUE) AND (Count(^TRT448) = 0)
    
            Data Elements:
    
            ^TRT454 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            */
            assertion = (report.TRT454 == true && Count(report.TRT448Collection) == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438019", Severity = ProcessMessageSeverity.Error,
                    Description = @"Exemption or rollover code not supplied",
                    LongDescription = @"If an exemption or rollover has been applied then the exemption or rollover code(s) must be supplied",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438019"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT454", Value = GetValueOrEmpty(report.TRT454) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438021
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438021()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438021
            If an exemption or rollover code is provided then the exemption or rollover applied indicator must be true.
    
            Legacy Rule Format:
            IF (COUNT(RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code <> NULL) > 0) AND (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator <> TRUE)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (Count(^TRT448) > 0) AND (^TRT454 <> TRUE)
    
            Data Elements:
    
            ^TRT454 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            */
            assertion = (Count(report.TRT448Collection) > 0 && report.TRT454 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438021", Severity = ProcessMessageSeverity.Error,
                    Description = @"Exemption or rollover applied indicator must be true",
                    LongDescription = @"If an exemption or rollover code(s) is provided then the exemption or rollover applied indicator must be true",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438021"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT454", Value = GetValueOrEmpty(report.TRT454) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP
    
            ^Context = RP
            */
            assertion = (report.RPExists != false && report.RPIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Closing
    
            ^Context = RP:RP.Closing
            */
            assertion = (report.RPClosingExists != false && report.RPClosingIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.Closing(Instant)
    
            ^Context = RP.Closing(Instant)
            */
            assertion = (report.RPClosingInstantExists != false && report.RPClosingInstantIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.GeneralPool
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Investor
    
            ^Context = RP:RP.Investor
            */
            assertion = (report.RPInvestorExists != false && report.RPInvestorIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_5()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.Investor:RP.Investor.YPrevious
    
            ^Context = RP.Investor:RP.Investor.YPrevious
            */
            assertion = (report.RPInvestorYPreviousExists != false && report.RPInvestorYPreviousIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_6()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.JAUS
    
            ^Context = RP:RP.JAUS
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_7()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JAUS:RP.JAUS.NonPrim
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_8()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JAUS:RP.JAUS.Prim
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_9()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.JFOREIGN
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_10()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_11()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_12()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.LimitedPartners
    
            ^Context = RP:RP.LimitedPartners
            */
            assertion = (report.RPLimitedPartnersExists != false && report.RPLimitedPartnersIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_13()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^Context = RP.LimitedPartners:RP.LimitedPartners.YPrevious
            */
            assertion = (report.RPLimitedPartnersYPreviousExists != false && report.RPLimitedPartnersYPreviousIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_14()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.LowCost
    
            ^Context = RP:RP.LowCost
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_15()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.NonPrim
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_16()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Opening
    
            ^Context = RP:RP.Opening
            */
            assertion = (report.RPOpeningExists != false && report.RPOpeningIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_17()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP.Opening(Instant)
    
            ^Context = RP.Opening(Instant)
            */
            assertion = (report.RPOpeningInstantExists != false && report.RPOpeningInstantIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningInstantOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_18()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Prim
    
            ^Context = RP:RP.Prim
            */
            assertion = (report.RPPrimExists != false && report.RPPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_19()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Share.NonPrim
    
            ^Context = RP:RP.Share.NonPrim
            */
            assertion = (report.RPShareNonPrimExists != false && report.RPShareNonPrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_20()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.Share.Prim
    
            ^Context = RP:RP.Share.Prim
            */
            assertion = (report.RPSharePrimExists != false && report.RPSharePrimIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438029_21()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438029
            Entity identifier scheme for ReportingParty must be "http://www.ato.gov.au/tfn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = "RprtPyType.xx.xx:ReportingParty") AND (entity.identifier.scheme <> "http://www.ato.gov.au/tfn")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^IdentifierScheme <> 'http://www.ato.gov.au/tfn')
    
            Data Elements:
    
            ^IdentifierScheme = RP:RP.TOFA
    
            ^Context = RP:RP.TOFA
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAIdentifierScheme != @"http://www.ato.gov.au/tfn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for ReportingParty must be ""http://www.ato.gov.au/tfn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438029"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438030
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438030()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438030
            Entity identifier scheme for Intermediary must be "http://www.ato.gov.au/abn"
    
            Legacy Rule Format:
            IF (RprtPyType.xx.xx:ReportPartyTypeDimension = “RprtPyType.xx.xx:Intermediary”) AND (entity.identifier.scheme <> “http://www.ato.gov.au/abn”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^Context <> NULL AND ^IdentifierScheme <> 'http://www.ato.gov.au/abn'
    
            Data Elements:
    
            ^IdentifierScheme = INT
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTIdentifierScheme != @"http://www.ato.gov.au/abn");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438030", Severity = ProcessMessageSeverity.Error,
                    Description = @"Entity identifier scheme for Intermediary must be ""http://www.ato.gov.au/abn""",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:entity/xbrli:identifier/@scheme",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438030"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438035
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438035()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438035
            Unstructured name must not contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+ have been removed from the technical business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRT250, '",","*"')
    
            Data Elements:
    
            ^TRT250 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = IsMatch(report.TRT250, @",|\*",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT250", Value = report.TRT250 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438035
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438035_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438035
            Unstructured name must not contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+ have been removed from the technical business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRT396, '",","*"')
    
            Data Elements:
    
            ^TRT396 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = IsMatch(report.TRT396, @",|\*",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT396", Value = report.TRT396 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438035
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438035_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438035
            Unstructured name must not contain invalid characters ~ ^ < > + , * 
            Invalid characters ~^<>+ have been removed from the technical business rule as they will be picked by the XBRL validator.
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonUnstructuredName.FullName.Text <> NULLORBLANK AND pyde.xx.xx:PersonUnstructuredName.FullName.Text CONTAINS SET(",","*"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ContainsSet(^TRT389, '",","*"')
    
            Data Elements:
    
            ^TRT389 = RP:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
            */
            assertion = IsMatch(report.TRT389, @",|\*",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438035", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unstructured name must not contain invalid characters ~ ^ < > + , *",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438035"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT389", Value = report.TRT389 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438051
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438051()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438051
            Date of opening balance must match with the start date of Reporting period
    
            Legacy Rule Format:
            IF RP.Opening(Instant):period.instantDate <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^InstantDate1 <> ^StartDate1
    
            Data Elements:
    
            ^StartDate1 = RP
    
            ^InstantDate1 = RP.Opening(Instant)
            */
            assertion = ((report.RPOpeningInstantExists == true) && (report.RPOpeningInstantInstantDate != report.RPStartDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438051", Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of opening balance must match with the start date of Reporting period",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438051"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438052
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438052()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438052
            Date of closing balance must match with the end date of Reporting period
    
            Legacy Rule Format:
            IF RP.Closing(Instant):period.instantDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^InstantDate1 <> ^EndDate
    
            Data Elements:
    
            ^EndDate = RP
    
            ^InstantDate1 = RP.Closing(Instant)
            */
            assertion = ((report.RPClosingInstantExists == true) && (report.RPClosingInstantInstantDate != report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438052", Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of closing balance must match with the end date of Reporting period",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438052"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = INT
    
            ^StartDate1 = RP
    
            ^Context = INT
            */
            assertion = (report.INTExists != false && report.INTStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Closing
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Closing
            */
            assertion = (report.RPClosingExists != false && report.RPClosingStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.GeneralPool
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Investor
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Investor
            */
            assertion = (report.RPInvestorExists != false && report.RPInvestorStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.Investor:RP.Investor.YPrevious
    
            ^StartDate1 = RP
    
            ^Context = RP.Investor:RP.Investor.YPrevious
            */
            assertion = (report.RPInvestorYPreviousExists != false && report.RPInvestorYPreviousStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_5()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.JAUS
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.JAUS
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_6()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.NonPrim
    
            ^StartDate1 = RP
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_7()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.Prim
    
            ^StartDate1 = RP
    
            ^Context = RP.JAUS:RP.JAUS.Prim
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_8()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.JFOREIGN
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.JFOREIGN
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_9()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^StartDate1 = RP
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_10()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^StartDate1 = RP
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_11()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.LimitedPartners
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.LimitedPartners
            */
            assertion = (report.RPLimitedPartnersExists != false && report.RPLimitedPartnersStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_12()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^StartDate1 = RP
    
            ^Context = RP.LimitedPartners:RP.LimitedPartners.YPrevious
            */
            assertion = (report.RPLimitedPartnersYPreviousExists != false && report.RPLimitedPartnersYPreviousStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_13()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.LowCost
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.LowCost
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_14()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.NonPrim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.NonPrim
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_15()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Opening
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Opening
            */
            assertion = (report.RPOpeningExists != false && report.RPOpeningStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_16()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Prim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Prim
            */
            assertion = (report.RPPrimExists != false && report.RPPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_17()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Share.NonPrim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Share.NonPrim
            */
            assertion = (report.RPShareNonPrimExists != false && report.RPShareNonPrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_18()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.Share.Prim
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.Share.Prim
            */
            assertion = (report.RPSharePrimExists != false && report.RPSharePrimStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438053_19()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438053
            Period start date within context must match start date within reporting party context
    
            Legacy Rule Format:
            IF period.startDate  <> RP:period.startDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^StartDate  <> ^StartDate1)
    
            Data Elements:
    
            ^StartDate = RP:RP.TOFA
    
            ^StartDate1 = RP
    
            ^Context = RP:RP.TOFA
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAStartDate != report.RPStartDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438053", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must match Reporting Party period start date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438053"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = INT
    
            ^EndDate1 = INT
    
            ^EndDate = RP
            */
            assertion = (report.INTExists != false && report.INTEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Closing
    
            ^EndDate1 = RP:RP.Closing
    
            ^EndDate = RP
            */
            assertion = (report.RPClosingExists != false && report.RPClosingEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
    
            ^EndDate1 = RP:RP.GeneralPool
    
            ^EndDate = RP
            */
            assertion = (report.RPGeneralPoolExists != false && report.RPGeneralPoolEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Investor
    
            ^EndDate1 = RP:RP.Investor
    
            ^EndDate = RP
            */
            assertion = (report.RPInvestorExists != false && report.RPInvestorEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.Investor:RP.Investor.YPrevious
    
            ^EndDate1 = RP.Investor:RP.Investor.YPrevious
    
            ^EndDate = RP
            */
            assertion = (report.RPInvestorYPreviousExists != false && report.RPInvestorYPreviousEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_5()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.JAUS
    
            ^EndDate1 = RP:RP.JAUS
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSExists != false && report.RPJAUSEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_6()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate1 = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSNonPrimExists != false && report.RPJAUSNonPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_7()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate1 = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate = RP
            */
            assertion = (report.RPJAUSPrimExists != false && report.RPJAUSPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_8()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.JFOREIGN
    
            ^EndDate1 = RP:RP.JFOREIGN
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNExists != false && report.RPJFOREIGNEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_9()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate1 = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNCFCLCExists != false && report.RPJFOREIGNCFCLCEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_10()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate1 = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate = RP
            */
            assertion = (report.RPJFOREIGNCFCUCExists != false && report.RPJFOREIGNCFCUCEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_11()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.LimitedPartners
    
            ^EndDate1 = RP:RP.LimitedPartners
    
            ^EndDate = RP
            */
            assertion = (report.RPLimitedPartnersExists != false && report.RPLimitedPartnersEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_12()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^EndDate1 = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^EndDate = RP
            */
            assertion = (report.RPLimitedPartnersYPreviousExists != false && report.RPLimitedPartnersYPreviousEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_13()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
    
            ^EndDate1 = RP:RP.LowCost
    
            ^EndDate = RP
            */
            assertion = (report.RPLowCostExists != false && report.RPLowCostEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_14()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.NonPrim
    
            ^EndDate1 = RP:RP.NonPrim
    
            ^EndDate = RP
            */
            assertion = (report.RPNonPrimExists != false && report.RPNonPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_15()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Opening
    
            ^EndDate1 = RP:RP.Opening
    
            ^EndDate = RP
            */
            assertion = (report.RPOpeningExists != false && report.RPOpeningEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_16()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Prim
    
            ^EndDate1 = RP:RP.Prim
    
            ^EndDate = RP
            */
            assertion = (report.RPPrimExists != false && report.RPPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_17()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Share.NonPrim
    
            ^EndDate1 = RP:RP.Share.NonPrim
    
            ^EndDate = RP
            */
            assertion = (report.RPShareNonPrimExists != false && report.RPShareNonPrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_18()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.Share.Prim
    
            ^EndDate1 = RP:RP.Share.Prim
    
            ^EndDate = RP
            */
            assertion = (report.RPSharePrimExists != false && report.RPSharePrimEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438054_19()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438054
            Period end date within context must match end date within reporting party context
    
            Legacy Rule Format:
            IF period.endDate <> RP:period.endDate
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^Context <> NULL) AND (^EndDate1 <> ^EndDate)
    
            Data Elements:
    
            ^Context = RP:RP.TOFA
    
            ^EndDate1 = RP:RP.TOFA
    
            ^EndDate = RP
            */
            assertion = (report.RPTOFAExists != false && report.RPTOFAEndDate != report.RPEndDate);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438054", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period end date must match Reporting Party period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:endDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438054"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438058
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438058()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438058
            Invalid context. The number of Reporting party: Investor contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Investor)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Investor
            */
            assertion = (Count(report.RPInvestorCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438058", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Investor contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438058"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438059
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438059()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438059
            Invalid context. The number of Reporting party:Limited Partners contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.LimitedPartners)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.LimitedPartners
            */
            assertion = (Count(report.RPLimitedPartnersCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438059", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Limited Partners contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438059"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438063
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438063()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438063
            If an Intermediary is present, Intermediary declaration statement accepted indicator must be supplied and must be “True”.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^TRT391 <> TRUE
    
            Data Elements:
    
            ^TRT391 = INT:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists != false && report.TRT391 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438063", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration statement accepted indicator is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration statement accepted indicator must be supplied and must be ""True"".",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438063"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT391", Value = GetValueOrEmpty(report.TRT391) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438064
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438064()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438064
            If an Intermediary is not present, Reporting Party declaration statement accepted indicator must be supplied and must be “True”.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.StatementAccepted.Indicator <> TRUE
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^TRT384 <> TRUE
    
            Data Elements:
    
            ^TRT384 = RP:Declaration:Declaration.StatementAccepted.Indicator WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists == false && report.TRT384 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438064", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration statement accepted indicator is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration statement accepted indicator must be supplied and must be ""True"".",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.StatementAccepted.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438064"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT384", Value = GetValueOrEmpty(report.TRT384) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438065
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438065()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438065
            If an Intermediary is present, Intermediary declaration signatory identifier must be supplied.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^TRT394 = BLANK
    
            Data Elements:
    
            ^TRT394 = INT:Declaration:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists != false && string.IsNullOrWhiteSpace(report.TRT394) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438065", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signatory identifier is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration signatory identifier must be supplied.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.SignatoryIdentifier.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438065"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT394", Value = report.TRT394 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438066
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438066()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438066
            If an Intermediary is not present, Reporting Party declaration signatory identifier must be supplied.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.SignatoryIdentifier.Text = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^TRT387 = BLANK
    
            Data Elements:
    
            ^TRT387 = RP:Declaration:Declaration.SignatoryIdentifier.Text WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists == false && string.IsNullOrWhiteSpace(report.TRT387) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438066", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signatory identifier is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration signatory identifier must be supplied.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.SignatoryIdentifier.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438066"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT387", Value = report.TRT387 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438067
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438067()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438067
            If an Intermediary is present, Intermediary declaration signature date must be supplied.
    
            Legacy Rule Format:
            IF INT<>NULL AND INT:pyin.xx.xx:Declaration.Signature.Date = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT <> NULL AND ^TRT393 = NULL
    
            Data Elements:
    
            ^TRT393 = INT:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists != false && report.TRT393 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438067", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signature date is required.",
                    LongDescription = @"If an Intermediary is present, Intermediary declaration signature date must be supplied.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438067"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT393", Value = GetValueOrEmptyDate(report.TRT393) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438068
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438068()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438068
            If an Intermediary is not present, Reporting Party declaration signature date must be supplied.
    
            Legacy Rule Format:
            IF INT=NULL AND RP:pyin.xx.xx:Declaration.Signature.Date = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^INT = NULL AND ^TRT386 = NULL
    
            Data Elements:
    
            ^TRT386 = RP:Declaration:Declaration.Signature.Date WHERE [Declaration.StatementType.Code='TrueAndCorrect']
    
            ^INT = INT
            */
            assertion = (report.INTExists == false && report.TRT386 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438068", Severity = ProcessMessageSeverity.Error,
                    Description = @"Declaration signature date is required.",
                    LongDescription = @"If an Intermediary is not present, Reporting Party declaration signature date must be supplied.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:Declaration.Signature.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438068"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT386", Value = GetValueOrEmptyDate(report.TRT386) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438081
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438081()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438081
            Invalid context. The number of Reporting party: Investor: YPrevious contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Investor.YPrevious)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.Investor:RP.Investor.YPrevious
            */
            assertion = (Count(report.RPInvestorYPreviousCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438081", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:Investor:YPrevious contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438081"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.438082
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN438082()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.438082
            Invalid context. The number of Reporting party:Limited Partners:YPrevious contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.LimitedPartners.YPrevious)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP.LimitedPartners:RP.LimitedPartners.YPrevious
            */
            assertion = (Count(report.RPLimitedPartnersYPreviousCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.438082", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:Limited Partners:YPrevious contexts must not exceed 1.",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.438082"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500130
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500130()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500130
            Person unstructured name must be provided.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF pyde.xx.xx:PersonUnstructuredName.FullName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^TRT250 = BLANK)
    
            Data Elements:
    
            ^TRT250 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
    
            ^PersonUnstructuredName = INT:PersonUnstructuredName
            */
            assertion = (report.PersonUnstructuredNameCollection_Contact_INTExists != false && string.IsNullOrWhiteSpace(report.TRT250) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500130", Severity = ProcessMessageSeverity.Error,
                    Description = @"Person unstructured name must be provided.",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500130"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT250", Value = report.TRT250 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500130
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500130_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500130
            Person unstructured name must be provided.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF pyde.xx.xx:PersonUnstructuredName.FullName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^TRT396 = BLANK)
    
            Data Elements:
    
            ^TRT396 = INT:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
    
            ^PersonUnstructuredName = INT:Declaration:PersonUnstructuredName
            */
            assertion = (report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTExists != false && string.IsNullOrWhiteSpace(report.TRT396) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500130", Severity = ProcessMessageSeverity.Error,
                    Description = @"Person unstructured name must be provided.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_INTOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500130"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT396", Value = report.TRT396 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500130
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500130_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500130
            Person unstructured name must be provided.
    
            Legacy Rule Format:
            WHERE IN TUPLE(personunstructuredname1.xx.xx:PersonUnstructuredName)
            IF pyde.xx.xx:PersonUnstructuredName.FullName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonUnstructuredName <> NULL) AND (^TRT389 = BLANK)
    
            Data Elements:
    
            ^TRT389 = RP:Declaration:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='DeclarationSignatory']
    
            ^PersonUnstructuredName = RP:Declaration:PersonUnstructuredName
            */
            assertion = (report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPExists != false && string.IsNullOrWhiteSpace(report.TRT389) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500130", Severity = ProcessMessageSeverity.Error,
                    Description = @"Person unstructured name must be provided.",
                    Location = "/xbrli:xbrl/tns:Declaration" + OccurrenceIndex(report.DeclarationCollection_TrueAndCorrect_RPOccurrenceIndex) + "/tns:PersonUnstructuredName" + OccurrenceIndex(report.Declaration_PersonUnstructuredNameCollection_DeclarationSignatory_RPOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500130"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT389", Value = report.TRT389 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500132()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^TRT161 = BLANK)
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MTR_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT161) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500132_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^TRT24 = BLANK)
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:MailRecipient:OrganisationNameDetails
            */
            assertion = (report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT24) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500132_2(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500132
                    Organisation name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
                    IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^OrganisationNameDetails <> NULL) AND (^TRT203 = BLANK)
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^OrganisationNameDetails = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                            Description = @"Organisation name must be present.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500132_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^TRT6 = BLANK)
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MN_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT6) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500132_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500132
            Organisation name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(organisationname2.xx.xx:OrganisationNameDetails)
            IF pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^OrganisationNameDetails <> NULL) AND (^TRT8 = BLANK)
    
            Data Elements:
    
            ^TRT8 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='P']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion = (report.OrganisationNameDetailsCollection_MN_P_RPExists != false && string.IsNullOrWhiteSpace(report.TRT8) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Organisation name must be present.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_P_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT8", Value = report.TRT8 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500133(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500133
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT511 = BLANK)
            
                    Data Elements:
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT511) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500133_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT162 = BLANK)
    
            Data Elements:
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT162) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500133_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT9 = BLANK)
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT9) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500133_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500133
            Address Line 1 must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT14 = BLANK)
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.TRT14) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                    Description = @"Address Line 1 must be present.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500133_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500133
                    Address Line 1 must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.Line1.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT205 = BLANK)
            
                    Data Elements:
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT205) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500133", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address Line 1 must be present.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500133"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500134(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500134
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT515 = BLANK)
            
                    Data Elements:
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT515) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500134_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT163 = BLANK)
    
            Data Elements:
    
            ^TRT163 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_BUS_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT163) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT163", Value = report.TRT163 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500134_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT10 = BLANK)
    
            Data Elements:
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_C_RPExists != false && string.IsNullOrWhiteSpace(report.TRT10) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500134_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500134
            Suburb/Town/City must be present in an address
    
            Legacy Rule Format:
            WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
            IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^AddressDetails <> NULL) AND (^TRT15 = BLANK)
    
            Data Elements:
    
            ^TRT15 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^AddressDetails = RP:AddressDetails
            */
            assertion = (report.AddressDetailsCollection_POS_P_RPExists != false && string.IsNullOrWhiteSpace(report.TRT15) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                    Description = @"Suburb/Town/City must be present in an address.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT15", Value = report.TRT15 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500134_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500134
                    Suburb/Town/City must be present in an address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(addressdetails2.xx.xx:AddressDetails)
                    IF pyde.xx.xx:AddressDetails.LocalityName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^AddressDetails <> NULL) AND (^TRT207 = BLANK)
            
                    Data Elements:
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT207) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500134", Severity = ProcessMessageSeverity.Error,
                            Description = @"Suburb/Town/City must be present in an address.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500134"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500135
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500135(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500135
                    Family Name must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text = BLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (^TRT200 = BLANK)
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PersonNameDetails = RP:StatementOfDistributionToBeneficiary:PersonNameDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPExists != false && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500135", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family Name must be present.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500135"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500135
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500135_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500135
            Family Name must be present
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.FamilyName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (^TRT20 = BLANK)
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
    
            ^PersonNameDetails = RP:MailRecipient:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPExists != false && string.IsNullOrWhiteSpace(report.TRT20) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500135", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Name must be present.",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500135"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500136
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500136()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500136
            BSB Number must be greater than 012000
    
            Legacy Rule Format:
            WHERE IN TUPLE(financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount) 
            IF pyid.xx.xx:FinancialInstitutionAccount.BankStateBranch.Number <> SET(012001- 999999)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FinancialInstitutionAccount <> NULL) AND (^TRT38 <> BLANK AND (IsNumeric(^TRT38) = 'FALSE' OR AsNumeric(^TRT38) < 12001 OR AsNumeric(^TRT38) > 999999 OR Length(^TRT38) <> 6))
    
            Data Elements:
    
            ^TRT38 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.BankStateBranch.Number
    
            ^FinancialInstitutionAccount = RP:FinancialInstitutionAccount
            */
            assertion = (report.FinancialInstitutionAccountCollectionExists != false && (string.IsNullOrWhiteSpace(report.TRT38) != true && (IsNumeric(report.TRT38) == false || AsNumeric(report.TRT38) < 12001 || AsNumeric(report.TRT38) > 999999 || Length(report.TRT38) != 6)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500136", Severity = ProcessMessageSeverity.Error,
                    Description = @"BSB Number must be greater than 012000.",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.BankStateBranch.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500136"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT38", Value = report.TRT38 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500137
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500137()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500137
            Account Number and Account Name must be present.
    
            Legacy Rule Format:
            WHERE IN TUPLE(financialinstitutionaccount1.xx.xx:FinancialInstitutionAccount)
            IF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number = BLANK OR pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text = BLANK
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^FinancialInstitutionAccount <> NULL) AND (^TRT39 = BLANK OR ^TRT40 = BLANK)
    
            Data Elements:
    
            ^TRT39 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionAccount.Number
    
            ^TRT40 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionAccountName.Text
    
            ^FinancialInstitutionAccount = RP:FinancialInstitutionAccount
            */
            assertion = (report.FinancialInstitutionAccountCollectionExists != false && (string.IsNullOrWhiteSpace(report.TRT39) == true || string.IsNullOrWhiteSpace(report.TRT40) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500137", Severity = ProcessMessageSeverity.Error,
                    Description = @"Account Number and Account Name must be present.",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccount.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500137"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT39", Value = report.TRT39 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT40", Value = report.TRT40 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500141(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500141
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
                    AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT517 <> BLANK) AND (NotInSet(^TRT517, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
            
                    Data Elements:
            
                    ^TRT517 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT517) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT517, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT517", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT517) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500141_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT164 <> BLANK) AND (NotInSet(^TRT164, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^TRT164 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT164) != true && !(IsMatch(report.TRT164, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_BUS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT164", Value = GetValueOrEmpty(report.TRT164) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500141_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT11 <> BLANK) AND (NotInSet(^TRT11, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^TRT11 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT11) != true && !(IsMatch(report.TRT11, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT11", Value = GetValueOrEmpty(report.TRT11) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500141_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500141
            For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
            Legacy Rule Format:
            IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
            AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^TRT16 <> BLANK) AND (NotInSet(^TRT16, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
    
            Data Elements:
    
            ^TRT16 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT16) != true && !(IsMatch(report.TRT16, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                    Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT16", Value = GetValueOrEmpty(report.TRT16) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500141_4(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500141
                    For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> NULLORBLANK)
                    AND (pyde.xx.xx:AddressDetails.StateOrTerritory.Code <> SET ("ACT","NSW","NT","QLD","SA","VIC","WA","TAS"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^TRT208 <> BLANK) AND (NotInSet(^TRT208, '"ACT","NSW","NT","QLD","SA","VIC","WA","TAS"'))
            
                    Data Elements:
            
                    ^TRT208 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT208) != true && !(IsMatch(statementOfDistributionToBeneficiary.TRT208, @"^(ACT|NSW|NT|QLD|SA|VIC|WA|TAS)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500141", Severity = ProcessMessageSeverity.Error,
                            Description = @"For an Australian address, state or territory must be one of: ACT, NSW, NT, QLD, SA, VIC, WA, TAS",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPOccurrenceIndex) + "/tns:AddressDetails.StateOrTerritory.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500141"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT208", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT208) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500142
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500142(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500142
                    Family name must contain at least one alphabetical character
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
                    IF pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z")
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (^PersonNameDetails <> NULL) AND (NotContainsSet(^TRT200, '"A-Z","a-z"'))
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^PersonNameDetails = RP:StatementOfDistributionToBeneficiary:PersonNameDetails
                    */
                    assertion = (statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPExists != false && !(IsMatch(statementOfDistributionToBeneficiary.TRT200, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500142", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name must contain at least one alphabetical character.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500142"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500142
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500142_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500142
            Family name must contain at least one alphabetical character
    
            Legacy Rule Format:
            WHERE IN TUPLE(personstructuredname3.xx.xx:PersonNameDetails)
            IF pyde.xx.xx:PersonNameDetails.FamilyName.Text DOES NOT CONTAIN SET("A-Z","a-z")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (^PersonNameDetails <> NULL) AND (NotContainsSet(^TRT20, '"A-Z","a-z"'))
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
    
            ^PersonNameDetails = RP:MailRecipient:PersonNameDetails
            */
            assertion = (report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPExists != false && !(IsMatch(report.TRT20, @"[A-Z]|[a-z]",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500142", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name must contain at least one alphabetical character.",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500142"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500145
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500145()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500145
            Account number must contain the following: A to Z a to z or 0 to 9
    
            Legacy Rule Format:
            IF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number <> NULLORBLANK AND ANY CHARACTER OF pyid.xx.xx:FinancialInstitutionAccount.FinancialInstitutionAccount.Number <> SET("a-z","A-Z","0-9")
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT39 <> BLANK AND NotCharacterInSet(^TRT39, '"a-z","A-Z","0-9"')
    
            Data Elements:
    
            ^TRT39 = RP:FinancialInstitutionAccount:FinancialInstitutionAccount.FinancialInstitutionAccount.Number
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT39) != true && !(IsMatch(report.TRT39, @"^[a-zA-Z0-9]*$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500145", Severity = ProcessMessageSeverity.Error,
                    Description = @"Account number contains invalid text",
                    LongDescription = @"Account number must contain the following: A to Z a to z or 0 to 9",
                    Location = "/xbrli:xbrl/tns:FinancialInstitutionAccount/tns:FinancialInstitutionAccount.FinancialInstitutionAccount.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500145"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT39", Value = report.TRT39 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500151
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500151(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500151
                    First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT201, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT201, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010436", Severity = ProcessMessageSeverity.Error,
                            Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500151
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500151_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500151
            First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.GivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT22, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRT22 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT22, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010436", Severity = ProcessMessageSeverity.Error,
                    Description = @"First name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.GivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500151"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT22", Value = GetValueOrEmpty(report.TRT22) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500152
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500152(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500152
                    Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT202, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT202, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.010448", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500152
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500152_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500152
            Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.OtherGivenName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT23, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRT23 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT23, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.010448", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other given name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.OtherGivenName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500152"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT23", Value = GetValueOrEmpty(report.TRT23) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.GEN.500153
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500153(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.GEN.500153
                    Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
                    Legacy Rule Format:
                    IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    (FoundSet(^TRT200, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = IsMatch(statementOfDistributionToBeneficiary.TRT200, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.500153", Severity = ProcessMessageSeverity.Error,
                            Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.GEN.500153
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOGEN500153_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.GEN.500153
            Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR
    
            Legacy Rule Format:
            IF (pyde.xx.xx:PersonNameDetails.FamilyName.Text = FOUND("ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"))
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            (FoundSet(^TRT20, '"ESQ","JNR","JP","MHA","MHR","MLA","MLC","MP","QC","SNR"'))
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = IsMatch(report.TRT20, @"(^(ESQ|JNR|JP|MHA|MHR|MLA|MLC|MP|QC|SNR)$)|(^(.*)( ESQ | JNR | JP | MHA | MHR | MLA | MLC | MP | QC | SNR )(.*)$)|(^(ESQ |JNR |JP |MHA |MHR |MLA |MLC |MP |QC |SNR )(.*)$)|(^(.*)( ESQ| JNR| JP| MHA| MHR| MLA| MLC| MP| QC| SNR)$)",RegexOptions.IgnoreCase);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500153", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family name cannot contain suffix ESQ, JNR, JP, MHA, MHR, MLA, MLC, MP, QC or SNR",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.GEN.500153"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432008()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432008
            Assessment calculation code must be:15, 16, 17, 36, 37
    
            Legacy Rule Format:
            IF [TRT230] <> NULL AND [TRT230] <> SET("15", "16", "17", "36", "37")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT230 <> NULL AND NotInSet(^TRT230, '"15", "16", "17", "36", "37"')
    
            Data Elements:
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT230 != null && !(IsMatch(report.TRT230, @"^(15|16|17|36|37)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432622", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code must be: 15, 16, 17, 36, 37",
                    LongDescription = @"The choice for resident trustee to be assessed to capital gains on behalf of beneficiaries assessment calculation code must be 15 - Trust where the deceased person died less than three years before the end of the income year; 16 - Trust where the deceased person died three or more than three years before the end of the income year; 36 - Inter vivos trusts - no beneficiary entitled to income; 37 - Inter vivos trusts - bankrupt estate or subsection 99A(2) of ITAA 1936 is to be applied.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432013()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432013
            Non-individual name AND Family name are both present. Only one name can be present.
    
            Legacy Rule Format:
            IF [TRT24] <> NULLORBLANK AND [TRT20] <> NULLORBLANK 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT20] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "MailRecipient") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:MailRecipient)
            [TRT24] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:MailRecipient)

            Technical Business Rule Format:
            ^TRT24 <> BLANK AND ^TRT20 <> BLANK
    
            Data Elements:
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT24) != true && string.IsNullOrWhiteSpace(report.TRT20) != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432013", Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of individual AND name of company must not both be present",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:PersonNameDetails" + OccurrenceIndex(report.MailRecipient_PersonNameDetailsCollection_LGL_MailRecipient_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432014
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432014()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432014
            Previous postal address is only required when an address change has occurred since the last return. Delete previous postal address or input the old address if a change of address has occurred.
    
            Legacy Rule Format:
            IF ([TRT14] = [TRT9] AND [TRT312] = [TRT306] AND [TRT15] = [TRT10] AND [TRT16] = [TRT11] AND [TRT17] = [TRT12]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT9] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT10] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT11] = TRT:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT12] = TRT:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT14] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT15] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT16] = TRT:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT17] = TRT:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT306] = TRT:RP:pyde.02.00:AddressDetails.Line2.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)
            [TRT312] = TRT:RP:pyde.02.00:AddressDetails.Line2.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "P") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            ^TRT14 = ^TRT9 AND ^TRT312 = ^TRT306 AND ^TRT15 = ^TRT10 AND ^TRT16 = ^TRT11 AND ^TRT17 = ^TRT12
    
            Data Elements:
    
            ^TRT14 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT11 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT12 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT15 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT16 = RP:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT17 = RP:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
    
            ^TRT306 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
    
            ^TRT312 = RP:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='P']
            */
            assertion = (report.TRT14 == report.TRT9 && report.TRT312 == report.TRT306 && report.TRT15 == report.TRT10 && report.TRT16 == report.TRT11 && report.TRT17 == report.TRT12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430009", Severity = ProcessMessageSeverity.Error,
                    Description = @"Previous postal address must not be the same as current postal address",
                    LongDescription = @"Previous postal address is only required when an address change has occurred since the last return.  Delete previous postal address or input the old address if a change of address has occurred.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_P_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432014"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT14", Value = report.TRT14 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT312", Value = GetValueOrEmpty(report.TRT312) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT306", Value = GetValueOrEmpty(report.TRT306) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT15", Value = report.TRT15 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT16", Value = GetValueOrEmpty(report.TRT16) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT11", Value = GetValueOrEmpty(report.TRT11) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT17", Value = GetValueOrEmpty(report.TRT17) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT12", Value = GetValueOrEmpty(report.TRT12) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432015
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432015()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432015
            Family name or non-individual name must be present.
    
            Legacy Rule Format:
            IF [TRT20] = NULLORBLANK AND [TRT24] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT20] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "MailRecipient") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:MailRecipient)
            [TRT24] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:MailRecipient)

            Technical Business Rule Format:
            ^TRT20 = BLANK AND ^TRT24 = BLANK
    
            Data Elements:
    
            ^TRT24 = RP:MailRecipient:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^TRT20 = RP:MailRecipient:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='MailRecipient']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT20) == true && string.IsNullOrWhiteSpace(report.TRT24) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432015", Severity = ProcessMessageSeverity.Error,
                    Description = @"Name of the trustee to whom notices should be sent must be present",
                    LongDescription = @"If the trustee is an individual, provide the name of the individual. If the trustee is a company, provide the name of the company",
                    Location = "/xbrli:xbrl/tns:MailRecipient/tns:OrganisationNameDetails" + OccurrenceIndex(report.MailRecipient_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT20", Value = report.TRT20 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT24", Value = report.TRT24 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432022
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432022(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432022
                    In the distribution statement, if the assessment calculation code is 140, there must be an S98(4) assessable amount.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214] = "140" AND [TRT229] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT229] = TRT:RP:rvctc4.02.00:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT214 = '140' AND ^TRT229 = NULL
            
                    Data Elements:
            
                    ^TRT229 = RP:StatementOfDistributionToBeneficiary:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT214 == @"140" && statementOfDistributionToBeneficiary.TRT229 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432022", Severity = ProcessMessageSeverity.Error,
                            Description = @"s98(4) assessable amount must be present",
                            LongDescription = @"In a Statement of Distribution, if the Assessment calculation code is 140, then there must be an amount present at 's98(4) assessable amount'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432022"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT229", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT229) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432023
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432023()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432023
            Trust code must be '039', '059', '071', '072', '077', '089', '090', '112', '179', '184', '203', '232' or '233'.
    
            Legacy Rule Format:
            IF [TRT32] <> NULL AND [TRT32] <> SET("039", "059", "071", "072", "077", "089", "090", "112", "179", "184", "203", "232", "233")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code

            Technical Business Rule Format:
            ^TRT32 <> NULL AND NotInSet(^TRT32, '"039", "059", "071", "072", "077", "089", "090", "112", "179", "184", "203", "232", "233"')
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
            assertion = (report.TRT32 != null && !(IsMatch(report.TRT32, @"^(039|059|071|072|077|089|090|112|179|184|203|232|233)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.442800", Severity = ProcessMessageSeverity.Error,
                    Description = @"Trust code must be  '039', '059', '071', '072', '077', '089', '090', '112', '179', '184', '203', '232' or '233'.",
                    LongDescription = @"Trust codes are: 039' - cash management unit trust; '059' - deceased estate;  '071' - discretionary investment trust; '072' - discretionary service trust; '077' - discretionary trading trust; '089' - fixed unit trust-other than public unit trust as described in 179 or 184; '090' - fixed trust-other than a fixed unit trust or public unit trust as described in 089, 179 or 184; '112' - hybrid trust-trust with fixed AND discretionary elements; '179' - public unit trust-listed other than cash management unit trust; '184' - public unit trust-unlisted other than cash management unit trust; '203' - special disability trust; '232' - Testamentary trust; '233' - CCIV sub-fund trust",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432023"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432024
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432024()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432024
            If the Type of trust is '059' (deceased estate), then the assessment calculation code for each Statement of Distribution must be 11,12,13,14, 111, 118, 119 or 120 and the assessment calculation code in the distribution statement where no beneficiary is entitled, if present, must be 15, 16 or 17.
    
            Legacy Rule Format:
            IF [TRT32] = "059" AND ((ANY OCCURRENCE OF ([TRT214] = SET("25", "26", "27", "28", "29", "30", "34", "35", "125", "126", "127", "128", "129", "134", "138", "139", "140"))) OR [TRT230] = SET("36", "37"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT32 = '059' AND ((AnyOccurrence(^TRT214, InSet(^TRT214, '"25", "26", "27", "28", "29", "30", "34", "35", "125", "126", "127", "128", "129", "134", "138", "139", "140"'))) OR InSet(^TRT230, '"36", "37"'))
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT32 == @"059" && ((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => IsMatch(TRT214Repeat.TRT214, @"^(25|26|27|28|29|30|34|35|125|126|127|128|129|134|138|139|140)$"))) || IsMatch(report.TRT230, @"^(36|37)$")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432725", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code(s) must be a Deceased trust estate code",
                    LongDescription = @"If the Type of trust is '059' (deceased estate), then the assessment calculation code for each Statement of Distribution must be 11 to 17, 119, 120 or 134 and the assessment calculation code  in the distribution statement where no beneficiary is entitled, if present, must be 36 or 37.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432024"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432025
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432025()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432025
            The type of trust estate is other than D (deceased estate) on the front cover AND the assessment calculation code in the distribution statement is 11,12,13,14, 111, 118, 119, 120 or the assessment calculation code in the distribution statement (no beneficiary presently entitled) is 15 to 17. The assessment calculation code is invalid for this type of trust.
    
            Legacy Rule Format:
            IF [TRT32] <> "059" AND (ANY OCCURRENCE OF [TRT214] = SET("11", "12", "13", "14", "111", "118", "119", "120") OR [TRT230] = SET("15", "16", "17"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT32 <> '059' AND AnyOccurrence(^TRT214, InSet(^TRT214, '"11", "12", "13", "14", "111", "118", "119", "120"') OR InSet(^TRT230, '"15", "16", "17"'))
    
            Data Elements:
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT32 != @"059" && (report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => IsMatch(TRT214Repeat.TRT214, @"^(11|12|13|14|111|118|119|120)$") || IsMatch(report.TRT230, @"^(15|16|17)$"))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432025", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code must be an Inter vivos trust estate code",
                    LongDescription = @"If the Type of trust estate is not a Deceased estate, (code '059') then the Assessment calculation codes in a beneficiary distribution statement must not be a deceased estate code",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:IncomeTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432025"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432026
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432026()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432026
            If the Type of trust is a special disability trust (code 203), then there must not be any 'income to which no beneficiary is presently entitled' and that Assessment calculation code must be blank
    
            Legacy Rule Format:
            IF [TRT230] <>  NULLORBLANK AND [TRT32] = "203" 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT230 <>  BLANK AND ^TRT32 = '203'
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT230) != true && report.TRT32 == @"203");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432026", Severity = ProcessMessageSeverity.Error,
                    Description = @"Special disability trust must not have income to which no beneficiary is presently entitled",
                    LongDescription = @"If the Type of trust is a special disability trust (code 203), then there must not be any 'income to which no beneficiary is presently entitled' and that Assessment calculation code must be blank",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432026"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432027
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432027()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432027
            If there is a Date of death, then the Type of trust must be a 'deceased estate' - code '059'
    
            Legacy Rule Format:
            IF [TRT34] <> NULL AND [TRT32] <> "059"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT34] = TRT:RP:pyde.02.00:PersonDemographicDetails.Death.Date

            Technical Business Rule Format:
            ^TRT34 <> NULL AND ^TRT32 <> '059'
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT34 = RP:PersonDemographicDetails.Death.Date
            */
            assertion = (report.TRT34 != null && report.TRT32 != @"059");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432027", Severity = ProcessMessageSeverity.Error,
                    Description = @"If there is a Date of death, then the Type of trust must be a 'deceased estate' - code '059'",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432027"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT34", Value = GetValueOrEmptyDate(report.TRT34) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432028
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432028()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432028
            Assessment calculation code 45 or 145 can only be used by Special Disability Trusts. (code 203)
    
            Legacy Rule Format:
            IF [TRT32] = "203" AND ANY OCCURRENCE OF([TRT214] <> SET("45", "145"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)

            Technical Business Rule Format:
            ^TRT32 = '203' AND AnyOccurrence(^TRT214, NotInSet(^TRT214, '"45", "145"'))
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT32 == @"203" && (report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => !(IsMatch(TRT214Repeat.TRT214, @"^(45|145)$",RegexOptions.IgnoreCase)))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432028", Severity = ProcessMessageSeverity.Error,
                    Description = @"Beneficiary assessment calculation code must be '45' or '145' for special disability trust",
                    LongDescription = @"Assessment calculation code 45 or 145 can only be used by Special Disability Trusts.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432028"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432029
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432029()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432029
            A beneficiary must NOT use a special disability Trust Assessment calculation code (code 45 or 145) unless the Type of trust is a special disability trust (code 203)
    
            Legacy Rule Format:
            IF [TRT32] <> "203" AND ANY OCCURRENCE OF([TRT214] = SET("45", "145"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)

            Technical Business Rule Format:
            ^TRT32 <> '203' AND AnyOccurrence(^TRT214, InSet(^TRT214, '"45", "145"'))
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT32 != @"203" && (report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => IsMatch(TRT214Repeat.TRT214, @"^(45|145)$"))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432029", Severity = ProcessMessageSeverity.Error,
                    Description = @"Beneficiary assessment calculation code must not be special disability trust code",
                    LongDescription = @"A beneficiary must NOT use a special disability Trust Assessment calculation code (code 45 or 145) unless the Type of trust is a special disability trust (code 230)",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432029"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432030
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432030()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432030
            Date of death must not be after the financial year of the return
    
            Legacy Rule Format:
            IF [TRT34] <> NULL AND [TRT34] > (([TRT1])&"-06-30") 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT34] = TRT:RP:pyde.02.00:PersonDemographicDetails.Death.Date

            Technical Business Rule Format:
            ^TRT34 <> NULL AND ^TRT34 > AsDate(Concat(^TRT1, '-06-30'))
    
            Data Elements:
    
            ^TRT34 = RP:PersonDemographicDetails.Death.Date
    
            ^TRT1 = RP:Report.TargetFinancial.Year
            */
            assertion = (report.TRT34 != null && report.TRT34.GetValueOrDefault() > AsDate(string.Concat(report.TRT1.GetValueOrDefault(), @"-06-30")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432030", Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of death must not be after the financial year of the return",
                    Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Death.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432030"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT34", Value = GetValueOrEmptyDate(report.TRT34) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432031
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432031()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432031
            Date of death must be present if the Type of trust estate is a deceased estate (code 059)
    
            Legacy Rule Format:
            IF [TRT32] = "059" AND [TRT34] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT34] = TRT:RP:pyde.02.00:PersonDemographicDetails.Death.Date

            Technical Business Rule Format:
            ^TRT32 = '059' AND ^TRT34 = NULL
    
            Data Elements:
    
            ^TRT34 = RP:PersonDemographicDetails.Death.Date
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
            assertion = (report.TRT32 == @"059" && report.TRT34 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432031", Severity = ProcessMessageSeverity.Error,
                    Description = @"Date of death must be present if the Type of trust estate is a deceased estate (code 059)",
                    Location = "/xbrli:xbrl/tns:PersonDemographicDetails.Death.Date",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432031"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT34", Value = GetValueOrEmptyDate(report.TRT34) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432038
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432038()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432038
            If Status of Business code is present, the Description of main business activity AND Industry code are also required
    
            Legacy Rule Format:
            IF [TRT43] <> NULLORBLANK AND ([TRT42] = NULL OR [TRT41] = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT41] = TRT:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            [TRT42] = TRT:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [TRT43] = TRT:RP:pyde.02.11:OrganisationDetails.ActivityEvent.Code

            Technical Business Rule Format:
            ^TRT43 <> BLANK AND (^TRT42 = NULL OR ^TRT41 = BLANK)
    
            Data Elements:
    
            ^TRT41 = RP:OrganisationDetails.MainIncomeActivity.Description
    
            ^TRT42 = RP:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            ^TRT43 = RP:OrganisationDetails.ActivityEvent.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT43) != true && (report.TRT42 == null || string.IsNullOrWhiteSpace(report.TRT41) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430016", Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of main business and industry code must be present",
                    LongDescription = @"If Status of Business code is present, the Description of main business activity and Industry code are also required",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432038"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT43", Value = GetValueOrEmpty(report.TRT43) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT42", Value = GetValueOrEmpty(report.TRT42) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT41", Value = GetValueOrEmpty(report.TRT41) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432039
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432039()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432039
            Description of main business activity AND total business income must be present
    
            Legacy Rule Format:
            IF [TRT42] <> NULL AND 
            (
            ([TRT41] = NULLORBLANK) OR
            ([TRT46] = NULL AND [TRT47] = NULL AND [TRT49] = NULL AND [TRT50] = NULL AND [TRT52] = NULL AND [TRT54] = NULL AND [TRT55] = NULL)
            ) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT41] = TRT:RP:pyde.02.05:OrganisationDetails.MainIncomeActivity.Description
            [TRT42] = TRT:RP:pyde.02.30:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            ^TRT42 <> NULL AND ((^TRT41 = BLANK) OR(^TRT46 = NULL AND ^TRT47 = NULL AND ^TRT49 = NULL AND ^TRT50 = NULL AND ^TRT52 = NULL AND ^TRT54 = NULL AND ^TRT55 = NULL))
    
            Data Elements:
    
            ^TRT41 = RP:OrganisationDetails.MainIncomeActivity.Description
    
            ^TRT42 = RP:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
            */
            assertion = (report.TRT42 != null && (string.IsNullOrWhiteSpace(report.TRT41) == true || report.TRT46 == null && report.TRT47 == null && report.TRT49 == null && report.TRT50 == null && report.TRT52 == null && report.TRT54 == null && report.TRT55 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430015", Severity = ProcessMessageSeverity.Error,
                    Description = @"Description of main business activity and total business income must be present",
                    LongDescription = @"If an Industry code is present, then the 'Description of main business activity' and business income (Primary Production and/or Non-Primary Production) must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.MainIncomeActivity.Description",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432039"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT42", Value = GetValueOrEmpty(report.TRT42) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT41", Value = GetValueOrEmpty(report.TRT41) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432041
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432041()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432041
            If business income (primary production or non-primary production) is present, an Industry code (ANZSIC) must be present
    
            Legacy Rule Format:
            IF ([TRT46] <> NULL OR [TRT47] <> NULL OR [TRT49] <> NULL OR [TRT50] <> NULL OR [TRT52] <> NULL OR [TRT54] <> NULL OR [TRT55] <> NULL) AND [TRT42] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT42] = TRT:RP:pyde.02.48:OrganisationDetails.OrganisationIndustry2006Extended.Code
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            (^TRT46 <> NULL OR ^TRT47 <> NULL OR ^TRT49 <> NULL OR ^TRT50 <> NULL OR ^TRT52 <> NULL OR ^TRT54 <> NULL OR ^TRT55 <> NULL) AND ^TRT42 = NULL
    
            Data Elements:
    
            ^TRT42 = RP:OrganisationDetails.OrganisationIndustry2006Extended.Code
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
            */
            assertion = ((report.TRT46 != null || report.TRT47 != null || report.TRT49 != null || report.TRT50 != null || report.TRT52 != null || report.TRT54 != null || report.TRT55 != null) && report.TRT42 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432041", Severity = ProcessMessageSeverity.Error,
                    Description = @"If  business income (primary production or non-primary production) is present, an Industry code (ANZSIC) must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationIndustry2006Extended.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432041"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT42", Value = GetValueOrEmpty(report.TRT42) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432045
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432045()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432045
            If Net primary production income/loss amount is present, then total business income from primary production (Gross payments where ABN not quoted, Assessable government industry payments or Other business income) must be present
    
            Legacy Rule Format:
            IF [TRT71] <> NULL AND ([TRT46] = NULL AND [TRT50] = NULL AND [TRT54] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^TRT71 <> NULL AND (^TRT46 = NULL AND ^TRT50 = NULL AND ^TRT54 = NULL)
    
            Data Elements:
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
            */
            assertion = (report.TRT71 != null && (report.TRT46 == null && report.TRT50 == null && report.TRT54 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432686", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP business income must be present",
                    LongDescription = @"Net primary production income/loss amount is present without total business income from primary production (Gross payments where ABN not quoted, Assessable government industry payments, Other business income).",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432045"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432046
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432046()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432046
            If Business name of main business or street address of main business is present, then total business income (primary production or non-primary production) must be present
    
            Legacy Rule Format:
            IF ([TRT161] <> NULLORBLANK OR [TRT162] <> NULLORBLANK) AND ([TRT46] = NULL AND [TRT47] = NULL AND [TRT49] = NULL AND [TRT50] = NULL AND [TRT52] = NULL AND [TRT54] = NULL AND [TRT55] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT161] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MTR" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails)
            [TRT162] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            (^TRT161 <> BLANK OR ^TRT162 <> BLANK) AND (^TRT46 = NULL AND ^TRT47 = NULL AND ^TRT49 = NULL AND ^TRT50 = NULL AND ^TRT52 = NULL AND ^TRT54 = NULL AND ^TRT55 = NULL)
    
            Data Elements:
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((string.IsNullOrWhiteSpace(report.TRT161) != true || string.IsNullOrWhiteSpace(report.TRT162) != true) && (report.TRT46 == null && report.TRT47 == null && report.TRT49 == null && report.TRT50 == null && report.TRT52 == null && report.TRT54 == null && report.TRT55 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430018", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total business income amount required",
                    LongDescription = @"Business name of main business or street address of main business is present and total business income (primary production or non-primary production) is missing at business income and expenses.",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432046"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432047
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432047()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432047
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT46] <> NULL AND [TRT46] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT46, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = NotMonetary(report.TRT46, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432047"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432048
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432048()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432048
            Net non-primary production income/loss amount is present without total business income non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding (excluding capital gains), Assessable government industry payments, Other business income).
    
            Legacy Rule Format:
            IF [TRT72] <> NULL AND ([TRT47] = NULL AND [TRT49] = NULL AND [TRT52] = NULL AND [TRT55] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^TRT72 <> NULL AND (^TRT47 = NULL AND ^TRT49 = NULL AND ^TRT52 = NULL AND ^TRT55 = NULL)
    
            Data Elements:
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
            */
            assertion = (report.TRT72 != null && (report.TRT47 == null && report.TRT49 == null && report.TRT52 == null && report.TRT55 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432748", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP business income must be present",
                    LongDescription = @"Net non-primary production income/loss amount is present without total business income non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding (excluding capital gains), Assessable government industry payments, Other business income).",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432048"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432049
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432049()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432049
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT47] <> NULL AND [TRT47] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT47, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = NotMonetary(report.TRT47, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.ABNNotQuotedPaymentGross.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432049"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432050
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432050()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432050
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT49] <> NULL AND [TRT49] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT49, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = NotMonetary(report.TRT49, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432050"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432052
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432052()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432052
            If Credit for tax withheld - foreign resident withholding (excluding capital gains) is present, then Non Primary production Gross payment subject to foreign resident withholding (excluding capital gains) must be present
    
            Legacy Rule Format:
            IF [TRT474] <> NULL AND [TRT49] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            ^TRT474 <> NULL AND ^TRT49 = NULL
    
            Data Elements:
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = (report.TRT474 != null && report.TRT49 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432750", Severity = ProcessMessageSeverity.Error,
                    Description = @"NPP Gross payments subject to foreign resident withholding (excluding capital gains) must be present",
                    LongDescription = @"If Credit for tax withheld - foreign resident withholding (excluding capital gains) is present, then Non Primary production Gross payment subject to foreign resident withholding (excluding capital gains) must be present.",
                    Location = "/xbrli:xbrl/tns:Remuneration.PaymentToForeignResidentGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432052"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432053
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432053()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432053
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT50] <> NULL AND [TRT50] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT50, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            */
            assertion = NotMonetary(report.TRT50, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432053"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432054
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432054()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432054
            If Primary production - Assessable government industry payments action code is present, Primary production - Assessable government industry payments amount must be present AND not zero
    
            Legacy Rule Format:
            IF [TRT361] = TRUE AND ([TRT50] = 0 OR [TRT50] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT361] = TRT:RP.JAUS.Prim:gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator

            Technical Business Rule Format:
            ^TRT361 = TRUE AND (^TRT50 = 0 OR ^TRT50 = NULL)
    
            Data Elements:
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT361 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator
            */
            assertion = (report.TRT361 == true && (report.TRT50 == 0 || report.TRT50 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430030", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP Assessable government industry payments amount must be present",
                    LongDescription = @"If Primary production - Assessable government industry payments action code is present, Primary production - Assessable government industry payments amount must be present and not zero",
                    Location = "/xbrli:xbrl/tns:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432054"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT361", Value = GetValueOrEmpty(report.TRT361) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432055
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432055()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432055
            If Non-primary production - Assessable government industry payments action code is present, Non-primary production - Assessable government industry payments must be present AND not zero
    
            Legacy Rule Format:
            IF [TRT362] = TRUE AND ([TRT52] = 0 OR [TRT52] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT362] = TRT:RP.JAUS.NonPrim:gfagg.02.02:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator

            Technical Business Rule Format:
            ^TRT362 = TRUE AND (^TRT52 = 0 OR ^TRT52 = NULL)
    
            Data Elements:
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT362 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Indicator
            */
            assertion = (report.TRT362 == true && (report.TRT52 == 0 || report.TRT52 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430032", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP Assessable government industry payments amount must be present",
                    LongDescription = @"If Non-primary production - Assessable government industry payments action code is present, Non-primary production - Assessable government industry payments must be present and not zero",
                    Location = "/xbrli:xbrl/tns:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432055"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT362", Value = GetValueOrEmpty(report.TRT362) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432056
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432056()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432056
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT52] <> NULL AND [TRT52] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT52, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            */
            assertion = NotMonetary(report.TRT52, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432056"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432057
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432057()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432057
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT54] <> NULL AND [TRT54] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT54, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
            */
            assertion = NotMonetary(report.TRT54, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432057"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432058
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432058()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432058
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT55] <> NULL AND [TRT55] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT55, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
            */
            assertion = NotMonetary(report.TRT55, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432058"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432059
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432059()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432059
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT56] <> NULL AND [TRT56] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT56] = TRT:RP.JAUS:bafpr1.02.02:Expense.ForeignResidentWithholding.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT56, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT56 = RP.JAUS:Expense.ForeignResidentWithholding.Amount
            */
            assertion = NotMonetary(report.TRT56, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.ForeignResidentWithholding.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432059"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT56", Value = GetValueOrEmpty(report.TRT56) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432060
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432060()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432060
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT57] <> NULL AND [TRT57] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT57] = TRT:RP.JAUS:lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT57, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT57 = RP.JAUS:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
            */
            assertion = NotMonetary(report.TRT57, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432060"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT57", Value = GetValueOrEmpty(report.TRT57) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432061
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432061()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432061
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT58] <> NULL AND [TRT58] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT58] = TRT:RP.JAUS:emsup.02.02:SuperannuationContribution.EmployerContributions.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT58, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT58 = RP.JAUS:SuperannuationContribution.EmployerContributions.Amount
            */
            assertion = NotMonetary(report.TRT58, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationContribution.EmployerContributions.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432061"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT58", Value = GetValueOrEmpty(report.TRT58) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432064
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432064()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432064
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT61] <> NULL AND [TRT61] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT61] = TRT:RP.JAUS:bafpr2.02.04:Expense.Operating.LeaseTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT61, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT61 = RP.JAUS:Expense.Operating.LeaseTotal.Amount
            */
            assertion = NotMonetary(report.TRT61, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.LeaseTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432064"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT61", Value = GetValueOrEmpty(report.TRT61) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432070
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432070()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432070
            The sum of deductions for 'Small business entity simplified depreciation' must not exceed the 'Depreciation expenses' amount
    
            Legacy Rule Format:
            IF [TRT65] < ([TRT189] + [TRT190])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT65] = TRT:RP.JAUS:bafpr1.02.37:Expense.DepreciationAndAmortisation.Amount
            [TRT189] = TRT:RP.LowCost:bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            [TRT190] = TRT:RP.GeneralPool:bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount

            Technical Business Rule Format:
            ^TRT65 < (^TRT189 + ^TRT190)
    
            Data Elements:
    
            ^TRT65 = RP.JAUS:Expense.DepreciationAndAmortisation.Amount
    
            ^TRT189 = RP.LowCost:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
    
            ^TRT190 = RP.GeneralPool:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = (report.TRT65.GetValueOrDefault() < report.TRT189.GetValueOrDefault() + report.TRT190.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432687", Severity = ProcessMessageSeverity.Error,
                    Description = @"Small business entity simplified depreciation must not exceed Depreciation expenses",
                    LongDescription = @"The sum of deductions for 'Small business entity simplified depreciation' must not exceed the 'Depreciation expenses' amount",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciationAndAmortisation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432070"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT65", Value = GetValueOrEmpty(report.TRT65) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT189", Value = GetValueOrEmpty(report.TRT189) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT190", Value = GetValueOrEmpty(report.TRT190) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432071
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432071()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432071
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT65] <> NULL AND [TRT65] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT65] = TRT:RP.JAUS:bafpr1.02.37:Expense.DepreciationAndAmortisation.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT65, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT65 = RP.JAUS:Expense.DepreciationAndAmortisation.Amount
            */
            assertion = NotMonetary(report.TRT65, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciationAndAmortisation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432071"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT65", Value = GetValueOrEmpty(report.TRT65) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432074
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432074()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432074
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT68] <> NULL AND [TRT68] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT68] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOtherTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT68, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT68 = RP.JAUS:Expense.DeductibleOtherTotal.Amount
            */
            assertion = NotMonetary(report.TRT68, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DeductibleOtherTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432074"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT68", Value = GetValueOrEmpty(report.TRT68) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432075
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432075()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432075
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT69] <> NULL AND [TRT69] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT69] = TRT:RP.JAUS:bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT69, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT69 = RP.JAUS:Income.ReconciliationAdjustmentTotal.Amount
            */
            assertion = NotMonetary(report.TRT69, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ReconciliationAdjustmentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432075"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT69", Value = GetValueOrEmpty(report.TRT69) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432078
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432078()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432078
            If Total business income from primary production (Assessable government industry payments, other business income) is present, Primary production Net primary production income or loss from business amount must be present
    
            Legacy Rule Format:
            IF ([TRT46] <> NULL OR [TRT50] <> NULL OR [TRT54] <> NULL) AND [TRT71] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            (^TRT46 <> NULL OR ^TRT50 <> NULL OR ^TRT54 <> NULL) AND ^TRT71 = NULL
    
            Data Elements:
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
            */
            assertion = ((report.TRT46 != null || report.TRT50 != null || report.TRT54 != null) && report.TRT71 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432688", Severity = ProcessMessageSeverity.Error,
                    Description = @"Primary Production Net income or loss from business must be present",
                    LongDescription = @"If Total business income from primary production (Assessable government industry payments, other business income) is present, Primary production Net primary production income or loss from business amount must be present",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432078"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432079
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432079()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432079
            The total of Primary production and Non-primary production Net business income must equal the total of all income amounts, plus Reconciliation items, minus the sum of all expenses amounts
    
            Legacy Rule Format:
            IF ([TRT46] + [TRT47] + [TRT49] + [TRT50] + [TRT52] + [TRT54] + [TRT55]) - ([TRT56] + [TRT57] + [TRT58] + [TRT467] + [TRT468] + [TRT61] + [TRT469] + [TRT470] + [TRT471] + [TRT65] + [TRT472] + [TRT473] + [TRT68]) + [TRT69] + [TRT447] <> ([TRT71] + [TRT72]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT56] = TRT:RP.JAUS:bafpr1.02.02:Expense.ForeignResidentWithholding.Amount
            [TRT57] = TRT:RP.JAUS:lrla.02.16:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
            [TRT58] = TRT:RP.JAUS:emsup.02.02:SuperannuationContribution.EmployerContributions.Amount
            [TRT61] = TRT:RP.JAUS:bafpr2.02.04:Expense.Operating.LeaseTotal.Amount
            [TRT65] = TRT:RP.JAUS:bafpr1.02.37:Expense.DepreciationAndAmortisation.Amount
            [TRT68] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOtherTotal.Amount
            [TRT69] = TRT:RP.JAUS:bafpr1.02.04:Income.ReconciliationAdjustmentTotal.Amount
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [TRT447] = TRT:RP.JAUS:bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount
            [TRT467] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.CostOfSales.Amount
            [TRT468] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.BadDebts.Amount
            [TRT469] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.Rent.Amount
            [TRT470] = TRT:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount
            [TRT471] = TRT:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount
            [TRT472] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount
            [TRT473] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount

            Technical Business Rule Format:
            (^TRT46 + ^TRT47 + ^TRT49 + ^TRT50 + ^TRT52 + ^TRT54 + ^TRT55) - (^TRT56 + ^TRT57 + ^TRT58 + ^TRT467 + ^TRT468 + ^TRT61 + ^TRT469 + ^TRT470 + ^TRT471 + ^TRT65 + ^TRT472 + ^TRT473 + ^TRT68) + ^TRT69 + ^TRT447 <> (^TRT71 + ^TRT72)
    
            Data Elements:
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
    
            ^TRT56 = RP.JAUS:Expense.ForeignResidentWithholding.Amount
    
            ^TRT57 = RP.JAUS:Remuneration.WagesAndSalaries.PaymentsContractorAndConsultant.Amount
    
            ^TRT58 = RP.JAUS:SuperannuationContribution.EmployerContributions.Amount
    
            ^TRT61 = RP.JAUS:Expense.Operating.LeaseTotal.Amount
    
            ^TRT65 = RP.JAUS:Expense.DepreciationAndAmortisation.Amount
    
            ^TRT68 = RP.JAUS:Expense.DeductibleOtherTotal.Amount
    
            ^TRT69 = RP.JAUS:Income.ReconciliationAdjustmentTotal.Amount
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
    
            ^TRT447 = RP.JAUS:Expense.ReconciliationAdjustmentTotal.Amount
    
            ^TRT467 = RP.JAUS:Expense.Operating.CostOfSales.Amount
    
            ^TRT468 = RP.JAUS:Expense.Operating.BadDebts.Amount
    
            ^TRT469 = RP.JAUS:Expense.Operating.Rent.Amount
    
            ^TRT470 = RP.JAUS:Expense.Operating.Interest.Amount
    
            ^TRT471 = RP.JAUS:Expense.Royalties.Amount
    
            ^TRT472 = RP.JAUS:Expense.Operating.MotorVehicle.Amount
    
            ^TRT473 = RP.JAUS:Expense.Operating.RepairsAndMaintenance.Amount
            */
            assertion = (report.TRT46.GetValueOrDefault() + report.TRT47.GetValueOrDefault() + report.TRT49.GetValueOrDefault() + report.TRT50.GetValueOrDefault() + report.TRT52.GetValueOrDefault() + report.TRT54.GetValueOrDefault() + report.TRT55.GetValueOrDefault() - (report.TRT56.GetValueOrDefault() + report.TRT57.GetValueOrDefault() + report.TRT58.GetValueOrDefault() + report.TRT467.GetValueOrDefault() + report.TRT468.GetValueOrDefault() + report.TRT61.GetValueOrDefault() + report.TRT469.GetValueOrDefault() + report.TRT470.GetValueOrDefault() + report.TRT471.GetValueOrDefault() + report.TRT65.GetValueOrDefault() + report.TRT472.GetValueOrDefault() + report.TRT473.GetValueOrDefault() + report.TRT68.GetValueOrDefault()) + report.TRT69.GetValueOrDefault() + report.TRT447.GetValueOrDefault() != report.TRT71.GetValueOrDefault() + report.TRT72.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430050", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Net income or loss must equal total Income, plus Reconciliation items, minus total Expenses",
                    LongDescription = @"The total of Primary production and Non-primary production Net business income must equal the total of all income amounts, plus Reconciliation items, minus the sum of all expenses amounts",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432079"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT56", Value = GetValueOrEmpty(report.TRT56) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT57", Value = GetValueOrEmpty(report.TRT57) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT58", Value = GetValueOrEmpty(report.TRT58) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT467", Value = GetValueOrEmpty(report.TRT467) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT468", Value = GetValueOrEmpty(report.TRT468) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT61", Value = GetValueOrEmpty(report.TRT61) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT469", Value = GetValueOrEmpty(report.TRT469) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT470", Value = GetValueOrEmpty(report.TRT470) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT471", Value = GetValueOrEmpty(report.TRT471) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT65", Value = GetValueOrEmpty(report.TRT65) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT472", Value = GetValueOrEmpty(report.TRT472) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT473", Value = GetValueOrEmpty(report.TRT473) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT68", Value = GetValueOrEmpty(report.TRT68) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT69", Value = GetValueOrEmpty(report.TRT69) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT447", Value = GetValueOrEmpty(report.TRT447) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432080
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432080()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432080
            If Tax withheld where ABN not quoted is present or Credit for tax withheld - Foreign resident withholding (excluding capital gains) is present, then Net income or loss from business must be present
    
            Legacy Rule Format:
            IF ([TRT73] <> NULL OR [TRT474] <> NULL) AND ([TRT71] = NULL AND [TRT72] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            (^TRT73 <> NULL OR ^TRT474 <> NULL) AND (^TRT71 = NULL AND ^TRT72 = NULL)
    
            Data Elements:
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = ((report.TRT73 != null || report.TRT474 != null) && (report.TRT71 == null && report.TRT72 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432752", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net income or loss from business must be present",
                    LongDescription = @"If Tax withheld where ABN not quoted is present or Credit for tax withheld - Foreign resident withholding is present (excluding capital gains), then Net income or loss from business must be present.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432080"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432081
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432081()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432081
            If an amount, greater than zero, is present in the distribution statement at share of income primary production, there must also be an amount greater than zero at primary production income or loss from business or primary production distribution from partnerships or primary production share of net income from trusts.
    
            Legacy Rule Format:
            IF ((ANY OCCURRENCE OF([TRT218]) > 0) OR [TRT234] > 0) AND ([TRT71] = NULL AND [TRT76] = NULL AND [TRT77] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ((AnyOccurrence(^TRT218, ^TRT218 > 0)) OR ^TRT234 > 0) AND (^TRT71 = NULL AND ^TRT76 = NULL AND ^TRT77 = NULL)
    
            Data Elements:
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
            */
            assertion = (((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT218Repeat => TRT218Repeat.TRT218.GetValueOrDefault() > 0)) || report.TRT234.GetValueOrDefault() > 0) && (report.TRT71 == null && report.TRT76 == null && report.TRT77 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432689", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP income/loss from business and/or PP distribution from partnerships or share of net income from trusts must be present",
                    LongDescription = @"If an amount greater than zero is present in a distribution statement at Share of income: primary production, then there must also be an amount greater than zero at primary production income or loss from business, or primary production distribution from partnerships, or primary production share of net income from trusts",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432081"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT218", Value = "TRT218" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432082
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432082()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432082
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT71] <> NULL AND [TRT71] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT71, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
            */
            assertion = NotMonetary(report.TRT71, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432082"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432083
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432083()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432083
            Total business income from non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding (excluding capital gains), Assessable government industry payments, Other business income) is present without net non-primary production income/loss amount.
    
            Legacy Rule Format:
            IF [TRT72] = NULL AND ([TRT47] <> NULL OR [TRT49] <> NULL OR [TRT52] <> NULL OR [TRT55] <> NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^TRT72 = NULL AND (^TRT47 <> NULL OR ^TRT49 <> NULL OR ^TRT52 <> NULL OR ^TRT55 <> NULL)
    
            Data Elements:
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
            */
            assertion = (report.TRT72 == null && (report.TRT47 != null || report.TRT49 != null || report.TRT52 != null || report.TRT55 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432751", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net non-PP income must be present",
                    LongDescription = @"Total business income from non-primary production (Gross payments where ABN not quoted, Gross payments subject to foreign resident withholding (excluding capital gains), Assessable government industry payments, Other business income) is present without net non-primary production income/loss amount.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432083"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432084
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432084()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432084
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT72] <> NULL AND [TRT72] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT72, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
            */
            assertion = NotMonetary(report.TRT72, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432084"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432085
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432085()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432085
            If either Primary production or Non-primary production Gross payments where ABN not quoted are greater than zero, then an amount at Tax withheld where ABN not quoted must be present
    
            Legacy Rule Format:
            IF ([TRT46] > 0 OR [TRT47] > 0) AND [TRT73] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            (^TRT46 > 0 OR ^TRT47 > 0) AND ^TRT73 = NULL
    
            Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = ((report.TRT46.GetValueOrDefault() > 0 || report.TRT47.GetValueOrDefault() > 0) && report.TRT73 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430058", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld where ABN not quoted must be present",
                    LongDescription = @"If either Primary production or Non-primary production Gross payments where ABN not quoted are greater than zero, then an amount at Tax withheld where ABN not quoted must be present",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432085"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432086
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432086()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432086
            Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (PP AND Non-PP) where ABN not quoted.
    
            Legacy Rule Format:
            IF [TRT73] > 0 AND [TRT73] > (([TRT46] + [TRT47]) *0.50) + 1 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            ^TRT73 > 0 AND ^TRT73 > ((^TRT46 + ^TRT47) *0.50) + 1
    
            Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = (report.TRT73.GetValueOrDefault() > 0 && report.TRT73.GetValueOrDefault() > (report.TRT46.GetValueOrDefault() + report.TRT47.GetValueOrDefault()) * 0.50M + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430056", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (PP and Non-PP) where ABN not quoted.",
                    LongDescription = @"Tax withheld where ABN not quoted cannot be greater than 50% of Gross payments (Primary production and Non-primary production) where ABN not quoted.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432086"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432087
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432087()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432087
            If Tax withheld where ABN not quoted is present there must be an amount at Gross payments where ABN not quoted for either Primary production or Non-primary production
    
            Legacy Rule Format:
            IF [TRT73] <> NULL AND [TRT46] = NULL AND [TRT47] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            ^TRT73 <> NULL AND ^TRT46 = NULL AND ^TRT47 = NULL
    
            Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
            */
            assertion = (report.TRT73 != null && report.TRT46 == null && report.TRT47 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430057", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP or Non-PP Gross payments where ABN not quoted must be present",
                    LongDescription = @"If Tax withheld where ABN not quoted is present there must be an amount at Gross payments where ABN not quoted for either Primary production or Non-primary production",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432087"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432089
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432089()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432089
            The sum of 'Credit for tax withheld where ABN not quoted' recorded in the Statements of distribution must equal the sum of 'Credit for tax withheld where ABN not quoted' plus 'Share of credit for tax withheld where ABN not quoted'
    
            Legacy Rule Format:
            IF SUM([TRT220]) + [TRT236] <> [TRT73] + [TRT84] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            [TRT84] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
            [TRT220] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT236] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            Sum(^TRT220) + ^TRT236 <> ^TRT73 + ^TRT84
    
            Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT84 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
    
            ^TRT220 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT236 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT220.GetValueOrDefault())) + report.TRT236.GetValueOrDefault() != report.TRT73.GetValueOrDefault() + report.TRT84.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432089", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Tax withheld where ABN not quoted must equal beneficiaries amounts",
                    LongDescription = @"The sum of 'Credit for tax withheld where ABN not quoted' recorded in the Statements of distribution must equal the sum of 'Credit for tax withheld where ABN not quoted' plus 'Share of credit for tax withheld where ABN not quoted'",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432089"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT220", Value = "TRT220" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT236", Value = GetValueOrEmpty(report.TRT236) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT84", Value = GetValueOrEmpty(report.TRT84) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432090
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432090()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432090
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT73] <> NULL AND [TRT73] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT73] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT73, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT73 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            */
            assertion = NotMonetary(report.TRT73, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432090"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT73", Value = GetValueOrEmpty(report.TRT73) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432091
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432091()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432091
            Credit for tax withheld - foreign resident withholding (excluding capital gains) is greater than 50% of Non PP Gross payments subject to foreign resident withholding (excluding capital gains).
    
            Legacy Rule Format:
            IF [TRT474] > 0 AND [TRT474] > ([TRT49] *0.50 + 1)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            ^TRT474 > 0 AND ^TRT474 > (^TRT49 *0.50 + 1)
    
            Data Elements:
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = (report.TRT474.GetValueOrDefault() > 0 && report.TRT474.GetValueOrDefault() > report.TRT49.GetValueOrDefault() * 0.50M + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432744", Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must not exceed 50% of Non PP gross payments",
                    LongDescription = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must not exceed 50% of Non-primary production Gross payments subject to foreign resident withholding (excluding capital gains).",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432091"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432092
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432092()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432092
            If Non-primary production Gross payments subject to foreign resident withholding (excluding capital gains) is greater than zero, then there must be an amount at Credit for tax withheld - foreign resident withholding (excluding capital gains)
    
            Legacy Rule Format:
            IF ([TRT49] > 0) AND [TRT474] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            (^TRT49 > 0) AND ^TRT474 = NULL
    
            Data Elements:
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
            */
            assertion = (report.TRT49.GetValueOrDefault() > 0 && report.TRT474 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432745", Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for tax withheld - foreign resident withholding (excluding capital gains) must be present",
                    LongDescription = @"If Non-primary production Gross payments subject to foreign resident withholding (excluding capital gains) is greater than zero, then there must be an amount at Credit for tax withheld - foreign resident withholding.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432092"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432093
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432093()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432093
            The sum of 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' recorded in the Statements of distribution must equal the sum of 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' plus 'Share of credit for tax withheld - foreign resident withholding (excluding capital gains)'
    
            Legacy Rule Format:
            IF SUM([TRT476]) + [TRT477] <> [TRT474] + [TRT87] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT87] = TRT:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            [TRT476] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT477] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            Sum(^TRT476) + ^TRT477 <> ^TRT474 + ^TRT87
    
            Data Elements:
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            ^TRT87 = RP.JAUS:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
    
            ^TRT476 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            ^TRT477 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT476.GetValueOrDefault())) + report.TRT477.GetValueOrDefault() != report.TRT474.GetValueOrDefault() + report.TRT87.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432746", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total Credit for Tax withheld - foreign resident withholding (excluding capital gains) must equal beneficiaries amounts",
                    LongDescription = @"The sum of 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' recorded in the Statements of distribution must equal the sum of 'Credit for tax withheld - foreign resident withholding (excluding capital gains)' plus 'Share of credit for tax withheld - foreign resident withholding (excluding capital gains)'.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432093"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT476", Value = "TRT476" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT477", Value = GetValueOrEmpty(report.TRT477) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT87", Value = GetValueOrEmpty(report.TRT87) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432097
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432097()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432097
            If there is an amount present at 'Primary production - Deductions relating to distribution', then there must also be an amount at 'Primary production - Distribution from partnerships' or 'Primary production - Share of net income from trusts'
    
            Legacy Rule Format:
            IF [TRT79] > 0 AND ([TRT76] = NULL AND [TRT77] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT79] = TRT:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            ^TRT79 > 0 AND (^TRT76 = NULL AND ^TRT77 = NULL)
    
            Data Elements:
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT79 = RP.JAUS.Prim:Expense.DistributionDeductionsOther.Amount
            */
            assertion = (report.TRT79.GetValueOrDefault() > 0 && (report.TRT76 == null && report.TRT77 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432690", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP - Income distribution from partnerships and/or share of net income from trusts must be present",
                    LongDescription = @"If there is an amount present at 'Primary production - Deductions relating to distribution', then there must also be an amount at 'Primary production - Distribution from partnerships' or 'Primary production - Share of net income from trusts'",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432097"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT79", Value = GetValueOrEmpty(report.TRT79) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432098
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432098()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432098
            If an amount is present at 'Deductions relating to distribution', 'Share of credit for tax withheld where ABN not quoted', 'Deductions relating to franked distributions from trusts', 'Share of franking credit from franked dividends', 'Share of credit for TFN amounts withheld from interest dividends and unit trust distributions', or 'Share of credit for tax withheld from foreign resident withholding (excluding capital gains)'; then there must be an amount present at 'Primary production - Distribution from partnerships', 'Primary production - Share of net income from trusts', 'Non-primary production - Distribution from partnerships less foreign income', 'Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions' and 'Non-primary production - Franked distributions from trusts'.
    
            Legacy Rule Format:
            IF ([TRT84] > 0 OR [TRT438] > 0 OR [TRT83] > 0 OR [TRT85] > 0 OR [TRT86] > 0 OR [TRT87] > 0) AND ([TRT76] = NULL AND [TRT77] = NULL AND [TRT80] = NULL AND [TRT81] = NULL AND [TRT437] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT83] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [TRT84] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
            [TRT85] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [TRT86] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [TRT87] = TRT:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [TRT438] = TRT:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^TRT84 > 0 OR ^TRT438 > 0 OR ^TRT83 > 0 OR ^TRT85 > 0 OR ^TRT86 > 0 OR ^TRT87 > 0) AND (^TRT76 = NULL AND ^TRT77 = NULL AND ^TRT80 = NULL AND ^TRT81 = NULL AND ^TRT437 = NULL)
    
            Data Elements:
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT83 = RP.JAUS.NonPrim:Expense.DistributionDeductionsOther.Amount
    
            ^TRT84 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
    
            ^TRT85 = RP.JAUS:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            ^TRT86 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            ^TRT87 = RP.JAUS:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
    
            ^TRT438 = RP.JAUS.NonPrim:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = ((report.TRT84.GetValueOrDefault() > 0 || report.TRT438.GetValueOrDefault() > 0 || report.TRT83.GetValueOrDefault() > 0 || report.TRT85.GetValueOrDefault() > 0 || report.TRT86.GetValueOrDefault() > 0 || report.TRT87.GetValueOrDefault() > 0) && (report.TRT76 == null && report.TRT77 == null && report.TRT80 == null && report.TRT81 == null && report.TRT437 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432753", Severity = ProcessMessageSeverity.Error,
                    Description = @"Distribution/share of net income from partnerships or trusts must be present",
                    LongDescription = @"If an amount is present at 'Deductions relating to distribution', 'Share of credit for tax withheld where ABN not quoted', 'Deductions relating to franked distributions from trusts', 'Share of franking credit from franked dividends', 'Share of credit for TFN amounts withheld from interest dividends and unit trust distributions', or 'Share of credit for tax withheld from foreign resident withholding (excluding capital gains)'; then there must be an amount present at 'Primary production - Distribution from partnerships', 'Primary production - Share of net income from trusts', 'Non-primary production - Distribution from partnerships less foreign income', 'Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions' and 'Non-primary production - Franked distributions from trusts'.",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432098"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT84", Value = GetValueOrEmpty(report.TRT84) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT438", Value = GetValueOrEmpty(report.TRT438) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT83", Value = GetValueOrEmpty(report.TRT83) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT85", Value = GetValueOrEmpty(report.TRT85) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT86", Value = GetValueOrEmpty(report.TRT86) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT87", Value = GetValueOrEmpty(report.TRT87) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432099
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432099()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432099
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT76] <> NULL AND [TRT76] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT76, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
            */
            assertion = NotMonetary(report.TRT76, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432099"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432100
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432100()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432100
            If the Primary production - Share of net income from trusts action code is present, then the Primary production - Share of net income from trusts must be present AND greater than zero. An action code is not required for negative or zero amounts
    
            Legacy Rule Format:
            IF [TRT353] <> NULL AND ([TRT77] = NULL OR [TRT77] <= 0) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT353] = TRT:RP.JAUS.Prim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT353 <> NULL AND (^TRT77 = NULL OR ^TRT77 <= 0)
    
            Data Elements:
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT353 = RP.JAUS.Prim:Party.TrustType.Code
            */
            assertion = (report.TRT353 != null && (report.TRT77 == null || report.TRT77.GetValueOrDefault() <= 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432659", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP - Share of net income from trusts amount must be greater than zero if action code is present",
                    LongDescription = @"If the Primary production - Share of net income from trusts action code is present, then the Primary production - Share of net income from trusts must be present and greater than zero. An action code is not required for negative or zero amounts",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432100"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT353", Value = GetValueOrEmpty(report.TRT353) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432102
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432102()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432102
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT79] <> NULL AND [TRT79] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT79] = TRT:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT79, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT79 = RP.JAUS.Prim:Expense.DistributionDeductionsOther.Amount
            */
            assertion = NotMonetary(report.TRT79, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432102"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT79", Value = GetValueOrEmpty(report.TRT79) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432103
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432103()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432103
            If there is an amount at Share of franking credits from franked dividends, and Share of credit for TFN amounts withheld from interest, dividends and unit trust distributions; then there must be an amount at Non-Primary Production Distribution from partnerships less foreign income; Share of net income from trusts less capital gains, foreign income and franked distributions; Deductions relating to distribution; Franked distributions from trusts or Deductions relating to franked distributions from trusts.
    
            Legacy Rule Format:
            IF ([TRT85] <> NULL AND [TRT86] <> NULL) AND ([TRT80] = NULL AND [TRT81] = NULL AND [TRT83] = NULL AND [TRT437] = NULL AND [TRT438] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT83] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [TRT85] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [TRT86] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [TRT438] = TRT:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^TRT85 <> NULL AND ^TRT86 <> NULL) AND (^TRT80 = NULL AND ^TRT81 = NULL AND ^TRT83 = NULL AND ^TRT437 = NULL AND ^TRT438 = NULL)
    
            Data Elements:
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT83 = RP.JAUS.NonPrim:Expense.DistributionDeductionsOther.Amount
    
            ^TRT85 = RP.JAUS:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            ^TRT86 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
    
            ^TRT438 = RP.JAUS.NonPrim:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = (report.TRT85 != null && report.TRT86 != null && (report.TRT80 == null && report.TRT81 == null && report.TRT83 == null && report.TRT437 == null && report.TRT438 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432692", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP distribution amount must be present",
                    LongDescription = @"If there is an amount at Share of franking credits from franked dividends, and Share of credit for TFN amounts withheld from interest, dividends and unit trust distributions; then there must be an amount at Non-Primary Production Distribution from partnerships less foreign income; Share of net income from trusts less capital gains, foreign income and franked distributions; Deductions relating to distribution; Franked distributions from trusts or Deductions relating to franked distributions from trusts.",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionLessForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432103"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT85", Value = GetValueOrEmpty(report.TRT85) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT86", Value = GetValueOrEmpty(report.TRT86) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT83", Value = GetValueOrEmpty(report.TRT83) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT438", Value = GetValueOrEmpty(report.TRT438) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432104
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432104()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432104
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT80] <> NULL AND [TRT80] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT80, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
            */
            assertion = NotMonetary(report.TRT80, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionLessForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432104"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432105
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432105()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432105
            If Non-primary production - Share of net income from trusts action code is present, then Non-primary production- Share of net income from trusts less capital gains, foreign income and franked distributions must be greater than zero. A trusts action code is not required for negative or zero amounts
    
            Legacy Rule Format:
            IF [TRT436] <> NULL AND ([TRT81] = NULL OR [TRT81] <= 0) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT436] = TRT:RP.JAUS.NonPrim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT436 <> NULL AND (^TRT81 = NULL OR ^TRT81 <= 0)
    
            Data Elements:
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT436 = RP.JAUS.NonPrim:Party.TrustType.Code
            */
            assertion = (report.TRT436 != null && (report.TRT81 == null || report.TRT81.GetValueOrDefault() <= 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432662", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP - Share of net income from trusts amount must be present",
                    LongDescription = @"If Non-primary production - Share of net income from trusts action code is present, then Non-primary production- Share of net income from trusts less capital gains, foreign income and franked distributions must be greater than zero.  A trusts action code is not required for negative or zero amounts",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432105"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT436", Value = GetValueOrEmpty(report.TRT436) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432107
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432107()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432107
            If there is an amount present at Non-primary production - Deductions relating to amounts shown at distribution from partnerships less foreign income and share of net income from trusts less capital gains, foreign income and franked distributions, there must be an amount at either Non-primary production - Distribution from partnerships less foreign income, or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions
    
            Legacy Rule Format:
            IF [TRT83] > 0 AND ([TRT80] = NULL AND [TRT81] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT83] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            ^TRT83 > 0 AND (^TRT80 = NULL AND ^TRT81 = NULL)
    
            Data Elements:
    
            ^TRT83 = RP.JAUS.NonPrim:Expense.DistributionDeductionsOther.Amount
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = (report.TRT83.GetValueOrDefault() > 0 && (report.TRT80 == null && report.TRT81 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432664", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP - Deductions relating to distributions/share of net income can not be more than zero",
                    LongDescription = @"If there is an amount present at Non-primary production - Deductions relating to amounts shown at distribution from partnerships less foreign income and share of net income from trusts less capital gains, foreign income and franked distributions, there must be an amount at either Non-primary production - Distribution from partnerships less foreign income, or Non-primary production - Share of net income from trusts less capital gains, foreign income and franked distributions",
                    Location = "/xbrli:xbrl/tns:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432107"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT83", Value = GetValueOrEmpty(report.TRT83) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432108
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432108()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432108
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT83] <> NULL AND [TRT83] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT83] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT83, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT83 = RP.JAUS.NonPrim:Expense.DistributionDeductionsOther.Amount
            */
            assertion = NotMonetary(report.TRT83, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DistributionDeductionsOther.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432108"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT83", Value = GetValueOrEmpty(report.TRT83) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432109
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432109()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432109
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT84] <> NULL AND [TRT84] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT84] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT84, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT84 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.TRT84, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432109"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT84", Value = GetValueOrEmpty(report.TRT84) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432110
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432110()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432110
            If Total net income is greater than zero, then the sum of 'Franking credit' recorded in the Statements of distribution must equal the sum of 'Share of franking credit from franked dividends' and 'Dividends:Franking credit'
    
            Legacy Rule Format:
            IF (([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] > 0) AND (SUM([TRT221]) + [TRT237] <> [TRT85] + [TRT98]) +/- 1)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT85] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            [TRT98] = TRT:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT221] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT237] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144) > 0 AND OutsideRange(Sum(^TRT221) + ^TRT237, ^TRT85 + ^TRT98,1)
    
            Data Elements:
    
            ^TRT85 = RP.JAUS:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
    
            ^TRT98 = RP.JAUS:IncomeTax.FrankingCredits.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
    
            ^TRT237 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() > 0 && OutsideRange((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT221.GetValueOrDefault())) + report.TRT237.GetValueOrDefault(), report.TRT85.GetValueOrDefault() + report.TRT98.GetValueOrDefault(), 1));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432110", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sum of Franking credit amounts must equal beneficiaries amounts",
                    LongDescription = @"If Total net income is greater than zero, then the sum of 'Franking credit' recorded in the Statements of distribution must equal the sum of 'Share of franking credit from franked dividends' and 'Dividends:Franking credit'",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432110"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT221", Value = "TRT221" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT237", Value = GetValueOrEmpty(report.TRT237) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT85", Value = GetValueOrEmpty(report.TRT85) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT98", Value = GetValueOrEmpty(report.TRT98) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432111
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432111()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432111
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT85] <> NULL AND [TRT85] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT85] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT85, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT85 = RP.JAUS:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.TRT85, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.FrankingCreditShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432111"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT85", Value = GetValueOrEmpty(report.TRT85) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432112
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432112()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432112
            The sum of 'TFN amounts withheld' recorded in the Statements of distribution must equal the sum of 'Share of credit for TFN amounts withheld from interest...' plus 'TFN amounts withheld from gross interest' plus 'TFN amounts withheld from dividends'
    
            Legacy Rule Format:
            IF SUM([TRT222]) + [TRT238] <> [TRT86] + [TRT95] + [TRT99]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT86] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            [TRT95] = TRT:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            [TRT99] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount
            [TRT222] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT238] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            Sum(^TRT222) + ^TRT238 <> ^TRT86 + ^TRT95 + ^TRT99
    
            Data Elements:
    
            ^TRT86 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
    
            ^TRT95 = RP.JAUS:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
    
            ^TRT99 = RP.JAUS:IncomeTax.DividendsTFNAmountsWithheld.Amount
    
            ^TRT222 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
    
            ^TRT238 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT222.GetValueOrDefault())) + report.TRT238.GetValueOrDefault() != report.TRT86.GetValueOrDefault() + report.TRT95.GetValueOrDefault() + report.TRT99.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432112", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld from interest must equal beneficiaries amount",
                    LongDescription = @"The sum of 'TFN amounts withheld' recorded in the Statements of distribution must equal the sum of 'Share of credit for TFN amounts withheld from interest...' plus 'TFN amounts withheld from gross interest' plus 'TFN amounts withheld from dividends'",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432112"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT222", Value = "TRT222" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT238", Value = GetValueOrEmpty(report.TRT238) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT86", Value = GetValueOrEmpty(report.TRT86) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT95", Value = GetValueOrEmpty(report.TRT95) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT99", Value = GetValueOrEmpty(report.TRT99) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432113
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432113()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432113
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT86] <> NULL AND [TRT86] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT86] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT86, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT86 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.TRT86, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromInterestAndDividendsWhereTFNNotQuotedShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432113"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT86", Value = GetValueOrEmpty(report.TRT86) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432114
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432114()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432114
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT87] <> NULL AND [TRT87] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT87] = TRT:RP.JAUS:rvctc4.02.00:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT87, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT87 = RP.JAUS:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.TRT87, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.CreditForTaxWithheldFromForeignResidentWithholdingShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432114"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT87", Value = GetValueOrEmpty(report.TRT87) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432118
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432118()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432118
            Interest deductions, capital works deductions or other rental deductions are greater than zero AND gross rent is absent.
    
            Legacy Rule Format:
            IF ([TRT90] > 0 OR [TRT91] > 0 OR [TRT92] > 0) AND [TRT89] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            (^TRT90 > 0 OR ^TRT91 > 0 OR ^TRT92 > 0) AND ^TRT89 = NULL
    
            Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = ((report.TRT90.GetValueOrDefault() > 0 || report.TRT91.GetValueOrDefault() > 0 || report.TRT92.GetValueOrDefault() > 0) && report.TRT89 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430079", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent must be present",
                    LongDescription = @"If Interest deductions, Capital works deductions, or Other rental deductions are greater than zero, Gross rent must be present",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432118"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432119
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432119()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432119
            Gross rent is greater than zero AND no rental expense items are present. (Rental expense items are interest deductions, capital works deductions AND other rental deductions). If there are no rental expenses then zero is acceptable in one of these fields.
    
            Legacy Rule Format:
            IF [TRT89] > 0 AND ([TRT90] = NULL AND [TRT91] = NULL AND [TRT92] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            ^TRT89 > 0 AND (^TRT90 = NULL AND ^TRT91 = NULL AND ^TRT92 = NULL)
    
            Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = (report.TRT89.GetValueOrDefault() > 0 && (report.TRT90 == null && report.TRT91 == null && report.TRT92 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430080", Severity = ProcessMessageSeverity.Error,
                    Description = @"Rent deductions must be present",
                    LongDescription = @"If Gross rent is greater than zero, then there must be an amount at Interest deductions, Capital works deductions or Other rental deductions. If there are no rental deductions, then zero must be present in at least one of these fields",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432119"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432121
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432121()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432121
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT89] <> NULL AND [TRT89] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT89, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
            */
            assertion = NotMonetary(report.TRT89, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Operating.RentalIncomeGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432121"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432123
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432123()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432123
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT90] <> NULL AND [TRT90] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT90, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
            */
            assertion = NotMonetary(report.TRT90, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432123"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432125
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432125()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432125
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT91] <> NULL AND [TRT91] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT91, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
            */
            assertion = NotMonetary(report.TRT91, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.CapitalWorksDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432125"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432127
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432127()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432127
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT92] <> NULL AND [TRT92] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT92, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = NotMonetary(report.TRT92, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432127"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432128
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432128()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432128
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT93] <> NULL AND [TRT93] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT93] = TRT:RP.JAUS:bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT93, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT93 = RP.JAUS:Income.ForestryManagedInvestmentScheme.Amount
            */
            assertion = NotMonetary(report.TRT93, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ForestryManagedInvestmentScheme.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432128"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT93", Value = GetValueOrEmpty(report.TRT93) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432129
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432129()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432129
            Where there are Deductions relating to Australian investment income greater than zero there must be income amounts at either Gross interest, Unfranked dividend amount or franked dividends.
    
            Legacy Rule Format:
            IF [TRT123] > 0 AND ([TRT94] = NULL AND [TRT96] = NULL AND [TRT97] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT94] = TRT:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT123] = TRT:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount

            Technical Business Rule Format:
            ^TRT123 > 0 AND (^TRT94 = NULL AND ^TRT96 = NULL AND ^TRT97 = NULL)
    
            Data Elements:
    
            ^TRT94 = RP.JAUS:Income.Interest.Gross.Amount
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
    
            ^TRT123 = RP.JAUS:Expense.AustralianInvestmentAllowableDeduction.Amount
            */
            assertion = (report.TRT123.GetValueOrDefault() > 0 && (report.TRT94 == null && report.TRT96 == null && report.TRT97 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430106", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross interest or dividend income must be present",
                    LongDescription = @"If Deductions relating to Australian investment income is greater than zero, then Gross interest, Unfranked amount, or Franked amount must be present",
                    Location = "/xbrli:xbrl/tns:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432129"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT123", Value = GetValueOrEmpty(report.TRT123) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT94", Value = GetValueOrEmpty(report.TRT94) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432130
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432130()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432130
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT94] <> NULL AND [TRT94] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT94] = TRT:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT94, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT94 = RP.JAUS:Income.Interest.Gross.Amount
            */
            assertion = NotMonetary(report.TRT94, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432130"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT94", Value = GetValueOrEmpty(report.TRT94) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432131
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432131()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432131
            TFN amounts deducted on interest are greater than or equal to 50% of gross interest.
    
            Legacy Rule Format:
            IF [TRT95] > 0 AND ([TRT94] * 0.5) <= [TRT95] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT94] = TRT:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [TRT95] = TRT:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            ^TRT95 > 0 AND (^TRT94 * 0.5) <= ^TRT95
    
            Data Elements:
    
            ^TRT95 = RP.JAUS:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
    
            ^TRT94 = RP.JAUS:Income.Interest.Gross.Amount
            */
            assertion = (report.TRT95.GetValueOrDefault() > 0 && report.TRT94.GetValueOrDefault() * 0.5M <= report.TRT95.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430093", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld from dividends must not exceed 50% of Gross interest",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432131"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT95", Value = GetValueOrEmpty(report.TRT95) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT94", Value = GetValueOrEmpty(report.TRT94) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432132
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432132()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432132
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT95] <> NULL AND [TRT95] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT95] = TRT:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT95, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT95 = RP.JAUS:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            */
            assertion = NotMonetary(report.TRT95, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432132"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT95", Value = GetValueOrEmpty(report.TRT95) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432133
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432133()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432133
            If an amount greater than zero is present at TFN amounts withheld from dividends, then there must also be an amount greater than zero at Unfranked amount or Franked amount.
    
            Legacy Rule Format:
            IF [TRT99] > 0 AND ([TRT96] = 0 OR [TRT96] = NULL) AND ([TRT97] = 0 OR [TRT97] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT99] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            ^TRT99 > 0 AND (^TRT96 = 0 OR ^TRT96 = NULL) AND (^TRT97 = 0 OR ^TRT97 = NULL)
    
            Data Elements:
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
    
            ^TRT99 = RP.JAUS:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = (report.TRT99.GetValueOrDefault() > 0 && (report.TRT96 == 0 || report.TRT96 == null) && (report.TRT97 == 0 || report.TRT97 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430095", Severity = ProcessMessageSeverity.Error,
                    Description = @"Dividends amounts must be present",
                    LongDescription = @"If an amount greater than zero is present at TFN amounts withheld from dividends, then there must also be an amount greater than zero at Unfranked amount or Franked amount",
                    Location = "/xbrli:xbrl/tns:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432133"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT99", Value = GetValueOrEmpty(report.TRT99) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432134
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432134()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432134
            The amount at TFN amounts withheld from dividends cannot be greater than 50% of Franked amount plus Unfranked amounts.
    
            Legacy Rule Format:
            IF ([TRT99] > 0 AND [TRT96] > 0 AND [TRT97] > 0) AND ([TRT96] + (([TRT97]*0.5) + 1)) < [TRT99] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT99] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            (^TRT99 > 0 AND ^TRT96 > 0 AND ^TRT97 > 0) AND (^TRT96 + ((^TRT97*0.5) + 1)) < ^TRT99
    
            Data Elements:
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
    
            ^TRT99 = RP.JAUS:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = (report.TRT99.GetValueOrDefault() > 0 && report.TRT96.GetValueOrDefault() > 0 && report.TRT97.GetValueOrDefault() > 0 && report.TRT96.GetValueOrDefault() + (report.TRT97.GetValueOrDefault() * 0.5M + 1) < report.TRT99.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430096", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amount withheld must not exceed unfranked dividends plus 50% of franked dividends",
                    LongDescription = @"The amount at TFN amounts withheld from dividends must not be greater than Unfranked amount plus 50% of Franked amount",
                    Location = "/xbrli:xbrl/tns:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432134"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT99", Value = GetValueOrEmpty(report.TRT99) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432135
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432135()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432135
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT96] <> NULL AND [TRT96] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT96, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
            */
            assertion = NotMonetary(report.TRT96, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.DividendsUnfranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432135"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432136
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432136()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432136
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT97] <> NULL AND [TRT97] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT97, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
            */
            assertion = NotMonetary(report.TRT97, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.DividendsFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432136"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432137
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432137()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432137
            Franking credit cannot be greater than 48% of dividends-franked amount.
    
            Legacy Rule Format:
            IF [TRT98] > 0 AND (([TRT97] * 0.48) < ([TRT98] - 1)) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT98] = TRT:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            ^TRT98 > 0 AND ((^TRT97 * 0.48) < (^TRT98 - 1))
    
            Data Elements:
    
            ^TRT98 = RP.JAUS:IncomeTax.FrankingCredits.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
            */
            assertion = (report.TRT98.GetValueOrDefault() > 0 && report.TRT97.GetValueOrDefault() * 0.48M < report.TRT98.GetValueOrDefault() - 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430098", Severity = ProcessMessageSeverity.Error,
                    Description = @"Franking credit cannot be greater than 48% of dividends-franked amount.",
                    LongDescription = @"Franking credit must not exceed 48% of Dividends - Franked amount",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432137"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT98", Value = GetValueOrEmpty(report.TRT98) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432138
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432138()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432138
            Dividends-franked amount has a value AND dividends-franking credit has no value.
    
            Legacy Rule Format:
            IF [TRT97] > 0 AND [TRT98] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT98] = TRT:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            ^TRT97 > 0 AND ^TRT98 = NULL
    
            Data Elements:
    
            ^TRT98 = RP.JAUS:IncomeTax.FrankingCredits.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
            */
            assertion = (report.TRT97.GetValueOrDefault() > 0 && report.TRT98 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430100", Severity = ProcessMessageSeverity.Error,
                    Description = @"Franking credit amount must be present",
                    LongDescription = @"If Dividends - Franked amount is present, then Franking credit must be present",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432138"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT98", Value = GetValueOrEmpty(report.TRT98) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432139
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432139()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432139
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT98] <> NULL AND [TRT98] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT98] = TRT:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT98, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT98 = RP.JAUS:IncomeTax.FrankingCredits.Amount
            */
            assertion = NotMonetary(report.TRT98, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432139"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT98", Value = GetValueOrEmpty(report.TRT98) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432140
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432140()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432140
            TFN amounts withheld from dividends is greater than 50% of the unfranked amount.
    
            Legacy Rule Format:
            IF [TRT99] > 0 AND (([TRT96]*0.5) + 1) < [TRT99] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [TRT99] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            ^TRT99 > 0 AND ((^TRT96*0.5) + 1) < ^TRT99
    
            Data Elements:
    
            ^TRT99 = RP.JAUS:IncomeTax.DividendsTFNAmountsWithheld.Amount
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
            */
            assertion = (report.TRT99.GetValueOrDefault() > 0 && report.TRT96.GetValueOrDefault() * 0.5M + 1 < report.TRT99.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430102", Severity = ProcessMessageSeverity.Error,
                    Description = @"TFN amounts withheld from dividends must not exceed 50% of the Unfranked amount",
                    Location = "/xbrli:xbrl/tns:IncomeTax.DividendsTFNAmountsWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432140"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT99", Value = GetValueOrEmpty(report.TRT99) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432141
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432141()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432141
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT99] <> NULL AND [TRT99] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT99] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.DividendsTFNAmountsWithheld.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT99, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT99 = RP.JAUS:IncomeTax.DividendsTFNAmountsWithheld.Amount
            */
            assertion = NotMonetary(report.TRT99, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.DividendsTFNAmountsWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432141"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT99", Value = GetValueOrEmpty(report.TRT99) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432142
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432142()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432142
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT100] <> NULL AND [TRT100] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT100] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitLumpSumPaymentTaxedElement.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "O") IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationLumpSumWithholdingPaymentDetails)

            Technical Business Rule Format:
            NotMonetary(^TRT100, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT100 = RP.JAUS:SuperannuationLumpSumWithholdingPaymentDetails:SuperannuationBenefit.DeathBenefitLumpSumPaymentTaxedElement.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
            */
            assertion = NotMonetary(report.TRT100, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationLumpSumWithholdingPaymentDetails" + OccurrenceIndex(report.SuperannuationLumpSumWithholdingPaymentDetailsCollection_O_RPJAUSOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitLumpSumPaymentTaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432142"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT100", Value = GetValueOrEmpty(report.TRT100) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432143
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432143()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432143
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT101] <> NULL AND [TRT101] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT101] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitLumpSumPaymentUntaxedElement.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "O") IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationLumpSumWithholdingPaymentDetails)

            Technical Business Rule Format:
            NotMonetary(^TRT101, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT101 = RP.JAUS:SuperannuationLumpSumWithholdingPaymentDetails:SuperannuationBenefit.DeathBenefitLumpSumPaymentUntaxedElement.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
            */
            assertion = NotMonetary(report.TRT101, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationLumpSumWithholdingPaymentDetails" + OccurrenceIndex(report.SuperannuationLumpSumWithholdingPaymentDetailsCollection_O_RPJAUSOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitLumpSumPaymentUntaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432143"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT101", Value = GetValueOrEmpty(report.TRT101) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432144
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432144()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432144
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:emsup.xx.xx:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount <> NULL 
            AND TRT:RP.JAus:emsup.xx.xx:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT102, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT102 = RP.JAUS:EmploymentTerminationPaymentDetails:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='D']
            */
            assertion = NotMonetary(report.TRT102, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:EmploymentTerminationPaymentDetails" + OccurrenceIndex(report.EmploymentTerminationPaymentDetailsCollection_D_RPJAUSOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432144"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT102", Value = GetValueOrEmpty(report.TRT102) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432144
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432144_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432144
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:emsup.xx.xx:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount <> NULL 
            AND TRT:RP.JAus:emsup.xx.xx:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT103, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT103 = RP.JAUS:EmploymentTerminationPaymentDetails:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
            */
            assertion = NotMonetary(report.TRT103, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:EmploymentTerminationPaymentDetails" + OccurrenceIndex(report.EmploymentTerminationPaymentDetailsCollection_O_RPJAUSOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432144"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT103", Value = GetValueOrEmpty(report.TRT103) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432146
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432146()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432146
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT104] <> NULL AND [TRT104] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT104] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.AustralianGovernmentPensionsAndAllowancesTaxWithheld.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT104, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT104 = RP.JAUS:IncomeTax.AustralianGovernmentPensionsAndAllowancesTaxWithheld.Amount
            */
            assertion = NotMonetary(report.TRT104, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.AustralianGovernmentPensionsAndAllowancesTaxWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432146"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT104", Value = GetValueOrEmpty(report.TRT104) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432148
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432148()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432148
            Tax withheld amounts from Australian government pensions or allowances are greater than or equal to the amount of Australian government pensions. Check both amounts AND correct as necessary.
    
            Legacy Rule Format:
            IF [TRT104] > 0 AND [TRT105] <= [TRT104]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT104] = TRT:RP.JAUS:rvctc1.02.00:IncomeTax.AustralianGovernmentPensionsAndAllowancesTaxWithheld.Amount
            [TRT105] = TRT:RP.JAUS:gfapc.02.01:Pension.Total.Amount

            Technical Business Rule Format:
            ^TRT104 > 0 AND ^TRT105 <= ^TRT104
    
            Data Elements:
    
            ^TRT105 = RP.JAUS:Pension.Total.Amount
    
            ^TRT104 = RP.JAUS:IncomeTax.AustralianGovernmentPensionsAndAllowancesTaxWithheld.Amount
            */
            assertion = (report.TRT104.GetValueOrDefault() > 0 && report.TRT105.GetValueOrDefault() <= report.TRT104.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432148", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax withheld from pension must not be greater than the total pension amount",
                    LongDescription = @"Tax withheld amounts from Australian government pensions or allowances must not be greater than or equal to the amount of Australian government pensions",
                    Location = "/xbrli:xbrl/tns:Pension.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432148"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT104", Value = GetValueOrEmpty(report.TRT104) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT105", Value = GetValueOrEmpty(report.TRT105) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432149
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432149()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432149
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT105] <> NULL AND [TRT105] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT105] = TRT:RP.JAUS:gfapc.02.01:Pension.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT105, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT105 = RP.JAUS:Pension.Total.Amount
            */
            assertion = NotMonetary(report.TRT105, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Pension.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432149"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT105", Value = GetValueOrEmpty(report.TRT105) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432152
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432152()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432152
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount <> NULL 
            AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT114, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT114 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount
            */
            assertion = NotMonetary(report.TRT114, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432152"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT114", Value = GetValueOrEmpty(report.TRT114) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432152
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432152_1(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432152
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount <> NULL 
                    AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT108, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT108 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT108, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432152"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT108", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT108) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432153
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432153(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432153
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT109] <> NULL AND [TRT109] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT109] = TRT:RP.JAUS:lrla.02.22:Remuneration.IndividualNonBusinessGross.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT109, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT109 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.IndividualNonBusinessGross.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT109, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:Remuneration.IndividualNonBusinessGross.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432153"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT109", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT109) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432154
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432154(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432154
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT110] <> NULL AND [TRT110] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT110] = TRT:RP.JAUS:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT110, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT110 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT110, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432154"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT110", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT110) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432155
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432155(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432155
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT111] <> NULL AND [TRT111] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT111] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT111, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT111 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT111, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432155"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT111", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT111) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432156
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432156(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432156
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT112] <> NULL AND [TRT112] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT112] = TRT:RP.JAUS:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT112, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT112 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT112, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432156"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT112", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT112) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432157
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432157(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432157
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT113] <> NULL AND [TRT113] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT113] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT113, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT113 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount
                    */
                    assertion = NotMonetary(individualNonBusinessWithholdingPaymentDetails.TRT113, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432157"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT113", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT113) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432159
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432159()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432159
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount <> NULL 
            AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT117, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT117 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:LumpSumInArrears:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount
            */
            assertion = NotMonetary(report.TRT117, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails/tns:LumpSumInArrears/tns:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432159"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT117", Value = GetValueOrEmpty(report.TRT117) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432159
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432159_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432159
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount <> NULL 
            AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT115, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT115 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount
            */
            assertion = NotMonetary(report.TRT115, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails/tns:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432159"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT115", Value = GetValueOrEmpty(report.TRT115) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432160
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432160()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432160
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount <> NULL 
            AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT118, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT118 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:LumpSumInArrears:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount
            */
            assertion = NotMonetary(report.TRT118, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails/tns:LumpSumInArrears/tns:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432160"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT118", Value = GetValueOrEmpty(report.TRT118) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432160
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432160_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432160
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount <> NULL 
            AND TRT:RP.JAus:rvctc2.xx.xx:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT116, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT116 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount
            */
            assertion = NotMonetary(report.TRT116, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails/tns:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432160"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT116", Value = GetValueOrEmpty(report.TRT116) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432163
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432163()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432163
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT119] <> NULL AND [TRT119] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT119] = TRT:RP.JAUS:lrla.02.00:Remuneration.BonusesAndAllowances.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT119, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT119 = RP.JAUS:Remuneration.BonusesAndAllowances.Amount
            */
            assertion = NotMonetary(report.TRT119, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.BonusesAndAllowances.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432163"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT119", Value = GetValueOrEmpty(report.TRT119) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432164
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432164(TRT2023.OtherAustralianIncome otherAustralianIncome, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432164
                    When other assessable Australian income is present (Item 14) in the Trust return, a description AND income amount is required.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:OtherAustralianIncome)
                    IF [TRT120] = NULLORBLANK OR [TRT121] = NULL 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT120] = TRT:RP.JAUS:bafpr1.02.00:Income.Other.Description IN TUPLE(trt.0011.lodge.req.02.00:OtherAustralianIncome)
                    [TRT121] = TRT:RP.JAUS:bafpr1.02.04:Income.Other.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherAustralianIncome)
        
                    Technical Business Rule Format:
                    ^TRT120 = BLANK OR ^TRT121 = NULL
            
                    Data Elements:
            
                    ^TRT120 = RP.JAUS:OtherAustralianIncome:Income.Other.Description
            
                    ^TRT121 = RP.JAUS:OtherAustralianIncome:Income.Other.Amount
                    */
                    assertion = (string.IsNullOrWhiteSpace(otherAustralianIncome.TRT120) == true || otherAustralianIncome.TRT121 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432164", Severity = ProcessMessageSeverity.Error,
                            Description = @"Each Other Australian income item must include both an amount and a description",
                            LongDescription = @"For each Other Australian income amount, a description must be present; and for each description, an amount must be present",
                            Location = "/xbrli:xbrl/tns:OtherAustralianIncome" + OccurrenceIndex(otherAustralianIncome.OccurrenceIndex) + "/tns:Income.Other.Description",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432164"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT120", Value = otherAustralianIncome.TRT120 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT121", Value = GetValueOrEmpty(otherAustralianIncome.TRT121) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432166
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432166(TRT2023.OtherAustralianIncome otherAustralianIncome, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432166
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT121] <> NULL AND [TRT121] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT121] = TRT:RP.JAUS:bafpr1.02.04:Income.Other.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherAustralianIncome)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT121, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT121 = RP.JAUS:OtherAustralianIncome:Income.Other.Amount
                    */
                    assertion = NotMonetary(otherAustralianIncome.TRT121, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:OtherAustralianIncome" + OccurrenceIndex(otherAustralianIncome.OccurrenceIndex) + "/tns:Income.Other.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432166"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT121", Value = GetValueOrEmpty(otherAustralianIncome.TRT121) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432167
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432167()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432167
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT122] <> NULL AND [TRT122] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT122] = TRT:RP.JAUS:bafpr1.02.04:Income.ExceptedTrustNet.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT122, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT122 = RP.JAUS:Income.ExceptedTrustNet.Amount
            */
            assertion = NotMonetary(report.TRT122, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExceptedTrustNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432167"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT122", Value = GetValueOrEmpty(report.TRT122) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432168
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432168()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432168
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT123] <> NULL AND [TRT123] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT123] = TRT:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT123, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT123 = RP.JAUS:Expense.AustralianInvestmentAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.TRT123, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.AustralianInvestmentAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432168"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT123", Value = GetValueOrEmpty(report.TRT123) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432169
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432169()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432169
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT124] <> NULL AND [TRT124] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT124] = TRT:RP.JAUS:bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT124, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT124 = RP.JAUS:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            */
            assertion = NotMonetary(report.TRT124, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.ForestryManagedInvestmentSchemeDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432169"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT124", Value = GetValueOrEmpty(report.TRT124) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432177
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432177(TRT2023.OtherDeductions otherDeductions, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432177
                    Details of other Australian source deductions is present, but the amount of other Australian source deductions is either missing or equal to zero.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:OtherDeductions)
                    IF [TRT128] <> NULLORBLANK AND ([TRT129] = 0 OR [TRT129] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT128] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Text IN TUPLE(trt.0011.lodge.req.02.00:OtherDeductions)
                    [TRT129] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherDeductions)
        
                    Technical Business Rule Format:
                    ^TRT128 <> BLANK AND (^TRT129 = 0 OR ^TRT129 = NULL)
            
                    Data Elements:
            
                    ^TRT129 = RP.JAUS:OtherDeductions:Expense.DeductibleOther.Amount
            
                    ^TRT128 = RP.JAUS:OtherDeductions:Expense.DeductibleOther.Text
                    */
                    assertion = (string.IsNullOrWhiteSpace(otherDeductions.TRT128) != true && (otherDeductions.TRT129 == 0 || otherDeductions.TRT129 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430110", Severity = ProcessMessageSeverity.Error,
                            Description = @"Other deductions amount must be present",
                            LongDescription = @"If Name of each item of deduction is present, then the Other deductions - Amount must be present and greater than zero",
                            Location = "/xbrli:xbrl/tns:OtherDeductions" + OccurrenceIndex(otherDeductions.OccurrenceIndex) + "/tns:Expense.DeductibleOther.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432177"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT128", Value = otherDeductions.TRT128 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT129", Value = GetValueOrEmpty(otherDeductions.TRT129) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432178
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432178(TRT2023.OtherDeductions otherDeductions, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432178
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT129] <> NULL AND [TRT129] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT129] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherDeductions)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT129, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT129 = RP.JAUS:OtherDeductions:Expense.DeductibleOther.Amount
                    */
                    assertion = NotMonetary(otherDeductions.TRT129, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:OtherDeductions" + OccurrenceIndex(otherDeductions.OccurrenceIndex) + "/tns:Expense.DeductibleOther.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432178"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT129", Value = GetValueOrEmpty(otherDeductions.TRT129) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432179
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432179()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432179
            The overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income other assessable foreign source income minus allowable tax losses of earlier income years deducted this year) AND the sum of the income amounts in the distribution statement is not equal to the overall net trust income.
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] > 0) AND ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] - [TRT144] <> SUM([TRT218]) + [TRT234] + SUM([TRT219]) + [TRT235] + SUM([TRT442]) + [TRT444] + SUM([TRT223]) + [TRT239] + [TRT446] +SUM([TRT224]) + [TRT240] + SUM([TRT225]) + [TRT241])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT223] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT224] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT225] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT239] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT240] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT241] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT446] = TRT:RP:rvctc3.02.08:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 > 0) AND (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 - ^TRT144 <> Sum(^TRT218) + ^TRT234 + Sum(^TRT219) + ^TRT235 + Sum(^TRT442) + ^TRT444 + Sum(^TRT223) + ^TRT239 + ^TRT446 +Sum(^TRT224) + ^TRT240 + Sum(^TRT225) + ^TRT241)
    
            Data Elements:
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
    
            ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
    
            ^TRT223 = RP:StatementOfDistributionToBeneficiary:Income.CapitalGainsNet.Amount
    
            ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT225 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.Net.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT239 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.CapitalGainsNet.Amount
    
            ^TRT240 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT241 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.Net.Amount
    
            ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
    
            ^TRT446 = RP:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() > 0 && report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() - report.TRT144.GetValueOrDefault() != (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT218.GetValueOrDefault())) + report.TRT234.GetValueOrDefault() + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT219.GetValueOrDefault())) + report.TRT235.GetValueOrDefault() + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT442.GetValueOrDefault())) + report.TRT444.GetValueOrDefault() + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT223.GetValueOrDefault())) + report.TRT239.GetValueOrDefault() + report.TRT446.GetValueOrDefault() + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT224.GetValueOrDefault())) + report.TRT240.GetValueOrDefault() + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT225.GetValueOrDefault())) + report.TRT241.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432179", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net trust income must equal distributed income",
                    LongDescription = @"If the overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses of earlier income years deducted this year) and the sum of the income amounts in the statements of distribution must equal the overall net trust income.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432179"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT218", Value = "TRT218" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT219", Value = "TRT219" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT442", Value = "TRT442" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT223", Value = "TRT223" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT239", Value = GetValueOrEmpty(report.TRT239) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT446", Value = GetValueOrEmpty(report.TRT446) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT224", Value = "TRT224" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT240", Value = GetValueOrEmpty(report.TRT240) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT225", Value = "TRT225" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT241", Value = GetValueOrEmpty(report.TRT241) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432180
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432180()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432180
            The amount of net Australian income/loss is not equal to the sum of all Australian income less all Australian source deductions.
    
            Legacy Rule Format:
            IF [TRT130] <> ([TRT71] + [TRT72] + [TRT76] + [TRT77] - [TRT79] + [TRT80] + [TRT81] - [TRT83] + [TRT437] - [TRT438] + [TRT89] - [TRT90] - [TRT91] - [TRT92] + [TRT93] + [TRT94] + [TRT96] + [TRT97] + [TRT98] + [TRT105] + SUM([TRT109]) + SUM([TRT111]) + (SUM([TRT113]) * 0.05) + [TRT115] + [TRT116] + [TRT117] + [TRT118] + [TRT119] + [TRT100] + [TRT101] + [TRT102] + [TRT103] + SUM([TRT121]) - ([TRT123] + [TRT439] + [TRT124] + SUM([TRT129]))) +/- 1 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT79] = TRT:RP.JAUS.Prim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT83] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Expense.DistributionDeductionsOther.Amount
            [TRT89] = TRT:RP.JAUS:bafpr2.02.00:Income.Operating.RentalIncomeGross.Amount
            [TRT90] = TRT:RP.JAUS:bafpr1.02.05:Expense.Interest.Amount
            [TRT91] = TRT:RP.JAUS:bafpr1.02.02:Expense.CapitalWorksDeduction.Amount
            [TRT92] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            [TRT93] = TRT:RP.JAUS:bafpr1.02.23:Income.ForestryManagedInvestmentScheme.Amount
            [TRT94] = TRT:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [TRT96] = TRT:RP.JAUS:bafpr1.02.05:Income.DividendsUnfranked.Amount
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT98] = TRT:RP.JAUS:rvctc1.02.03:IncomeTax.FrankingCredits.Amount
            [TRT100] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitLumpSumPaymentTaxedElement.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "O") IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationLumpSumWithholdingPaymentDetails)
            [TRT101] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitLumpSumPaymentUntaxedElement.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "O") IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationLumpSumWithholdingPaymentDetails)
            [TRT102] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "D") IN TUPLE(trt.0011.lodge.req.02.00:EmploymentTerminationPaymentDetails)
            [TRT103] = TRT:RP.JAUS:emsup.02.01:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE (crigi.02.00:InsuranceBenefits.BeneficiaryType.Code = "O") IN TUPLE(trt.0011.lodge.req.02.00:EmploymentTerminationPaymentDetails)
            [TRT105] = TRT:RP.JAUS:gfapc.02.01:Pension.Total.Amount
            [TRT109] = TRT:RP.JAUS:lrla.02.22:Remuneration.IndividualNonBusinessGross.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
            [TRT111] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
            [TRT113] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
            [TRT115] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationIncomeStreamWithholdingPaymentDetails)
            [TRT116] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationIncomeStreamWithholdingPaymentDetails)
            [TRT117] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount IN TUPLE(trt.0011.lodge.req.02.00:LumpSumInArrears) IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationIncomeStreamWithholdingPaymentDetails)
            [TRT118] = TRT:RP.JAUS:rvctc2.02.00:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount IN TUPLE(trt.0011.lodge.req.02.00:LumpSumInArrears) IN TUPLE(trt.0011.lodge.req.02.00:SuperannuationIncomeStreamWithholdingPaymentDetails)
            [TRT119] = TRT:RP.JAUS:lrla.02.00:Remuneration.BonusesAndAllowances.Amount
            [TRT121] = TRT:RP.JAUS:bafpr1.02.04:Income.Other.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherAustralianIncome)
            [TRT123] = TRT:RP.JAUS:bafpr1.02.04:Expense.AustralianInvestmentAllowableDeduction.Amount
            [TRT124] = TRT:RP.JAUS:bafpr1.02.23:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
            [TRT129] = TRT:RP.JAUS:bafpr1.02.04:Expense.DeductibleOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:OtherDeductions)
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [TRT438] = TRT:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount
            [TRT439] = TRT:RP.JAUS:rvctc2.02.06:IncomeTax.Deduction.DistributionFranked.Amount

            Technical Business Rule Format:
            OutsideRange(^TRT130, ^TRT71 + ^TRT72 + ^TRT76 + ^TRT77 - ^TRT79 + ^TRT80 + ^TRT81 - ^TRT83 + ^TRT437 - ^TRT438 + ^TRT89 - ^TRT90 - ^TRT91 - ^TRT92 + ^TRT93 + ^TRT94 + ^TRT96 + ^TRT97 + ^TRT98 + ^TRT105 + Sum(^TRT109) + Sum(^TRT111) + (Sum(^TRT113) * 0.05) + ^TRT115 + ^TRT116 + ^TRT117 + ^TRT118 + ^TRT119 + ^TRT100 + ^TRT101 + ^TRT102 + ^TRT103 + Sum(^TRT121) - (^TRT123 + ^TRT439 + ^TRT124 + Sum(^TRT129)) , 1)
    
            Data Elements:
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT79 = RP.JAUS.Prim:Expense.DistributionDeductionsOther.Amount
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT83 = RP.JAUS.NonPrim:Expense.DistributionDeductionsOther.Amount
    
            ^TRT89 = RP.JAUS:Income.Operating.RentalIncomeGross.Amount
    
            ^TRT90 = RP.JAUS:Expense.Interest.Amount
    
            ^TRT91 = RP.JAUS:Expense.CapitalWorksDeduction.Amount
    
            ^TRT92 = RP.JAUS:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
    
            ^TRT93 = RP.JAUS:Income.ForestryManagedInvestmentScheme.Amount
    
            ^TRT94 = RP.JAUS:Income.Interest.Gross.Amount
    
            ^TRT96 = RP.JAUS:Income.DividendsUnfranked.Amount
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
    
            ^TRT98 = RP.JAUS:IncomeTax.FrankingCredits.Amount
    
            ^TRT100 = RP.JAUS:SuperannuationLumpSumWithholdingPaymentDetails:SuperannuationBenefit.DeathBenefitLumpSumPaymentTaxedElement.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
    
            ^TRT101 = RP.JAUS:SuperannuationLumpSumWithholdingPaymentDetails:SuperannuationBenefit.DeathBenefitLumpSumPaymentUntaxedElement.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
    
            ^TRT102 = RP.JAUS:EmploymentTerminationPaymentDetails:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='D']
    
            ^TRT103 = RP.JAUS:EmploymentTerminationPaymentDetails:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount WHERE [InsuranceBenefits.BeneficiaryType.Code='O']
    
            ^TRT105 = RP.JAUS:Pension.Total.Amount
    
            ^TRT109 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.IndividualNonBusinessGross.Amount
    
            ^TRT111 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount
    
            ^TRT113 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount
    
            ^TRT115 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount
    
            ^TRT116 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount
    
            ^TRT117 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:LumpSumInArrears:IncomeTax.Superannuation.TaxableComponentTaxedElement.Amount
    
            ^TRT118 = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails:LumpSumInArrears:IncomeTax.Superannuation.TaxableComponentUntaxedElement.Amount
    
            ^TRT119 = RP.JAUS:Remuneration.BonusesAndAllowances.Amount
    
            ^TRT121 = RP.JAUS:OtherAustralianIncome:Income.Other.Amount
    
            ^TRT123 = RP.JAUS:Expense.AustralianInvestmentAllowableDeduction.Amount
    
            ^TRT124 = RP.JAUS:Expense.ForestryManagedInvestmentSchemeDeduction.Amount
    
            ^TRT129 = RP.JAUS:OtherDeductions:Expense.DeductibleOther.Amount
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
    
            ^TRT438 = RP.JAUS.NonPrim:IncomeTax.Deduction.TrustDistributionFranked.Amount
    
            ^TRT439 = RP.JAUS:IncomeTax.Deduction.DistributionFranked.Amount
            */
            assertion = OutsideRange(report.TRT130.GetValueOrDefault(), report.TRT71.GetValueOrDefault() + report.TRT72.GetValueOrDefault() + report.TRT76.GetValueOrDefault() + report.TRT77.GetValueOrDefault() - report.TRT79.GetValueOrDefault() + report.TRT80.GetValueOrDefault() + report.TRT81.GetValueOrDefault() - report.TRT83.GetValueOrDefault() + report.TRT437.GetValueOrDefault() - report.TRT438.GetValueOrDefault() + report.TRT89.GetValueOrDefault() - report.TRT90.GetValueOrDefault() - report.TRT91.GetValueOrDefault() - report.TRT92.GetValueOrDefault() + report.TRT93.GetValueOrDefault() + report.TRT94.GetValueOrDefault() + report.TRT96.GetValueOrDefault() + report.TRT97.GetValueOrDefault() + report.TRT98.GetValueOrDefault() + report.TRT105.GetValueOrDefault() + (report.IndividualNonBusinessWithholdingPaymentDetailsCollection == null ? 0 : report.IndividualNonBusinessWithholdingPaymentDetailsCollection.Sum(f => f.TRT109.GetValueOrDefault())) + (report.IndividualNonBusinessWithholdingPaymentDetailsCollection == null ? 0 : report.IndividualNonBusinessWithholdingPaymentDetailsCollection.Sum(f => f.TRT111.GetValueOrDefault())) + (report.IndividualNonBusinessWithholdingPaymentDetailsCollection == null ? 0 : report.IndividualNonBusinessWithholdingPaymentDetailsCollection.Sum(f => f.TRT113.GetValueOrDefault())) * 0.05M + report.TRT115.GetValueOrDefault() + report.TRT116.GetValueOrDefault() + report.TRT117.GetValueOrDefault() + report.TRT118.GetValueOrDefault() + report.TRT119.GetValueOrDefault() + report.TRT100.GetValueOrDefault() + report.TRT101.GetValueOrDefault() + report.TRT102.GetValueOrDefault() + report.TRT103.GetValueOrDefault() + (report.OtherAustralianIncomeCollection == null ? 0 : report.OtherAustralianIncomeCollection.Sum(f => f.TRT121.GetValueOrDefault())) - (report.TRT123.GetValueOrDefault() + report.TRT439.GetValueOrDefault() + report.TRT124.GetValueOrDefault() + (report.OtherDeductionsCollection == null ? 0 : report.OtherDeductionsCollection.Sum(f => f.TRT129.GetValueOrDefault()))), 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432613", Severity = ProcessMessageSeverity.Error,
                    Description = @"Net Australian income must equal income less deductions",
                    LongDescription = @"The amount of net Australian income/loss is not equal to the sum of all Australian income less all Australian source deductions.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432180"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT79", Value = GetValueOrEmpty(report.TRT79) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT83", Value = GetValueOrEmpty(report.TRT83) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT438", Value = GetValueOrEmpty(report.TRT438) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT89", Value = GetValueOrEmpty(report.TRT89) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT90", Value = GetValueOrEmpty(report.TRT90) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT91", Value = GetValueOrEmpty(report.TRT91) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT92", Value = GetValueOrEmpty(report.TRT92) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT93", Value = GetValueOrEmpty(report.TRT93) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT94", Value = GetValueOrEmpty(report.TRT94) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT96", Value = GetValueOrEmpty(report.TRT96) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT98", Value = GetValueOrEmpty(report.TRT98) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT105", Value = GetValueOrEmpty(report.TRT105) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT109", Value = "TRT109" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT111", Value = "TRT111" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT113", Value = "TRT113" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT115", Value = GetValueOrEmpty(report.TRT115) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT116", Value = GetValueOrEmpty(report.TRT116) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT117", Value = GetValueOrEmpty(report.TRT117) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT118", Value = GetValueOrEmpty(report.TRT118) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT119", Value = GetValueOrEmpty(report.TRT119) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT100", Value = GetValueOrEmpty(report.TRT100) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT101", Value = GetValueOrEmpty(report.TRT101) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT102", Value = GetValueOrEmpty(report.TRT102) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT103", Value = GetValueOrEmpty(report.TRT103) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT121", Value = "TRT121" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT123", Value = GetValueOrEmpty(report.TRT123) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT439", Value = GetValueOrEmpty(report.TRT439) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT124", Value = GetValueOrEmpty(report.TRT124) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT129", Value = "TRT129" });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432181
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432181()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432181
            There is an overall trust loss (i.e. the sum of net Australian income/loss plus net capital gains, plus attributed foreign income plus other assessable foreign source income minus allowable tax losses of earlier income years deducted this year is less than zero) AND income distribution amount/s is/are present in the distribution statement (distribution to a beneficiary or no beneficiary presently entitled). Income cannot be distributed when there is an overall trust loss. Delete the income from the distribution statement.
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] <= 0) AND (SUM([TRT218]) + SUM([TRT219]) + SUM([TRT442]) + SUM([TRT223]) + SUM([TRT224]) + SUM([TRT225]) + SUM([TRT217]) + [TRT233] + [TRT234] + [TRT235] + [TRT444] + [TRT239] + [TRT240] + [TRT241] + [TRT446] <> 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT223] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT224] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT225] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT233] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT239] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT240] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT241] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT446] = TRT:RP:rvctc3.02.08:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 <= 0) AND (Sum(^TRT218) + Sum(^TRT219) + Sum(^TRT442) + Sum(^TRT223) + Sum(^TRT224) + Sum(^TRT225) + Sum(^TRT217) + ^TRT233 + ^TRT234 + ^TRT235 + ^TRT444 + ^TRT239 + ^TRT240 + ^TRT241 + ^TRT446 <> 0)
    
            Data Elements:
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
    
            ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
    
            ^TRT223 = RP:StatementOfDistributionToBeneficiary:Income.CapitalGainsNet.Amount
    
            ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT225 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.Net.Amount
    
            ^TRT233 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT239 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.CapitalGainsNet.Amount
    
            ^TRT240 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT241 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.Net.Amount
    
            ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
    
            ^TRT446 = RP:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() <= 0 && (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT218.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT219.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT442.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT223.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT224.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT225.GetValueOrDefault())) + (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT217.GetValueOrDefault())) + report.TRT233.GetValueOrDefault() + report.TRT234.GetValueOrDefault() + report.TRT235.GetValueOrDefault() + report.TRT444.GetValueOrDefault() + report.TRT239.GetValueOrDefault() + report.TRT240.GetValueOrDefault() + report.TRT241.GetValueOrDefault() + report.TRT446.GetValueOrDefault() != 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432181", Severity = ProcessMessageSeverity.Error,
                    Description = @"Zero net income or an overall trust loss cannot be distributed",
                    LongDescription = @"If there is an overall trust loss (i.e. the sum of net Australian income/loss plus net capital gains, plus attributed foreign income plus other assessable foreign source income minus allowable tax losses of earlier income years deducted this year, is less than zero) there can not be income distribution amounts present in any statement of distribution",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432181"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT218", Value = "TRT218" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT219", Value = "TRT219" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT442", Value = "TRT442" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT223", Value = "TRT223" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT224", Value = "TRT224" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT225", Value = "TRT225" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT217", Value = "TRT217" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT233", Value = GetValueOrEmpty(report.TRT233) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT239", Value = GetValueOrEmpty(report.TRT239) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT240", Value = GetValueOrEmpty(report.TRT240) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT241", Value = GetValueOrEmpty(report.TRT241) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT446", Value = GetValueOrEmpty(report.TRT446) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432188
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432188()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432188
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT130] <> NULL AND [TRT130] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT130, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
            */
            assertion = NotMonetary(report.TRT130, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432188"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432191(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432191
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> NULL 
                    AND TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT223, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT223 = RP:StatementOfDistributionToBeneficiary:Income.CapitalGainsNet.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT223, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.CapitalGainsNet.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432191"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT223", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT223) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432191_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432191
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> NULL 
            AND TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT133, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
            */
            assertion = NotMonetary(report.TRT133, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432191
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432191_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432191
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> NULL 
            AND TRT:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT239, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT239 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.CapitalGainsNet.Amount
            */
            assertion = NotMonetary(report.TRT239, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432191"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT239", Value = GetValueOrEmpty(report.TRT239) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432192
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432192()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432192
            Where an amount has been entered at Net capital gain, the answer to the question "Did you have a CGT event during the year" must be Yes.
    
            Legacy Rule Format:
            IF [TRT133] > 0 AND [TRT131] = FALSE 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT131] = TRT:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount

            Technical Business Rule Format:
            ^TRT133 > 0 AND ^TRT131 = FALSE
    
            Data Elements:
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT131 = RP:CapitalGainsTax.Event.Indicator
            */
            assertion = (report.TRT133.GetValueOrDefault() > 0 && report.TRT131 == false);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432192", Severity = ProcessMessageSeverity.Error,
                    Description = @"Answer to the 'Did you have a CGT event..?' question must be 'yes'",
                    LongDescription = @"If an amount is present at Net capital gain, then the answer to the question 'Did you have a CGT event during the year?' must be 'yes' (true)",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432192"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT131", Value = GetValueOrEmpty(report.TRT131) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432193
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432193()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432193
            The overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income other assessable foreign source income minus allowable tax losses of earlier income years deducted this year), AND the sum of Attributed Foreign Income (including no beneficiary presently entitled) in the distribution statement, is not equal to the sum of Attributed Foreign Income amounts of the trust.
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] > 0) AND (SUM([TRT224]) + [TRT240] <> [TRT134] + [TRT136]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT224] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT240] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 > 0) AND (Sum(^TRT224) + ^TRT240 <> ^TRT134 + ^TRT136)
    
            Data Elements:
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT240 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AttributedForeignIncome.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() > 0 && (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT224.GetValueOrDefault())) + report.TRT240.GetValueOrDefault() != report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432193", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income must equal distributed amounts",
                    LongDescription = @"If the overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses), then the sum of Attributed Foreign Income in the Statements of distribution must equal to the sum of the Attributed Foreign Income amounts for the Trust",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432193"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT224", Value = "TRT224" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT240", Value = GetValueOrEmpty(report.TRT240) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432194
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432194()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432194
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT134] <> NULL AND [TRT134] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT134, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
            */
            assertion = NotMonetary(report.TRT134, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCLCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432194"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432196
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432196()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432196
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT136] <> NULL AND [TRT136] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT136, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
            */
            assertion = NotMonetary(report.TRT136, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.Attributed.Amount[@contextRef='" + report.RPJFOREIGNCFCUCId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432196"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432199
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432199()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432199
            For attributed foreign income, a yes response to the question "did the entity have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?" is required when there are amounts present at either listed country or unlisted country.
    
            Legacy Rule Format:
            IF ([TRT134] > 0 OR [TRT136] > 0) AND [TRT138] <> TRUE 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT138] = TRT:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator

            Technical Business Rule Format:
            (^TRT134 > 0 OR ^TRT136 > 0) AND ^TRT138 <> TRUE
    
            Data Elements:
    
            ^TRT138 = RP.JFOREIGN:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
            */
            assertion = ((report.TRT134.GetValueOrDefault() > 0 || report.TRT136.GetValueOrDefault() > 0) && report.TRT138 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432712", Severity = ProcessMessageSeverity.Error,
                    Description = @"Answer to Attributed foreign income question must be 'yes'",
                    LongDescription = @"If Attributed foreign income - Listed country or Unlisted country is greater than zero, then a yes (true) response to the question ""Did you have either a direct or indirect interest in a foreign trust, controlled foreign company or transferor trust?"" is required",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432199"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT138", Value = GetValueOrEmpty(report.TRT138) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432202
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432202()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432202
            Other assessable foreign source income 'Gross' amount must not be less than 'Net' amount
    
            Legacy Rule Format:
            IF [TRT140] < [TRT141] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT140] = TRT:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^TRT140 < ^TRT141
    
            Data Elements:
    
            ^TRT140 = RP.JFOREIGN:Income.Other.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
            */
            assertion = (report.TRT140.GetValueOrDefault() < report.TRT141.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430120", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other assessable foreign source income 'Gross' amount must not be less than 'Net' amount",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432202"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT140", Value = GetValueOrEmpty(report.TRT140) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432203
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432203()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432203
            The sum of PP share of net income from trusts plus non-PP share of net income from trusts plus non-PP franked distributions from trusts plus net capital gains plus attributed foreign income plus other assessable net foreign source income plus Aust'n franking credits from a NZ coy is less than zero which indicates that an overall trust loss is being distributed between trusts. Adjust distributed amounts until the overall negative amount no longer exists (i.e. make the sum equal to zero).
    
            Legacy Rule Format:
            IF ([TRT141] = NULL OR [TRT141] = 0) AND ([TRT77] + [TRT81] + [TRT437] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] < 0 )
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            (^TRT141 = NULL OR ^TRT141 = 0) AND (^TRT77 + ^TRT81 + ^TRT437 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 < 0)
    
            Data Elements:
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
            */
            assertion = ((report.TRT141 == null || report.TRT141 == 0) && report.TRT77.GetValueOrDefault() + report.TRT81.GetValueOrDefault() + report.TRT437.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() < 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432683", Severity = ProcessMessageSeverity.Error,
                    Description = @"An overall trust loss must not be distributed between trusts",
                    LongDescription = @"The sum of Share of net income from trusts, plus Net capital gains,  plus Attributed foreign income, plus Other assessable net foreign source income, plus Aust'n franking credits from a NZ coy, must not total less than zero. This indicates that an overall trust loss is being distributed between trusts.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432203"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432204
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432204()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432204
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT141] <> NULL AND [TRT141] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT141, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
            */
            assertion = NotMonetary(report.TRT141, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432204"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432205
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432205()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432205
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT142] <> NULL AND [TRT142] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT142] = TRT:RP.JFOREIGN:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT142, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT142 = RP.JFOREIGN:Income.InternationalDealings.TaxOffset.Amount
            */
            assertion = NotMonetary(report.TRT142, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.TaxOffset.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432205"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT142", Value = GetValueOrEmpty(report.TRT142) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432206
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432206()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432206
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT143] <> NULL AND [TRT143] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT143, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = NotMonetary(report.TRT143, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432206"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432207
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432207()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432207
            The amount at Tax losses Deducted cannot exceed the sum of 'Net Aust'n income or loss', plus 'Net capital gain', plus 'Attributed foreign income' amounts, plus 'Net - Other assessable foreign income', plus 'Aust'n franking credits from a NZ coy' .
    
            Legacy Rule Format:
            IF [TRT144] > 0 AND ([TRT144] > [TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
            ^TRT144 > 0 AND (^TRT144 > ^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143)
    
            Data Elements:
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = (report.TRT144.GetValueOrDefault() > 0 && report.TRT144.GetValueOrDefault() > report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432207", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax losses deducted must not exceed total income.",
                    LongDescription = @"The amount at 'Tax losses deducted' must not be greater than the sum of 'Net Aust'n income or loss', plus 'Net capital gain', plus 'Attributed foreign income' amounts, plus 'Net - Other assessable foreign income', plus 'Aust'n franking credits from a NZ coy'",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432207"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432209
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432209()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432209
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT144] <> NULL AND [TRT144] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT144, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
            */
            assertion = NotMonetary(report.TRT144, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TaxLossesDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432209"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432211
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432211()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432211
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT145] <> NULL AND [TRT145] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT145] = TRT:RP.Closing:rvctc3.02.02:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT145, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT145 = RP.Closing:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount
            */
            assertion = NotMonetary(report.TRT145, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Tax.Losses.CarriedForward.LaterIncomeYearsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432211"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT145", Value = GetValueOrEmpty(report.TRT145) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432227
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432227()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432227
            All current assets is greater than total assets.
    
            Legacy Rule Format:
            IF [TRT156] <> NULL AND  [TRT157] < [TRT156]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT156] = TRT:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [TRT157] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            ^TRT156 <> NULL AND  ^TRT157 < ^TRT156
    
            Data Elements:
    
            ^TRT156 = RP.Closing(Instant):Assets.CurrentTotal.Amount
    
            ^TRT157 = RP.Closing(Instant):Assets.Total.Amount
            */
            assertion = (report.TRT156 != null && report.TRT157.GetValueOrDefault() < report.TRT156.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430194", Severity = ProcessMessageSeverity.Error,
                    Description = @"All current assets must not exceed Total assets",
                    Location = "/xbrli:xbrl/tns:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432227"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT156", Value = GetValueOrEmpty(report.TRT156) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT157", Value = GetValueOrEmpty(report.TRT157) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432228
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432228()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432228
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT156] <> NULL AND [TRT156] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT156] = TRT:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT156, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT156 = RP.Closing(Instant):Assets.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.TRT156, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432228"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT156", Value = GetValueOrEmpty(report.TRT156) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432229
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432229()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432229
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT157] <> NULL AND [TRT157] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT157] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT157, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT157 = RP.Closing(Instant):Assets.Total.Amount
            */
            assertion = NotMonetary(report.TRT157, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432229"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT157", Value = GetValueOrEmpty(report.TRT157) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432230
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432230()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432230
            All current liabilities is greater than total liabilities.
    
            Legacy Rule Format:
            IF [TRT158] <> NULL AND [TRT159] < [TRT158]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT158] = TRT:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [TRT159] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            ^TRT158 <> NULL AND ^TRT159 < ^TRT158
    
            Data Elements:
    
            ^TRT158 = RP.Closing(Instant):Liabilities.CurrentTotal.Amount
    
            ^TRT159 = RP.Closing(Instant):Liabilities.Total.Amount
            */
            assertion = (report.TRT158 != null && report.TRT159.GetValueOrDefault() < report.TRT158.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430195", Severity = ProcessMessageSeverity.Error,
                    Description = @"All current liabilities must not exceed Total liabilities",
                    Location = "/xbrli:xbrl/tns:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432230"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT158", Value = GetValueOrEmpty(report.TRT158) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT159", Value = GetValueOrEmpty(report.TRT159) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432231
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432231()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432231
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT158] <> NULL AND [TRT158] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT158] = TRT:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT158, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT158 = RP.Closing(Instant):Liabilities.CurrentTotal.Amount
            */
            assertion = NotMonetary(report.TRT158, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Liabilities.CurrentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432231"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT158", Value = GetValueOrEmpty(report.TRT158) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432232
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432232()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432232
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT159] <> NULL AND [TRT159] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT159] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT159, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT159 = RP.Closing(Instant):Liabilities.Total.Amount
            */
            assertion = NotMonetary(report.TRT159, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Liabilities.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432232"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT159", Value = GetValueOrEmpty(report.TRT159) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432234
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432234()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432234
            Total business income (primary production or non-primary production) is present at business income AND expenses AND either the business name or business street address is missing.
    
            Legacy Rule Format:
            IF ([TRT46] <> NULL OR [TRT47] <> NULL OR [TRT49] <> NULL OR [TRT50] <> NULL OR [TRT52] <> NULL OR [TRT54] <> NULL OR [TRT55] <> NULL) AND ([TRT161] = NULLORBLANK OR [TRT162] = NULLORBLANK) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT161] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MTR" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails)
            [TRT162] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails)

            Technical Business Rule Format:
            (^TRT46 <> NULL OR ^TRT47 <> NULL OR ^TRT49 <> NULL OR ^TRT50 <> NULL OR ^TRT52 <> NULL OR ^TRT54 <> NULL OR ^TRT55 <> NULL) AND (^TRT161 = BLANK OR ^TRT162 = BLANK)
    
            Data Elements:
    
            ^TRT161 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MTR'].[OrganisationNameDetails.Currency.Code='C']
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
    
            ^TRT162 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            */
            assertion = ((report.TRT46 != null || report.TRT47 != null || report.TRT49 != null || report.TRT50 != null || report.TRT52 != null || report.TRT54 != null || report.TRT55 != null) && (string.IsNullOrWhiteSpace(report.TRT161) == true || string.IsNullOrWhiteSpace(report.TRT162) == true));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430129", Severity = ProcessMessageSeverity.Error,
                    Description = @"Business name and business address of main business must be present",
                    LongDescription = @"If Total business income (Primary production or Non-primary production) is present at business income and expenses, then both the Business name and Business address of main business must be present",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MTR_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432234"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT161", Value = report.TRT161 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT162", Value = report.TRT162 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432235
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432235()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432235
            Net business income (PP or non-PP) is present in business income AND deductions AND there are no business declaration items present.
    
            Legacy Rule Format:
            IF ([TRT71] <> NULL OR [TRT72] <> NULL) AND ([TRT166] = NULL AND [TRT167] = NULL AND [TRT168] = NULL AND [TRT170] = NULL AND [TRT156] = NULL AND [TRT157] = NULL AND [TRT171] = NULL AND [TRT158] = NULL AND [TRT159] = NULL AND [TRT172] = NULL AND [TRT174] = NULL AND [TRT181] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT71] = TRT:RP.JAUS.Prim:bafpr1.02.23:Income.Net.Amount
            [TRT72] = TRT:RP.JAUS.NonPrim:bafpr1.02.23:Income.Net.Amount
            [TRT156] = TRT:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [TRT157] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.Total.Amount
            [TRT158] = TRT:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [TRT159] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.Total.Amount
            [TRT166] = TRT:RP.Opening(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [TRT167] = TRT:RP:bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount
            [TRT168] = TRT:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [TRT170] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount
            [TRT171] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount
            [TRT172] = TRT:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [TRT174] = TRT:RP:lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount
            [TRT181] = TRT:RP:lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount

            Technical Business Rule Format:
            (^TRT71 <> NULL OR ^TRT72 <> NULL) AND (^TRT166 = NULL AND ^TRT167 = NULL AND ^TRT168 = NULL AND ^TRT170 = NULL AND ^TRT156 = NULL AND ^TRT157 = NULL AND ^TRT171 = NULL AND ^TRT158 = NULL AND ^TRT159 = NULL AND ^TRT172 = NULL AND ^TRT174 = NULL AND ^TRT181 = NULL)
    
            Data Elements:
    
            ^TRT166 = RP.Opening(Instant):Assets.Inventories.Total.Amount
    
            ^TRT71 = RP.JAUS.Prim:Income.Net.Amount
    
            ^TRT72 = RP.JAUS.NonPrim:Income.Net.Amount
    
            ^TRT156 = RP.Closing(Instant):Assets.CurrentTotal.Amount
    
            ^TRT157 = RP.Closing(Instant):Assets.Total.Amount
    
            ^TRT158 = RP.Closing(Instant):Liabilities.CurrentTotal.Amount
    
            ^TRT159 = RP.Closing(Instant):Liabilities.Total.Amount
    
            ^TRT167 = RP:Expense.Purchases.AndOtherCosts.Amount
    
            ^TRT168 = RP.Closing(Instant):Assets.Inventories.Total.Amount
    
            ^TRT170 = RP.Closing(Instant):Assets.TradeandReceivablesOther.Amount
    
            ^TRT171 = RP.Closing(Instant):Liabilities.TradeAndOtherPayablesTotal.Amount
    
            ^TRT172 = RP:Remuneration.WagesAndSalaries.Total.Amount
    
            ^TRT174 = RP:Remuneration.PaymentToRelatedPartiesGross.Amount
    
            ^TRT181 = RP:Remuneration.FringeBenefits.EmployeeContribution.Amount
            */
            assertion = ((report.TRT71 != null || report.TRT72 != null) && (report.TRT166 == null && report.TRT167 == null && report.TRT168 == null && report.TRT170 == null && report.TRT156 == null && report.TRT157 == null && report.TRT171 == null && report.TRT158 == null && report.TRT159 == null && report.TRT172 == null && report.TRT174 == null && report.TRT181 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432235", Severity = ProcessMessageSeverity.Error,
                    Description = @"Business and professional items amounts must be present",
                    LongDescription = @"If Net business income is present under Business income and deductions, then at least one Business and professional items amount must be present",
                    Location = "/xbrli:xbrl/tns:Assets.Inventories.Total.Amount[@contextRef='" + report.RPOpeningInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432235"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT71", Value = GetValueOrEmpty(report.TRT71) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT72", Value = GetValueOrEmpty(report.TRT72) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT166", Value = GetValueOrEmpty(report.TRT166) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT167", Value = GetValueOrEmpty(report.TRT167) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT168", Value = GetValueOrEmpty(report.TRT168) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT170", Value = GetValueOrEmpty(report.TRT170) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT156", Value = GetValueOrEmpty(report.TRT156) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT157", Value = GetValueOrEmpty(report.TRT157) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT171", Value = GetValueOrEmpty(report.TRT171) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT158", Value = GetValueOrEmpty(report.TRT158) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT159", Value = GetValueOrEmpty(report.TRT159) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT172", Value = GetValueOrEmpty(report.TRT172) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT174", Value = GetValueOrEmpty(report.TRT174) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT181", Value = GetValueOrEmpty(report.TRT181) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432236
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432236()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432236
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT166] <> NULL AND [TRT166] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT166] = TRT:RP.Opening(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT166, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT166 = RP.Opening(Instant):Assets.Inventories.Total.Amount
            */
            assertion = NotMonetary(report.TRT166, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.Inventories.Total.Amount[@contextRef='" + report.RPOpeningInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432236"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT166", Value = GetValueOrEmpty(report.TRT166) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432237
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432237()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432237
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT167] <> NULL AND [TRT167] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT167] = TRT:RP:bafpr2.02.02:Expense.Purchases.AndOtherCosts.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT167, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT167 = RP:Expense.Purchases.AndOtherCosts.Amount
            */
            assertion = NotMonetary(report.TRT167, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Purchases.AndOtherCosts.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432237"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT167", Value = GetValueOrEmpty(report.TRT167) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432238
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432238()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432238
            Closing stock amount must be present if Code is present
    
            Legacy Rule Format:
            IF ([TRT169] <> NULL) AND ([TRT168] = NULL OR [TRT168] = 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT168] = TRT:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [TRT169] = TRT:RP.Closing(Instant):bafpo7.02.18:Assets.Inventories.ValuationMethod.Code

            Technical Business Rule Format:
            (^TRT169 <> NULL) AND (^TRT168 = NULL OR ^TRT168 = 0)
    
            Data Elements:
    
            ^TRT168 = RP.Closing(Instant):Assets.Inventories.Total.Amount
    
            ^TRT169 = RP.Closing(Instant):Assets.Inventories.ValuationMethod.Code
            */
            assertion = (report.TRT169 != null && (report.TRT168 == null || report.TRT168 == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430197", Severity = ProcessMessageSeverity.Error,
                    Description = @"Closing stock amount must be present if Code is present",
                    Location = "/xbrli:xbrl/tns:Assets.Inventories.Total.Amount[@contextRef='" + report.RPClosingInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432238"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT169", Value = GetValueOrEmpty(report.TRT169) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT168", Value = GetValueOrEmpty(report.TRT168) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432239
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432239()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432239
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT168] <> NULL AND [TRT168] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT168] = TRT:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT168, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT168 = RP.Closing(Instant):Assets.Inventories.Total.Amount
            */
            assertion = NotMonetary(report.TRT168, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.Inventories.Total.Amount[@contextRef='" + report.RPClosingInstantId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432239"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT168", Value = GetValueOrEmpty(report.TRT168) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432240
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432240()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432240
            Under business income and expenses, the closing stock amount is present; however the closing stock action code is missing. The code must be present to indicate the method used to value closing stock.
    
            Legacy Rule Format:
            IF ([TRT169] = NULL) AND ([TRT168] >0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT168] = TRT:RP.Closing(Instant):bafpo7.02.04:Assets.Inventories.Total.Amount
            [TRT169] = TRT:RP.Closing(Instant):bafpo7.02.18:Assets.Inventories.ValuationMethod.Code

            Technical Business Rule Format:
            (^TRT169 = NULL) AND (^TRT168 >0)
    
            Data Elements:
    
            ^TRT169 = RP.Closing(Instant):Assets.Inventories.ValuationMethod.Code
    
            ^TRT168 = RP.Closing(Instant):Assets.Inventories.Total.Amount
            */
            assertion = (report.TRT169 == null && report.TRT168.GetValueOrDefault() > 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430198", Severity = ProcessMessageSeverity.Error,
                    Description = @"Closing stock action code must be present if Closing stock amount is present",
                    Location = "/xbrli:xbrl/tns:Assets.Inventories.ValuationMethod.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432240"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT169", Value = GetValueOrEmpty(report.TRT169) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT168", Value = GetValueOrEmpty(report.TRT168) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432241
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432241()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432241
            Trade debtors is greater than all current assets.
    
            Legacy Rule Format:
            IF [TRT170] <> NULL AND ([TRT156] < [TRT170])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT156] = TRT:RP.Closing(Instant):bafpo1.02.00:Assets.CurrentTotal.Amount
            [TRT170] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount

            Technical Business Rule Format:
            ^TRT170 <> NULL AND (^TRT156 < ^TRT170)
    
            Data Elements:
    
            ^TRT170 = RP.Closing(Instant):Assets.TradeandReceivablesOther.Amount
    
            ^TRT156 = RP.Closing(Instant):Assets.CurrentTotal.Amount
            */
            assertion = (report.TRT170 != null && report.TRT156.GetValueOrDefault() < report.TRT170.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430199", Severity = ProcessMessageSeverity.Error,
                    Description = @"Trade debtors amount must not exceed All current assets amount",
                    Location = "/xbrli:xbrl/tns:Assets.TradeandReceivablesOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432241"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT170", Value = GetValueOrEmpty(report.TRT170) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT156", Value = GetValueOrEmpty(report.TRT156) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432242
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432242()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432242
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT170] <> NULL AND [TRT170] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT170] = TRT:RP.Closing(Instant):bafpo1.02.13:Assets.TradeandReceivablesOther.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT170, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT170 = RP.Closing(Instant):Assets.TradeandReceivablesOther.Amount
            */
            assertion = NotMonetary(report.TRT170, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.TradeandReceivablesOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432242"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT170", Value = GetValueOrEmpty(report.TRT170) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432243
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432243()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432243
            Trade creditors is greater than all current liabilities.
    
            Legacy Rule Format:
            IF [TRT171] <> NULL AND ([TRT158] < [TRT171])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT158] = TRT:RP.Closing(Instant):bafpo1.02.00:Liabilities.CurrentTotal.Amount
            [TRT171] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount

            Technical Business Rule Format:
            ^TRT171 <> NULL AND (^TRT158 < ^TRT171)
    
            Data Elements:
    
            ^TRT171 = RP.Closing(Instant):Liabilities.TradeAndOtherPayablesTotal.Amount
    
            ^TRT158 = RP.Closing(Instant):Liabilities.CurrentTotal.Amount
            */
            assertion = (report.TRT171 != null && report.TRT158.GetValueOrDefault() < report.TRT171.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430200", Severity = ProcessMessageSeverity.Error,
                    Description = @"Trade creditors cannot exceed all current liabilities in Financial and other information",
                    Location = "/xbrli:xbrl/tns:Liabilities.TradeAndOtherPayablesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432243"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT171", Value = GetValueOrEmpty(report.TRT171) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT158", Value = GetValueOrEmpty(report.TRT158) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432244
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432244()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432244
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT171] <> NULL AND [TRT171] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT171] = TRT:RP.Closing(Instant):bafpo1.02.13:Liabilities.TradeAndOtherPayablesTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT171, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT171 = RP.Closing(Instant):Liabilities.TradeAndOtherPayablesTotal.Amount
            */
            assertion = NotMonetary(report.TRT171, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Liabilities.TradeAndOtherPayablesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432244"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT171", Value = GetValueOrEmpty(report.TRT171) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432245
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432245()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432245
            In other business and professional items, the type code indicating the expense component where the salary and wage expenses for total salary and wage expenses is present, however, total salary and wage expenses amount is missing.
    
            Legacy Rule Format:
            IF [TRT173] <> NULL AND ([TRT172] = 0 OR [TRT172] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT172] = TRT:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [TRT173] = TRT:RP:lrla.02.05:Remuneration.WagesAndSalariesAction.Code

            Technical Business Rule Format:
            ^TRT173 <> NULL AND (^TRT172 = 0 OR ^TRT172 = NULL)
    
            Data Elements:
    
            ^TRT172 = RP:Remuneration.WagesAndSalaries.Total.Amount
    
            ^TRT173 = RP:Remuneration.WagesAndSalariesAction.Code
            */
            assertion = (report.TRT173 != null && (report.TRT172 == 0 || report.TRT172 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430131", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total salary and wage expenses amount must be present if the action code is present",
                    LongDescription = @"If a Total salary and wage expenses action code is present an amount must be present. Either enter an amount or delete the action code",
                    Location = "/xbrli:xbrl/tns:Remuneration.WagesAndSalaries.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432245"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT173", Value = GetValueOrEmpty(report.TRT173) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT172", Value = GetValueOrEmpty(report.TRT172) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432246
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432246()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432246
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT172] <> NULL AND [TRT172] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT172] = TRT:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT172, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT172 = RP:Remuneration.WagesAndSalaries.Total.Amount
            */
            assertion = NotMonetary(report.TRT172, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.WagesAndSalaries.Total.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432246"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT172", Value = GetValueOrEmpty(report.TRT172) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432247
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432247()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432247
            In other business and professional items, total salary and wage expenses amount is present; however, the type code indicating the expense component where the salary and wage expenses have been reported is missing.
    
            Legacy Rule Format:
            IF [TRT172] > 0 AND [TRT173] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT172] = TRT:RP:lrla.02.17:Remuneration.WagesAndSalaries.Total.Amount
            [TRT173] = TRT:RP:lrla.02.05:Remuneration.WagesAndSalariesAction.Code

            Technical Business Rule Format:
            ^TRT172 > 0 AND ^TRT173 = NULL
    
            Data Elements:
    
            ^TRT173 = RP:Remuneration.WagesAndSalariesAction.Code
    
            ^TRT172 = RP:Remuneration.WagesAndSalaries.Total.Amount
            */
            assertion = (report.TRT172.GetValueOrDefault() > 0 && report.TRT173 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430132", Severity = ProcessMessageSeverity.Error,
                    Description = @"Total salary and wage expenses action code must be present",
                    LongDescription = @"If a Total salary and wage expenses amount is present, then an action code must be present",
                    Location = "/xbrli:xbrl/tns:Remuneration.WagesAndSalariesAction.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432247"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT172", Value = GetValueOrEmpty(report.TRT172) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT173", Value = GetValueOrEmpty(report.TRT173) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432248
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432248()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432248
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT174] <> NULL AND [TRT174] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT174] = TRT:RP:lrla.02.00:Remuneration.PaymentToRelatedPartiesGross.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT174, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT174 = RP:Remuneration.PaymentToRelatedPartiesGross.Amount
            */
            assertion = NotMonetary(report.TRT174, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.PaymentToRelatedPartiesGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432248"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT174", Value = GetValueOrEmpty(report.TRT174) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432249
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432249()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432249
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT175] <> NULL AND [TRT175] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT175] = TRT:RP:bafpr3.02.20:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT175, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT175 = RP:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount
            */
            assertion = NotMonetary(report.TRT175, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciatingAssets.IntangibleFirstDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432249"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT175", Value = GetValueOrEmpty(report.TRT175) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432250
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432250()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432250
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT176] <> NULL AND [TRT176] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT176] = TRT:RP:bafpr3.02.20:Expense.DepreciatingAssets.OtherFirstDeducted.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT176, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT176 = RP:Expense.DepreciatingAssets.OtherFirstDeducted.Amount
            */
            assertion = NotMonetary(report.TRT176, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.DepreciatingAssets.OtherFirstDeducted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432250"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT176", Value = GetValueOrEmpty(report.TRT176) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432251
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432251()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432251
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT177] <> NULL AND [TRT177] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT177] = TRT:RP:bafpo7.02.02:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT177, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT177 = RP:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount
            */
            assertion = NotMonetary(report.TRT177, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.DepreciatingAssets.IntangibleTerminationValue.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432251"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT177", Value = GetValueOrEmpty(report.TRT177) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432252
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432252()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432252
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT178] <> NULL AND [TRT178] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT178] = TRT:RP:bafpo7.02.02:Assets.DepreciatingAssets.OtherTerminationValue.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT178, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT178 = RP:Assets.DepreciatingAssets.OtherTerminationValue.Amount
            */
            assertion = NotMonetary(report.TRT178, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Assets.DepreciatingAssets.OtherTerminationValue.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432252"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT178", Value = GetValueOrEmpty(report.TRT178) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432253
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432253()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432253
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT179] <> NULL AND [TRT179] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT179] = TRT:RP:bafpr1.02.02:Expense.ProjectPoolAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT179, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT179 = RP:Expense.ProjectPoolAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.TRT179, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.ProjectPoolAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432253"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT179", Value = GetValueOrEmpty(report.TRT179) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432254
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432254()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432254
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT180] <> NULL AND [TRT180] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT180] = TRT:RP:bafpr1.02.02:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT180, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT180 = RP:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount
            */
            assertion = NotMonetary(report.TRT180, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.CapitalExpenditureSpecifiedAllowableDeduction.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432254"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT180", Value = GetValueOrEmpty(report.TRT180) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432255
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432255()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432255
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT181] <> NULL AND [TRT181] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT181] = TRT:RP:lrla.02.00:Remuneration.FringeBenefits.EmployeeContribution.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT181, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT181 = RP:Remuneration.FringeBenefits.EmployeeContribution.Amount
            */
            assertion = NotMonetary(report.TRT181, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Remuneration.FringeBenefits.EmployeeContribution.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432255"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT181", Value = GetValueOrEmpty(report.TRT181) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432257
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432257()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432257
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT182] <> NULL AND [TRT182] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT182] = TRT:RP.JFOREIGN:bafpr1.02.05:Expense.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT182, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT182 = RP.JFOREIGN:Expense.Interest.Amount
            */
            assertion = NotMonetary(report.TRT182, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Interest.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432257"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT182", Value = GetValueOrEmpty(report.TRT182) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432259
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432259()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432259
            Overseas royalty expenses under business and professional items, is greater than total royalty expenses in the business income and expenses section.
    
            Legacy Rule Format:
            IF [TRT183] > 0 AND [TRT183] > [TRT471] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount
            [TRT471] = TRT:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            ^TRT183 > 0 AND ^TRT183 > ^TRT471
    
            Data Elements:
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
    
            ^TRT471 = RP.JAUS:Expense.Royalties.Amount
            */
            assertion = (report.TRT183.GetValueOrDefault() > 0 && report.TRT183.GetValueOrDefault() > report.TRT471.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430040", Severity = ProcessMessageSeverity.Error,
                    Description = @"Overseas royalty expenses must not exceed Total royalty expenses",
                    LongDescription = @"Overseas royalty expenses - under Business and professional items must not be greater than Total royalty expenses - under Business income and expenses",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432259"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT471", Value = GetValueOrEmpty(report.TRT471) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432260
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432260()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432260
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT183] <> NULL AND [TRT183] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT183] = TRT:RP.JFOREIGN:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT183, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT183 = RP.JFOREIGN:Expense.Royalties.Amount
            */
            assertion = NotMonetary(report.TRT183, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432260"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT183", Value = GetValueOrEmpty(report.TRT183) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432261
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432261()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432261
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT184] <> NULL AND [TRT184] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT184] = TRT:RP:bafpr1.02.28:Expense.LandcareOperationsAcceleratedDepreciation.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT184, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT184 = RP:Expense.LandcareOperationsAcceleratedDepreciation.Amount
            */
            assertion = NotMonetary(report.TRT184, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.LandcareOperationsAcceleratedDepreciation.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432261"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT184", Value = GetValueOrEmpty(report.TRT184) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432263
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432263()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432263
            Unpaid present entitlement to a private company does not contain an amount and code is D or X.
    
            Legacy Rule Format:
            IF [TRT187] = SET("D", "X") AND ([TRT186] = 0 OR [TRT186] = NULL) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT186] = TRT:RP.Closing(Instant):rvctc1.02.00:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount
            [TRT187] = TRT:RP:rvctc1.02.00:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Code

            Technical Business Rule Format:
            InSet(^TRT187, '"D", "X"') AND (^TRT186 = 0 OR ^TRT186 = NULL)
    
            Data Elements:
    
            ^TRT186 = RP.Closing(Instant):IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount
    
            ^TRT187 = RP:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Code
            */
            assertion = (IsMatch(report.TRT187, @"^(D|X)$") && (report.TRT186 == 0 || report.TRT186 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432263", Severity = ProcessMessageSeverity.Error,
                    Description = @"No amount in Unpaid present entitlement to a private company.",
                    LongDescription = @"Unpaid present entitlement to a private company does not contain an amount and code is D or X.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432263"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT187", Value = GetValueOrEmpty(report.TRT187) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT186", Value = GetValueOrEmpty(report.TRT186) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432264
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432264()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432264
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT186] <> NULL AND [TRT186] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT186] = TRT:RP.Closing(Instant):rvctc1.02.00:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT186, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT186 = RP.Closing(Instant):IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount
            */
            assertion = NotMonetary(report.TRT186, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432264"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT186", Value = GetValueOrEmpty(report.TRT186) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432265
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432265()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432265
            Unpaid present entitlement to a private company code box contains an invalid character. The valid characters are D or X.
    
            Legacy Rule Format:
            IF [TRT186] > 0 AND [TRT187] <> SET("D", "X") 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT186] = TRT:RP.Closing(Instant):rvctc1.02.00:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount
            [TRT187] = TRT:RP:rvctc1.02.00:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Code

            Technical Business Rule Format:
            ^TRT186 > 0 AND NotInSet(^TRT187, '"D", "X"')
    
            Data Elements:
    
            ^TRT187 = RP:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Code
    
            ^TRT186 = RP.Closing(Instant):IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Amount
            */
            assertion = (report.TRT186.GetValueOrDefault() > 0 && !(IsMatch(report.TRT187, @"^(D|X)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432265", Severity = ProcessMessageSeverity.Error,
                    Description = @"Unpaid present entitlement to a private company code must be 'D' - Dividend, or 'X' - No dividend",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PrivateCompanyUnpaidPresentEntitlement.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432265"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT186", Value = GetValueOrEmpty(report.TRT186) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT187", Value = GetValueOrEmpty(report.TRT187) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432275
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432275()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432275
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT194] <> NULL AND [TRT194] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT194] = TRT:RP:rvctc2.02.00:IncomeTax.MedicareLevy.BeneficiarySpouseTaxableIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT194, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT194 = RP:IncomeTax.MedicareLevy.BeneficiarySpouseTaxableIncome.Amount
            */
            assertion = NotMonetary(report.TRT194, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.BeneficiarySpouseTaxableIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432275"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT194", Value = GetValueOrEmpty(report.TRT194) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432276
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432276()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432276
            The number of dependent children and students shown for Medicare levy reduction or exemption cannot be greater than 12.
    
            Legacy Rule Format:
            IF [TRT195] > 12 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT195] = TRT:RP:rvctc2.02.14:IncomeTax.MedicareLevy.DependentChildrenAndStudents.Count

            Technical Business Rule Format:
            ^TRT195 > 12
    
            Data Elements:
    
            ^TRT195 = RP:IncomeTax.MedicareLevy.DependentChildrenAndStudents.Count
            */
            assertion = (report.TRT195.GetValueOrDefault() > 12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432276", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of dependent children and students is greater than 12",
                    LongDescription = @"The number of dependent children and students shown for Medicare levy reduction or exemption cannot be greater than 12.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.DependentChildrenAndStudents.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432276"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT195", Value = report.TRT195.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432277
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432277()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432277
            The number of days full levy exemption and/or the number of days half levy exemption exceed 365 (366 in leap years).
    
            Legacy Rule Format:
            IF [TRT196] + [TRT198] > ([TRT1]&"-06-30") - (([TRT1]-1)&"-07-01") +1  
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT196] = TRT:RP:rvctc2.02.15:IncomeTax.MedicareLevy.ExemptionFullDays.Count
            [TRT198] = TRT:RP:rvctc2.02.15:IncomeTax.MedicareLevy.ExemptionHalfDays.Count

            Technical Business Rule Format:
            (^TRT196 + ^TRT198) > (DaysInDuration(AsDate(Concat(^TRT1, '-06-30')) - AsDate(Concat(^TRT1-1, '-07-01'))) + 1)
    
            Data Elements:
    
            ^TRT196 = RP:IncomeTax.MedicareLevy.ExemptionFullDays.Count
    
            ^TRT1 = RP:Report.TargetFinancial.Year
    
            ^TRT198 = RP:IncomeTax.MedicareLevy.ExemptionHalfDays.Count
            */
            assertion = (report.TRT196.GetValueOrDefault() + report.TRT198.GetValueOrDefault() > (AsDate(string.Concat(report.TRT1.GetValueOrDefault(), @"-06-30")) - AsDate(string.Concat(report.TRT1.GetValueOrDefault() - 1, @"-07-01"))).Days + 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432614", Severity = ProcessMessageSeverity.Error,
                    Description = @"Levy exemption days exceeds limit",
                    LongDescription = @"The number of days full levy exemption and/or the number of days half levy exemption exceed 365 (366 in leap years).",
                    Location = "/xbrli:xbrl/tns:IncomeTax.MedicareLevy.ExemptionFullDays.Count",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432277"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT196", Value = report.TRT196.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT198", Value = report.TRT198.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432280
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432280(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432280
                    A trust can only make an overall distribution of net income to each individual or non-individual beneficiary. Please adjust distributed income until the overall negative amount no longer exists.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF ([TRT200] <> NULLORBLANK OR [TRT203] <> NULLORBLANK) AND ([TRT219] + [TRT218] + [TRT442] + [TRT223] + [TRT224] + [TRT225] + [TRT217] < 0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT223] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT224] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT225] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT200 <> BLANK OR ^TRT203 <> BLANK) AND (^TRT219 + ^TRT218 + ^TRT442 + ^TRT223 + ^TRT224 + ^TRT225 + ^TRT217 < 0)
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT223 = RP:StatementOfDistributionToBeneficiary:Income.CapitalGainsNet.Amount
            
                    ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
            
                    ^TRT225 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.Net.Amount
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
                    */
                    assertion = ((string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) != true) && statementOfDistributionToBeneficiary.TRT219.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT218.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT442.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT223.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT224.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT225.GetValueOrDefault() + statementOfDistributionToBeneficiary.TRT217.GetValueOrDefault() < 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432280", Severity = ProcessMessageSeverity.Error,
                            Description = @"Trust beneficiary cannot have a zero or a negative income distribution",
                            LongDescription = @"A trust can only make an overall distribution of net income to each individual or non-individual beneficiary.  Please adjust distributed income until the overall negative amount no longer exists.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432280"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT223", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT223) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT224", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT224) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT225", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT225) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT217", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT217) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432281
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432281(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432281
                    Either an individual or non-individual name must be present in each Statement of distribution
    
                    Legacy Rule Format:
                    WHERE IN 
                    TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214]  <> NULL AND ([TRT200] = NULLORBLANK AND [TRT203] = NULLORBLANK)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT214  <> NULL AND (^TRT200 = BLANK AND ^TRT203 = BLANK)
            
                    Data Elements:
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT214 != null && (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) == true && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) == true));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432281", Severity = ProcessMessageSeverity.Error,
                            Description = @"Either an individual or non-individual name must be present in each Statement of distribution",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPOccurrenceIndex) + "/tns:PersonNameDetails.FamilyName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432281"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432282
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432282(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432282
                    Family name and non-individual name line 1 are both present in the distribution statement. Please delete the incorrect name as only one individual or non-individual name may be present.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT200] <> NULLORBLANK AND [TRT203] <> NULLORBLANK 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT200 <> BLANK AND ^TRT203 <> BLANK
            
                    Data Elements:
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) != true && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) != true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432615", Severity = ProcessMessageSeverity.Error,
                            Description = @"Both family name and company name present",
                            LongDescription = @"Family name and non-individual name line 1 are both present in the distribution statement. Please delete the incorrect name as only one individual or non-individual name may be present.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:OrganisationNameDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432282"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432283
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432283(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432283
                    Foreign income tax offsets are greater than zero in distribution statement and there is no Attributed foreign income or Other assessable foreign source income.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary) 
                    IF [TRT226] > 0  AND [TRT224] = NULL AND [TRT225] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT224] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT225] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT226] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT226 > 0  AND ^TRT224 = NULL AND ^TRT225 = NULL
            
                    Data Elements:
            
                    ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
            
                    ^TRT225 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.Net.Amount
            
                    ^TRT226 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.TaxOffset.Amount
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT226.GetValueOrDefault() > 0 && statementOfDistributionToBeneficiary.TRT224 == null && statementOfDistributionToBeneficiary.TRT225 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432283", Severity = ProcessMessageSeverity.Error,
                            Description = @"Attributed foreign income or Other assessable foreign source income must be present",
                            LongDescription = @"If 'Foreign income tax offsets' is greater than zero in a Statement of Distribution',  then there must be 'Attributed foreign income' or 'Other assessable foreign source income' within a beneficiary's Statement of Distribution or against the 'Income to which no beneficiary is presently entitled'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AttributedForeignIncome.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432283"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT226", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT226) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT224", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT224) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT225", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT225) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432284
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432284(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432284
                    TFN or address must be present in each Statement of Distribution
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT211] = NULL AND COUNT (TUPLE(address2.xx.xx:AddressDetails)) = 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^StatementOfDistributionToBeneficiary <> NULL AND ^TRT211 = NULL AND Count(^AddressDetails) = 0 AND Count(^AddressDetails2) = 0
            
                    Data Elements:
            
                    ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
            
                    ^StatementOfDistributionToBeneficiary = RP:StatementOfDistributionToBeneficiary
            
                    ^AddressDetails2 = RP:StatementOfDistributionToBeneficiary:AddressDetails
                    */
                    assertion = (report.StatementOfDistributionToBeneficiaryCollectionExists != false && statementOfDistributionToBeneficiary.TRT211 == null && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPCount) == 0 && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPCount) == 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.430141", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN or address must be present in each Statement of Distribution",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432284"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT211", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT211) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432285
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432285()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432285
            The TFN provided for the Trustee Beneficiary or a Partner in the distribution statement should not be the same as the TFN of the Trust making the statement or the TFN of the Partnership return.
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF[TRT211]) = RP:entity.indentifier.TFN
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)

            Technical Business Rule Format:
            (AnyOccurrence(^TRT211, ^TRT211 = ^TFN))
    
            Data Elements:
    
            ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
    
            ^TFN = RP
            */
            assertion = (report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT211Repeat => TRT211Repeat.TRT211 == report.RPIdentifierTFN));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430142", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax file number must not be the TFN of the Trust or Partnership",
                    LongDescription = @"The TFN provided for a Trustee or Partner in a Statement of distribution must not be the same as the TFN of the Trust or Partnership",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432285"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT211", Value = "TRT211" });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432286
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432286(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432286
                    Beneficiary's identifier is present in the distribution statement without a beneficiary’s tax file number.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT212] <> NULLORBLANK AND [TRT211] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT212] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumberTrailingNumeric.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT212 <> BLANK AND ^TRT211 = NULL
            
                    Data Elements:
            
                    ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
            
                    ^TRT212 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumberTrailingNumeric.Identifier
                    */
                    assertion = (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT212) != true && statementOfDistributionToBeneficiary.TRT211 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432286", Severity = ProcessMessageSeverity.Error,
                            Description = @"Beneficiary's tax file number must be present if the TFN identifier is present",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432286"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT212", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT212) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT211", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT211) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432287
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432287(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432287
                    Resident beneficiary details must include TFN and Name. Only one individidual or non-individual name may be present.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214] = SET("14", "35")  AND [TRT344] = TRUE AND [TRT211] = NULL AND [TRT200] = NULLORBLANK AND [TRT203] = NULLORBLANK
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    InSet(^TRT214, '"14", "35"')  AND ^TRT344 = TRUE AND ^TRT211 = NULL AND ^TRT200 = BLANK AND ^TRT203 = BLANK
            
                    Data Elements:
            
                    ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
                    */
                    assertion = (IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(14|35)$") && statementOfDistributionToBeneficiary.TRT344 == true && statementOfDistributionToBeneficiary.TRT211 == null && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) == true && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432287", Severity = ProcessMessageSeverity.Error,
                            Description = @"To make a correct TB statement, TFN must be present for resident trustee beneficiary",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumber.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432287"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT211", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT211) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432288
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432288(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432288
                    TFN trailing number must be in the range 1 to 49.
    
                    Legacy Rule Format:
                    IF [TRT212] <> NULL AND [TRT212] <> SET(1-49) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT212] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumberTrailingNumeric.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT212 <> NULL AND (^TRT212 < 1 OR ^TRT212 > 49)
            
                    Data Elements:
            
                    ^TRT212 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumberTrailingNumeric.Identifier
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT212 != null && (AsNumeric(statementOfDistributionToBeneficiary.TRT212) < 1 || AsNumeric(statementOfDistributionToBeneficiary.TRT212) > 49));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432288", Severity = ProcessMessageSeverity.Error,
                            Description = @"TFN trailing number must be in the range 1 to 49.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Identifiers.TaxFileNumberTrailingNumeric.Identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432288"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT212", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT212) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432289
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432289(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432289
                    Assessment calculation code is 11, 25, 26, 27, 28, 29, 111, 125,126,127,128 129 in the distribution statement and beneficiary's birth date is missing.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214] = SET("11", "25", "26", "27", "28", "29", "111", "125", "126", "127", "128", "129") AND [TRT213] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT213] = TRT:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    InSet(^TRT214, '"11", "25", "26", "27", "28", "29", "111", "125", "126", "127", "128", "129"') AND ^TRT213 = NULL
            
                    Data Elements:
            
                    ^TRT213 = RP:StatementOfDistributionToBeneficiary:PersonDemographicDetails.Birth.Date
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(11|25|26|27|28|29|111|125|126|127|128|129)$") && statementOfDistributionToBeneficiary.TRT213 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432289", Severity = ProcessMessageSeverity.Error,
                            Description = @"Beneficiary's date of birth must be present",
                            LongDescription = @"If the Assessment calculation code in the distribution statement is 11, 25 to 29, 111, 125 to 129, the beneficiary's Date of birth must be present",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:PersonDemographicDetails.Birth.Date",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432289"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT213", Value = GetValueOrEmptyDate(statementOfDistributionToBeneficiary.TRT213) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432290
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432290()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432290
            Assessment calculation code is 111, 118, 119, 120, 125,126,127,128,129, 134, 138, 139 or 140 in the distribution statement and Other Attachments Schedule is missing.
    
            Legacy Rule Format:
            IF ANY OCCURRENCE OF [TRT214] = SET("111", "118", "119", "120", "125", "126", "127", "128", "129", "134", "138", "139", "140") AND [TRT405] = NULLORBLANK
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT405] = TRT:RP:bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text

            Technical Business Rule Format:
            AnyOccurrence(^TRT214, InSet(^TRT214, '"111", "118", "119", "120", "125", "126", "127", "128", "129", "134", "138", "139", "140"')) AND ^TRT405 = BLANK
    
            Data Elements:
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
    
            ^TRT405 = RP:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => IsMatch(TRT214Repeat.TRT214, @"^(111|118|119|120|125|126|127|128|129|134|138|139|140)$"))) && string.IsNullOrWhiteSpace(report.TRT405) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432723", Severity = ProcessMessageSeverity.Error,
                    Description = @"Further information must be provided at Other attachments schedule",
                    LongDescription = @"If the Assessment calculation code in a Statement of Distribution is a non-resident code (111, 118, 119, 120, 125 to 129, 134, 138, 139 or 140) then further details must be supplied using the 'Other Attachments' schedule",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:IncomeTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432290"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT405", Value = GetValueOrEmpty(report.TRT405) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432292
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432292(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432292
                    Primary production income, non-primary production income, credit for tax withheld where ABN not quoted, Div 6AA eligible income, franked distributions, franking credit, TFN credit or Australian franking credit from a NZ company is present in the distribution statement and the assessment calculation code is missing.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF ([TRT218] <> NULL OR [TRT219] <> NULL  OR [TRT442] <> NULL OR [TRT220] > 0 OR [TRT549] > 0 OR [TRT221] > 0 OR [TRT222] > 0 OR [TRT217] > 0) AND [TRT214] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT220] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT221] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT222] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT549] = TRT:RP:bafpr1.02.41:Income.TaxableMinorBeneficiary.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT218 <> NULL OR ^TRT219 <> NULL  OR ^TRT442 <> NULL OR ^TRT220 > 0 OR ^TRT549 > 0 OR ^TRT221 > 0 OR ^TRT222 > 0 OR ^TRT217 > 0) AND ^TRT214 = NULL
            
                    Data Elements:
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            
                    ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT220 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    ^TRT222 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
            
                    ^TRT549 = RP:StatementOfDistributionToBeneficiary:Income.TaxableMinorBeneficiary.Amount
                    */
                    assertion = ((statementOfDistributionToBeneficiary.TRT218 != null || statementOfDistributionToBeneficiary.TRT219 != null || statementOfDistributionToBeneficiary.TRT442 != null || statementOfDistributionToBeneficiary.TRT220.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT549.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT221.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT222.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT217.GetValueOrDefault() > 0) && statementOfDistributionToBeneficiary.TRT214 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432616", Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment calculation code must be present",
                            LongDescription = @"If a Statement of Distribution contains an amount at Primary production income, Non-primary production income, Credit for tax withheld where ABN not quoted, Franked distributions, Franking credit, TFN amounts withheld, or Australian franking credit from a NZ company, then an Assessment calculation code must be present in the statement.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AssessmentCalculation.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432292"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT220", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT220) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT549", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT549) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT221", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT221) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT222", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT222) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT217", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT217) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432293
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432293(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432293
                    If within a Statement of Distribution the answer to the TB statement question is 'no' (false), then an Assessment calculation code must be present
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF ([TRT200] <> NULLORBLANK OR [TRT203] <> NULLORBLANK) AND [TRT344] <> TRUE AND [TRT214] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT200 <> BLANK OR ^TRT203 <> BLANK) AND ^TRT344 <> TRUE AND ^TRT214 = NULL
            
                    Data Elements:
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
                    */
                    assertion = ((string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) != true) && statementOfDistributionToBeneficiary.TRT344 != true && statementOfDistributionToBeneficiary.TRT214 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432293", Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment calculation code must be present",
                            LongDescription = @"If within a Statement of Distribution the answer to the TB statement question is either 'no' (false) or absent, then an Assessment calculation code must be present",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AssessmentCalculation.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432293"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432294
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432294(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432294
                    Address of beneficiary must be present and be an overseas address
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214] = SET("120", "140") AND [TRT344] = TRUE AND (([TRT200] = NULLORBLANK AND [TRT203] = NULLORBLANK) OR (([TRT205] = NULLORBLANK OR [TRT207] = NULLORBLANK OR [TRT382] = NULL OR [TRT320] <> TRUE) AND ([TRT511] = NULLORBLANK OR [TRT515] = NULLORBLANK OR [TRT519] = NULL OR [TRT508] <> TRUE)))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT205] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT207] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT320] = TRT:RP:pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT382] = TRT:RP:pyde.02.08:AddressDetails.Country.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT508] = TRT:RP:pyde.02.00:AddressDetails.OverseasAddress.Indicator WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT511] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT515] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT519] = TRT:RP:pyde.02.08:AddressDetails.Country.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    InSet(^TRT214, '"120", "140"') AND ^TRT344 = TRUE AND ((^TRT200 = BLANK AND ^TRT203 = BLANK) OR ((^TRT205 = BLANK OR ^TRT207 = BLANK OR ^TRT382 = NULL OR ^TRT320 <> TRUE) AND (^TRT511 = BLANK OR ^TRT515 = BLANK OR ^TRT519 = NULL OR ^TRT508 <> TRUE)))
            
                    Data Elements:
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT320 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
            
                    ^TRT382 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT508 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.OverseasAddress.Indicator WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT519 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = (IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(120|140)$") && statementOfDistributionToBeneficiary.TRT344 == true && (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) == true && string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) == true || (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT205) == true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT207) == true || statementOfDistributionToBeneficiary.TRT382 == null || statementOfDistributionToBeneficiary.TRT320 != true) && (string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT511) == true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT515) == true || statementOfDistributionToBeneficiary.TRT519 == null || statementOfDistributionToBeneficiary.TRT508 != true)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432294", Severity = ProcessMessageSeverity.Error,
                            Description = @"Address of beneficiary must be present and be an overseas address",
                            LongDescription = @"If you are making a TB Statement for a non-resident beneficiary (using Assessment calculation code 120 or 140) the address of the beneficiary must be present in the Statement of Distribution. The overseas address indicator must be 'yes' (true)",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AssessmentCalculation.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432294"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT382", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT382) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT320", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT320) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT519", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT519) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT508", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT508) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432295
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432295(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432295
                    Where a trust is making a TB statement by answering the TB question Yes, the Assessment calculation code should be NULL, 35, 140, 14 or 120.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF  [TRT214] <> NULL AND [TRT214] <> SET("14", "35", "120", "140") AND [TRT344] = TRUE
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT214 <> NULL AND NotInSet(^TRT214, '"14", "35", "120", "140"') AND ^TRT344 = TRUE
            
                    Data Elements:
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT214 != null && !(IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(14|35|120|140)$",RegexOptions.IgnoreCase)) && statementOfDistributionToBeneficiary.TRT344 == true);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432295", Severity = ProcessMessageSeverity.Error,
                            Description = @"TB statement must not be made if the distribution is not to a trustee beneficiary",
                            LongDescription = @"If within a Statement of Distribution the answer to the 'TB statement' question is 'yes' (true), then the 'Assessment calculation code' must be a trustee beneficiary code (14, 35, 120 or 140) or blank.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AssessmentCalculation.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432295"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432296
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432296(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432296
                    Where credit for tax withheld - foreign resident withholding (excluding capital gains) is greater than zero, there must be an amount at either share of income - primary or non-primary production or franked distributions. Beneficiary entitled.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT476] > 0 AND ([TRT218] = NULL AND [TRT219] = NULL AND [TRT442] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT476] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT476 > 0 AND (^TRT218 = NULL AND ^TRT219 = NULL AND ^TRT442 = NULL)
            
                    Data Elements:
            
                    ^TRT476 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT476.GetValueOrDefault() > 0 && (statementOfDistributionToBeneficiary.TRT218 == null && statementOfDistributionToBeneficiary.TRT219 == null && statementOfDistributionToBeneficiary.TRT442 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432738", Severity = ProcessMessageSeverity.Error,
                            Description = @"PP or Non PP Share of income or franked distributions must be present",
                            LongDescription = @"If within a Statement of Distribution there is an amount greater than zero at 'Credit for tax withheld - foreign resident withholding (excluding capital gains)', then there must be an amount at either 'Share of income - primary production'  or 'Share of income - non-primary production' or 'Franked distributions' within the Statement of Distribution.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432296"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT476", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT476) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432298
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432298()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432298
            If the trust is in an income position, and Australian franking credits from a New Zealand franking company is present, then this amount must be greater than or equal to the amount of Australian franking credits from a New Zealand franking company in the distribution statement (beneficiary and non-beneficiary).
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] > 0) AND [TRT143] > 0 AND [TRT143] < SUM([TRT217]) + [TRT233] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT233] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 > 0) AND ^TRT143 > 0 AND ^TRT143 < Sum(^TRT217) + ^TRT233
    
            Data Elements:
    
            ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT233 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() > 0 && report.TRT143.GetValueOrDefault() > 0 && report.TRT143.GetValueOrDefault() < (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT217.GetValueOrDefault())) + report.TRT233.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432667", Severity = ProcessMessageSeverity.Error,
                    Description = @"Australian franking credits from a NZ franking company amount must not be less than distributed amounts",
                    LongDescription = @"If the overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses), and 'Australian franking credits from a NZ franking company' is present for the trust, then this amount must be greater than or equal to the amount of 'Australian franking credits from a NZ franking company' in the Statements of distribution",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432298"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT217", Value = "TRT217" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT233", Value = GetValueOrEmpty(report.TRT233) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432299
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432299(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432299
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount <> NULL 
                    AND TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT217, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT217, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432299"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT217", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT217) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432299
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432299_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432299
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount <> NULL 
            AND TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT233, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT233 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            */
            assertion = NotMonetary(report.TRT233, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432299"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT233", Value = GetValueOrEmpty(report.TRT233) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432300
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432300(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432300
                    If value is included in entity type, TFN or any of the Beneficiary Name and address fields, then an amount must be included at Primary production (PP) or non PP income or Franked distributions or Franking credit or TFN amounts withheld or Australian franking credit from a NZ franking company or the TB Statement-Tax preferred amounts or TB Statement -Untaxed part of share of net income for each beneficiary listed in the distribution statement.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF ([TRT432]<>NULL OR [TRT211]<>NULLORBLANK OR [TRT212]<>NULLORBLANK OR [TRT200]<>NULLORBLANK OR [TRT201]<>NULLORBLANK OR [TRT202]<>NULLORBLANK OR [TRT203] <> NULLORBLANK OR [TRT213]<>NULL OR [TRT205]<>NULLORBLANK OR [TRT206]<>NULLORBLANK OR [TRT207]<>NULLORBLANK OR [TRT208]<>NULL OR [TRT209]<>NULL OR [TRT382]<>NULL OR [TRT511]<>NULLORBLANK OR [TRT512]<>NULLORBLANK OR [TRT515]<>NULLORBLANK OR [TRT517]<>NULL OR [TRT516]<>NULL OR [TRT519]<>NULL) AND [TRT218]=NULL AND [TRT219]=NULL AND [TRT442]= NULL AND [TRT221] =NULL AND [TRT222]=NULL AND [TRT217]=NULL AND [TRT345]=NULL AND [TRT346]= NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT200] = TRT:RP:pyde.02.00:PersonNameDetails.FamilyName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT201] = TRT:RP:pyde.02.00:PersonNameDetails.GivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT202] = TRT:RP:pyde.02.00:PersonNameDetails.OtherGivenName.Text WHERE (pyde.02.00:PersonNameDetails.PersonNameType.Code = "LGL" AND pyde.02.05:PersonNameDetails.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT203] = TRT:RP:pyde.02.50:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = "MN" AND pyde.02.00:OrganisationNameDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT205] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT206] = TRT:RP:pyde.02.00:AddressDetails.Line2.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT207] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT208] = TRT:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT209] = TRT:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT212] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumberTrailingNumeric.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT213] = TRT:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT221] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT222] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT345] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT346] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT382] = TRT:RP:pyde.02.08:AddressDetails.Country.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "RES" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT511] = TRT:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT512] = TRT:RP:pyde.02.00:AddressDetails.Line2.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT515] = TRT:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT516] = TRT:RP:pyde.02.00:AddressDetails.Postcode.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT517] = TRT:RP:pyde.02.00:AddressDetails.StateOrTerritory.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT519] = TRT:RP:pyde.02.08:AddressDetails.Country.Code WHERE (pyde.02.01:AddressDetails.Usage.Code = "BUS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(trt.0011.lodge.req.02.00:AddressDetails) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT432 <> NULL OR ^TRT211 <>BLANK OR ^TRT212 <>BLANK OR ^TRT200 <>BLANK OR ^TRT201 <>BLANK OR ^TRT202 <>BLANK OR ^TRT203 <> BLANK OR ^TRT213 <> NULL OR ^TRT205 <>BLANK OR ^TRT206 <>BLANK OR ^TRT207 <>BLANK OR ^TRT208 <> NULL OR ^TRT209 <> NULL OR ^TRT382 <> NULL OR ^TRT511 <>BLANK OR ^TRT512 <>BLANK OR ^TRT515 <>BLANK OR ^TRT517 <> NULL OR ^TRT516 <> NULL OR ^TRT519 <> NULL) AND ^TRT218=NULL AND ^TRT219=NULL AND ^TRT442= NULL AND ^TRT221 =NULL AND ^TRT222=NULL AND ^TRT217=NULL AND ^TRT345=NULL AND ^TRT346= NULL
            
                    Data Elements:
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT200 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.FamilyName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT201 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.GivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT202 = RP:StatementOfDistributionToBeneficiary:PersonNameDetails:PersonNameDetails.OtherGivenName.Text WHERE [PersonNameDetails.PersonNameType.Code='LGL'].[PersonNameDetails.Usage.Code='Contact']
            
                    ^TRT203 = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
            
                    ^TRT205 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT206 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT207 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT208 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT209 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
            
                    ^TRT212 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumberTrailingNumeric.Identifier
            
                    ^TRT213 = RP:StatementOfDistributionToBeneficiary:PersonDemographicDetails.Birth.Date
            
                    ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    ^TRT222 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            
                    ^TRT345 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount
            
                    ^TRT346 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount
            
                    ^TRT382 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='RES'].[AddressDetails.Currency.Code='C']
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
            
                    ^TRT511 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT512 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Line2.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT515 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT516 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Postcode.Text WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT517 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.StateOrTerritory.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
            
                    ^TRT519 = RP:StatementOfDistributionToBeneficiary:AddressDetails:AddressDetails.Country.Code WHERE [AddressDetails.Usage.Code='BUS'].[AddressDetails.Currency.Code='C']
                    */
                    assertion = ((statementOfDistributionToBeneficiary.TRT432 != null || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT211) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT212) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT200) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT201) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT202) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT203) != true || statementOfDistributionToBeneficiary.TRT213 != null || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT205) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT206) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT207) != true || statementOfDistributionToBeneficiary.TRT208 != null || statementOfDistributionToBeneficiary.TRT209 != null || statementOfDistributionToBeneficiary.TRT382 != null || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT511) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT512) != true || string.IsNullOrWhiteSpace(statementOfDistributionToBeneficiary.TRT515) != true || statementOfDistributionToBeneficiary.TRT517 != null || statementOfDistributionToBeneficiary.TRT516 != null || statementOfDistributionToBeneficiary.TRT519 != null) && statementOfDistributionToBeneficiary.TRT218 == null && statementOfDistributionToBeneficiary.TRT219 == null && statementOfDistributionToBeneficiary.TRT442 == null && statementOfDistributionToBeneficiary.TRT221 == null && statementOfDistributionToBeneficiary.TRT222 == null && statementOfDistributionToBeneficiary.TRT217 == null && statementOfDistributionToBeneficiary.TRT345 == null && statementOfDistributionToBeneficiary.TRT346 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432637", Severity = ProcessMessageSeverity.Error,
                            Description = @"Each Statement of Distribution must contain distribution amounts",
                            LongDescription = @"A Statement of distribution must include an amount against at least one of: Primary production income, Non-primary production income, Franked distributions, Franking credit, TFN amounts withheld, Australian franking credit from a NZ company, TB Statement-Tax preferred amounts or TB Statement-Untaxed part of share of net income.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432300"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT211", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT211) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT212", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT212) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT200", Value = statementOfDistributionToBeneficiary.TRT200 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT201", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT201) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT202", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT202) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT203", Value = statementOfDistributionToBeneficiary.TRT203 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT213", Value = GetValueOrEmptyDate(statementOfDistributionToBeneficiary.TRT213) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT205", Value = statementOfDistributionToBeneficiary.TRT205 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT206", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT206) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT207", Value = statementOfDistributionToBeneficiary.TRT207 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT208", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT208) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT209", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT209) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT382", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT382) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT511", Value = statementOfDistributionToBeneficiary.TRT511 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT512", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT512) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT515", Value = statementOfDistributionToBeneficiary.TRT515 });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT517", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT517) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT516", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT516) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT519", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT519) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT221", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT221) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT222", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT222) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT217", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT217) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT345", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT345) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT346", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT346) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432301
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432301(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432301
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT218] <> NULL AND [TRT218] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT218, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT218, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432301"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432302
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432302(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432302
                    If an amount greater than zero is present at 'Credit for tax withheld where ABN not quoted' against a Statement of Distribution, then there must be Primary production or Non-primary production income or franked distributions amount against the beneficiary
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary) 
                    IF [TRT220] > 0  AND [TRT218] = NULL AND [TRT219] = NULL  AND [TRT442] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT218] = TRT:RP.Prim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT220] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT220 > 0  AND ^TRT218 = NULL AND ^TRT219 = NULL  AND ^TRT442 = NULL
            
                    Data Elements:
            
                    ^TRT220 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            
                    ^TRT218 = RP.Prim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT220.GetValueOrDefault() > 0 && statementOfDistributionToBeneficiary.TRT218 == null && statementOfDistributionToBeneficiary.TRT219 == null && statementOfDistributionToBeneficiary.TRT442 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432619", Severity = ProcessMessageSeverity.Error,
                            Description = @"PP or Non-PP income distribution or franked distribution amount must be present in Statement of distribution",
                            LongDescription = @"If an amount greater than zero is present at 'Credit for tax withheld where ABN not quoted' against a Statement of Distribution, then there must be Primary production or Non-primary production income or franked distributions amount against the beneficiary.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432302"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT220", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT220) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT218", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT218) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432303
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432303(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432303
                    Franking credit or TFN amount withheld is present in the distribution statement and there is no non-primary production income amount or franked distributions amount. Zero is acceptable when there is an overall trust loss.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF ([TRT221] > 0 OR [TRT222] > 0) AND [TRT219] = NULL AND [TRT442] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT221] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT222] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT442] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT221 > 0 OR ^TRT222 > 0) AND ^TRT219 = NULL AND ^TRT442 = NULL
            
                    Data Elements:
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
            
                    ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    ^TRT222 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
                    */
                    assertion = ((statementOfDistributionToBeneficiary.TRT221.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT222.GetValueOrDefault() > 0) && statementOfDistributionToBeneficiary.TRT219 == null && statementOfDistributionToBeneficiary.TRT442 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432618", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-PP income or franked distributions amount must be present in Statement of distribution",
                            LongDescription = @"If within a Statement of Distribution ''Franking credit' or 'TFN amount withheld' is present, then there must be an amount at 'Non-primary production income' or 'Franked distributions'. Zero is acceptable if there is an overall trust loss.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432303"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT221", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT221) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT222", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT222) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432304
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432304(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432304
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT219] <> NULL AND [TRT219] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT219] = TRT:RP.NonPrim:bafpr1.02.04:Income.BeneficiaryShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT219, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT219 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.BeneficiaryShare.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT219, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.BeneficiaryShare.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432304"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT219", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT219) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432306
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432306(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432306
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount <> NULL 
                    AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT220, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT220 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT220, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432306"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT220", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT220) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432306
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432306_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432306
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount <> NULL 
            AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT236, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT236 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
            */
            assertion = NotMonetary(report.TRT236, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432306"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT236", Value = GetValueOrEmpty(report.TRT236) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432307
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432307(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432307
                    There is an overall trust loss (the sum of net Australian income/loss, net capital gains, attributed foreign income other assessable foreign source income minus allowable tax losses of earlier income years deducted this year is less than or equal to zero) and franking credits are present in the distribution statement and Australian franking credits from a NZ company (distribution to a beneficiary or no beneficiary presently entitled). Franking credits cannot be distributed when there is an overall trust loss. Delete the franking credits from the distribution statement.
    
                    Legacy Rule Format:
                    IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] <= 0) AND ((ANY OCCURRENCE OF [TRT217] <> NULL OR [TRT221] <> NULL)  OR ([TRT237] <> NULL OR [TRT233] <> NULL))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
                    [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
                    [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
                    [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
                    [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
                    [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
                    [TRT217] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT221] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT233] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
                    [TRT237] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
        
                    Technical Business Rule Format:
                    (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 <= 0) AND ((AnyOccurrence(^TRT217, ^TRT217 <> NULL) OR ^TRT221 <> NULL)  OR (^TRT237 <> NULL OR ^TRT233 <> NULL))
            
                    Data Elements:
            
                    ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            
                    ^TRT130 = RP.JAUS:Income.Net.Amount
            
                    ^TRT133 = RP:Income.CapitalGainsNet.Amount
            
                    ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
            
                    ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
            
                    ^TRT141 = RP.JFOREIGN:Income.Net.Amount
            
                    ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
            
                    ^TRT217 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT233 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            
                    ^TRT237 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
                    */
                    assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() <= 0 && ((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT217Repeat => TRT217Repeat.TRT217 != null)) || statementOfDistributionToBeneficiary.TRT221 != null || (report.TRT237 != null || report.TRT233 != null)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432307", Severity = ProcessMessageSeverity.Error,
                            Description = @"Franking credits must not be present when the trust has zero net income or incurs an overall loss",
                            LongDescription = @"If there is an overall trust loss (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses is less than or equal to zero) then franking credits and Australian franking credits from a NZ company must not be present in a Statement of Distribution. Franking credits cannot be distributed when there is an overall trust loss. Delete the franking credits from the Statement of Distribution",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432307"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT217", Value = "TRT217" });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT221", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT221) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT237", Value = GetValueOrEmpty(report.TRT237) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT233", Value = GetValueOrEmpty(report.TRT233) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432308
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432308(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432308
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount <> NULL 
                    AND TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT221, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT221 = RP:StatementOfDistributionToBeneficiary:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT221, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432308"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT221", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT221) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432308
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432308_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432308
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount <> NULL 
            AND TRT:RP:rvctc2.xx.xx:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT237, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT237 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
            */
            assertion = NotMonetary(report.TRT237, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432308"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT237", Value = GetValueOrEmpty(report.TRT237) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432309
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432309(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432309
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount <> NULL 
                    AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT222, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT222 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT222, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432309"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT222", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT222) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432309
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432309_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432309
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount <> NULL 
            AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT238, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT238 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
            */
            assertion = NotMonetary(report.TRT238, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432309"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT238", Value = GetValueOrEmpty(report.TRT238) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432311
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432311()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432311
            Foreign income tax offsets are greater than zero where no beneficiary entitled and there is no Attributed foreign income or Other assessable foreign source income.
    
            Legacy Rule Format:
            IF [TRT242] > 0  AND [TRT240] = NULL AND [TRT241] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT240] = TRT:RP:rvctc1.02.00:IncomeTax.AttributedForeignIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT241] = TRT:RP:bafpr3.02.14:Income.InternationalDealings.Net.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT242] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT242 > 0  AND ^TRT240 = NULL AND ^TRT241 = NULL
    
            Data Elements:
    
            ^TRT242 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.TaxOffset.Amount
    
            ^TRT240 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AttributedForeignIncome.Amount
    
            ^TRT241 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.Net.Amount
            */
            assertion = (report.TRT242.GetValueOrDefault() > 0 && report.TRT240 == null && report.TRT241 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432311", Severity = ProcessMessageSeverity.Error,
                    Description = @"Attributed foreign income or Other assessable foreign source income must be present",
                    LongDescription = @"If 'Foreign income tax offsets' is greater than zero in the 'Income to which no beneficiary is presently entitled', then there must be 'Attributed foreign income' or 'Other assessable foreign source income' against the 'Income to which no beneficiary is presently entitled'",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.InternationalDealings.TaxOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432311"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT242", Value = GetValueOrEmpty(report.TRT242) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT240", Value = GetValueOrEmpty(report.TRT240) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT241", Value = GetValueOrEmpty(report.TRT241) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432312
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432312(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432312
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc1.xx.xx:IncomeTax.AttributedForeignIncome.Amount <> NULL 
                    AND TRT:RP:rvctc1.xx.xx:IncomeTax.AttributedForeignIncome.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT224, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT224 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AttributedForeignIncome.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT224, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AttributedForeignIncome.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432312"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT224", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT224) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432312
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432312_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432312
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc1.xx.xx:IncomeTax.AttributedForeignIncome.Amount <> NULL 
            AND TRT:RP:rvctc1.xx.xx:IncomeTax.AttributedForeignIncome.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT240, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT240 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AttributedForeignIncome.Amount
            */
            assertion = NotMonetary(report.TRT240, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.AttributedForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432312"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT240", Value = GetValueOrEmpty(report.TRT240) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432314
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432314()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432314
            There is zero net income or an overall trust loss (the sum of net Australian income/loss, net capital gains, attributed foreign income other assessable foreign source income minus allowable tax losses of earlier income years deducted this year is less than or equal to zero) and distributed Foreign income tax offsets are present in the distribution statements (distribution to a beneficiary or no beneficiary presently entitled). Foreign income tax offsets cannot be distributed when there is zero net income or an overall trust loss. Delete the Foreign income tax offsets from the distribution statement.
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] <= 0 ) AND ((ANY OCCURRENCE OF [TRT226] <> NULL) OR [TRT242] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT226] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT242] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 <= 0) AND ((AnyOccurrence(^TRT226, ^TRT226 <> NULL)) OR ^TRT242 <> NULL)
    
            Data Elements:
    
            ^TRT226 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.TaxOffset.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT242 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.TaxOffset.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() <= 0 && ((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT226Repeat => TRT226Repeat.TRT226 != null)) || report.TRT242 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432314", Severity = ProcessMessageSeverity.Error,
                    Description = @"Foreign income tax offsets must not be present if trust has zero net income or incurs a loss",
                    LongDescription = @"If there is zero net income or an overall trust loss (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses is less than or equal to zero) then a 'Foreign income tax offset' amount must not be present in a Statement of Distribution or at 'Income to which no beneficiary is presently entitled..'. Foreign income tax offsets cannot be distributed when there is zero net income or an overall trust loss. Delete the Foreign income tax offsets from the distribution statement.",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:Income.InternationalDealings.TaxOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432314"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT226", Value = "TRT226" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT242", Value = GetValueOrEmpty(report.TRT242) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432315
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432315()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432315
            The overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income other assessable foreign source income minus allowable tax losses of earlier income years deducted this year) and the sum of Foreign income tax offsets (including no beneficiary presently entitled) is not equal to the Foreign income tax offsets.
    
            Legacy Rule Format:
            IF ([TRT130] + [TRT133] + [TRT134] + [TRT136] + [TRT141] + [TRT143] - [TRT144] > 0) AND (SUM([TRT226]) + [TRT242] <> [TRT142]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT134] = TRT:RP.JFOREIGN.CFCLC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT136] = TRT:RP.JFOREIGN.CFCUC:bafpr3.02.02:Income.InternationalDealings.Attributed.Amount
            [TRT141] = TRT:RP.JFOREIGN:bafpr1.02.23:Income.Net.Amount
            [TRT142] = TRT:RP.JFOREIGN:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount
            [TRT143] = TRT:RP.JFOREIGN:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
            [TRT144] = TRT:RP:rvctc2.02.00:IncomeTax.Deduction.TaxLossesDeducted.Amount
            [TRT226] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT242] = TRT:RP:bafpr3.02.18:Income.InternationalDealings.TaxOffset.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT130 + ^TRT133 + ^TRT134 + ^TRT136 + ^TRT141 + ^TRT143 - ^TRT144 > 0) AND (Sum(^TRT226) + ^TRT242 <> ^TRT142)
    
            Data Elements:
    
            ^TRT226 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.TaxOffset.Amount
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT134 = RP.JFOREIGN.CFCLC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT136 = RP.JFOREIGN.CFCUC:Income.InternationalDealings.Attributed.Amount
    
            ^TRT141 = RP.JFOREIGN:Income.Net.Amount
    
            ^TRT142 = RP.JFOREIGN:Income.InternationalDealings.TaxOffset.Amount
    
            ^TRT143 = RP.JFOREIGN:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT144 = RP:IncomeTax.Deduction.TaxLossesDeducted.Amount
    
            ^TRT242 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.TaxOffset.Amount
            */
            assertion = (report.TRT130.GetValueOrDefault() + report.TRT133.GetValueOrDefault() + report.TRT134.GetValueOrDefault() + report.TRT136.GetValueOrDefault() + report.TRT141.GetValueOrDefault() + report.TRT143.GetValueOrDefault() - report.TRT144.GetValueOrDefault() > 0 && (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT226.GetValueOrDefault())) + report.TRT242.GetValueOrDefault() != report.TRT142.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432315", Severity = ProcessMessageSeverity.Error,
                    Description = @"Foreign income tax offsets amount must equal distributed amounts",
                    LongDescription = @"If the overall net trust income is greater than zero (the sum of net Australian income/loss, net capital gains, attributed foreign income, other assessable foreign source income; minus allowable tax losses), then the sum of Foreign income tax offsets in the Statements of Distribution, including 'Income to which no beneficiary is presently entitled', must equal the Foreign income tax offsets' of the trust",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary/tns:Income.InternationalDealings.TaxOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432315"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT134", Value = GetValueOrEmpty(report.TRT134) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT136", Value = GetValueOrEmpty(report.TRT136) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT141", Value = GetValueOrEmpty(report.TRT141) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT143", Value = GetValueOrEmpty(report.TRT143) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT144", Value = GetValueOrEmpty(report.TRT144) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT226", Value = "TRT226" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT242", Value = GetValueOrEmpty(report.TRT242) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT142", Value = GetValueOrEmpty(report.TRT142) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432316
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432316(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432316
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:bafpr3.xx.xx:Income.InternationalDealings.TaxOffset.Amount <> NULL 
                    AND TRT:RP:bafpr3.xx.xx:Income.InternationalDealings.TaxOffset.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT226, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT226 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.TaxOffset.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT226, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.InternationalDealings.TaxOffset.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432316"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT226", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT226) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432316
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432316_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432316
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:bafpr3.xx.xx:Income.InternationalDealings.TaxOffset.Amount <> NULL 
            AND TRT:RP:bafpr3.xx.xx:Income.InternationalDealings.TaxOffset.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT242, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT242 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.TaxOffset.Amount
            */
            assertion = NotMonetary(report.TRT242, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.InternationalDealings.TaxOffset.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432316"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT242", Value = GetValueOrEmpty(report.TRT242) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432318
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432318(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432318
                    In the statement of distribution, if the assessment calculation code is 134, 138 or 139, there must be an S98(3) assessable amount.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT214] = SET("134","138", "139") AND [TRT228] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT228] = TRT:RP:rvctc4.02.00:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    InSet(^TRT214, '"134","138", "139"') AND ^TRT228 = NULL
            
                    Data Elements:
            
                    ^TRT228 = RP:StatementOfDistributionToBeneficiary:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(134|138|139)$") && statementOfDistributionToBeneficiary.TRT228 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432318", Severity = ProcessMessageSeverity.Error,
                            Description = @"s98(3) assessable amount must be present",
                            LongDescription = @"In a Statement of Distribution, if the Assessment calculation code is 134, 138 or 139, then there must be an amount present at 's98(3) assessable amount'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432318"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT228", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT228) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432319
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432319(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432319
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT228] <> NULL AND [TRT228] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT228] = TRT:RP:rvctc4.02.00:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT228, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT228 = RP:StatementOfDistributionToBeneficiary:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT228, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiaryNonTrustee.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432319"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT228", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT228) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432320
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432320(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432320
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT229] <> NULL AND [TRT229] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT229] = TRT:RP:rvctc4.02.00:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT229, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT229 = RP:StatementOfDistributionToBeneficiary:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT229, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:InternationalDealings.AssessableIncomeNonResidentBeneficiaryTrustee.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432320"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT229", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT229) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432321
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432321()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432321
            PP income, Non-PP income, credit for tax withheld where no ABN quoted, franked distributions, franking credit, or TFN credit or Aust'n franking credit from a NZ coy is present in the distribution statement (no beneficiary presently entitled) and the assessment calculation code no beneficiary presently entitled is missing.
    
            Legacy Rule Format:
            WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:IncomeNoBeneficiaryPresentlyEntitled)
            IF ([TRT234] <> NULL OR [TRT235] <> NULL OR [TRT444] <> NULL OR [TRT236] > 0 OR [TRT237] > 0 OR [TRT238] > 0 OR [TRT233] > 0) AND [TRT230] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT233] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT236] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT237] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT238] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^IncomeNoBeneficiaryPresentlyEntitled <> NULL) AND ((^TRT234 <> NULL OR ^TRT235 <> NULL OR ^TRT444 <> NULL OR ^TRT236 > 0 OR ^TRT237 > 0 OR ^TRT238 > 0 OR ^TRT233 > 0) AND ^TRT230 = NULL)
    
            Data Elements:
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
    
            ^TRT233 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromNewZealandCompanies.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT236 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT237 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
    
            ^TRT238 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
    
            ^IncomeNoBeneficiaryPresentlyEntitled = RP:IncomeNoBeneficiaryPresentlyEntitled
            */
            assertion = (report.IncomeNoBeneficiaryPresentlyEntitledCollectionExists != false && ((report.TRT234 != null || report.TRT235 != null || report.TRT444 != null || report.TRT236.GetValueOrDefault() > 0 || report.TRT237.GetValueOrDefault() > 0 || report.TRT238.GetValueOrDefault() > 0 || report.TRT233.GetValueOrDefault() > 0) && report.TRT230 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432621", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code must be present",
                    LongDescription = @"If the 'Income to which no beneficiary is presently entitled' Statement of Distribution contains an amount at Primary production income, Non-primary production income, Credit for tax withheld where ABN not quoted, Franked distributions, Franking credit, TFN amounts withheld, or Australian franking credit from a NZ company, then an Assessment calculation code must be present in the statement.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432321"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT236", Value = GetValueOrEmpty(report.TRT236) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT237", Value = GetValueOrEmpty(report.TRT237) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT238", Value = GetValueOrEmpty(report.TRT238) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT233", Value = GetValueOrEmpty(report.TRT233) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432324
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432324()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432324
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT234] <> NULL AND [TRT234] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT234, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
            */
            assertion = NotMonetary(report.TRT234, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.IncomeOrLossTaxable.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432324"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432325
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432325()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432325
            Franking credit or TFN amount withheld is present in the distribution statement (no beneficiary presently entitled) and there is no non-primary production income distribution or franked distributions amount (no beneficiary presently entitled). Zero is acceptable when there is an overall trust loss.
    
            Legacy Rule Format:
            WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:IncomeNoBeneficiaryPresentlyEntitled)
            IF ([TRT237] > 0 OR [TRT238] > 0) AND [TRT235] = NULL AND [TRT444] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT237] = TRT:RP:rvctc2.02.00:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT238] = TRT:RP:rvctc2.02.04:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^IncomeNoBeneficiaryPresentlyEntitled <> NULL) AND ((^TRT237 > 0 OR ^TRT238 > 0) AND ^TRT235 = NULL AND ^TRT444 = NULL)
    
            Data Elements:
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT237 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.FrankingCredits.ReceivedFromAustralianCompanies.Amount
    
            ^TRT238 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldWhereTFNNotQuoted.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
    
            ^IncomeNoBeneficiaryPresentlyEntitled = RP:IncomeNoBeneficiaryPresentlyEntitled
            */
            assertion = (report.IncomeNoBeneficiaryPresentlyEntitledCollectionExists != false && ((report.TRT237.GetValueOrDefault() > 0 || report.TRT238.GetValueOrDefault() > 0) && report.TRT235 == null && report.TRT444 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432623", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-PP income or franked distributions amount must be present in 'no beneficiary entitled' distribution",
                    LongDescription = @"If within the 'Income to which no beneficiary is presently entitled' Statement of Distribution, a 'Franking credit' or 'TFN amount withheld' amount is present, then there must be an amount at 'Non-primary production income' or 'franked distributions'. Zero is acceptable if there is an overall trust loss.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.IncomeOrLossTaxable.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432325"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT237", Value = GetValueOrEmpty(report.TRT237) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT238", Value = GetValueOrEmpty(report.TRT238) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432326
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432326()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432326
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT235] <> NULL AND [TRT235] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT235, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
            */
            assertion = NotMonetary(report.TRT235, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.IncomeOrLossTaxable.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432326"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432335
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432335()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432335
            There is a 'yes' response to one or more of the questions on the trust return at Overseas Transactions, Beneficiary under legal disability, or Non-resident trust and the Other Attachments Schedule is missing.
    
            Legacy Rule Format:
            IF ([TRT244] = TRUE OR [TRT149] = TRUE OR [TRT245] = "NonResidentNoPermEstab") AND [TRT405] = NULLORBLANK 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT149] = TRT:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator
            [TRT244] = TRT:RP:rvctc1.02.00:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [TRT405] = TRT:RP:bafot.02.00:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text

            Technical Business Rule Format:
            (^TRT244 = TRUE OR ^TRT149 = TRUE OR ^TRT245 = 'NonResidentNoPermEstab') AND ^TRT405 = BLANK
    
            Data Elements:
    
            ^TRT244 = RP:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator
    
            ^TRT149 = RP:Residency.TaxPurposesPersonStatus.Indicator
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
    
            ^TRT405 = RP:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = ((report.TRT244 == true || report.TRT149 == true || report.TRT245 == @"NonResidentNoPermEstab") && string.IsNullOrWhiteSpace(report.TRT405) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432335", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other attachments schedule must be present",
                    LongDescription = @"If a non-resident beneficiary or a beneficiary under a legal disability is entitled to a share of trust income, or if the trust is a non-resident trust, then further information must be provided against the 'Other Attachments' element",
                    Location = "/xbrli:xbrl/tns:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432335"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT244", Value = GetValueOrEmpty(report.TRT244) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT149", Value = GetValueOrEmpty(report.TRT149) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT405", Value = GetValueOrEmpty(report.TRT405) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432336
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432336()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432336
            Code value must be "Resident" or "NonResidentNoPermEstab".
    
            Legacy Rule Format:
            IF [TRT245] <> NULL AND [TRT245] <> SET("Resident", "NonResidentNoPermEstab")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code

            Technical Business Rule Format:
            ^TRT245 <> NULL AND NotInSet(^TRT245, '"Resident", "NonResidentNoPermEstab"')
    
            Data Elements:
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
            */
            assertion = (report.TRT245 != null && !(IsMatch(report.TRT245, @"^(Resident|NonResidentNoPermEstab)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432704", Severity = ProcessMessageSeverity.Error,
                    Description = @"Code value must be ""Resident"" or ""NonResidentNoPermEstab""",
                    LongDescription = @"To indicate the trust is a non-resident trust, use code 'NonResidentNoPermEstab' - Non Resident. To indicate the trust is not a non-resident trust use code 'Resident'.",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432336"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432337
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432337()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432337
            There is a 'yes' response to the question concerning whether the trust estate is a non-resident trust estate and the amount of income derived outside Australia to which no beneficiary is presently entitled is missing.
    
            Legacy Rule Format:
            IF [TRT245] = "NonResidentNoPermEstab" AND [TRT246] = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [TRT246] = TRT:RP:rvctc4.02.00:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount

            Technical Business Rule Format:
            ^TRT245 = 'NonResidentNoPermEstab' AND ^TRT246 = NULL
    
            Data Elements:
    
            ^TRT246 = RP:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
            */
            assertion = (report.TRT245 == @"NonResidentNoPermEstab" && report.TRT246 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432705", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-resident trust - amount of income must be present",
                    LongDescription = @"If the answer to the question 'Is the trust a non-resident trust' is 'NonResidentNoPermEstab', indicating non-resident, then an amount must be present at '...the amount of income derived outside Australia to which no beneficiary is presently entitled...'",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432337"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT246", Value = GetValueOrEmpty(report.TRT246) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432338
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432338()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432338
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT246] <> NULL AND [TRT246] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT246] = TRT:RP:rvctc4.02.00:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT246, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT246 = RP:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount
            */
            assertion = NotMonetary(report.TRT246, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432338"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT246", Value = GetValueOrEmpty(report.TRT246) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432339
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432339()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432339
            There is a 'no' response to the question concerning whether the trust estate is a non resident trust estate and there is an amount of income derived outside Australia to which no beneficiary is presently entitled.
    
            Legacy Rule Format:
            IF [TRT245] = "Resident" AND [TRT246] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [TRT246] = TRT:RP:rvctc4.02.00:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount

            Technical Business Rule Format:
            ^TRT245 = 'Resident' AND ^TRT246 <> NULL
    
            Data Elements:
    
            ^TRT246 = RP:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
            */
            assertion = (report.TRT245 == @"Resident" && report.TRT246 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432703", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-resident trust income amount must not be present",
                    LongDescription = @"If the answer to the question 'Is the trust a non-resident trust' is 'Resident', indicating 'resident', then an amount must NOT be present at '...the amount of income derived outside Australia to which no beneficiary is presently entitled...'",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.NonResidentTrustNonAustralianSourcedIncomeNoPresentEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432339"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT246", Value = GetValueOrEmpty(report.TRT246) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432342
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432342()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432342
            The total amount at National rental affordability scheme tax offset (beneficiary and non beneficiary) must equal the amount at National rental affordability scheme tax offset entitlement.
    
            Legacy Rule Format:
            IF SUM([TRT342]) + [TRT343] <> [TRT340] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT340] = TRT:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            [TRT342] = TRT:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT343] = TRT:RP:gfati.02.01:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            Sum(^TRT342) + ^TRT343 <> ^TRT340
    
            Data Elements:
    
            ^TRT340 = RP:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
    
            ^TRT342 = RP:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
    
            ^TRT343 = RP:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT342.GetValueOrDefault())) + report.TRT343.GetValueOrDefault() != report.TRT340.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432342", Severity = ProcessMessageSeverity.Error,
                    Description = @"National rental affordability scheme tax offset amount must equal distributed amounts",
                    LongDescription = @"The sum of 'Share of National rental affordability scheme tax offset' amounts in the Statements of Distribution must equal the amount at 'National rental affordability scheme tax offset entitlement' for the trust",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432342"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT342", Value = "TRT342" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT343", Value = GetValueOrEmpty(report.TRT343) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT340", Value = GetValueOrEmpty(report.TRT340) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432343
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432343()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432343
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> NULL 
            AND TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT340, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT340 = RP:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            */
            assertion = NotMonetary(report.TRT340, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432343"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT340", Value = GetValueOrEmpty(report.TRT340) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432343
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432343_1(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432343
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> NULL 
                    AND TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT342, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT342 = RP:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT342, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432343"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT342", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT342) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432343
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432343_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432343
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> NULL 
            AND TRT:RP:gfati.xx.xx:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT343, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT343 = RP:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount
            */
            assertion = NotMonetary(report.TRT343, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:TaxOffsetClaim.NationalRentalAffordabilitySchemeEntitlement.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432343"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT343", Value = GetValueOrEmpty(report.TRT343) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432347
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432347(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432347
                    Where the trustee has indicated that they are making a TB statement, by answering the TB question Yes, there must be a value greater than 0 at 'Tax preferred amounts' or Untaxed part of share of net income' .
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT344] = TRUE AND ([TRT345] = 0 OR [TRT345] = NULL) AND ([TRT346] = 0 OR [TRT346] = NULL)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT345] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT346] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT344 = TRUE AND (^TRT345 = 0 OR ^TRT345 = NULL) AND (^TRT346 = 0 OR ^TRT346 = NULL)
            
                    Data Elements:
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
            
                    ^TRT345 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount
            
                    ^TRT346 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT344 == true && (statementOfDistributionToBeneficiary.TRT345 == 0 || statementOfDistributionToBeneficiary.TRT345 == null) && (statementOfDistributionToBeneficiary.TRT346 == 0 || statementOfDistributionToBeneficiary.TRT346 == null));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432347", Severity = ProcessMessageSeverity.Error,
                            Description = @"TB Statement information amount must be present",
                            LongDescription = @"If within a Statement of Distribution the answer to the question 'TB statement' is 'yes' (true), then there must be an amount greater than zero at 'Tax preferred amounts' or Untaxed part of share of net income'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.TrustBeneficiaryStatement.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432347"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT345", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT345) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT346", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT346) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432348
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432348(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432348
                    Where the trust has indicated that they are not making a TB statement, by answering the TB question No, 'Tax preferred amounts' or Untaxed part of share of net income' must be blank or equal to zero.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                    IF [TRT344] <> TRUE AND ([TRT345] > 0 OR [TRT346] > 0) 
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT344] = TRT:RP:rvctc1.02.03:IncomeTax.TrustBeneficiaryStatement.Indicator IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT345] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT346] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT344 <> TRUE AND (^TRT345 > 0 OR ^TRT346 > 0)
            
                    Data Elements:
            
                    ^TRT344 = RP:StatementOfDistributionToBeneficiary:IncomeTax.TrustBeneficiaryStatement.Indicator
            
                    ^TRT345 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount
            
                    ^TRT346 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT344 != true && (statementOfDistributionToBeneficiary.TRT345.GetValueOrDefault() > 0 || statementOfDistributionToBeneficiary.TRT346.GetValueOrDefault() > 0));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432348", Severity = ProcessMessageSeverity.Error,
                            Description = @"TB Statement information amount must NOT be present",
                            LongDescription = @"If within a Statement of Distribution the answer to the question 'TB statement' is not 'yes' (false or blank), then there must NOT be an amount greater than zero at 'Tax preferred amounts' or Untaxed part of share of net income'",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.TrustBeneficiaryStatement.Indicator",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432348"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT344", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT344) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT345", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT345) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT346", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT346) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432349
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432349(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432349
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT345] <> NULL AND [TRT345] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT345] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT345, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT345 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT345, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.Superannuation.TrusteeBeneficiaryTaxPreferred.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432349"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT345", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT345) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432350
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432350(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432350
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT346] <> NULL AND [TRT346] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT346] = TRT:RP:rvctc2.02.02:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT346, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT346 = RP:StatementOfDistributionToBeneficiary:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT346, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.Superannuation.TrusteeBeneficiaryUntaxedPartShareOfNetIncome.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432350"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT346", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT346) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432352
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432352()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432352
            Primary production- Share of net income from trusts is greater than zero and there is no primary production-Distribution from trusts action code.
    
            Legacy Rule Format:
            IF [TRT77] > 0 AND [TRT353] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT353] = TRT:RP.JAUS.Prim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT77 > 0 AND ^TRT353 = NULL
    
            Data Elements:
    
            ^TRT353 = RP.JAUS.Prim:Party.TrustType.Code
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
            */
            assertion = (report.TRT77.GetValueOrDefault() > 0 && report.TRT353 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432660", Severity = ProcessMessageSeverity.Error,
                    Description = @"Share of net income from trusts action code must be present",
                    LongDescription = @"If Primary production- Share of net income from trusts is greater than zero, the Primary production - Share of net income from trusts action code must be present",
                    Location = "/xbrli:xbrl/tns:Party.TrustType.Code[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432352"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT353", Value = GetValueOrEmpty(report.TRT353) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432353
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432353()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432353
            When there is an amount present for non-primary production distribution from trusts, there must be a trust action code.
    
            Legacy Rule Format:
            IF [TRT81] > 0 AND [TRT436] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT436] = TRT:RP.JAUS.NonPrim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT81 > 0 AND ^TRT436 = NULL
    
            Data Elements:
    
            ^TRT436 = RP.JAUS.NonPrim:Party.TrustType.Code
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = (report.TRT81.GetValueOrDefault() > 0 && report.TRT436 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432663", Severity = ProcessMessageSeverity.Error,
                    Description = @"Distribution from trusts action code must be present",
                    LongDescription = @"If Non-primary production - Share of net income from trusts, less net capital gain, foreign income and franked distributions is greater than zero, the Non-primary production - Distribution from trusts action code must be present",
                    Location = "/xbrli:xbrl/tns:Party.TrustType.Code[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432353"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT436", Value = GetValueOrEmpty(report.TRT436) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432360
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432360()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432360
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT356] <> NULL AND [TRT356] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT356] = TRT:RP.TOFA:bafpr1.02.04:Income.GainsTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT356, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT356 = RP.TOFA:Income.GainsTotal.Amount
            */
            assertion = NotMonetary(report.TRT356, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.GainsTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432360"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT356", Value = GetValueOrEmpty(report.TRT356) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432361
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432361()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432361
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT357] <> NULL AND [TRT357] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT357] = TRT:RP.TOFA:bafpr1.02.04:Expense.LossesTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT357, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT357 = RP.TOFA:Expense.LossesTotal.Amount
            */
            assertion = NotMonetary(report.TRT357, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.LossesTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432361"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT357", Value = GetValueOrEmpty(report.TRT357) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432365
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432365()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432365
            Current name of trust must be present
    
            Legacy Rule Format:
            IF COUNT TUPLE (xbrli/Organisationname2.xx.xx:OrganisationNameDetails WHERE TUPLE ELEMENT (pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = "MN") AND (pyde.xx.xx:OrganisationNameDetails.Currency.Code = "C") AND (pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text  <> NULLORBLANK)) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Not(Count(^OrganisationNameDetails) = 1 AND ^TRT6 <> BLANK)
    
            Data Elements:
    
            ^TRT6 = RP:OrganisationNameDetails:OrganisationNameDetails.OrganisationalName.Text WHERE [OrganisationNameDetails.OrganisationalNameType.Code='MN'].[OrganisationNameDetails.Currency.Code='C']
    
            ^OrganisationNameDetails = RP:OrganisationNameDetails
            */
            assertion =  !((Count(report.OrganisationNameDetailsCollection_MN_C_RPCount) == 1 && string.IsNullOrWhiteSpace(report.TRT6) != true) );
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432365", Severity = ProcessMessageSeverity.Error,
                    Description = @"Current name of trust must be present",
                    LongDescription = @"There must be one instance of the current main Name of the trust. This name must be submitted with a Currency Code 'C' and a Type of 'MN'",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails" + OccurrenceIndex(report.OrganisationNameDetailsCollection_MN_C_RPLastOccurrenceIndex) + "/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432365"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT6", Value = report.TRT6 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432366
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432366()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432366
            Current postal address of the trust must be present
    
            Legacy Rule Format:
            IF COUNT TUPLE (xbrli/address2.xx.xx:AddressDetails WHERE TUPLE ELEMENT (pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (pyde.xx.xx:AddressDetails.Currency.Code = "C") AND (pyde.xx.xx:AddressDetails.Line1.Text <> NULLORBLANK)) <> 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT9 = BLANK
    
            Data Elements:
    
            ^TRT9 = RP:AddressDetails:AddressDetails.Line1.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT9) == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432366", Severity = ProcessMessageSeverity.Error,
                    Description = @"Current postal address of the trust must be present",
                    LongDescription = @"There must be one instance of the current postal address of the Trust. This address must be submitted with a Currency Code 'C' and a Usage of 'POS'. Line 1, Suburb/Town/City must be present",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432366"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT9", Value = report.TRT9 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432367
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432367()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432367
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF (TRT:RP:pyin.xx.xx:Report.TargetFinancial.Year = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT1 = NULL
    
            Data Elements:
    
            ^TRT1 = RP:Report.TargetFinancial.Year
            */
            assertion = (report.TRT1 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432367"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432368
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432368()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432368
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT32] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code

            Technical Business Rule Format:
            ^TRT32 = NULL
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
            assertion = (report.TRT32 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432368"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432369
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432369()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432369
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT35] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT35] = TRT:RP:pyin.02.00:Report.Statement.Summary.PaymentRefundOrNil.Code

            Technical Business Rule Format:
            ^TRT35 = NULL
    
            Data Elements:
    
            ^TRT35 = RP:Report.Statement.Summary.PaymentRefundOrNil.Code
            */
            assertion = (report.TRT35 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Report.Statement.Summary.PaymentRefundOrNil.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432369"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT35", Value = report.TRT35 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432370
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432370()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432370
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT36] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT36] = TRT:RP:pyin.02.28:Lodgment.FinalReturn.Indicator

            Technical Business Rule Format:
            ^TRT36 = NULL
    
            Data Elements:
    
            ^TRT36 = RP:Lodgment.FinalReturn.Indicator
            */
            assertion = (report.TRT36 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Lodgment.FinalReturn.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432370"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT36", Value = GetValueOrEmpty(report.TRT36) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432373
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432373()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432373
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT130] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT130] = TRT:RP.JAUS:bafpr1.02.23:Income.Net.Amount

            Technical Business Rule Format:
            ^TRT130 = NULL
    
            Data Elements:
    
            ^TRT130 = RP.JAUS:Income.Net.Amount
            */
            assertion = (report.TRT130 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432373"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT130", Value = GetValueOrEmpty(report.TRT130) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432374
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432374()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432374
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT131] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT131] = TRT:RP:rvctc3.02.00:CapitalGainsTax.Event.Indicator

            Technical Business Rule Format:
            ^TRT131 = NULL
    
            Data Elements:
    
            ^TRT131 = RP:CapitalGainsTax.Event.Indicator
            */
            assertion = (report.TRT131 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.Event.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432374"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT131", Value = GetValueOrEmpty(report.TRT131) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432375
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432375()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432375
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT138] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT138] = TRT:RP.JFOREIGN:bafpr3.02.03:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator

            Technical Business Rule Format:
            ^TRT138 = NULL
    
            Data Elements:
    
            ^TRT138 = RP.JFOREIGN:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator
            */
            assertion = (report.TRT138 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Income.InternationalDealings.AttributedForeignIncomeForeignTrustorControlledForeignCompanyorTransferorTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432375"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT138", Value = GetValueOrEmpty(report.TRT138) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432377
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432377()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432377
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT148] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT148] = TRT:RP:rvctc4.02.10:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator

            Technical Business Rule Format:
            ^TRT148 = NULL
    
            Data Elements:
    
            ^TRT148 = RP:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator
            */
            assertion = (report.TRT148 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.RelatedPartiesTransactionsExcessAggregateValue.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432377"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT148", Value = GetValueOrEmpty(report.TRT148) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432378
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432378()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432378
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT149] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT149] = TRT:RP:pyde.02.25:Residency.TaxPurposesPersonStatus.Indicator

            Technical Business Rule Format:
            ^TRT149 = NULL
    
            Data Elements:
    
            ^TRT149 = RP:Residency.TaxPurposesPersonStatus.Indicator
            */
            assertion = (report.TRT149 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesPersonStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432378"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT149", Value = GetValueOrEmpty(report.TRT149) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432379
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432379()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432379
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT151] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT151] = TRT:RP:rvctc4.02.11:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator

            Technical Business Rule Format:
            ^TRT151 = NULL
    
            Data Elements:
    
            ^TRT151 = RP:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator
            */
            assertion = (report.TRT151 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.TransactionsWithTaxHavenCountries.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432379"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT151", Value = GetValueOrEmpty(report.TRT151) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432383
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432383()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432383
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT244] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT244] = TRT:RP:rvctc1.02.00:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator

            Technical Business Rule Format:
            ^TRT244 = NULL
    
            Data Elements:
    
            ^TRT244 = RP:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator
            */
            assertion = (report.TRT244 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:IncomeTax.BeneficiaryUnderLegalDisabilityAlsoPresentlyEntitledToIncomeFromAnotherTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432383"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT244", Value = GetValueOrEmpty(report.TRT244) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432387
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432387()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432387
            The must be no more than 20 instances of Other Australian Income - Tax withheld details
    
            Legacy Rule Format:
            IF COUNT (TUPLE(trt.0011.lodge.req.xx.xx:IndividualNonBusinessWithholdingPaymentDetails)) > 20
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^IndividualNonBusinessWithholdingPaymentDetails) > 20
    
            Data Elements:
    
            ^IndividualNonBusinessWithholdingPaymentDetails = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails
            */
            assertion = (Count(report.IndividualNonBusinessWithholdingPaymentDetailsCollectionCount) > 20);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432387", Severity = ProcessMessageSeverity.Error,
                    Description = @"The must be no more than 20 instances of Other Australian Income - Tax withheld details",
                    Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(report.IndividualNonBusinessWithholdingPaymentDetailsCollection == null ? 0 : (report.IndividualNonBusinessWithholdingPaymentDetailsCollection.Count() == 0 ? 0 : report.IndividualNonBusinessWithholdingPaymentDetailsCollection.Last().OccurrenceIndex)) + "/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432387"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432389
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432389()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432389
            Other deductions must not exceed 99 instances
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:OtherDeductions) > 99
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^OtherDeductions) > 99
    
            Data Elements:
    
            ^OtherDeductions = RP.JAUS:OtherDeductions
            */
            assertion = (Count(report.OtherDeductionsCollectionCount) > 99);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430109", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other deductions must not exceed 99 instances",
                    Location = "/xbrli:xbrl/tns:OtherDeductions" + OccurrenceIndex(report.OtherDeductionsCollection == null ? 0 : (report.OtherDeductionsCollection.Count() == 0 ? 0 : report.OtherDeductionsCollection.Last().OccurrenceIndex)) + "/tns:Expense.DeductibleOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432389"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432413
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432413()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432413
            Family trust revocation code must be 'R' for revocation or 'V' for variation
    
            Legacy Rule Format:
            IF [TRT29] <> NULLORBLANK AND [TRT29] <> SET("R","V")
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT29] = TRT:RP:rvctc3.02.03:Elections.FamilyTrustElectionRevocation.Code

            Technical Business Rule Format:
            ^TRT29 <> BLANK AND NotInSet(^TRT29, '"R","V"')
    
            Data Elements:
    
            ^TRT29 = RP:Elections.FamilyTrustElectionRevocation.Code
            */
            assertion = (string.IsNullOrWhiteSpace(report.TRT29) != true && !(IsMatch(report.TRT29, @"^(R|V)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.432413", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family trust revocation code must be 'R' for revocation or 'V' for variation",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionRevocation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432413"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT29", Value = GetValueOrEmpty(report.TRT29) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432418
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432418(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432418
                    Assessment calculation code must be:11-14,25-30,34,35,45,111,118 -120,125-129,134,138,139,140,145
    
                    Legacy Rule Format:
                    IF [TRT214] <> NULL AND [TRT214] <> SET("11", "12", "13", "14", "25", "26", "27", "28", "29", "30", "34", "35", "45", "111", "118", "119", "120", "125", "126", "127", "128", "129", "134", "138", "139", "140", "145")
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT214 <> NULL AND NotInSet(^TRT214, '"11", "12", "13", "14", "25", "26", "27", "28", "29", "30", "34", "35", "45", "111", "118", "119", "120", "125", "126", "127", "128", "129", "134", "138", "139", "140", "145"')
            
                    Data Elements:
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT214 != null && !(IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(11|12|13|14|25|26|27|28|29|30|34|35|45|111|118|119|120|125|126|127|128|129|134|138|139|140|145)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432796", Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment calculation code must be:11-14,25-30,34,35,45,111,118 -120,125-129,134,138,139,140,145",
                            LongDescription = @"Beneficiary Assessment calculation code must be: RESIDENT - INTER VIVOS TRUSTS: 25 - Over 18 years of age under a legal disability or an excepted person; 26 - A prescribed person receiving excepted income only; 27 - A prescribed person receiving eligible income only; 28 - A prescribed person receiving excepted and eligible income only; 29 - A prescribed person receiving eligible income from more than one trust; 30 - Not under any legal disability; 34 - A company; 35 - A trustee. NON-RESIDENT - INTER VIVOS TRUSTS: 125 - Over 18 years of age under a legal disability or an excepted person; 126 - A prescribed person receiving excepted income only; 127 - A prescribed person receiving eligible income only; 128 - A prescribed person receiving excepted and eligible income only; 129 - A prescribed person receiving eligible income from more than one trust; 134 - A base rate entity; 138 - Not under any legal disability; 139 - A company; 140  - A trustee. RESIDENT - DECEASED ESTATE: 11 - Under a legal disability; 12 - Not under a legal disability; 13 - A company; 14 - A trustee. NON-RESIDENT DECEASED ESTATE: 111 - Under a legal disability; 118 - Not under a legal disability; 119 - A company; 120 - A trustee. SPECIAL DISABILITY TRUSTS: 45 - Resident beneficiary; 145 - Non-resident beneficiary.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.AssessmentCalculation.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432418"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432419
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432419()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432419
            The number of ETP payments for a dependent beneficiary must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:EmploymentTerminationPaymentDetails WHERE TUPLE ELEMENT crigi.xx.xx:InsuranceBenefits.BeneficiaryType.Code = "D") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^EmploymentTerminationPaymentDetails) > 1
    
            Data Elements:
    
            ^EmploymentTerminationPaymentDetails = RP.JAUS:EmploymentTerminationPaymentDetails
            */
            assertion = (Count(report.EmploymentTerminationPaymentDetailsCollection_D_RPJAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432419", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of ETP payments for a dependent beneficiary must not exceed 1.",
                    LongDescription = @"You may provide a maximum of two instances of Eligible termination payments. One for where the beneficiary is a dependant and another where the beneficiary is a non-dependant.",
                    Location = "/xbrli:xbrl/tns:EmploymentTerminationPaymentDetails" + OccurrenceIndex(report.EmploymentTerminationPaymentDetailsCollection_D_RPJAUSLastOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432419"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432421
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432421()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432421
            If an amount greater than zero is present at TFN amounts withheld from gross interest, then an amount greater than zero must also be present at Gross interest.
    
            Legacy Rule Format:
            IF [TRT95] > 0 AND ([TRT94] = 0 OR [TRT94] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT94] = TRT:RP.JAUS:bafpr2.02.02:Income.Interest.Gross.Amount
            [TRT95] = TRT:RP.JAUS:rvctc1.02.16:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount

            Technical Business Rule Format:
            ^TRT95 > 0 AND (^TRT94 = 0 OR ^TRT94 = NULL)
    
            Data Elements:
    
            ^TRT94 = RP.JAUS:Income.Interest.Gross.Amount
    
            ^TRT95 = RP.JAUS:IncomeTax.TFNAmountsWithheldFromGrossInterest.Amount
            */
            assertion = (report.TRT95.GetValueOrDefault() > 0 && (report.TRT94 == 0 || report.TRT94 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430092", Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross interest amount greater than zero must be present",
                    LongDescription = @"If an amount greater than zero is present at TFN amounts withheld from gross interest, then an amount greater than zero must also be present at Gross interest",
                    Location = "/xbrli:xbrl/tns:Income.Interest.Gross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432421"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT95", Value = GetValueOrEmpty(report.TRT95) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT94", Value = GetValueOrEmpty(report.TRT94) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432425
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432425(TRT2023.IndividualNonBusinessWithholdingPaymentDetails individualNonBusinessWithholdingPaymentDetails, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432425
                    At least one amount must be present for Other Australian Income - withholding details
    
                    Legacy Rule Format:
                    WHERE IN TUPLE(trt.0011.lodge.req.xx.xx:IndividualNonBusinessWithholdingPaymentDetails)
                    IF [TRT108] = NULL AND [TRT109] = NULL AND [TRT110] = NULL AND [TRT111] = NULL AND [TRT112] = NULL AND [TRT113] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT108] = TRT:RP.JAUS:rvctc2.02.12:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
                    [TRT109] = TRT:RP.JAUS:lrla.02.22:Remuneration.IndividualNonBusinessGross.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
                    [TRT110] = TRT:RP.JAUS:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
                    [TRT111] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
                    [TRT112] = TRT:RP.JAUS:rvctc2.02.14:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
                    [TRT113] = TRT:RP.JAUS:lrla.02.22:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount IN TUPLE(trt.0011.lodge.req.02.00:IndividualNonBusinessWithholdingPaymentDetails)
        
                    Technical Business Rule Format:
                    ^TRT108 = NULL AND ^TRT109 = NULL AND ^TRT110 = NULL AND ^TRT111 = NULL AND ^TRT112 = NULL AND ^TRT113 = NULL
            
                    Data Elements:
            
                    ^TRT108 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount
            
                    ^TRT109 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.IndividualNonBusinessGross.Amount
            
                    ^TRT110 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumA.Amount
            
                    ^TRT111 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumA.Amount
            
                    ^TRT112 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldUnusedAnnualOrLongServiceLeavePaymentLumpSumB.Amount
            
                    ^TRT113 = RP.JAUS:IndividualNonBusinessWithholdingPaymentDetails:Remuneration.UnusedAnnualLongServiceLeavePaymentLumpSumB.Amount
                    */
                    assertion = (individualNonBusinessWithholdingPaymentDetails.TRT108 == null && individualNonBusinessWithholdingPaymentDetails.TRT109 == null && individualNonBusinessWithholdingPaymentDetails.TRT110 == null && individualNonBusinessWithholdingPaymentDetails.TRT111 == null && individualNonBusinessWithholdingPaymentDetails.TRT112 == null && individualNonBusinessWithholdingPaymentDetails.TRT113 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432425", Severity = ProcessMessageSeverity.Error,
                            Description = @"At least one amount must be present for Other Australian Income - withholding details",
                            LongDescription = @"For each ABN that is provided against Other Australian income - Withholding details, at least one payment amount must be present",
                            Location = "/xbrli:xbrl/tns:IndividualNonBusinessWithholdingPaymentDetails" + OccurrenceIndex(individualNonBusinessWithholdingPaymentDetails.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432425"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT108", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT108) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT109", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT109) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT110", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT110) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT111", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT111) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT112", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT112) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT113", Value = GetValueOrEmpty(individualNonBusinessWithholdingPaymentDetails.TRT113) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432430
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432430()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432430
            The number of Other Australian income items must not exceed 50
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:OtherAustralianIncome) > 50
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^OtherAustralianIncome) > 50
    
            Data Elements:
    
            ^OtherAustralianIncome = RP.JAUS:OtherAustralianIncome
            */
            assertion = (Count(report.OtherAustralianIncomeCollectionCount) > 50);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432430", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Other Australian income items must not exceed 50",
                    Location = "/xbrli:xbrl/tns:OtherAustralianIncome" + OccurrenceIndex(report.OtherAustralianIncomeCollection == null ? 0 : (report.OtherAustralianIncomeCollection.Count() == 0 ? 0 : report.OtherAustralianIncomeCollection.Last().OccurrenceIndex)) + "/tns:Income.Other.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432430"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432431
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432431()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432431
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT140] <> NULL AND [TRT140] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT140] = TRT:RP.JFOREIGN:bafpr1.02.04:Income.Other.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT140, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT140 = RP.JFOREIGN:Income.Other.Amount
            */
            assertion = NotMonetary(report.TRT140, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432431"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT140", Value = GetValueOrEmpty(report.TRT140) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432560
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432560()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432560
            Invalid context. The number of Reporting Party Closing contexts must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Closing)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Closing
            */
            assertion = (Count(report.RPClosingCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432560", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting Party Closing contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432560"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432561
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432561()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432561
            Maximum Length for Suburb/Town is 60 characters
    
            Legacy Rule Format:
            IF TRT:RP:pyde.xx.xx:AddressDetails.LocalityName.Text <> NULLORBLANK AND (LENGTH(TRT:RP:pyde.xx.xx:AddressDetails.LocalityName.Text) > 60)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT10) > 60
    
            Data Elements:
    
            ^TRT10 = RP:AddressDetails:AddressDetails.LocalityName.Text WHERE [AddressDetails.Usage.Code='POS'].[AddressDetails.Currency.Code='C']
            */
            assertion = (Length(report.TRT10) > 60);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432561", Severity = ProcessMessageSeverity.Error,
                    Description = @"Maximum Length for Suburb/Town is 60 characters",
                    LongDescription = @"Field has incorrect format. The maximum number of characters for Suburb/Town is 60.",
                    Location = "/xbrli:xbrl/tns:AddressDetails" + OccurrenceIndex(report.AddressDetailsCollection_POS_C_RPOccurrenceIndex) + "/tns:AddressDetails.LocalityName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432561"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT10", Value = report.TRT10 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = INT
    
            ^EndDate = INT
            */
            assertion = ((report.INTExists == true) && (report.INTStartDate >= report.INTEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.INTOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP
    
            ^EndDate = RP
            */
            assertion = ((report.RPExists == true) && (report.RPStartDate >= report.RPEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_2()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Closing
    
            ^EndDate = RP:RP.Closing
            */
            assertion = ((report.RPClosingExists == true) && (report.RPClosingStartDate >= report.RPClosingEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPClosingOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_3()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.GeneralPool
    
            ^EndDate = RP:RP.GeneralPool
            */
            assertion = ((report.RPGeneralPoolExists == true) && (report.RPGeneralPoolStartDate >= report.RPGeneralPoolEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_4()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Investor
    
            ^EndDate = RP:RP.Investor
            */
            assertion = ((report.RPInvestorExists == true) && (report.RPInvestorStartDate >= report.RPInvestorEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_5()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.Investor:RP.Investor.YPrevious
    
            ^EndDate = RP.Investor:RP.Investor.YPrevious
            */
            assertion = ((report.RPInvestorYPreviousExists == true) && (report.RPInvestorYPreviousStartDate >= report.RPInvestorYPreviousEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPInvestorYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_6()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.JAUS
    
            ^EndDate = RP:RP.JAUS
            */
            assertion = ((report.RPJAUSExists == true) && (report.RPJAUSStartDate >= report.RPJAUSEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_7()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.NonPrim
    
            ^EndDate = RP.JAUS:RP.JAUS.NonPrim
            */
            assertion = ((report.RPJAUSNonPrimExists == true) && (report.RPJAUSNonPrimStartDate >= report.RPJAUSNonPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_8()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JAUS:RP.JAUS.Prim
    
            ^EndDate = RP.JAUS:RP.JAUS.Prim
            */
            assertion = ((report.RPJAUSPrimExists == true) && (report.RPJAUSPrimStartDate >= report.RPJAUSPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJAUSPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_9()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.JFOREIGN
    
            ^EndDate = RP:RP.JFOREIGN
            */
            assertion = ((report.RPJFOREIGNExists == true) && (report.RPJFOREIGNStartDate >= report.RPJFOREIGNEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_10()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
    
            ^EndDate = RP.JFOREIGN:RP.JFOREIGN.CFCLC
            */
            assertion = ((report.RPJFOREIGNCFCLCExists == true) && (report.RPJFOREIGNCFCLCStartDate >= report.RPJFOREIGNCFCLCEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCLCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_11()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
    
            ^EndDate = RP.JFOREIGN:RP.JFOREIGN.CFCUC
            */
            assertion = ((report.RPJFOREIGNCFCUCExists == true) && (report.RPJFOREIGNCFCUCStartDate >= report.RPJFOREIGNCFCUCEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPJFOREIGNCFCUCOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_12()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.LimitedPartners
    
            ^EndDate = RP:RP.LimitedPartners
            */
            assertion = ((report.RPLimitedPartnersExists == true) && (report.RPLimitedPartnersStartDate >= report.RPLimitedPartnersEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_13()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP.LimitedPartners:RP.LimitedPartners.YPrevious
    
            ^EndDate = RP.LimitedPartners:RP.LimitedPartners.YPrevious
            */
            assertion = ((report.RPLimitedPartnersYPreviousExists == true) && (report.RPLimitedPartnersYPreviousStartDate >= report.RPLimitedPartnersYPreviousEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLimitedPartnersYPreviousOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_14()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.LowCost
    
            ^EndDate = RP:RP.LowCost
            */
            assertion = ((report.RPLowCostExists == true) && (report.RPLowCostStartDate >= report.RPLowCostEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_15()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.NonPrim
    
            ^EndDate = RP:RP.NonPrim
            */
            assertion = ((report.RPNonPrimExists == true) && (report.RPNonPrimStartDate >= report.RPNonPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_16()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Opening
    
            ^EndDate = RP:RP.Opening
            */
            assertion = ((report.RPOpeningExists == true) && (report.RPOpeningStartDate >= report.RPOpeningEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPOpeningOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_17()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Prim
    
            ^EndDate = RP:RP.Prim
            */
            assertion = ((report.RPPrimExists == true) && (report.RPPrimStartDate >= report.RPPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_18()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Share.NonPrim
    
            ^EndDate = RP:RP.Share.NonPrim
            */
            assertion = ((report.RPShareNonPrimExists == true) && (report.RPShareNonPrimStartDate >= report.RPShareNonPrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_19()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.Share.Prim
    
            ^EndDate = RP:RP.Share.Prim
            */
            assertion = ((report.RPSharePrimExists == true) && (report.RPSharePrimStartDate >= report.RPSharePrimEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432562
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432562_20()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432562
            Period start date must be prior to Period end date
    
            Legacy Rule Format:
            IF period.startDate >= period.endDate WHERE CONTEXT <> SET(“RPI.Opening”, “RPI.Closing”)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^StartDate >= ^EndDate
    
            Data Elements:
    
            ^StartDate = RP:RP.TOFA
    
            ^EndDate = RP:RP.TOFA
            */
            assertion = ((report.RPTOFAExists == true) && (report.RPTOFAStartDate >= report.RPTOFAEndDate));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432562", Severity = ProcessMessageSeverity.Error,
                    Description = @"Period start date must be prior to Period end date",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPTOFAOccurrenceIndex) + "/xbrli:period/xbrli:startDate",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432562"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432567
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432567()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432567
            The number of ETP payments for a non-dependent beneficiary must not exceed 1.
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:EmploymentTerminationPaymentDetails WHERE TUPLE ELEMENT crigi.xx.xx:InsuranceBenefits.BeneficiaryType.Code = "O") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^EmploymentTerminationPaymentDetails) > 1
    
            Data Elements:
    
            ^EmploymentTerminationPaymentDetails = RP.JAUS:EmploymentTerminationPaymentDetails
            */
            assertion = (Count(report.EmploymentTerminationPaymentDetailsCollection_O_RPJAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432567", Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of ETP payments for a non-dependent beneficiary must not exceed 1.",
                    LongDescription = @"You may provide a maximum of two instances of Eligible termination payments. One for where the beneficiary is a dependant and another where the beneficiary is a non-dependant.",
                    Location = "/xbrli:xbrl/tns:EmploymentTerminationPaymentDetails" + OccurrenceIndex(report.EmploymentTerminationPaymentDetailsCollection_O_RPJAUSLastOccurrenceIndex) + "/tns:SuperannuationBenefit.DeathBenefitEmploymentTerminationPaymentTaxableComponent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432567"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432569
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432569(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432569
                    Current Residential Address required for individual beneficiary.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF COUNT(TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)) = 1 AND (COUNT(TUPLE(address2.xx.xx:AddressDetails WHERE TUPLE EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "RES") AND (pyde.xx.xx:AddressDetails.Currency.Code = "C"))) <> 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^StatementOfDistributionToBeneficiary <> NULL AND Count(^PersonNameDetails) = 1 AND (Count(^AddressDetails)) <> 1
            
                    Data Elements:
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
            
                    ^PersonNameDetails = RP:StatementOfDistributionToBeneficiary:PersonNameDetails
            
                    ^StatementOfDistributionToBeneficiary = RP:StatementOfDistributionToBeneficiary
                    */
                    assertion = (report.StatementOfDistributionToBeneficiaryCollectionExists != false && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPCount) == 1 && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPCount) != 1);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432569", Severity = ProcessMessageSeverity.Error,
                            Description = @"Current Residential Address required for individual beneficiary.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_RES_C_RPLastOccurrenceIndex) + "/tns:AddressDetails.Usage.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432569"} },
                        };
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432570
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432570()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432570
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT245] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code

            Technical Business Rule Format:
            ^TRT245 = NULL
    
            Data Elements:
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
            */
            assertion = (report.TRT245 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432570"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432572
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432572()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432572
            Telephone must include area code for landline numbers.
    
            Legacy Rule Format:
            WHERE IN TUPLE(electroniccontacttelephone1.xx.xx:ElectronicContactTelephone)
            IF pyde.xx.xx:ElectronicContact.Telephone.ServiceLine.Code = "02" AND pyde.xx.xx:ElectronicContact.Telephone.Minimal.Number <> NULL AND pyde.xx.xx:ElectronicContact.Telephone.Area.Code = NULL 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT334 = '02' AND ^TRT249 <> NULL AND ^TRT248 = NULL
    
            Data Elements:
    
            ^TRT248 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Area.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']
    
            ^TRT334 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.ServiceLine.Code WHERE [ElectronicContact.Telephone.Usage.Code='03']
    
            ^TRT249 = INT:ElectronicContactTelephone:ElectronicContact.Telephone.Minimal.Number WHERE [ElectronicContact.Telephone.Usage.Code='03']
            */
            assertion = (report.TRT334 == @"02" && report.TRT249 != null && report.TRT248 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432572", Severity = ProcessMessageSeverity.Error,
                    Description = @"Telephone must include area code for landline numbers.",
                    LongDescription = @"Telephone must include area code for landline numbers.",
                    Location = "/xbrli:xbrl/tns:ElectronicContactTelephone" + OccurrenceIndex(report.ElectronicContactTelephoneCollection_03_INTOccurrenceIndex) + "/tns:ElectronicContact.Telephone.Area.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432572"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT334", Value = report.TRT334 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT249", Value = report.TRT249 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT248", Value = GetValueOrEmpty(report.TRT248) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432573
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432573()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432573
            Contact name must not exceed 200 characters in length
    
            Legacy Rule Format:
            IF LENGTH([TRT250]) > 200
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT250] = TRT:INT:pyde.02.00:PersonUnstructuredName.FullName.Text WHERE (pyde.02.05:PersonUnstructuredName.Usage.Code = "Contact") IN TUPLE(trt.0011.lodge.req.02.00:PersonUnstructuredName)

            Technical Business Rule Format:
            Length(^TRT250) > 200
    
            Data Elements:
    
            ^TRT250 = INT:PersonUnstructuredName:PersonUnstructuredName.FullName.Text WHERE [PersonUnstructuredName.Usage.Code='Contact']
            */
            assertion = (Length(report.TRT250) > 200);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432573", Severity = ProcessMessageSeverity.Error,
                    Description = @"Contact name must not exceed 200 characters in length",
                    Location = "/xbrli:xbrl/tns:PersonUnstructuredName" + OccurrenceIndex(report.PersonUnstructuredNameCollection_Contact_INTOccurrenceIndex) + "/tns:PersonUnstructuredName.FullName.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432573"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT250", Value = report.TRT250 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432577
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432577()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432577
            Schedule A must not exceed 3000 characters
    
            Legacy Rule Format:
            IF LENGTH(bafot.xx.xx:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text) > 3000
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Length(^TRT405) > 3000
    
            Data Elements:
    
            ^TRT405 = RP:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text
            */
            assertion = (Length(report.TRT405) > 3000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432577", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Other attachments schedule must not exceed 3000 characters",
                    LongDescription = @"Tthe 'Other attachments' schedule is represented by the 'Regulatory Disclosures. General Information About Financial Statements.Text' element",
                    Location = "/xbrli:xbrl/tns:RegulatoryDisclosures.GeneralInformationAboutFinancialStatements.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432577"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT405", Value = GetValueOrEmpty(report.TRT405) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432578
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432578()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432578
            In the distribution statement the sum of Share of credit for TFN amounts withheld from payments from closely held trusts to which beneficiaries are entitled to and to which beneficiaries are not entitled to is not equal to the Credit for TFN amounts withheld from payments from closely held trust.
    
            Legacy Rule Format:
            IF [TRT418] <> (SUM([TRT419]) + [TRT435])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT418] = TRT:RP.JAUS:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
            [TRT419] = TRT:RP:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT435] = TRT:RP:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT418 <> (Sum(^TRT419) + ^TRT435)
    
            Data Elements:
    
            ^TRT418 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
    
            ^TRT419 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
    
            ^TRT435 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
            */
            assertion = (report.TRT418.GetValueOrDefault() != (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT419.GetValueOrDefault())) + report.TRT435.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432624", Severity = ProcessMessageSeverity.Error,
                    Description = @"Sum of Share of credit for TFN amounts withheld not equal to Credit for TFN amounts withheld",
                    LongDescription = @"In the distribution statement the sum of Share of credit for TFN amounts withheld from payments from closely held trusts to which beneficiaries are entitled to and to which beneficiaries are not entitled to is not equal to the Credit for TFN amounts withheld from payments from closely held trusts.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432578"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT418", Value = GetValueOrEmpty(report.TRT418) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT419", Value = "TRT419" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT435", Value = GetValueOrEmpty(report.TRT435) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432579
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432579()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432579
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT418] <> NULL AND [TRT418] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT418] = TRT:RP.JAUS:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT418, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT418 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount
            */
            assertion = NotMonetary(report.TRT418, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrustShareReceivedIndirectly.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432579"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT418", Value = GetValueOrEmpty(report.TRT418) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432581
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432581(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432581
                    Family name or non-individual name line 1 is present in the distribution statement without the entity type of beneficiary code.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF [TRT432] = NULL
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^TRT432 = NULL
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT432 == null);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432581", Severity = ProcessMessageSeverity.Error,
                            Description = @"In distribution statement the entity type of beneficiary code must be present",
                            LongDescription = @"Family name or non-individual name line 1 is present in the distribution statement without the entity type of beneficiary code.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432581"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432582
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432582(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432582
                    Individual name must be present for individual beneficiaries.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF COUNT(TUPLE(prsnstrcnm3.xx.xx:PersonNameDetails)) = 0 AND 
                          [TRT432] = 'I'
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^StatementOfDistributionToBeneficiary <> NULL AND Count(^PersonNameDetails) = 0 AND ^TRT432 = 'I'
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^PersonNameDetails = RP:StatementOfDistributionToBeneficiary:PersonNameDetails
            
                    ^StatementOfDistributionToBeneficiary = RP:StatementOfDistributionToBeneficiary
                    */
                    assertion = (report.StatementOfDistributionToBeneficiaryCollectionExists != false && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_PersonNameDetailsCollection_LGL_Contact_RPCount) == 0 && statementOfDistributionToBeneficiary.TRT432 == @"I");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432582", Severity = ProcessMessageSeverity.Error,
                            Description = @"Individual name must be present for individual beneficiaries.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432582"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432583
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432583(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432583
                    Non-individual name must be present for non-individual beneficiaries.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF COUNT(TUPLE(orgname2.xx.xx:OrganisationNameDetails)) = 0 AND 
                           [TRT432] <> 'I'
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    ^StatementOfDistributionToBeneficiary <> NULL AND Count(^OrganisationNameDetails) = 0 AND ^TRT432 <> 'I'
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^OrganisationNameDetails = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails
            
                    ^StatementOfDistributionToBeneficiary = RP:StatementOfDistributionToBeneficiary
                    */
                    assertion = (report.StatementOfDistributionToBeneficiaryCollectionExists != false && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPCount) == 0 && statementOfDistributionToBeneficiary.TRT432 != @"I");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432583", Severity = ProcessMessageSeverity.Error,
                            Description = @"Non-individual name must be present for non-individual beneficiaries.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432583"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432584
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432584(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432584
                    Date of birth must be present for individual beneficiaries.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF ([TRT213] = NULL AND [TRT432] = 'I') THEN
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT213] = TRT:RP:pyde.02.00:PersonDemographicDetails.Birth.Date IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (^TRT213 = NULL AND ^TRT432 = 'I')
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^TRT213 = RP:StatementOfDistributionToBeneficiary:PersonDemographicDetails.Birth.Date
                    */
                    assertion = (statementOfDistributionToBeneficiary.TRT213 == null && statementOfDistributionToBeneficiary.TRT432 == @"I");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432584", Severity = ProcessMessageSeverity.Error,
                            Description = @"Date of birth must be present for individual beneficiaries.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432584"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT213", Value = GetValueOrEmptyDate(statementOfDistributionToBeneficiary.TRT213) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432585
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432585(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432585
                    Assessment calculation codes are not correct for company beneficiaries.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF ([TRT214] <> SET("13","34","119","134","139") AND [TRT432] = 'C') THEN
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (NotInSet(^TRT214, '"13","34","119","134","139"') AND ^TRT432 = 'C')
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (!(IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(13|34|119|134|139)$",RegexOptions.IgnoreCase)) && statementOfDistributionToBeneficiary.TRT432 == @"C");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432585", Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment calculation codes are not correct for company beneficiaries.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432585"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432586
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432586(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432586
                    Assessment calculation codes are not correct for trustee beneficiaries
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF ([TRT214] <> ("14","35","45","120","140","145") AND [TRT432] = 'T') THEN
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
                    [TRT432] = TRT:RP:rvctc1.02.04:IncomeTax.BeneficiaryEntityType.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    (NotInSet(^TRT214, '"14","35","45","120","140","145"') AND ^TRT432 = 'T')
            
                    Data Elements:
            
                    ^TRT432 = RP:StatementOfDistributionToBeneficiary:IncomeTax.BeneficiaryEntityType.Code
            
                    ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
                    */
                    assertion = (!(IsMatch(statementOfDistributionToBeneficiary.TRT214, @"^(14|35|45|120|140|145)$",RegexOptions.IgnoreCase)) && statementOfDistributionToBeneficiary.TRT432 == @"T");
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432586", Severity = ProcessMessageSeverity.Error,
                            Description = @"Assessment calculation codes are not correct for trustee beneficiaries",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.BeneficiaryEntityType.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432586"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT214", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT214) });
            
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT432", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT432) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432587
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432587(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432587
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount <> NULL 
                    AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT419, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT419 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT419, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432587"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT419", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT419) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432587
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432587_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432587
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount <> NULL 
            AND TRT:RP:rvctc2.xx.xx:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT435, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT435 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
            */
            assertion = NotMonetary(report.TRT435, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432587"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT435", Value = GetValueOrEmpty(report.TRT435) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432588
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432588(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432588
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT433] <> NULL AND [TRT433] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT433] = TRT:RP:bafpr1.02.05:Expense.DistributionBeneficiaryCloselyHeldTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:AnnualTrusteePaymentReportInformation) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT433, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT433 = RP:StatementOfDistributionToBeneficiary:AnnualTrusteePaymentReportInformation:Expense.DistributionBeneficiaryCloselyHeldTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT433, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432588", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AnnualTrusteePaymentReportInformation/tns:Expense.DistributionBeneficiaryCloselyHeldTrust.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432588"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT433", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT433) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432589
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432589(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432589
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT434] <> NULL AND [TRT434] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT434] = TRT:RP:rvctc2.02.03:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:AnnualTrusteePaymentReportInformation) IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT434, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT434 = RP:StatementOfDistributionToBeneficiary:AnnualTrusteePaymentReportInformation:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT434, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432589", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AnnualTrusteePaymentReportInformation/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldFromCloselyHeldTrust.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432589"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT434", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT434) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432593
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432593()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432593
            The beneficiary TFN reported must not be reported more than once.
    
            Legacy Rule Format:
            IF [TRT211] <> NULL AND [TRT211] = ANY OTHER OCCURRENCE OF [TRT211]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT211] = TRT:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)

            Technical Business Rule Format:
            HasDuplicateValues(^TRT211)
    
            Data Elements:
    
            ^TRT211 = RP:StatementOfDistributionToBeneficiary:Identifiers.TaxFileNumber.Identifier
            */
            assertion = HasDuplicateValues(report.StatementOfDistributionToBeneficiaryCollection == null ? null : report.StatementOfDistributionToBeneficiaryCollection.Select(f => f.TRT211).Cast<object>().ToArray());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432593", Severity = ProcessMessageSeverity.Error,
                    Description = @"The beneficiary TFN reported must not be reported more than once.",
                    Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary[" + report.StatementOfDistributionToBeneficiaryCollection[DuplicateValueIndex(report.StatementOfDistributionToBeneficiaryCollection.Select(f => f.TRT211))].OccurrenceIndex + "]/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432593"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432594
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432594()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432594
            Client's reference must not exceed 12 characters in length
    
            Legacy Rule Format:
            IF LENGTH([TRT251]) > 12
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT251] = TRT:INT:pyid.02.00:Identifiers.TaxAgentClientReference.Text

            Technical Business Rule Format:
            Length(^TRT251) > 12
    
            Data Elements:
    
            ^TRT251 = INT:Identifiers.TaxAgentClientReference.Text
            */
            assertion = (Length(report.TRT251) > 12);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432594", Severity = ProcessMessageSeverity.Error,
                    Description = @"Client's reference must not exceed 12 characters in length",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxAgentClientReference.Text",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432594"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT251", Value = GetValueOrEmpty(report.TRT251) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432600
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432600()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432600
            An amount greater than zero is present at credit for tax withheld where ABN not quoted (beneficiary not entitled) and there is no primary production or non-primary production income distribution or franked distributions amount.
    
            Legacy Rule Format:
            IF [TRT236] > 0  AND [TRT234] = NULL AND [TRT235] = NULL AND [TRT444] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT236] = TRT:RP:rvctc2.02.00:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT236 > 0  AND ^TRT234 = NULL AND ^TRT235 = NULL AND ^TRT444 = NULL
    
            Data Elements:
    
            ^TRT236 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
            */
            assertion = (report.TRT236.GetValueOrDefault() > 0 && report.TRT234 == null && report.TRT235 == null && report.TRT444 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432620", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP or Non-PP income distribution or franked distributions must be present in Statement of distribution",
                    LongDescription = @"If an amount greater than zero is present at 'Credit for tax withheld where ABN not quoted' against 'Income to which no beneficiary is presently entitled...', then there must be Primary production or Non-primary production income or franked distributions amount for 'income to which no beneficiary is presently entitled'.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForTaxWithheldWhereABNNotQuoted.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432600"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT236", Value = GetValueOrEmpty(report.TRT236) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432601
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432601()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432601
            IncomeNoBeneficiaryPresentlyEntitled tuple can only appear once
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:IncomeNoBeneficiaryPresentlyEntitled) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^IncomeNoBeneficiaryPresentlyEntitled) > 1
    
            Data Elements:
    
            ^IncomeNoBeneficiaryPresentlyEntitled = RP:IncomeNoBeneficiaryPresentlyEntitled
            */
            assertion = (Count(report.IncomeNoBeneficiaryPresentlyEntitledCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432601", Severity = ProcessMessageSeverity.Error,
                    Description = @"IncomeNoBeneficiaryPresentlyEntitled tuple can only appear once",
                    LongDescription = @"IncomeNoBeneficiaryPresentlyEntitled tuple can only appear once. Please contact your software vendor in order to rectify this error.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled" + OccurrenceIndex(report.IncomeNoBeneficiaryPresentlyEntitledCollectionCount) + "/tns:IncomeTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432601"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432604
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432604()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432604
            SuperannuationIncomeStreamWithholdingPaymentDetails tuple can only appear once
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:SuperannuationIncomeStreamWithholdingPaymentDetails) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^SuperannuationIncomeStreamWithholdingPaymentDetails) > 1
    
            Data Elements:
    
            ^SuperannuationIncomeStreamWithholdingPaymentDetails = RP.JAUS:SuperannuationIncomeStreamWithholdingPaymentDetails
            */
            assertion = (Count(report.SuperannuationIncomeStreamWithholdingPaymentDetailsCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432604", Severity = ProcessMessageSeverity.Error,
                    Description = @"SuperannuationIncomeStreamWithholdingPaymentDetails tuple can only appear once",
                    LongDescription = @"SuperannuationIncomeStreamWithholdingPaymentDetails tuple can only appear once. Please contact your software vendor in order to rectify this error.",
                    Location = "/xbrli:xbrl/tns:SuperannuationIncomeStreamWithholdingPaymentDetails" + OccurrenceIndex(report.SuperannuationIncomeStreamWithholdingPaymentDetailsCollectionCount) + "/tns:IncomeTax.PayAsYouGoWithholding.TaxWithheld.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432604"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432605
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432605()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432605
            SuperannuationLumpSumWithholdingPaymentDetails tuple can only appear once
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:SuperannuationLumpSumWithholdingPaymentDetails) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^SuperannuationLumpSumWithholdingPaymentDetails) > 1
    
            Data Elements:
    
            ^SuperannuationLumpSumWithholdingPaymentDetails = RP.JAUS:SuperannuationLumpSumWithholdingPaymentDetails
            */
            assertion = (Count(report.SuperannuationLumpSumWithholdingPaymentDetailsCollection_O_RPJAUSCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432605", Severity = ProcessMessageSeverity.Error,
                    Description = @"SuperannuationLumpSumWithholdingPaymentDetails tuple can only appear once",
                    LongDescription = @"SuperannuationLumpSumWithholdingPaymentDetails tuple can only appear once. Please contact your software vendor in order to rectify this error.",
                    Location = "/xbrli:xbrl/tns:SuperannuationLumpSumWithholdingPaymentDetails" + OccurrenceIndex(report.SuperannuationLumpSumWithholdingPaymentDetailsCollection_O_RPJAUSLastOccurrenceIndex) + "/tns:InsuranceBenefits.DeathBenefit.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432605"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432606
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432606()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432606
            MailRecipient tuple can only appear once
    
            Legacy Rule Format:
            IF COUNT TUPLE(trt.0011.lodge.req.xx.xx:MailRecipient) > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^MailRecipient) > 1
    
            Data Elements:
    
            ^MailRecipient = RP:MailRecipient
            */
            assertion = (Count(report.MailRecipientCollectionCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432606", Severity = ProcessMessageSeverity.Error,
                    Description = @"MailRecipient tuple can only appear once",
                    LongDescription = @"MailRecipient tuple can only appear once. Please contact your software vendor in order to rectify this error.",
                    Location = "/xbrli:xbrl/tns:MailRecipient" + OccurrenceIndex(report.MailRecipientCollectionCount) + "/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432606"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432607
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432607()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432607
            Where credit for tax withheld - foreign resident withholding (excluding capital gains) is greater than zero, there must be an amount at either share of income - primary or non-primary production or franked distributions. No beneficiary entitled.
    
            Legacy Rule Format:
            IF [TRT477] > 0 AND ([TRT234] = NULL AND [TRT235] = NULL  AND [TRT444] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT234] = TRT:RP.Prim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT235] = TRT:RP.NonPrim:rvctc1.02.00:IncomeTax.IncomeOrLossTaxable.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT444] = TRT:RP:bafpr1.02.09:Income.DistributionFranked.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT477] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT477 > 0 AND (^TRT234 = NULL AND ^TRT235 = NULL  AND ^TRT444 = NULL)
    
            Data Elements:
    
            ^TRT477 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
    
            ^TRT234 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT235 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.IncomeOrLossTaxable.Amount
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
            */
            assertion = (report.TRT477.GetValueOrDefault() > 0 && (report.TRT234 == null && report.TRT235 == null && report.TRT444 == null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432741", Severity = ProcessMessageSeverity.Error,
                    Description = @"PP or Non PP Share of income or franked distributions must be present",
                    LongDescription = @"If within a Statement of Distribution there is an amount greater than zero at 'Credit for tax withheld - foreign resident withholding (excluding capital gains)', then there must be an amount at either 'Share of income - primary production'  or 'Share of income - non-primary production' or 'Franked distributions' where 'Income to which no beneficiary is presently entitled'.",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432607"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT477", Value = GetValueOrEmpty(report.TRT477) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT234", Value = GetValueOrEmpty(report.TRT234) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT235", Value = GetValueOrEmpty(report.TRT235) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432609
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432609()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432609
            Family Trust Election Status - Specified income year must not be less than 1995 or greater than the current reporting year.
    
            Legacy Rule Format:
            IF ([TRT28] <> NULL) AND ([TRT28] < 1995 OR [TRT28] > [TRT1])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT28] = TRT:RP:rvctc3.02.00:Elections.FamilyTrustElectionStatus.Year

            Technical Business Rule Format:
            (^TRT28 <> NULL) AND (^TRT28 < 1995 OR ^TRT28 > ^TRT1)
    
            Data Elements:
    
            ^TRT28 = RP:Elections.FamilyTrustElectionStatus.Year
    
            ^TRT1 = RP:Report.TargetFinancial.Year
            */
            assertion = (report.TRT28 != null && (report.TRT28.GetValueOrDefault() < 1995 || report.TRT28.GetValueOrDefault() > report.TRT1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432609", Severity = ProcessMessageSeverity.Error,
                    Description = @"Family Trust Election Status - Specified income year must not be less than 1995 or greater than the current reporting year.",
                    Location = "/xbrli:xbrl/tns:Elections.FamilyTrustElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432609"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT28", Value = GetValueOrEmpty(report.TRT28) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432610
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432610()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432610
            Interposed Entity Election Status - Specified income year must not be less than 1995 or greater than the current reporting year.
    
            Legacy Rule Format:
            IF ([TRT30] <> NULL) AND ([TRT30] < 1995 OR [TRT30] > [TRT1])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT30] = TRT:RP:rvctc3.02.00:Elections.InterposedEntityElectionStatus.Year

            Technical Business Rule Format:
            (^TRT30 <> NULL) AND (^TRT30 < 1995 OR ^TRT30 > ^TRT1)
    
            Data Elements:
    
            ^TRT30 = RP:Elections.InterposedEntityElectionStatus.Year
    
            ^TRT1 = RP:Report.TargetFinancial.Year
            */
            assertion = (report.TRT30 != null && (report.TRT30.GetValueOrDefault() < 1995 || report.TRT30.GetValueOrDefault() > report.TRT1.GetValueOrDefault()));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432610", Severity = ProcessMessageSeverity.Error,
                    Description = @"Interposed Entity Election Status - Specified income year must not be less than 1995 or greater than the current reporting year.",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432610"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT30", Value = GetValueOrEmpty(report.TRT30) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432611
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432611()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432611
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT447] <> NULL AND [TRT447] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT447] = TRT:RP.JAUS:bafpr1.02.10:Expense.ReconciliationAdjustmentTotal.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT447, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT447 = RP.JAUS:Expense.ReconciliationAdjustmentTotal.Amount
            */
            assertion = NotMonetary(report.TRT447, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.ReconciliationAdjustmentTotal.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432611"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT447", Value = GetValueOrEmpty(report.TRT447) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432612
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432612()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432612
            If tax offset code is present then the tax offset amount must be present and greater than zero
    
            Legacy Rule Format:
            IF ([TRT463] = 0 OR [TRT463] = NULL) AND [TRT464] <> NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT463] = TRT:RP:rvctc1.02.17:IncomeTax.TaxOffsetRefundableOther.Amount
            [TRT464] = TRT:RP:rvctc1.02.18:IncomeTax.TaxOffsetRefundableOther.Code

            Technical Business Rule Format:
            (^TRT463 = 0 OR ^TRT463 = NULL) AND ^TRT464 <> NULL
    
            Data Elements:
    
            ^TRT464 = RP:IncomeTax.TaxOffsetRefundableOther.Code
    
            ^TRT463 = RP:IncomeTax.TaxOffsetRefundableOther.Amount
            */
            assertion = ((report.TRT463 == 0 || report.TRT463 == null) && report.TRT464 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432640", Severity = ProcessMessageSeverity.Error,
                    Description = @"Tax offset amount is equal to zero or not present but code is present",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetRefundableOther.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432612"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT463", Value = GetValueOrEmpty(report.TRT463) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT464", Value = GetValueOrEmpty(report.TRT464) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432613
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432613(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432613
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT465] <> NULL AND [TRT465] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT465] = TRT:RP:rvctc1.02.32:IncomeTax.ExplorationCreditsDistributed.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT465, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT465 = RP:StatementOfDistributionToBeneficiary:IncomeTax.ExplorationCreditsDistributed.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT465, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.ExplorationCreditsDistributed.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432613"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT465", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT465) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432631
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432631()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432631
            If there is an amount present at non-primary production deductions relating to franked distributions from trusts, then there must be an amount at non-primary production franked distributions from trusts.
    
            Legacy Rule Format:
            IF [TRT438] > 0 AND [TRT437] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount
            [TRT438] = TRT:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            ^TRT438 > 0 AND ^TRT437 = NULL
    
            Data Elements:
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
    
            ^TRT438 = RP.JAUS.NonPrim:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = (report.TRT438.GetValueOrDefault() > 0 && report.TRT437 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432631", Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-primary production franked distributions from trusts amount must be present",
                    LongDescription = @"Non-primary production 'Deductions relating to franked distributions from trusts' is greater than zero, however no non-primary production 'Franked distributions from trusts' amount exists.",
                    Location = "/xbrli:xbrl/tns:Income.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432631"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT438", Value = GetValueOrEmpty(report.TRT438) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432632
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432632()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432632
            The sum of Share of income of the trust estate (beneficary entitled) and the Share of income of the trust estate (no beneficiary entitled) in the Statement of distribution, must equal the Income of the Trust Estate.
    
            Legacy Rule Format:
            IF (ANY OCCURRENCE OF [TRT441] > 0 OR [TRT443] > 0) AND 
            ((SUM([TRT441]) + [TRT443]) <> [TRT440])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT440] = TRT:RP:bafpr1.02.10:Income.TrustEstateIncome.Amount
            [TRT441] = TRT:RP:bafpr1.02.10:Income.TrustEstateIncomeShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT443] = TRT:RP:bafpr1.02.10:Income.TrustEstateIncomeShare.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (AnyOccurrence(^TRT441, ^TRT441 > 0) OR ^TRT443 > 0) AND ((Sum(^TRT441) + ^TRT443) <> ^TRT440)
    
            Data Elements:
    
            ^TRT440 = RP:Income.TrustEstateIncome.Amount
    
            ^TRT441 = RP:StatementOfDistributionToBeneficiary:Income.TrustEstateIncomeShare.Amount
    
            ^TRT443 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.TrustEstateIncomeShare.Amount
            */
            assertion = (((report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT441Repeat => TRT441Repeat.TRT441.GetValueOrDefault() > 0)) || report.TRT443.GetValueOrDefault() > 0) && (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT441.GetValueOrDefault())) + report.TRT443.GetValueOrDefault() != report.TRT440.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432632", Severity = ProcessMessageSeverity.Error,
                    Description = @"Share of income of the trust estate is incorrect",
                    LongDescription = @"The sum of Share of income of the trust estate (beneficiary entitled and no beneficiary entitled) does not equal the income of the trust estate amount.",
                    Location = "/xbrli:xbrl/tns:Income.TrustEstateIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432632"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT441", Value = "TRT441" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT443", Value = GetValueOrEmpty(report.TRT443) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT440", Value = GetValueOrEmpty(report.TRT440) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432633
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432633()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432633
            If No beneficiary presently entitled assessment calculation code and Choice for resident trustee to be assessed to capital gains on behalf of beneficiaries assessment calculation code both exist, the assessment calculation code value for both must be identical.
    
            Legacy Rule Format:
            IF [TRT230] <> NULL AND [TRT445] <> NULL AND [TRT230] <> [TRT445]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)
            [TRT445] = TRT:RP:rvctc3.02.08:CapitalGainsTax.AssessmentCalculation.Code

            Technical Business Rule Format:
            ^TRT230 <> NULL AND ^TRT445 <> NULL AND ^TRT230 <> ^TRT445
    
            Data Elements:
    
            ^TRT445 = RP:CapitalGainsTax.AssessmentCalculation.Code
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT230 != null && report.TRT445 != null && report.TRT230 != report.TRT445);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432633", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code is not correct",
                    LongDescription = @"No beneficiary presently entitled assessment calculation code and Choice for resident trusteee to be assessed to capital gains on behalf of beneficiaries assessment calculation code are present, but the assessment calculation code value of each are not identical.",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432633"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = GetValueOrEmpty(report.TRT230) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT445", Value = GetValueOrEmpty(report.TRT445) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432634
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432634()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432634
            Where there are deductions relating to franked distributions greater than zero, then there must be an income amount at franked dividends.
    
            Legacy Rule Format:
            IF [TRT439] > 0 AND [TRT97] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT97] = TRT:RP.JAUS:bafpr1.02.00:Income.DividendsFranked.Amount
            [TRT439] = TRT:RP.JAUS:rvctc2.02.06:IncomeTax.Deduction.DistributionFranked.Amount

            Technical Business Rule Format:
            ^TRT439 > 0 AND ^TRT97 = NULL
    
            Data Elements:
    
            ^TRT97 = RP.JAUS:Income.DividendsFranked.Amount
    
            ^TRT439 = RP.JAUS:IncomeTax.Deduction.DistributionFranked.Amount
            */
            assertion = (report.TRT439.GetValueOrDefault() > 0 && report.TRT97 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432634", Severity = ProcessMessageSeverity.Error,
                    Description = @"Franked Dividends Amount must be present",
                    LongDescription = @"If 'Deductions relating to franked distributions' is greater than zero, then there must be a 'Dividends - Franked Amount'",
                    Location = "/xbrli:xbrl/tns:Income.DividendsFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432634"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT439", Value = GetValueOrEmpty(report.TRT439) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT97", Value = GetValueOrEmpty(report.TRT97) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432635
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432635()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432635
            Amount of capital gains on which the trustee has chosen to be assessed on behalf of beneficiaries is present and the assessment calculation code is missing.
    
            Legacy Rule Format:
            IF [TRT446] <> NULL AND  [TRT445] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT445] = TRT:RP:rvctc3.02.08:CapitalGainsTax.AssessmentCalculation.Code
            [TRT446] = TRT:RP:rvctc3.02.08:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount

            Technical Business Rule Format:
            ^TRT446 <> NULL AND  ^TRT445 = NULL
    
            Data Elements:
    
            ^TRT445 = RP:CapitalGainsTax.AssessmentCalculation.Code
    
            ^TRT446 = RP:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount
            */
            assertion = (report.TRT446 != null && report.TRT445 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432635", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code must be present",
                    LongDescription = @"Choice for resident trustee to be assessed to capital gains on behalf of beneficiaries assessment calculation code required.",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.AssessmentCalculation.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432635"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT446", Value = GetValueOrEmpty(report.TRT446) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT445", Value = GetValueOrEmpty(report.TRT445) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432636
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432636()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432636
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT437] <> NULL AND [TRT437] <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT437] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT437, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT437 = RP.JAUS.NonPrim:Income.TrustDistributionFranked.Amount
            */
            assertion = NotMonetary(report.TRT437, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432636"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT437", Value = GetValueOrEmpty(report.TRT437) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432637
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432637()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432637
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT438] <> NULL AND [TRT438] <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT438] = TRT:RP.JAUS.NonPrim:rvctc2.02.05:IncomeTax.Deduction.TrustDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT438, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT438 = RP.JAUS.NonPrim:IncomeTax.Deduction.TrustDistributionFranked.Amount
            */
            assertion = NotMonetary(report.TRT438, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TrustDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432637"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT438", Value = GetValueOrEmpty(report.TRT438) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432638
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432638()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432638
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT439] <> NULL AND [TRT439] <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT439] = TRT:RP.JAUS:rvctc2.02.06:IncomeTax.Deduction.DistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT439, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT439 = RP.JAUS:IncomeTax.Deduction.DistributionFranked.Amount
            */
            assertion = NotMonetary(report.TRT439, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.DistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432638"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT439", Value = GetValueOrEmpty(report.TRT439) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432639
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432639()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432639
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT440] <> NULL AND [TRT440] <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT440] = TRT:RP:bafpr1.02.10:Income.TrustEstateIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT440, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT440 = RP:Income.TrustEstateIncome.Amount
            */
            assertion = NotMonetary(report.TRT440, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.TrustEstateIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432639"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT440", Value = GetValueOrEmpty(report.TRT440) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432640
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432640(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432640
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF bafpr1.xx.xx:Income.TrustEstateIncomeShare.Amount <> NULL AND  bafpr1.xx.xx:Income.TrustEstateIncomeShare.Amount  <> MONETARY (U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT441, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT441 = RP:StatementOfDistributionToBeneficiary:Income.TrustEstateIncomeShare.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT441, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.TrustEstateIncomeShare.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432640"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT441", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT441) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432640
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432640_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432640
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr1.xx.xx:Income.TrustEstateIncomeShare.Amount <> NULL AND  bafpr1.xx.xx:Income.TrustEstateIncomeShare.Amount  <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT443, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT443 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.TrustEstateIncomeShare.Amount
            */
            assertion = NotMonetary(report.TRT443, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.TrustEstateIncomeShare.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432640"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT443", Value = GetValueOrEmpty(report.TRT443) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432641
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432641(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432641
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF bafpr1.xx.xx:Income.DistributionFranked.Amount <> NULL AND bafpr1.xx.xx:Income.DistributionFranked.Amount <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT442, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT442 = RP:StatementOfDistributionToBeneficiary:Income.DistributionFranked.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT442, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.DistributionFranked.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432641"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT442", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT442) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432641
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432641_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432641
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF bafpr1.xx.xx:Income.DistributionFranked.Amount <> NULL AND bafpr1.xx.xx:Income.DistributionFranked.Amount <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT444, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT444 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.DistributionFranked.Amount
            */
            assertion = NotMonetary(report.TRT444, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.DistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432641"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT444", Value = GetValueOrEmpty(report.TRT444) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432642
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432642()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432642
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT446] <> NULL AND [TRT446] <> MONETARY (U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT446] = TRT:RP:rvctc3.02.08:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT446, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT446 = RP:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount
            */
            assertion = NotMonetary(report.TRT446, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.TrusteeAssessedCapitalGain.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432642"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT446", Value = GetValueOrEmpty(report.TRT446) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432658
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432658()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432658
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT77] <> NULL AND [TRT77] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT77, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
            */
            assertion = NotMonetary(report.TRT77, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432658"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432661
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432661()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432661
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT81] <> NULL AND [TRT81] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT81, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            */
            assertion = NotMonetary(report.TRT81, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432661"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432666
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432666()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432666
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT155] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT155] = TRT:RP:bafpr1.02.13:Income.PersonalServicesIncomeIncluded.Indicator

            Technical Business Rule Format:
            ^TRT155 = NULL
    
            Data Elements:
    
            ^TRT155 = RP:Income.PersonalServicesIncomeIncluded.Indicator
            */
            assertion = (report.TRT155 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncomeIncluded.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432666"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT155", Value = GetValueOrEmpty(report.TRT155) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432668
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432668()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432668
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT440] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT440] = TRT:RP:bafpr1.02.10:Income.TrustEstateIncome.Amount

            Technical Business Rule Format:
            ^TRT440 = NULL
    
            Data Elements:
    
            ^TRT440 = RP:Income.TrustEstateIncome.Amount
            */
            assertion = (report.TRT440 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Income.TrustEstateIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432668"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT440", Value = GetValueOrEmpty(report.TRT440) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432681
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432681()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432681
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT449] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT449] = TRT:RP:bafpo6.02.15:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator

            Technical Business Rule Format:
            ^TRT449 = NULL
    
            Data Elements:
    
            ^TRT449 = RP:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator
            */
            assertion = (report.TRT449 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:Liabilities.ThinCapitalisation.ProvisionsApplied.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432681"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT449", Value = GetValueOrEmpty(report.TRT449) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432693
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432693(string TRT448, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432693
                    Invalid exemption or rollover code
    
                    Legacy Rule Format:
                    IF [TRT448] <> NULL AND [TRT448] <> SET (DOMAIN(TRTCGTExemptionOrRolloverCodes))
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT448] = TRT:RP:rvctc3.02.47:CapitalGainsTax.ExemptionOrRolloverApplied.Code
        
                    Technical Business Rule Format:
                    ^TRT448 <> NULL AND NotInDomain(^TRT448, 'Trtcgtexemptionorrollovercodes')
            
                    Data Elements:
            
                    ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                    */
                    assertion = (TRT448 != null && !(IsMatch(TRT448, @"^(A - Small business active asset reduction|B - Small business retirement exemption|C - Small business roll-over|D - Small business 15 year exemption|E - Foreign resident CGT exemption|F - Scrip for scrip roll-over|H - Demerger exemption|I - Main residence exemption|J - Capital gains disregarded as a result of the sale of a pre-CGT asset|K - Disposal or creation of assets in a wholly-owned company|L - Replacement asset roll-overs|M - Exchange of shares or units|N - Exchange of rights or options|O - Exchange of shares in one company for shares in another company|P - Exchange of units in a unit trust for shares in a company|Q - Trust restructure roll-over|R - Demerger roll-over|S - Same asset roll-overs|T - Small Business Restructure Roll-over|U - Early stage investor|V - Venture capital investment|X - Other exemptions and roll-overs)$",RegexOptions.IgnoreCase)));
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432693", Severity = ProcessMessageSeverity.Error,
                            Description = @"Invalid exemption or rollover code",
                            Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[" + itemIndex1 + "]",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432693"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT448", Value = GetValueOrEmpty(TRT448) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432694
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432694()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432694
            Exemption or rollover code must be unique
    
            Legacy Rule Format:
            IF [TRT448] <> NULL AND [TRT448] = ANY OTHER OCCURRENCE OF [TRT448]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT448] = TRT:RP:rvctc3.02.47:CapitalGainsTax.ExemptionOrRolloverApplied.Code

            Technical Business Rule Format:
            HasDuplicateValues(^TRT448)
    
            Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
            */
            assertion = HasDuplicateValues(report.TRT448Collection);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432694", Severity = ProcessMessageSeverity.Error,
                    Description = @"Exemption or rollover code must be unique",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code" + OccurrenceIndex(DuplicateValueIndex(report.TRT448Collection) + 1) + "",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432694"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432696
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432696()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432696
            Invalid context. The number of Reporting party:LowCost contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.LowCost)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.LowCost
            */
            assertion = (Count(report.RPLowCostCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432696", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:LowCost contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPLowCostLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432696"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432697
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432697()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432697
            Invalid context. The number of Reporting party:GeneralPool contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.GeneralPool)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.GeneralPool
            */
            assertion = (Count(report.RPGeneralPoolCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432697", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party:GeneralPool contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPGeneralPoolLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432697"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432701
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432701()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432701
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT147] <> NULL AND [TRT147] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT147] = TRT:RP.Opening:gfati.02.05:TaxOffsetClaim.LandcareAndWaterFacilityBroughtForward.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT147, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT147 = RP.Opening:TaxOffsetClaim.LandcareAndWaterFacilityBroughtForward.Amount
            */
            assertion = NotMonetary(report.TRT147, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.LandcareAndWaterFacilityBroughtForward.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432701"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT147", Value = GetValueOrEmpty(report.TRT147) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432702
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432702(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432702
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF TRT:RP:rvctc4.xx.xx:Income.InternationalDealings.Net.Amount <> NULL 
                    AND TRT:RP:rvctc4.xx.xx:Income.InternationalDealings.Net.Amount <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT225, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT225 = RP:StatementOfDistributionToBeneficiary:Income.InternationalDealings.Net.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT225, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.InternationalDealings.Net.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432702"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT225", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT225) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432702
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432702_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432702
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF TRT:RP:rvctc4.xx.xx:Income.InternationalDealings.Net.Amount <> NULL 
            AND TRT:RP:rvctc4.xx.xx:Income.InternationalDealings.Net.Amount <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT241, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT241 = RP:IncomeNoBeneficiaryPresentlyEntitled:Income.InternationalDealings.Net.Amount
            */
            assertion = NotMonetary(report.TRT241, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.InternationalDealings.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432702"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT241", Value = GetValueOrEmpty(report.TRT241) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432703
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432703()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432703
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT475] <> NULL AND [TRT475] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT475] = TRT:RP.Closing(Instant):rvctc3.02.16:Capital.Losses.CarriedForward.Net.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT475, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT475 = RP.Closing(Instant):Capital.Losses.CarriedForward.Net.Amount
            */
            assertion = NotMonetary(report.TRT475, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432703"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT475", Value = GetValueOrEmpty(report.TRT475) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432704
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432704()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432704
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT467] <> NULL AND [TRT467] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT467] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.CostOfSales.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT467, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT467 = RP.JAUS:Expense.Operating.CostOfSales.Amount
            */
            assertion = NotMonetary(report.TRT467, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.CostOfSales.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432704"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT467", Value = GetValueOrEmpty(report.TRT467) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432705
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432705()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432705
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT468] <> NULL AND [TRT468] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT468] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.BadDebts.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT468, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT468 = RP.JAUS:Expense.Operating.BadDebts.Amount
            */
            assertion = NotMonetary(report.TRT468, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.BadDebts.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432705"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT468", Value = GetValueOrEmpty(report.TRT468) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432708
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432708()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432708
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT471] <> NULL AND [TRT471] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT471] = TRT:RP.JAUS:bafpr1.02.27:Expense.Royalties.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT471, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT471 = RP.JAUS:Expense.Royalties.Amount
            */
            assertion = NotMonetary(report.TRT471, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Royalties.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432708"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT471", Value = GetValueOrEmpty(report.TRT471) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432709
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432709()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432709
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT472] <> NULL AND [TRT472] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT472] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.MotorVehicle.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT472, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT472 = RP.JAUS:Expense.Operating.MotorVehicle.Amount
            */
            assertion = NotMonetary(report.TRT472, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.MotorVehicle.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432709"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT472", Value = GetValueOrEmpty(report.TRT472) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432710
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432710()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432710
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT473] <> NULL AND [TRT473] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT473] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.RepairsAndMaintenance.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT473, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT473 = RP.JAUS:Expense.Operating.RepairsAndMaintenance.Amount
            */
            assertion = NotMonetary(report.TRT473, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.RepairsAndMaintenance.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432710"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT473", Value = GetValueOrEmpty(report.TRT473) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432711
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432711()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432711
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT474] <> NULL AND [TRT474] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT474] = TRT:RP.JAUS:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT474, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT474 = RP.JAUS:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = NotMonetary(report.TRT474, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432711"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT474", Value = GetValueOrEmpty(report.TRT474) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432712
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432712()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432712
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT469] <> NULL AND [TRT469] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT469] = TRT:RP.JAUS:bafpr2.02.16:Expense.Operating.Rent.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT469, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT469 = RP.JAUS:Expense.Operating.Rent.Amount
            */
            assertion = NotMonetary(report.TRT469, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.Rent.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432712"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT469", Value = GetValueOrEmpty(report.TRT469) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432713
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432713()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432713
            If the amount in tax offset is greater than zero then tax offsets code must be present
    
            Legacy Rule Format:
            IF [TRT463] > 0 AND [TRT464] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT463] = TRT:RP:rvctc1.02.17:IncomeTax.TaxOffsetRefundableOther.Amount
            [TRT464] = TRT:RP:rvctc1.02.18:IncomeTax.TaxOffsetRefundableOther.Code

            Technical Business Rule Format:
            ^TRT463 > 0 AND ^TRT464 = NULL
    
            Data Elements:
    
            ^TRT463 = RP:IncomeTax.TaxOffsetRefundableOther.Amount
    
            ^TRT464 = RP:IncomeTax.TaxOffsetRefundableOther.Code
            */
            assertion = (report.TRT463.GetValueOrDefault() > 0 && report.TRT464 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432638", Severity = ProcessMessageSeverity.Error,
                    Description = @"Code not present for tax offset amount greater than zero",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetRefundableOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432713"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT463", Value = GetValueOrEmpty(report.TRT463) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT464", Value = GetValueOrEmpty(report.TRT464) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432714
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432714()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432714
            The Share of Other Refundable Tax Offsets plus the sum of Exploration Credits is not equal to Other Refundable Tax Offset
    
            Legacy Rule Format:
            IF SUM([TRT465]) + [TRT466] <> [TRT463] 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT463] = TRT:RP:rvctc1.02.17:IncomeTax.TaxOffsetRefundableOther.Amount
            [TRT465] = TRT:RP:rvctc1.02.32:IncomeTax.ExplorationCreditsDistributed.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT466] = TRT:RP:rvctc1.02.17:IncomeTax.TaxOffsetRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            Sum(^TRT465) + ^TRT466 <> ^TRT463
    
            Data Elements:
    
            ^TRT463 = RP:IncomeTax.TaxOffsetRefundableOther.Amount
    
            ^TRT465 = RP:StatementOfDistributionToBeneficiary:IncomeTax.ExplorationCreditsDistributed.Amount
    
            ^TRT466 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.TaxOffsetRefundableOther.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT465.GetValueOrDefault())) + report.TRT466.GetValueOrDefault() != report.TRT463.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432639", Severity = ProcessMessageSeverity.Error,
                    Description = @"The Share of Other Refundable Tax Offsets plus the sum of Exploration Credits is not equal to Other Refundable Tax Offset",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetRefundableOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432714"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT465", Value = "TRT465" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT466", Value = GetValueOrEmpty(report.TRT466) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT463", Value = GetValueOrEmpty(report.TRT463) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432715
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432715()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432715
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT470] <> NULL AND [TRT470] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT470] = TRT:RP.JAUS:bafpr2.02.17:Expense.Operating.Interest.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT470, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT470 = RP.JAUS:Expense.Operating.Interest.Amount
            */
            assertion = NotMonetary(report.TRT470, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.Operating.Interest.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432715"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT470", Value = GetValueOrEmpty(report.TRT470) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432716
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432716(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432716
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT476] <> NULL AND [TRT476] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT476] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT476, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT476 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT476, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432716"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT476", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT476) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432717
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432717()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432717
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF rvctc1.xx.xx:IncomeTax.TaxOffsetRefundableOther.Amount <> NULL 
            AND rvctc1.xx.xx:IncomeTax.TaxOffsetRefundableOther.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT463, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT463 = RP:IncomeTax.TaxOffsetRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT463, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetRefundableOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432717"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT463", Value = GetValueOrEmpty(report.TRT463) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432717
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432717_1()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432717
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF rvctc1.xx.xx:IncomeTax.TaxOffsetRefundableOther.Amount <> NULL 
            AND rvctc1.xx.xx:IncomeTax.TaxOffsetRefundableOther.Amount <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT466, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT466 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.TaxOffsetRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT466, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.TaxOffsetRefundableOther.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432717"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT466", Value = GetValueOrEmpty(report.TRT466) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432718
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432718()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432718
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT478] <> NULL AND [TRT478] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT478] = TRT:RP.JAUS:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT478, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT478 = RP.JAUS:Income.SmallBusinessEntityNet.Amount
            */
            assertion = NotMonetary(report.TRT478, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.SmallBusinessEntityNet.Amount[@contextRef='" + report.RPJAUSId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432718"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT478", Value = GetValueOrEmpty(report.TRT478) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432719
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432719(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432719
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT479] <> NULL AND [TRT479] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT479] = TRT:RP:bafpr1.02.04:Income.SmallBusinessEntityNet.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT479, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT479 = RP:StatementOfDistributionToBeneficiary:Income.SmallBusinessEntityNet.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT479, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.SmallBusinessEntityNet.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432719"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT479", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT479) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432721
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432721()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432721
            Other refundable tax offsets code box contains an invalid value. The valid value is Multiple Tax Offsets or Exploration credit refundable tax offset
    
            Legacy Rule Format:
            IF [TRT463] > 0 AND [TRT464] <> SET("MultipleTaxOffsets", "ExplorationCreditRefundableTaxOffset") 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT463] = TRT:RP:rvctc1.02.17:IncomeTax.TaxOffsetRefundableOther.Amount
            [TRT464] = TRT:RP:rvctc1.02.18:IncomeTax.TaxOffsetRefundableOther.Code

            Technical Business Rule Format:
            ^TRT463 > 0 AND NotInSet(^TRT464, '"MultipleTaxOffsets", "ExplorationCreditRefundableTaxOffset"')
    
            Data Elements:
    
            ^TRT464 = RP:IncomeTax.TaxOffsetRefundableOther.Code
    
            ^TRT463 = RP:IncomeTax.TaxOffsetRefundableOther.Amount
            */
            assertion = (report.TRT463.GetValueOrDefault() > 0 && !(IsMatch(report.TRT464, @"^(MultipleTaxOffsets|ExplorationCreditRefundableTaxOffset)$",RegexOptions.IgnoreCase)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432724", Severity = ProcessMessageSeverity.Error,
                    Description = @"Other refundable tax offsets code must be 'MultipleTaxOffsets' or 'ExplorationCreditRefundableTaxOffset'",
                    LongDescription = @"Where a value has been entered in 'Other refundable tax offsets' then the 'Other refundable tax offsets code' response must be either 'MultipleTaxOffsets' or 'ExplorationCreditRefundableTaxOffset'",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetRefundableOther.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432721"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT463", Value = GetValueOrEmpty(report.TRT463) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT464", Value = GetValueOrEmpty(report.TRT464) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432730
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432730()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432730
            Exploration credit refundable tax offset cannot be claimed by non-resident entities
    
            Legacy Rule Format:
            IF [TRT245] = "NonResidentNoPermEstab" AND [TRT464] = "ExplorationCreditRefundableTaxOffset"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [TRT464] = TRT:RP:rvctc1.02.18:IncomeTax.TaxOffsetRefundableOther.Code

            Technical Business Rule Format:
            ^TRT245 = 'NonResidentNoPermEstab' AND ^TRT464 = 'ExplorationCreditRefundableTaxOffset'
    
            Data Elements:
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
    
            ^TRT464 = RP:IncomeTax.TaxOffsetRefundableOther.Code
            */
            assertion = (report.TRT245 == @"NonResidentNoPermEstab" && report.TRT464 == @"ExplorationCreditRefundableTaxOffset");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432730", Severity = ProcessMessageSeverity.Error,
                    Description = @"Exploration credit tax offset cannot be claimed",
                    LongDescription = @"Exploration credit refundable tax offset cannot be claimed by non-resident entities",
                    Location = "/xbrli:xbrl/tns:Residency.TaxPurposesOrganisationStatus.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432730"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT464", Value = GetValueOrEmpty(report.TRT464) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432731
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432731()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432731
            Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged no later than the 15th of June of the current year, unless it is an amendment. After the 15th of June, a return for that lodgment year must be submitted from the 1st of July, using the new service associated to that specific lodgment year.
    
            Legacy Rule Format:
            IF ([TRT1] <> SET(2023, 2024)) OR ([TRT1] = 2024 AND DATE(Today) > 2024-06-15 AND [TRT414] <> TRUE)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT414] = TRT:RP:pyin.02.28:Report.Amendment.Indicator

            Technical Business Rule Format:
            (NotInSet(^TRT1, '"2023", "2024"')) OR (^TRT1 = 2024 AND Today() > AsDate('2024-06-15') AND ^TRT414 <> TRUE)
    
            Data Elements:
    
            ^TRT1 = RP:Report.TargetFinancial.Year
    
            ^TRT414 = RP:Report.Amendment.Indicator
            */
            assertion = (!(IsMatch(report.TRT1.GetValueOrDefault().ToString(), @"^(2023|2024)$",RegexOptions.IgnoreCase)) || report.TRT1.GetValueOrDefault() == 2024 && DateTime.Now.Date > AsDate(@"2024-06-15") && report.TRT414 != true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.500127", Severity = ProcessMessageSeverity.Error,
                    Description = @"Year of return is invalid",
                    LongDescription = @"Year of return must be either the current or next (future) lodgment year associated with this service. If a future year lodgment is required, then it must be lodged no later than the 15th of June of the current year, unless it is an amendment. After the 15th of June, a return for that lodgment year must be submitted from the 1st of July, using the new service associated to that specific lodgment year.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432731"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT414", Value = GetValueOrEmpty(report.TRT414) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432739
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432739(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432739
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT481] <> NULL AND [TRT481] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT481] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT481, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT481 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT481, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432739"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT481", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT481) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432742
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432742()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432742
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT482] <> NULL AND [TRT482] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT482] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT482, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT482 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
            */
            assertion = NotMonetary(report.TRT482, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432742"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT482", Value = GetValueOrEmpty(report.TRT482) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432747
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432747()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432747
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT480] <> NULL AND [TRT480] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT480] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT480, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT480 = RP:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
            */
            assertion = NotMonetary(report.TRT480, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432747"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT480", Value = GetValueOrEmpty(report.TRT480) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432765
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432765()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432765
            The total amount at Credit for amounts withheld from foreign resident capital gains withholding (beneficiary and non beneficiary) must equal the amount at Credit for amounts withheld from foreign resident capital gains withholding.
    
            Legacy Rule Format:
            IF [TRT480] <> (SUM([TRT481]) + [TRT482]) 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT480] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
            [TRT481] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT482] = TRT:RP:rvctc2.02.22:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT480 <> (Sum(^TRT481) + ^TRT482)
    
            Data Elements:
    
            ^TRT480 = RP:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
    
            ^TRT481 = RP:StatementOfDistributionToBeneficiary:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
    
            ^TRT482 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount
            */
            assertion = (report.TRT480.GetValueOrDefault() != (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT481.GetValueOrDefault())) + report.TRT482.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432765", Severity = ProcessMessageSeverity.Error,
                    Description = @"Credit for amounts withheld from foreign resident capital gains withholding amount must equal distributed amounts",
                    LongDescription = @"The sum of income of the Credit for amounts withheld from foreign resident capital gains withholding (beneficiary and non beneficiary) does not equal the income of the trust Credit for amounts withheld from foreign resident capital gains withholding.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditForCapitalGainsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432765"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT480", Value = GetValueOrEmpty(report.TRT480) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT481", Value = "TRT481" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT482", Value = GetValueOrEmpty(report.TRT482) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432778
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432778()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432778
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT477] <> NULL AND [TRT477] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT477] = TRT:RP:rvctc2.02.13:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT477, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT477 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount
            */
            assertion = NotMonetary(report.TRT477, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:IncomeTax.PayAsYouGoWithholding.CreditForAmountsWithheldFromForeignResidents.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432778"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT477", Value = GetValueOrEmpty(report.TRT477) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432783
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432783()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432783
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT500] <> NULL AND [TRT500] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT500] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT500, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT500 = RP.LimitedPartners:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT500, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPLimitedPartnersId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432783"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT500", Value = GetValueOrEmpty(report.TRT500) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432784
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432784()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432784
            The total amount at Early stage venture capital limited partnership tax offset (beneficiary and no beneficiary presently entitled) must equal the amount at Early stage venture capital limited partnership tax offset.
    
            Legacy Rule Format:
            IF [TRT500] <> (SUM([TRT502]) + [TRT504])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT500] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount
            [TRT502] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT504] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT500 <> (Sum(^TRT502) + ^TRT504)
    
            Data Elements:
    
            ^TRT500 = RP.LimitedPartners:TaxOffsetClaim.NonRefundableOther.Amount
    
            ^TRT502 = RP.LimitedPartners:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NonRefundableOther.Amount
    
            ^TRT504 = RP.LimitedPartners:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = (report.TRT500.GetValueOrDefault() != (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT502.GetValueOrDefault())) + report.TRT504.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432784", Severity = ProcessMessageSeverity.Error,
                    Description = @"Early stage venture capital limited partnership tax offset amount must equal distributed amounts.",
                    LongDescription = @"The total amount at Early stage venture capital limited partnership tax offset (beneficiary and no beneficiary presently entitled) must equal the amount at Early stage venture capital limited partnership tax offset.",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPLimitedPartnersId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432784"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT500", Value = GetValueOrEmpty(report.TRT500) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT502", Value = "TRT502" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT504", Value = GetValueOrEmpty(report.TRT504) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432785
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432785()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432785
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT501] <> NULL AND [TRT501] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT501] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT501, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT501 = RP.Investor:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT501, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432785"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT501", Value = GetValueOrEmpty(report.TRT501) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432786
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432786()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432786
            The sum of Early stage investor tax offset and Early stage investor tax offset carried forward from previous year- No beneficiary presently entitled cannot exceed $200,000.
    
            Legacy Rule Format:
            IF ([TRT501] <> NULL OR [TRT507] <> NULL) AND ([TRT501] + [TRT507] > 200000)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT501] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount
            [TRT507] = TRT:RP.Investor.YPrevious:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (^TRT501 <> NULL OR ^TRT507 <> NULL) AND (^TRT501 + ^TRT507 > 200000)
    
            Data Elements:
    
            ^TRT501 = RP.Investor:TaxOffsetClaim.NonRefundableOther.Amount
    
            ^TRT507 = RP.Investor.YPrevious:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = ((report.TRT501 != null || report.TRT507 != null) && report.TRT501.GetValueOrDefault() + report.TRT507.GetValueOrDefault() > 200000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432786", Severity = ProcessMessageSeverity.Error,
                    Description = @"The sum of Early stage investor tax offset and Early stage investor tax offset carried forward from previous year cannot exceed $200,000.",
                    LongDescription = @"This offset is capped at $200,000 per investor and their affiliates combined each income year. This cap applies to the sum of the current year entitlement and tax offset carried forward from the previous year.",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432786"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT501", Value = GetValueOrEmpty(report.TRT501) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT507", Value = GetValueOrEmpty(report.TRT507) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432787
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432787()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432787
            The total of amounts at Early stage investor tax offset (beneficiary and no beneficiary presently entitled) must equal the amount at Early stage investor tax offset.
    
            Legacy Rule Format:
            IF [TRT501] <> (SUM([TRT503]) + [TRT506])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT501] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount
            [TRT503] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT506] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT501 <> (Sum(^TRT503) + ^TRT506)
    
            Data Elements:
    
            ^TRT501 = RP.Investor:TaxOffsetClaim.NonRefundableOther.Amount
    
            ^TRT503 = RP.Investor:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NonRefundableOther.Amount
    
            ^TRT506 = RP.Investor:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = (report.TRT501.GetValueOrDefault() != (report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT503.GetValueOrDefault())) + report.TRT506.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432787", Severity = ProcessMessageSeverity.Error,
                    Description = @"Early stage investor tax offset amount must equal distributed amounts.",
                    LongDescription = @"The total of amounts at Early stage investor tax offset (beneficiary and no beneficiary presently entitled) must equal the amount at Early stage investor tax offset.",
                    Location = "/xbrli:xbrl/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432787"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT501", Value = GetValueOrEmpty(report.TRT501) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT503", Value = "TRT503" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT506", Value = GetValueOrEmpty(report.TRT506) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432788
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432788(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432788
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT502] <> NULL AND [TRT502] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT502] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT502, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT502 = RP.LimitedPartners:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NonRefundableOther.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT502, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPLimitedPartnersId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432788"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT502", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT502) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432789
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432789(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432789
                    Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
                    Legacy Rule Format:
                    IF [TRT503] <> NULL AND [TRT503] <> MONETARY(U,13,2)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT503] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT503, 'U', 13, 2)
            
                    Data Elements:
            
                    ^TRT503 = RP.Investor:StatementOfDistributionToBeneficiary:TaxOffsetClaim.NonRefundableOther.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT503, @"U", 13, 2);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432789"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT503", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT503) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432790
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432790()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432790
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT504] <> NULL AND [TRT504] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT504] = TRT:RP.LimitedPartners:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT504, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT504 = RP.LimitedPartners:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT504, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPLimitedPartnersId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432790"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT504", Value = GetValueOrEmpty(report.TRT504) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432791
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432791()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432791
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT505] <> NULL AND [TRT505] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT505] = TRT:RP.LimitedPartners.YPrevious:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT505, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT505 = RP.LimitedPartners.YPrevious:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT505, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPLimitedPartnersYPreviousId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432791"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT505", Value = GetValueOrEmpty(report.TRT505) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432792
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432792()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432792
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT506] <> NULL AND [TRT506] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT506] = TRT:RP.Investor:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT506, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT506 = RP.Investor:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT506, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432792"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT506", Value = GetValueOrEmpty(report.TRT506) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432793
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432793()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432793
            Monetary field, must be unsigned, maximum of 13 chars, may contain a decimal point and up to 2 decimals
    
            Legacy Rule Format:
            IF [TRT507] <> NULL AND [TRT507] <> MONETARY(U,13,2)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT507] = TRT:RP.Investor.YPrevious:gfati.02.06:TaxOffsetClaim.NonRefundableOther.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT507, 'U', 13, 2)
    
            Data Elements:
    
            ^TRT507 = RP.Investor.YPrevious:IncomeNoBeneficiaryPresentlyEntitled:TaxOffsetClaim.NonRefundableOther.Amount
            */
            assertion = NotMonetary(report.TRT507, @"U", 13, 2);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:TaxOffsetClaim.NonRefundableOther.Amount[@contextRef='" + report.RPInvestorYPreviousId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432793"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT507", Value = GetValueOrEmpty(report.TRT507) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432794
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432794(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432794
                    Current Business Address required for non-individual beneficiary.
    
                    Legacy Rule Format:
                    WHERE IN TUPLE (trt.0011.lodge.req.xx.xx:StatementOfDistributionToBeneficiary)
                       IF COUNT(TUPLE(orgname2.xx.xx:OrganisationNameDetails)) = 1 AND (COUNT(TUPLE(address2.xx.xx:AddressDetails WHERE TUPLE EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "BUS") AND (pyde.xx.xx:AddressDetails.Currency.Code = "C"))) <> 1
                       RETURN VALIDATION MESSAGE
                    ENDIF
        
                    Technical Business Rule Format:
                    ^StatementOfDistributionToBeneficiary <> NULL AND Count(^OrganisationNameDetails) = 1 AND Count(^AddressDetails) <> 1
            
                    Data Elements:
            
                    ^AddressDetails = RP:StatementOfDistributionToBeneficiary:AddressDetails
            
                    ^OrganisationNameDetails = RP:StatementOfDistributionToBeneficiary:OrganisationNameDetails
            
                    ^StatementOfDistributionToBeneficiary = RP:StatementOfDistributionToBeneficiary
                    */
                    assertion = (report.StatementOfDistributionToBeneficiaryCollectionExists != false && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_OrganisationNameDetailsCollection_MN_C_RPCount) == 1 && Count(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPCount) != 1);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.TRT.432794", Severity = ProcessMessageSeverity.Error,
                            Description = @"Current Business Address required for non-individual beneficiary.",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:AddressDetails" + OccurrenceIndex(statementOfDistributionToBeneficiary.StatementOfDistributionToBeneficiary_AddressDetailsCollection_BUS_C_RPLastOccurrenceIndex) + "/tns:AddressDetails.Usage.Code",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432794"} },
                        };
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432800
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432800()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432800
            Mandatory field not supplied
    
            Legacy Rule Format:
            IF [TRT520] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            [TRT520] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrust.Indicator

            Technical Business Rule Format:
            ^TRT520 = NULL
    
            Data Elements:
    
            ^TRT520 = RP:OrganisationDetails.ManagedInvestmentTrust.Indicator
            */
            assertion = (report.TRT520 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.001001", Severity = ProcessMessageSeverity.Error,
                    Description = @"Mandatory field not supplied",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.ManagedInvestmentTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432800"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT520", Value = GetValueOrEmpty(report.TRT520) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432801
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432801()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432801
            If the trust is not a Managed Investment Trust (MIT), then MIT type code and election into capital account treatment must be left blank.
    
            Legacy Rule Format:
            IF ([TRT520] = FALSE) AND ([TRT521] <> NULL OR [TRT352] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            
            [TRT352] = TRT:RP:rvctc3.02.24:Elections.ManagedInvestmentTrusts.Indicator
            [TRT520] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrust.Indicator
            [TRT521] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrustType.Code

            Technical Business Rule Format:
            (^TRT520 = FALSE) AND (^TRT521 <> NULL OR ^TRT352 <> NULL)
    
            Data Elements:
    
            ^TRT520 = RP:OrganisationDetails.ManagedInvestmentTrust.Indicator
    
            ^TRT352 = RP:Elections.ManagedInvestmentTrusts.Indicator
    
            ^TRT521 = RP:OrganisationDetails.ManagedInvestmentTrustType.Code
            */
            assertion = (report.TRT520 == false && (report.TRT521 != null || report.TRT352 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432801", Severity = ProcessMessageSeverity.Error,
                    Description = @"Managed investment trust information not required.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.ManagedInvestmentTrust.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432801"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT520", Value = GetValueOrEmpty(report.TRT520) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT521", Value = GetValueOrEmpty(report.TRT521) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT352", Value = GetValueOrEmpty(report.TRT352) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432802
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432802()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432802
            If the trust is a Managed Investment Trust (MIT) and the type of trust is either Cash Management Unit Trust (code 039), or Public Unit Trust (listed) - other than a cash management unit trust (code 179), or Public Unit Trust (unlisted) - other than a cash management unit trust (code 184), or Fixed Unit Trust - other than a public unit trust described in '179' or '184' (code 089), or Fixed Trust - other than a fixed unit trust or public unit trust described in '089', '179', or '184' (code 090), then MIT type code must be provided.
    
            Legacy Rule Format:
            IF [TRT521] = NULL AND ([TRT520] = TRUE AND [TRT32] = SET("039", "179", "184", "089", "090"))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT520] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrust.Indicator
            [TRT521] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrustType.Code

            Technical Business Rule Format:
            ^TRT521 = NULL AND (^TRT520 = TRUE AND InSet(^TRT32, '"039", "179", "184", "089", "090"'))
    
            Data Elements:
    
            ^TRT521 = RP:OrganisationDetails.ManagedInvestmentTrustType.Code
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT520 = RP:OrganisationDetails.ManagedInvestmentTrust.Indicator
            */
            assertion = (report.TRT521 == null && (report.TRT520 == true && IsMatch(report.TRT32, @"^(039|179|184|089|090)$")));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432802", Severity = ProcessMessageSeverity.Error,
                    Description = @"Managed investment trust type code required.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.ManagedInvestmentTrustType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432802"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT521", Value = GetValueOrEmpty(report.TRT521) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT520", Value = GetValueOrEmpty(report.TRT520) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432805
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432805()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432805
            If the trust is a managed investment trust (MIT), has the trustee made an election into capital account treatment?' must be completed if the trust is a Managed Investment Trust (MIT).
    
            Legacy Rule Format:
            IF [TRT352] = NULL  AND [TRT520] = TRUE
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT352] = TRT:RP:rvctc3.02.24:Elections.ManagedInvestmentTrusts.Indicator
            [TRT520] = TRT:RP:pyde.02.45:OrganisationDetails.ManagedInvestmentTrust.Indicator

            Technical Business Rule Format:
            ^TRT352 = NULL  AND ^TRT520 = TRUE
    
            Data Elements:
    
            ^TRT352 = RP:Elections.ManagedInvestmentTrusts.Indicator
    
            ^TRT520 = RP:OrganisationDetails.ManagedInvestmentTrust.Indicator
            */
            assertion = (report.TRT352 == null && report.TRT520 == true);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432805", Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital account treatment election required.",
                    Location = "/xbrli:xbrl/tns:Elections.ManagedInvestmentTrusts.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432805"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT352", Value = GetValueOrEmpty(report.TRT352) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT520", Value = GetValueOrEmpty(report.TRT520) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432807
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432807(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432807
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT537] <> NULL AND [TRT537] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT537] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT537, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT537 = RP.Prim:StatementOfDistributionToBeneficiary:Income.NonConcessionalManagedInvestmentTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT537, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432807"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT537", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT537) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432808
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432808(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432808
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT538] <> NULL AND [TRT538] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT538] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT538, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT538 = RP.Prim:StatementOfDistributionToBeneficiary:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT538, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432808"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT538", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT538) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432809
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432809(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432809
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT539] <> NULL AND [TRT539] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT539] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT539, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT539 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.NonConcessionalManagedInvestmentTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT539, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432809"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT539", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT539) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432810
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432810(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432810
                    Monetary field, may be signed, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT540] <> NULL AND [TRT540] <> MONETARY(S,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT540] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT540, 'S', 11, 0)
            
                    Data Elements:
            
                    ^TRT540 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT540, @"S", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432810"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT540", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT540) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432811
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432811(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432811
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT541] <> NULL AND [TRT541] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT541] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT541, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT541 = RP:StatementOfDistributionToBeneficiary:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT541, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432811"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT541", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT541) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432812
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432812(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432812
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT542] <> NULL AND [TRT542] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT542] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT542, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT542 = RP:StatementOfDistributionToBeneficiary:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT542, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432812"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT542", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT542) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432813
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432813()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432813
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT543] <> NULL AND [TRT543] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT543] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT543, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT543 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT543, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432813"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT543", Value = GetValueOrEmpty(report.TRT543) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432814
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432814()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432814
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT544] <> NULL AND [TRT544] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT544] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT544, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT544 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT544, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432814"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT544", Value = GetValueOrEmpty(report.TRT544) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432815
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432815()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432815
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT545] <> NULL AND [TRT545] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT545] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT545, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT545 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT545, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432815"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT545", Value = GetValueOrEmpty(report.TRT545) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432816
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432816()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432816
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT546] <> NULL AND [TRT546] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT546] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT546, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT546 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT546, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432816"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT546", Value = GetValueOrEmpty(report.TRT546) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432817
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432817()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432817
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT547] <> NULL AND [TRT547] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT547] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT547, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT547 = RP:IncomeNoBeneficiaryPresentlyEntitled:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = NotMonetary(report.TRT547, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432817"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT547", Value = GetValueOrEmpty(report.TRT547) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432818
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432818()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432818
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT548] <> NULL AND [TRT548] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT548] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            NotMonetary(^TRT548, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT548 = RP:IncomeNoBeneficiaryPresentlyEntitled:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = NotMonetary(report.TRT548, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeNoBeneficiaryPresentlyEntitled/tns:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432818"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT548", Value = GetValueOrEmpty(report.TRT548) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432819
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432819()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432819
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT536] <> NULL AND [TRT536] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT536] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT536, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT536 = RP:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = NotMonetary(report.TRT536, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432819"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT536", Value = GetValueOrEmpty(report.TRT536) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432820
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432820()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432820
            Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT535] <> NULL AND [TRT535] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT535] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT535, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT535 = RP:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = NotMonetary(report.TRT535, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432820"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT535", Value = GetValueOrEmpty(report.TRT535) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432821
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432821()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432821
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT525] <> NULL AND [TRT525] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT525] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT525, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT525 = RP.JAUS.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT525, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432821"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT525", Value = GetValueOrEmpty(report.TRT525) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432822
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432822()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432822
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT526] <> NULL AND [TRT526] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT526] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT526, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT526 = RP.JAUS.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT526, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432822"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT526", Value = GetValueOrEmpty(report.TRT526) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432823
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432823()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432823
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT523] <> NULL AND [TRT523] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT523] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT523, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT523 = RP.JAUS.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT523, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432823"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT523", Value = GetValueOrEmpty(report.TRT523) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432824
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432824()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432824
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT524] <> NULL AND [TRT524] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT524] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT524, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT524 = RP.JAUS.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT524, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432824"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT524", Value = GetValueOrEmpty(report.TRT524) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432825
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432825()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432825
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT531] <> NULL AND [TRT531] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT531] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT531, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT531 = RP.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT531, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432825"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT531", Value = GetValueOrEmpty(report.TRT531) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432826
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432826()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432826
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT532] <> NULL AND [TRT532] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT532] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT532, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT532 = RP.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT532, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432826"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT532", Value = GetValueOrEmpty(report.TRT532) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432827
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432827()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432827
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT527] <> NULL AND [TRT527] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT527] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT527, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT527 = RP.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT527, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432827"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT527", Value = GetValueOrEmpty(report.TRT527) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432828
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432828()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432828
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT528] <> NULL AND [TRT528] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT528] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT528, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT528 = RP.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT528, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432828"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT528", Value = GetValueOrEmpty(report.TRT528) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432829
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432829()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432829
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT533] <> NULL AND [TRT533] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT533] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT533, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT533 = RP.Share.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT533, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPShareNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432829"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT533", Value = GetValueOrEmpty(report.TRT533) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432830
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432830()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432830
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT534] <> NULL AND [TRT534] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT534] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT534, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT534 = RP.Share.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT534, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPShareNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432830"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT534", Value = GetValueOrEmpty(report.TRT534) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432831
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432831()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432831
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT529] <> NULL AND [TRT529] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT529] = TRT:RP.Share.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT529, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT529 = RP.Share.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT529, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPSharePrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432831"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT529", Value = GetValueOrEmpty(report.TRT529) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432832
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432832()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432832
            Monetary field, may be signed, maximum of 11 chars, no decimals
    
            Legacy Rule Format:
            IF [TRT530] <> NULL AND [TRT530] <> MONETARY(S,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT530] = TRT:RP.Share.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT530, 'S', 11, 0)
    
            Data Elements:
    
            ^TRT530 = RP.Share.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = NotMonetary(report.TRT530, @"S", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPSharePrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432832"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT530", Value = GetValueOrEmpty(report.TRT530) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432833
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432833()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432833
            Invalid context. The number of Reporting party: Share: Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Share.Prim)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Share.Prim
            */
            assertion = (Count(report.RPSharePrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432833", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Share: Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPSharePrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432833"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432834
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432834()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432834
            Invalid context. The number of Reporting party: Share: Non-Primary Production contexts must not exceed 1
    
            Legacy Rule Format:
            IF COUNT(CONTEXT(RP.Share.NonPrim)) > 1 
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            Count(^Context) > 1
    
            Data Elements:
    
            ^Context = RP:RP.Share.NonPrim
            */
            assertion = (Count(report.RPShareNonPrimCount) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432834", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid context. The number of Reporting party: Share: Non-Primary Production contexts must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context" + OccurrenceIndex(report.RPShareNonPrimLastOccurrenceIndex),
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432834"} },
                };
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432863
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432863()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432863
            If the type of trust is '232' (testamentary trust), then the assessment calculation code for each Statement of Distribution must be 25, 26, 27, 28, 29, 30, 34, 35, 125, 126, 127, 128, 129, 134, 138, 139 or 140 and the assessment calculation code in the distribution statement where no beneficiary is entitled, if present, must be 36 or 37.
    
            Legacy Rule Format:
            IF [TRT32] = "232" AND (ANY OCCURRENCE OF [TRT214] <> SET("25", "26", "27", "28", "29", "30", "34", "35", "125", "126", "127", "128", "129", "134", "138", "139", "140") OR ([TRT230] <> NULL AND [TRT230] <> SET("36", "37")))
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT214] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT230] = TRT:RP:rvctc1.02.44:IncomeTax.AssessmentCalculation.Code IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            ^TRT32 = '232' AND AnyOccurrence(^TRT214, NotInSet(^TRT214, '"25", "26", "27", "28", "29", "30", "34", "35", "125", "126", "127", "128", "129", "134", "138", "139", "140"') OR (^TRT230 <> NULL AND NotInSet(^TRT230, '"36", "37"')))
    
            Data Elements:
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
    
            ^TRT214 = RP:StatementOfDistributionToBeneficiary:IncomeTax.AssessmentCalculation.Code
    
            ^TRT230 = RP:IncomeNoBeneficiaryPresentlyEntitled:IncomeTax.AssessmentCalculation.Code
            */
            assertion = (report.TRT32 == @"232" && (report.StatementOfDistributionToBeneficiaryCollection == null ? false : report.StatementOfDistributionToBeneficiaryCollection.Any(TRT214Repeat => !(IsMatch(TRT214Repeat.TRT214, @"^(25|26|27|28|29|30|34|35|125|126|127|128|129|134|138|139|140)$",RegexOptions.IgnoreCase)) || report.TRT230 != null && !(IsMatch(report.TRT230, @"^(36|37)$",RegexOptions.IgnoreCase)))));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432863", Severity = ProcessMessageSeverity.Error,
                    Description = @"Assessment calculation code(s) must be a testamentary trust code.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.OrganisationType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432863"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT214", Value = "Count(TRT214)" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT230", Value = "TRT230" });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432864
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432864(TRT2023.StatementOfDistributionToBeneficiary statementOfDistributionToBeneficiary, int itemIndex1)
      {
        ProcessMessageDocument processMessage;
        bool assertion;
            
                    /*  VR.ATO.TRT.432864
                    Monetary field, must be unsigned, maximum of 11 chars, no decimals
    
                    Legacy Rule Format:
                    IF [TRT549] <> NULL AND [TRT549] <> MONETARY(U,11,0)
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [TRT549] = TRT:RP:bafpr1.02.41:Income.TaxableMinorBeneficiary.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
        
                    Technical Business Rule Format:
                    NotMonetary(^TRT549, 'U', 11, 0)
            
                    Data Elements:
            
                    ^TRT549 = RP:StatementOfDistributionToBeneficiary:Income.TaxableMinorBeneficiary.Amount
                    */
                    assertion = NotMonetary(statementOfDistributionToBeneficiary.TRT549, @"U", 11, 0);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                            Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                            LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                            Location = "/xbrli:xbrl/tns:StatementOfDistributionToBeneficiary" + OccurrenceIndex(statementOfDistributionToBeneficiary.OccurrenceIndex) + "/tns:Income.TaxableMinorBeneficiary.Amount",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432864"} },
                        };
                        processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "TRT549", Value = GetValueOrEmpty(statementOfDistributionToBeneficiary.TRT549) });
            
                        response.Add(processMessage);
                    }
              }
              #endregion 
    
      #region VR.ATO.TRT.432871
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432871()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432871
            Number of assets you are opting out for must be unsigned, maximum of 5 numeric characters, no decimals and must not be 0
    
            Legacy Rule Format:
            IF LENGTH([TRT552]) > 5 OR [TRT552] <= 0 AND [TRT552] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT552] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number

            Technical Business Rule Format:
            Length(^TRT552) > 5 OR ^TRT552 <= 0 AND ^TRT552 <> NULL
    
            Data Elements:
    
            ^TRT552 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            */
            assertion = (Length(report.TRT552.GetValueOrDefault()) > 5 || report.TRT552.GetValueOrDefault() <= 0 && report.TRT552 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432871"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT552", Value = report.TRT552.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432872
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432872()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432872
            When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets is "Some eligible assets" or "All eligible assets", Number of assets you are opting out for must be provided
    
            Legacy Rule Format:
            IF(InSet([TRT551], '"Some eligible assets", "All eligible assets"') AND [TRT552] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT552] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            [TRT551] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code

            Technical Business Rule Format:
            (InSet(^TRT551, '"Some eligible assets", "All eligible assets"') AND ^TRT552 = NULL)
    
            Data Elements:
    
            ^TRT552 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            */
            assertion = (IsMatch(report.TRT551, @"^(Some eligible assets|All eligible assets)$") && report.TRT552 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432907", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are opting out for must be provided",
                    LongDescription = @"When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets is ""Some eligible assets"" or ""All eligible assets"", Number of assets you are opting out for must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432872"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT552", Value = report.TRT552.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432873
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432873()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432873
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals and must not be 0
    
            Legacy Rule Format:
            IF [TRT553] <> MONETARY(U,11,0) OR [TRT553] = 0 AND [TRT553] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT553] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT553, 'U', 11, 0) OR ^TRT553 = 0 AND ^TRT553 <> NULL
    
            Data Elements:
    
            ^TRT553 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            */
            assertion = (NotMonetary(report.TRT553, @"U", 11, 0) || report.TRT553 == 0 && report.TRT553 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432873"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT553", Value = GetValueOrEmpty(report.TRT553) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432874
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432874()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432874
            When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets is "Some eligible assets" or "All eligible assets", Value of assets you are opting out for must be provided
    
            Legacy Rule Format:
            IF(InSet([TRT551], '"Some eligible assets", "All eligible assets"') AND [TRT553] = NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT553] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            [TRT551] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code

            Technical Business Rule Format:
            (InSet(^TRT551, '"Some eligible assets", "All eligible assets"') AND ^TRT553 = NULL)
    
            Data Elements:
    
            ^TRT553 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            */
            assertion = (IsMatch(report.TRT551, @"^(Some eligible assets|All eligible assets)$") && report.TRT553 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432908", Severity = ProcessMessageSeverity.Error,
                    Description = @"Value of assets you are opting out for must be provided",
                    LongDescription = @"When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets is ""Some eligible assets"" or ""All eligible assets"", Value of assets you are opting out for must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432874"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT553", Value = GetValueOrEmpty(report.TRT553) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432875
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432875()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432875
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals and must not be 0
    
            Legacy Rule Format:
            IF [TRT554] <> MONETARY(U,11,0) OR [TRT554] = 0 AND [TRT554] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT554] =  TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT554, 'U', 11, 0) OR ^TRT554 = 0 AND ^TRT554 <> NULL
    
            Data Elements:
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (NotMonetary(report.TRT554, @"U", 11, 0) || report.TRT554 == 0 && report.TRT554 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432875"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432876
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432876()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432876
            When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets? is "Some eligible assets", Temporary full expensing deductions must be provided
    
            Legacy Rule Format:
            IF [TRT551] = "Some eligible assets" AND [TRT554] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRT551] =  TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code

            Technical Business Rule Format:
            ^TRT551 = 'Some eligible assets' AND ^TRT554 = NULL
    
            Data Elements:
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            */
            assertion = (report.TRT551 == @"Some eligible assets" && report.TRT554 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432876", Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must be provided",
                    LongDescription = @"When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets? is ""Some eligible assets"", Temporary full expensing deductions must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432876"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432877
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432877()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432877
            Number of assets you are claiming for? must be unsigned, maximum of 5 numeric characters, no decimals and must not be 0
    
            Legacy Rule Format:
            IF LENGTH([TRT555]) > 5 OR [TRT555] <= 0 AND [TRT555] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT555] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            Length(^TRT555) > 5 OR ^TRT555 <= 0 AND ^TRT555 <> NULL
    
            Data Elements:
    
            ^TRT555 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
            */
            assertion = (Length(report.TRT555.GetValueOrDefault()) > 5 || report.TRT555.GetValueOrDefault() <= 0 && report.TRT555 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432877"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT555", Value = report.TRT555.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432880
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432880()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432880
            When Temporary full expensing deductions has a value, Number of assets you are claiming for? must be provided
    
            Legacy Rule Format:
            IF [TRT554] <> NULL AND [TRT555] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT555] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            ^TRT554 <> NULL AND ^TRT555 = NULL
    
            Data Elements:
    
            ^TRT555 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRT554 != null && report.TRT555 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432880", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number assets you are claiming for? must be provided",
                    LongDescription = @"When Temporary full expensing deductions has a value, Number of assets you are claiming for? must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432880"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT555", Value = report.TRT555.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432887
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432887()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432887
            Monetary field must be unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [TRT561] <> NULL AND [TRT561] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT561] = TRT:RP:bafpr1.02.44:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount

            Technical Business Rule Format:
            NotMonetary(^TRT561, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT561 = RP:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount
            */
            assertion = NotMonetary(report.TRT561, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432887"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT561", Value = GetValueOrEmpty(report.TRT561) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432888
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432888()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432888
            Subsequent year accelerated depreciation deductions for assets using Backing Business Investment must be greater than 0
    
            Legacy Rule Format:
            IF [TRT561] <> NULL AND [TRT561] = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT561] =  TRT:RP:bafpr1.02.44:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount

            Technical Business Rule Format:
            ^TRT561 <> NULL AND ^TRT561 = 0
    
            Data Elements:
    
            ^TRT561 = RP:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount
            */
            assertion = (report.TRT561 != null && report.TRT561 == 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432888", Severity = ProcessMessageSeverity.Error,
                    Description = @"Subsequent year accelerated depreciation deductions for assets using Backing Business Investment must be greater than 0",
                    Location = "/xbrli:xbrl/tns:Expense.SubsequentYearAcceleratedDepreciationDeductionsForAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432888"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT561", Value = GetValueOrEmpty(report.TRT561) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432891
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432891()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432891
            When Temporary full expensing deductions or Deduction for certain assets (small business entity) are present, the Aggregated turnover range must be provided
    
            Legacy Rule Format:
            IF ([TRT563] = NULL) AND ([TRT554] <> NULL OR [TRT189] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT563] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRT189] = TRT:RP.bafpr3.02.11:Expense.DepreciatingAssets.SmallBusinessEntity.Amount

            Technical Business Rule Format:
            (^TRT563 = NULL) AND (^TRT554 <> NULL OR ^TRT189 <> NULL)
    
            Data Elements:
    
            ^TRT563 = RP:Income.AggregatedTurnoverRange.Code
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
    
            ^TRT189 = RP.LowCost:Expense.DepreciatingAssets.SmallBusinessEntity.Amount
            */
            assertion = (report.TRT563 == null && (report.TRT554 != null || report.TRT189 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432911", Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover range must be provided",
                    LongDescription = @"When Temporary full expensing deductions or Deduction for certain assets (small business entity) are present, the Aggregated turnover range must be provided",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnoverRange.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432891"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT563", Value = GetValueOrEmpty(report.TRT563) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT189", Value = GetValueOrEmpty(report.TRT189) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432893
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432893()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432893
            Monetary field must be unsigned, maximum of 12 numeric characters, no decimals and must not be 0
    
            Legacy Rule Format:
            IF ([TRT564] <> MONETARY(U,12,0) OR [TRT564] = 0) AND ([TRT564] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT564] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount

            Technical Business Rule Format:
            (NotMonetary(^TRT564, 'U', 12, 0) OR ^TRT564 = 0) AND (^TRT564 <> NULL)
    
            Data Elements:
    
            ^TRT564 = RP:Income.AggregatedTurnover.Amount
            */
            assertion = ((NotMonetary(report.TRT564, @"U", 12, 0) || report.TRT564 == 0) && report.TRT564 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432893"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT564", Value = GetValueOrEmpty(report.TRT564) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432894
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432894()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432894
            When P is selected at aggregated turnover range, Aggregated turnover must be provided
    
            Legacy Rule Format:
            IF [TRT564] = NULL AND [TRT563] = "P"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT563] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code
            [TRT564] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount

            Technical Business Rule Format:
            ^TRT564 = NULL AND ^TRT563 = 'P'
    
            Data Elements:
    
            ^TRT564 = RP:Income.AggregatedTurnover.Amount
    
            ^TRT563 = RP:Income.AggregatedTurnoverRange.Code
            */
            assertion = (report.TRT564 == null && report.TRT563 == @"P");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432912", Severity = ProcessMessageSeverity.Error,
                    Description = @"Aggregated turnover must be provided",
                    LongDescription = @"When P is selected at aggregated turnover range, Aggregated turnover must be provided",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432894"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT564", Value = GetValueOrEmpty(report.TRT564) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT563", Value = GetValueOrEmpty(report.TRT563) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432896
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432896()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432896
            If the company has self-assessed that they are a Country by Country Reporting Entity (CbCRE), then the Significant Global Entity (SGE) indicator must be true.
    
            Legacy Rule Format:
            IF [TRT550] = TRUE AND ([TRT487] = NULL OR [TRT487] = FALSE)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT487] = TRT:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            [TRT550] = TRT:RP:rvctc4.02.17:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator

            Technical Business Rule Format:
            ^TRT550 = TRUE AND (^TRT487 = NULL OR ^TRT487 = FALSE)
    
            Data Elements:
    
            ^TRT550 = RP:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator
    
            ^TRT487 = RP:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            */
            assertion = (report.TRT550 == true && (report.TRT487 == null || report.TRT487 == false));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432896", Severity = ProcessMessageSeverity.Error,
                    Description = @"A reporting party who indicates that they are a Country-by-country reporting entity must also indicate that they are an SGE",
                    Location = "/xbrli:xbrl/tns:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432896"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT550", Value = GetValueOrEmpty(report.TRT550) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT487", Value = GetValueOrEmpty(report.TRT487) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432897
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432897()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432897
            Primary production- Share of net income from trusts action code of "C" is not valid for Trust
    
            Legacy Rule Format:
            IF [TRT353] = "C"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT353] = TRT:RP.JAUS.Prim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT353 = 'C'
    
            Data Elements:
    
            ^TRT353 = RP.JAUS.Prim:Party.TrustType.Code
            */
            assertion = (report.TRT353 == @"C");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432897", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid Primary production- Share of net income from trusts action code.",
                    Location = "/xbrli:xbrl/tns:Party.TrustType.Code[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432897"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT353", Value = GetValueOrEmpty(report.TRT353) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432898
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432898()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432898
            Non-primary production- share of net income from trusts, less net capital gain, foreign income and franked distributions action code of "C" is not valid for Trust
    
            Legacy Rule Format:
            IF [TRT436] = "C"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT436] = TRT:RP.JAUS.NonPrim:pyde.02.05:Party.TrustType.Code

            Technical Business Rule Format:
            ^TRT436 = 'C'
    
            Data Elements:
    
            ^TRT436 = RP.JAUS.NonPrim:Party.TrustType.Code
            */
            assertion = (report.TRT436 == @"C");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432898", Severity = ProcessMessageSeverity.Error,
                    Description = @"Invalid Non-primary production- share of net income from trusts, less net capital gain, foreign income and franked distributions action code.",
                    Location = "/xbrli:xbrl/tns:Party.TrustType.Code[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432898"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT436", Value = GetValueOrEmpty(report.TRT436) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432899
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432899()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432899
            When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets? is "All eligible assets", Temporary full expensing deductions must not be provided
    
            Legacy Rule Format:
            IF [TRT551] = "All eligible assets" AND [TRT554] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRT551] =  TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code

            Technical Business Rule Format:
            ^TRT551 = 'All eligible assets' AND ^TRT554 <> NULL
    
            Data Elements:
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            */
            assertion = (report.TRT551 == @"All eligible assets" && report.TRT554 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432899", Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must not be provided",
                    LongDescription = @"When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets? is ""All eligible assets"", Temporary full expensing deductions must not be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432899"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432900
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432900()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432900
            When Number of assets you are claiming for is provided, then Temporary full expensing deductions must be provided
    
            Legacy Rule Format:
            IF [TRT554] = NULL AND [TRT555] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRT555] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            ^TRT554 = NULL AND ^TRT555 <> NULL
    
            Data Elements:
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
    
            ^TRT555 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
            */
            assertion = (report.TRT554 == null && report.TRT555 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432900", Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must be provided",
                    LongDescription = @"When Number of assets you are claiming for is provided, then Temporary full expensing deductions must be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432900"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT555", Value = report.TRT555.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432901
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432901()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432901
            When Aggregated turnover has a value of $5 billion or more, Temporary full expensing deductions must not be provided
    
            Legacy Rule Format:
            IF [TRT564] >= 5000000000 AND [TRT554] <> NULL 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT564] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount

            Technical Business Rule Format:
            ^TRT564 >= 5000000000 AND ^TRT554 <> NULL
    
            Data Elements:
    
            ^TRT564 = RP:Income.AggregatedTurnover.Amount
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            */
            assertion = (report.TRT564.GetValueOrDefault() >= 5000000000 && report.TRT554 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432901", Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary full expensing deductions must not be provided",
                    LongDescription = @"When Aggregated turnover has a value of $5 billion or more, Temporary full expensing deductions must not be provided",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432901"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT564", Value = GetValueOrEmpty(report.TRT564) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432906
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432906()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432906
            When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets does not have a value, then Number of assets you are opting out for and Value of assets you are opting out for must not be provided
    
            Legacy Rule Format:
            IF [TRT551] = NULL AND ([TRT552] <> NULL OR [TRT553] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT551] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRT552] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            [TRT553] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount

            Technical Business Rule Format:
            ^TRT551 = NULL AND (^TRT552 <> NULL OR ^TRT553 <> NULL)
    
            Data Elements:
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRT552 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
    
            ^TRT553 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            */
            assertion = (report.TRT551 == null && (report.TRT552 != null || report.TRT553 != null));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432906", Severity = ProcessMessageSeverity.Error,
                    Description = @"Number of assets you are opting out for and Value of assets you are opting out for must not be provided",
                    LongDescription = @"When Are you making a choice to opt out of temporary full expensing for some or all of your eligible assets does not have a value, then Number of assets you are opting out for and Value of assets you are opting out for must not be provided",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432906"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT552", Value = report.TRT552.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT553", Value = GetValueOrEmpty(report.TRT553) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432920
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432920()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432920
            'Do you own shares that are referable to a sub-fund within the same CCIV as you?' must be completed if the trust is a CCIV sub-fund trust.
    
            Legacy Rule Format:
            IF [TRT32] = "233" AND [TRT566] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT566] = TRT:RP:bafpo7.02.33:Assets.Investment.CrossOtherCCIVSubfund.Indicator

            Technical Business Rule Format:
            ^TRT32 = '233' AND ^TRT566 = NULL
    
            Data Elements:
    
            ^TRT566 = RP:Assets.Investment.CrossOtherCCIVSubfund.Indicator
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
            assertion = (report.TRT32 == @"233" && report.TRT566 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432920", Severity = ProcessMessageSeverity.Error,
                    Description = @"'Do you own shares that are referable to a sub-fund within the same CCIV as you?' must be completed when type of trust is CCIV sub-fund trust.",
                    Location = "/xbrli:xbrl/tns:Assets.Investment.CrossOtherCCIVSubfund.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432920"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT566", Value = GetValueOrEmpty(report.TRT566) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432921
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432921()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432921
            'Do you own shares that are referable to a sub-fund within the same CCIV as you?' must be left blank if when the trust is not a CCIV sub-fund trust.
    
            Legacy Rule Format:
            IF [TRT32] <> "233" AND [TRT566] <> NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT32] = TRT:RP:pyde.02.51:OrganisationDetails.OrganisationType.Code
            [TRT566] = TRT:RP:bafpo7.02.33:Assets.Investment.CrossOtherCCIVSubfund.Indicator

            Technical Business Rule Format:
            ^TRT32 <> '233' AND ^TRT566 <> NULL
    
            Data Elements:
    
            ^TRT566 = RP:Assets.Investment.CrossOtherCCIVSubfund.Indicator
    
            ^TRT32 = RP:OrganisationDetails.OrganisationType.Code
            */
            assertion = (report.TRT32 != @"233" && report.TRT566 != null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432921", Severity = ProcessMessageSeverity.Error,
                    Description = @"'Do you own shares that are referable to a sub-fund within the same CCIV as you?' must not be completed when trust type is not a CCIV sub-fund trust.",
                    Location = "/xbrli:xbrl/tns:Assets.Investment.CrossOtherCCIVSubfund.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432921"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT32", Value = report.TRT32 });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT566", Value = GetValueOrEmpty(report.TRT566) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432922
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432922()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432922
            Temporary Full Expensing (TFE) fields cannot be included in a 2023-24 early lodgment.
    
            Legacy Rule Format:
            IF [TRT1] = 2024 AND TRT:RP:period.startDate >= "2023-07-01" AND ([TRT551] <> NULL OR [TRT552] <> NULL OR [TRT553] <> NULL OR [TRT554] <> NULL OR [TRT555] <> NULL)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT1] = TRT:RP:pyin.02.37:Report.TargetFinancial.Year
            [TRT551] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
            [TRT552] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
            [TRT553] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
            [TRT554] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
            [TRT555] = TRT:RP:rvctc2.02.36:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number

            Technical Business Rule Format:
            ^TRT1 = 2024 AND ^StartDate1 >=  AsDate('2023-07-01')  AND (^TRT551 <> NULL OR ^TRT552 <> NULL OR ^TRT553 <> NULL OR ^TRT554 <> NULL OR ^TRT555 <> NULL)
    
            Data Elements:
    
            ^TRT555 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number
    
            ^TRT1 = RP:Report.TargetFinancial.Year
    
            ^TRT551 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Code
    
            ^TRT552 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Number
    
            ^TRT553 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssetsOptOut.Amount
    
            ^TRT554 = RP:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Amount
    
            ^StartDate1 = RP
            */
            assertion = ((report.RPExists == true) && (report.TRT1.GetValueOrDefault() == 2024 && report.RPStartDate >= AsDate(@"2023-07-01") && (report.TRT551 != null || report.TRT552 != null || report.TRT553 != null || report.TRT554 != null || report.TRT555 != null)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432922", Severity = ProcessMessageSeverity.Error,
                    Description = @"Temporary Full Expensing (TFE) is not applicable after 30/06/2023.",
                    LongDescription = @"One or more fields for Temporary Full Expensing (TFE) have been completed on this return. TFE is not applicable after 30/06/2023. Please correct these fields before lodging.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.TemporaryFullExpensingEligibleAssets.Number",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432922"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT1", Value = GetValueOrEmpty(report.TRT1) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT551", Value = GetValueOrEmpty(report.TRT551) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT552", Value = report.TRT552.GetValueOrDefault().ToString() });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT553", Value = GetValueOrEmpty(report.TRT553) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT554", Value = GetValueOrEmpty(report.TRT554) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT555", Value = report.TRT555.GetValueOrDefault().ToString() });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432923
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432923()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432923
            Monetary field - unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [TRT567] <> NULL AND [TRT567] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT567, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT567 = RP:Expense.EmployeeTrainingBonus.Amount
            */
            assertion = NotMonetary(report.TRT567, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.EmployeeTrainingBonus.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432923"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT567", Value = GetValueOrEmpty(report.TRT567) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432924
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432924()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432924
            If there is an amount greater than zero at Small business skills and training boost, there must be an amount at Reconciliation items- Expense reconciliation adjustments- Australian business expenses
    
            Legacy Rule Format:
            IF [TRT567] > 0 AND [TRT447] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT567 > 0 AND ^TRT447 = NULL
    
            Data Elements:
    
            ^TRT567 = RP:Expense.EmployeeTrainingBonus.Amount
    
            ^TRT447 = RP.JAUS:Expense.ReconciliationAdjustmentTotal.Amount
            */
            assertion = (report.TRT567.GetValueOrDefault() > 0 && report.TRT447 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432924", Severity = ProcessMessageSeverity.Error,
                    Description = @"Reconciliation items- Expense reconciliation adjustments- Australian business expenses must be present",
                    LongDescription = @"If there is an amount greater than zero at Small business skills and training boost, there must be an amount at Reconciliation items- Expense reconciliation adjustments- Australian business expenses",
                    Location = "/xbrli:xbrl/tns:Expense.EmployeeTrainingBonus.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432924"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT567", Value = GetValueOrEmpty(report.TRT567) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT447", Value = GetValueOrEmpty(report.TRT447) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432925
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432925()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432925
            Monetary field - unsigned, maximum of 11 numeric characters, no decimals
    
            Legacy Rule Format:
            IF [TRT568] <> NULL AND [TRT568] <> MONETARY(U,11,0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            NotMonetary(^TRT568, 'U', 11, 0)
    
            Data Elements:
    
            ^TRT568 = RP:Expense.TechnologyInvestmentBonus.Amount
            */
            assertion = NotMonetary(report.TRT568, @"U", 11, 0);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.400011", Severity = ProcessMessageSeverity.Error,
                    Description = @"Field incorrect format. Exceeds maximum digits or decimals and/or contains illegal characters",
                    LongDescription = @"Field is in incorrect format for a monetary field. Field either exceeds maximum digits or decimals and/or contains illegal characters and/or for a signed field does not have the sign as the first character",
                    Location = "/xbrli:xbrl/tns:Expense.TechnologyInvestmentBonus.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432925"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT568", Value = GetValueOrEmpty(report.TRT568) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.432926
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRT432926()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.432926
            If there is an amount greater than zero at Small business technology investment boost, there must be an amount at Reconciliation items- Expense reconciliation adjustments- Australian business expenses
    
            Legacy Rule Format:
            IF [TRT568] > 0 AND [TRT447] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF

            Technical Business Rule Format:
            ^TRT568 > 0 AND ^TRT447 = NULL
    
            Data Elements:
    
            ^TRT568 = RP:Expense.TechnologyInvestmentBonus.Amount
    
            ^TRT447 = RP.JAUS:Expense.ReconciliationAdjustmentTotal.Amount
            */
            assertion = (report.TRT568.GetValueOrDefault() > 0 && report.TRT447 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.432926", Severity = ProcessMessageSeverity.Error,
                    Description = @"Reconciliation items- Expense reconciliation adjustments- Australian business expenses must be present",
                    LongDescription = @"If there is an amount greater than zero at Small business technology investment boost, there must be an amount at Reconciliation items- Expense reconciliation adjustments- Australian business expenses",
                    Location = "/xbrli:xbrl/tns:Expense.TechnologyInvestmentBonus.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.432926"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT568", Value = GetValueOrEmpty(report.TRT568) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT447", Value = GetValueOrEmpty(report.TRT447) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00001
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00001()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00001
            If the 'Exemption or rollover code' is 'T - Small Business Restructure Roll-over' then the value entered in 'total business income' cannot be more than 10 million dollars.
    
            Legacy Rule Format:
            IF ANY OCCURRENCE OF ([TRT448]) = "T - Small Business Restructure Roll-over" AND (([TRT46] + [TRT50] + [TRT54] + [TRT47] + [TRT49] + [TRT52] + [TRT55]) > 10000000)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT46] = TRT:RP.JAUS.Prim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT47] = TRT:RP.JAUS.NonPrim:lrla.02.00:Remuneration.ABNNotQuotedPaymentGross.Amount
            [TRT49] = TRT:RP.JAUS.NonPrim:lrla.02.21:Remuneration.PaymentToForeignResidentGross.Amount
            [TRT50] = TRT:RP.JAUS.Prim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT52] = TRT:RP.JAUS.NonPrim:gfagg.02.00:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT448] = TRT:RP:rvctc3.02.47:CapitalGainsTax.ExemptionOrRolloverApplied.Code

            Technical Business Rule Format:
            AnyOccurrence(^TRT448, ^TRT448 = 'T - Small Business Restructure Roll-over') AND ((^TRT46 + ^TRT50 + ^TRT54 + ^TRT47 + ^TRT49 + ^TRT52 + ^TRT55) > 10000000)
    
            Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^TRT46 = RP.JAUS.Prim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT47 = RP.JAUS.NonPrim:Remuneration.ABNNotQuotedPaymentGross.Amount
    
            ^TRT49 = RP.JAUS.NonPrim:Remuneration.PaymentToForeignResidentGross.Amount
    
            ^TRT50 = RP.JAUS.Prim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT52 = RP.JAUS.NonPrim:GovernmentFunding.GovernmentIndustryPaymentsAssessable.Amount
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
            */
            assertion = ((report.TRT448Collection == null ? false : report.TRT448Collection.Any(TRT448Repeat => TRT448Repeat == @"T - Small Business Restructure Roll-over")) && report.TRT46.GetValueOrDefault() + report.TRT50.GetValueOrDefault() + report.TRT54.GetValueOrDefault() + report.TRT47.GetValueOrDefault() + report.TRT49.GetValueOrDefault() + report.TRT52.GetValueOrDefault() + report.TRT55.GetValueOrDefault() > 10000000);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00001", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Total business income amount incorrect",
                    LongDescription = @"To apply the small business restructure roll-over, each party to the transfer must be a small business entity, or an entity that is either (a) an affiliate, (b) connected with an entity, or (c) a partner in a partnership, that is a small business entity.",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00001"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = "TRT448" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT46", Value = GetValueOrEmpty(report.TRT46) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT50", Value = GetValueOrEmpty(report.TRT50) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT47", Value = GetValueOrEmpty(report.TRT47) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT49", Value = GetValueOrEmpty(report.TRT49) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT52", Value = GetValueOrEmpty(report.TRT52) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00002
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00002()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00002
            If the rollover applied code is "T - Small Business Restructure Roll-over", then the residency status of the trust must be 'Resident'.
    
            Legacy Rule Format:
            IF ANY OCCURRENCE OF ([TRT448]) = "T - Small Business Restructure Roll-over" AND [TRT245] <> "Resident" 
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT245] = TRT:RP:pyde.02.10:Residency.TaxPurposesOrganisationStatus.Code
            [TRT448] = TRT:RP:rvctc3.02.47:CapitalGainsTax.ExemptionOrRolloverApplied.Code

            Technical Business Rule Format:
            AnyOccurrence(^TRT448, ^TRT448 = 'T - Small Business Restructure Roll-over') AND ^TRT245 <> 'Resident'
    
            Data Elements:
    
            ^TRT448 = RP:CapitalGainsTax.ExemptionOrRolloverApplied.Code
    
            ^TRT245 = RP:Residency.TaxPurposesOrganisationStatus.Code
            */
            assertion = ((report.TRT448Collection == null ? false : report.TRT448Collection.Any(TRT448Repeat => TRT448Repeat == @"T - Small Business Restructure Roll-over")) && report.TRT245 != @"Resident");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00002", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Residency status of the trust must be 'Resident', unless eligible under certain exceptions.",
                    LongDescription = @"If the rollover applied code is ""T - Small Business Restructure Roll-over"", then the residency status of the trust must be 'Resident'.",
                    Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00002"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT448", Value = "TRT448" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT245", Value = report.TRT245 });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00003
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00003()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00003
            NCMI from non-primary production - Business income and excluded from NCMI from non-primary production - Business income should be included in non-PP income - Other business income.
    
            Legacy Rule Format:
            IF [TRT55] < ([TRT525] + [TRT526])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT55] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.Other.Amount
            [TRT525] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT526] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT55 < (^TRT525 + ^TRT526)
    
            Data Elements:
    
            ^TRT55 = RP.JAUS.NonPrim:Income.Other.Amount
    
            ^TRT525 = RP.JAUS.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT526 = RP.JAUS.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT55.GetValueOrDefault() < report.TRT525.GetValueOrDefault() + report.TRT526.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00003", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - business income from non-primary production are included at non-PP income - Other business income.",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00003"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT55", Value = GetValueOrEmpty(report.TRT55) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT525", Value = GetValueOrEmpty(report.TRT525) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT526", Value = GetValueOrEmpty(report.TRT526) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00004
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00004()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00004
            NCMI from non-primary production - Distribution from partnerships less foreign income and excluded from NCMI from non-primary production - Distribution from partnerships less foreign income should be included in non-PP income - Distribution from partnerships less foreign income.
    
            Legacy Rule Format:
            IF [TRT80] < ([TRT531] + [TRT532])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT80] = TRT:RP.JAUS.NonPrim:bafpr1.02.04:Income.PartnershipDistributionLessForeignIncome.Amount
            [TRT531] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT532] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT80 < (^TRT531 + ^TRT532)
    
            Data Elements:
    
            ^TRT80 = RP.JAUS.NonPrim:Income.PartnershipDistributionLessForeignIncome.Amount
    
            ^TRT531 = RP.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT532 = RP.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT80.GetValueOrDefault() < report.TRT531.GetValueOrDefault() + report.TRT532.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00004", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - partnership income from non-primary production are included at non-PP income - Distribution from partnerships less foreign income.",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionLessForeignIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00004"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT80", Value = GetValueOrEmpty(report.TRT80) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT531", Value = GetValueOrEmpty(report.TRT531) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT532", Value = GetValueOrEmpty(report.TRT532) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00005
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00005()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00005
            NCMI from non-primary production - Share of net income from trusts less net capital gain, foreign income and franked distributions and excluded from NCMI from non-primary production - Share of net income from trusts less net capital gain, foreign income and franked distributions should be included in non-PP income - Share of net income from trusts less net capital gain, foreign income and franked distributions.
    
            Legacy Rule Format:
            IF [TRT81] < ([TRT533] + [TRT534])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT81] = TRT:RP.JAUS.NonPrim:bafpr1.02.13:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
            [TRT533] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT534] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT81 < (^TRT533 + ^TRT534)
    
            Data Elements:
    
            ^TRT81 = RP.JAUS.NonPrim:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount
    
            ^TRT533 = RP.Share.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT534 = RP.Share.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT81.GetValueOrDefault() < report.TRT533.GetValueOrDefault() + report.TRT534.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00005", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - share of net income from trusts from non-primary production  are included at non-PP income - Share of net income from trusts less net capital gains, foreign income and franked distributions.",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNetExcludeNetCapitalGainsAndForeignIncomeAndDistributionFranked.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00005"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT81", Value = GetValueOrEmpty(report.TRT81) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT533", Value = GetValueOrEmpty(report.TRT533) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT534", Value = GetValueOrEmpty(report.TRT534) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00006
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00006()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00006
            NCMI from primary production - Business income and excluded from NCMI from primary production - Business income should be included in PP income - Other business income.
    
            Legacy Rule Format:
            IF [TRT54] < ([TRT523] + [TRT524])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT54] = TRT:RP.JAUS.Prim:bafpr1.02.04:Income.Other.Amount
            [TRT523] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT524] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT54 < (^TRT523 + ^TRT524)
    
            Data Elements:
    
            ^TRT54 = RP.JAUS.Prim:Income.Other.Amount
    
            ^TRT523 = RP.JAUS.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT524 = RP.JAUS.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT54.GetValueOrDefault() < report.TRT523.GetValueOrDefault() + report.TRT524.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00006", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - business income from primary production are included at PP income - Other business income.",
                    Location = "/xbrli:xbrl/tns:Income.Other.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00006"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT54", Value = GetValueOrEmpty(report.TRT54) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT523", Value = GetValueOrEmpty(report.TRT523) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT524", Value = GetValueOrEmpty(report.TRT524) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00007
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00007()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00007
            NCMI from primary production - Distribution from partnerships and excluded from NCMI from primary production - Distribution from partnerships should be included in PP income - Distribution from partnerships.
    
            Legacy Rule Format:
            IF [TRT76] < ([TRT527] + [TRT528])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT76] = TRT:RP.JAUS.Prim:bafpr1.02.02:Income.PartnershipDistributionGross.Amount
            [TRT527] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT528] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT76 < (^TRT527 + ^TRT528)
    
            Data Elements:
    
            ^TRT76 = RP.JAUS.Prim:Income.PartnershipDistributionGross.Amount
    
            ^TRT527 = RP.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT528 = RP.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT76.GetValueOrDefault() < report.TRT527.GetValueOrDefault() + report.TRT528.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00007", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - partnership income from primary production are included at PP income - Distribution from partnerships.",
                    Location = "/xbrli:xbrl/tns:Income.PartnershipDistributionGross.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00007"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT76", Value = GetValueOrEmpty(report.TRT76) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT527", Value = GetValueOrEmpty(report.TRT527) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT528", Value = GetValueOrEmpty(report.TRT528) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00008
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00008()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00008
            NCMI from primary production - Share of net income from trusts and excluded from NCMI from primary production - Share of net income from trusts should be included in PP income - Share of net income from trusts.
    
            Legacy Rule Format:
            IF [TRT77] < ([TRT529] + [TRT530])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT77] = TRT:RP.JAUS.Prim:bafpr1.02.14:Income.TrustShareNet.Amount
            [TRT529] = TRT:RP.Share.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT530] = TRT:RP.Share.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount

            Technical Business Rule Format:
            ^TRT77 < (^TRT529 + ^TRT530)
    
            Data Elements:
    
            ^TRT77 = RP.JAUS.Prim:Income.TrustShareNet.Amount
    
            ^TRT529 = RP.Share.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT530 = RP.Share.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = (report.TRT77.GetValueOrDefault() < report.TRT529.GetValueOrDefault() + report.TRT530.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00008", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure NCMI and Excluded from NCMI - share of net income from trusts from primary production are included at PP income - Share of net income from trusts.",
                    Location = "/xbrli:xbrl/tns:Income.TrustShareNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00008"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT77", Value = GetValueOrEmpty(report.TRT77) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT529", Value = GetValueOrEmpty(report.TRT529) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT530", Value = GetValueOrEmpty(report.TRT530) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00009
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00009()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00009
            The total amount at NCMI from primary production (beneficiary and no beneficiary presently entitled) cannot exceed the sum of NCMI from primary production - Business income, NCMI from primary production - Distribution from partnerships and NCMI from primary production - Share of net income from trusts.
    
            Legacy Rule Format:
            IF (SUM([TRT537]) + [TRT543]) > ([TRT523] + [TRT527] + [TRT529])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT523] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT527] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT529] = TRT:RP.Share.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT537] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT543] = TRT:RP.Prim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT537) + ^TRT543) > (^TRT523 + ^TRT527 + ^TRT529)
    
            Data Elements:
    
            ^TRT523 = RP.JAUS.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT527 = RP.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT529 = RP.Share.Prim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT537 = RP.Prim:StatementOfDistributionToBeneficiary:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT543 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT537.GetValueOrDefault())) + report.TRT543.GetValueOrDefault() > report.TRT523.GetValueOrDefault() + report.TRT527.GetValueOrDefault() + report.TRT529.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00009", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at NCMI from primary production (beneficiary and no beneficiary presently entitled) does not exceed net NCMI from primary production.",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00009"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT537", Value = "TRT537" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT543", Value = GetValueOrEmpty(report.TRT543) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT523", Value = GetValueOrEmpty(report.TRT523) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT527", Value = GetValueOrEmpty(report.TRT527) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT529", Value = GetValueOrEmpty(report.TRT529) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00010
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00010()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00010
            The total amount at Excluded from NCMI from primary production (beneficiary and no beneficiary presently entitled) cannot exceed the sum of excluded from NCMI from primary production - Business income, excluded from NCMI from primary production - Distribution from partnerships and excluded from NCMI from primary production - Share of net income from trusts.
    
            Legacy Rule Format:
            IF (SUM([TRT538]) + [TRT544]) > ([TRT524] + [TRT528] + [TRT530])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT524] = TRT:RP.JAUS.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT528] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT530] = TRT:RP.Share.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT538] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT544] = TRT:RP.Prim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT538) + ^TRT544) > (^TRT524 + ^TRT528 + ^TRT530)
    
            Data Elements:
    
            ^TRT524 = RP.JAUS.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT528 = RP.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT530 = RP.Share.Prim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT538 = RP.Prim:StatementOfDistributionToBeneficiary:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT544 = RP.Prim:IncomeNoBeneficiaryPresentlyEntitled:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT538.GetValueOrDefault())) + report.TRT544.GetValueOrDefault() > report.TRT524.GetValueOrDefault() + report.TRT528.GetValueOrDefault() + report.TRT530.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00010", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at Excluded from NCMI from primary production (beneficiary and no beneficiary presently entitled) does not exceed net Excluded from NCMI from primary production.",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00010"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT538", Value = "TRT538" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT544", Value = GetValueOrEmpty(report.TRT544) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT524", Value = GetValueOrEmpty(report.TRT524) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT528", Value = GetValueOrEmpty(report.TRT528) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT530", Value = GetValueOrEmpty(report.TRT530) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00011
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00011()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00011
            The total amount at NCMI from non-primary production (beneficiary and no beneficiary presently entitled) cannot exceed the sum of NCMI from non-primary production - Business income, NCMI from non-primary production - Distribution from partnerships less foreign income and NCMI from non-primary production - Share of net income from trusts less net capital gain, foreign income and franked distributions.
    
            Legacy Rule Format:
            IF (SUM([TRT539]) + [TRT545])> ([TRT525] + [TRT531] + [TRT533])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT525] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT531] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT533] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount
            [TRT539] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT545] = TRT:RP.NonPrim:bafpr1.02.40:Income.NonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT539) + ^TRT545)> (^TRT525 + ^TRT531 + ^TRT533)
    
            Data Elements:
    
            ^TRT525 = RP.JAUS.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT531 = RP.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT533 = RP.Share.NonPrim:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT539 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.NonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT545 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:Income.NonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT539.GetValueOrDefault())) + report.TRT545.GetValueOrDefault() > report.TRT525.GetValueOrDefault() + report.TRT531.GetValueOrDefault() + report.TRT533.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00011", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at NCMI from non-primary production (beneficiary and no beneficiary presently entitled) does not exceed net NCMI from non-primary production.",
                    Location = "/xbrli:xbrl/tns:Income.NonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00011"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT539", Value = "TRT539" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT545", Value = GetValueOrEmpty(report.TRT545) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT525", Value = GetValueOrEmpty(report.TRT525) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT531", Value = GetValueOrEmpty(report.TRT531) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT533", Value = GetValueOrEmpty(report.TRT533) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00012
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00012()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00012
            The total amount at Excluded from NCMI from non-primary production (beneficiary and no beneficiary presently entitled) cannot exceed the sum of excluded from NCMI from non-primary production - Business income, excluded from NCMI from non-primary production - Distribution from partnerships less foreign income and excluded from NCMI from non-primary production - Share of net income from trusts less net capital gain, foreign income and franked distributions.
    
            Legacy Rule Format:
            IF (SUM([TRT540]) + [TRT546]) > ([TRT526] + [TRT532] + [TRT534])
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT526] = TRT:RP.JAUS.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT532] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT534] = TRT:RP.Share.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            [TRT540] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT546] = TRT:RP.NonPrim:bafpr1.02.40:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT540) + ^TRT546) > (^TRT526 + ^TRT532 + ^TRT534)
    
            Data Elements:
    
            ^TRT526 = RP.JAUS.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT532 = RP.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT534 = RP.Share.NonPrim:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT540 = RP.NonPrim:StatementOfDistributionToBeneficiary:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
    
            ^TRT546 = RP.NonPrim:IncomeNoBeneficiaryPresentlyEntitled:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT540.GetValueOrDefault())) + report.TRT546.GetValueOrDefault() > report.TRT526.GetValueOrDefault() + report.TRT532.GetValueOrDefault() + report.TRT534.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00012", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at Excluded from NCMI from non-primary production (beneficiary and no beneficiary presently entitled) does not exceed net Excluded from NCMI from non-primary production.",
                    Location = "/xbrli:xbrl/tns:Income.ExcludedFromNonConcessionalManagedInvestmentTrust.Amount[@contextRef='" + report.RPJAUSNonPrimId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00012"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT540", Value = "TRT540" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT546", Value = GetValueOrEmpty(report.TRT546) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT526", Value = GetValueOrEmpty(report.TRT526) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT532", Value = GetValueOrEmpty(report.TRT532) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT534", Value = GetValueOrEmpty(report.TRT534) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00013
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00013()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00013
            The total amount at capital gains NCMI (beneficiary and no beneficiary presently entitled) cannot exceed net capital gains NCMI.
    
            Legacy Rule Format:
            IF (SUM([TRT541]) + [TRT547]) > [TRT535]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT535] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
            [TRT541] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT547] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT541) + ^TRT547) > ^TRT535
    
            Data Elements:
    
            ^TRT535 = RP:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
    
            ^TRT541 = RP:StatementOfDistributionToBeneficiary:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
    
            ^TRT547 = RP:IncomeNoBeneficiaryPresentlyEntitled:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT541.GetValueOrDefault())) + report.TRT547.GetValueOrDefault() > report.TRT535.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00013", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at capital gains NCMI (beneficiary and no beneficiary presently entitled) does not exceed net capital gains NCMI.",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00013"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT541", Value = "TRT541" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT547", Value = GetValueOrEmpty(report.TRT547) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT535", Value = GetValueOrEmpty(report.TRT535) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00014
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00014()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00014
            The total amount at capital gains Excluded from NCMI (beneficiary and no beneficiary presently entitled) cannot exceed net capital gains Excluded from NCMI.
    
            Legacy Rule Format:
            IF (SUM([TRT542]) + [TRT548]) > [TRT536]
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT536] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
            [TRT542] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:StatementOfDistributionToBeneficiary)
            [TRT548] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount IN TUPLE(trt.0011.lodge.req.02.00:IncomeNoBeneficiaryPresentlyEntitled)

            Technical Business Rule Format:
            (Sum(^TRT542) + ^TRT548) > ^TRT536
    
            Data Elements:
    
            ^TRT536 = RP:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
    
            ^TRT542 = RP:StatementOfDistributionToBeneficiary:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
    
            ^TRT548 = RP:IncomeNoBeneficiaryPresentlyEntitled:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = ((report.StatementOfDistributionToBeneficiaryCollection == null ? 0 : report.StatementOfDistributionToBeneficiaryCollection.Sum(f => f.TRT542.GetValueOrDefault())) + report.TRT548.GetValueOrDefault() > report.TRT536.GetValueOrDefault());
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00014", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure the total amount at capital gains Excluded from NCMI (beneficiary and no beneficiary presently entitled) does not exceed net capital gains Excluded from NCMI.",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00014"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT542", Value = "TRT542" });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT548", Value = GetValueOrEmpty(report.TRT548) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT536", Value = GetValueOrEmpty(report.TRT536) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00015
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00015()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00015
            Net capital gain cannot be blank if either of capital gains NCMI or capital gains excluded from NCMI are greater than zero.
    
            Legacy Rule Format:
            IF [TRT133] = NULL AND ([TRT535] > 0 OR [TRT536] > 0)
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT133] = TRT:RP:bafpr1.02.00:Income.CapitalGainsNet.Amount
            [TRT535] = TRT:RP:rvctc3.02.49:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
            [TRT536] = TRT:RP:rvctc3.02.49:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount

            Technical Business Rule Format:
            ^TRT133 = NULL AND (^TRT535 > 0 OR ^TRT536 > 0)
    
            Data Elements:
    
            ^TRT133 = RP:Income.CapitalGainsNet.Amount
    
            ^TRT535 = RP:Capital.Gains.NonConcessionalManagedInvestmentTrustIncome.Amount
    
            ^TRT536 = RP:Capital.Gains.ExcludedFromNonConcessionalManagedInvestmentTrustIncome.Amount
            */
            assertion = (report.TRT133 == null && (report.TRT535.GetValueOrDefault() > 0 || report.TRT536.GetValueOrDefault() > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00015", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Ensure net capital gains is not blank if either of capital gains NCMI or capital gains Excluded from NCMI are greater than zero.",
                    Location = "/xbrli:xbrl/tns:Income.CapitalGainsNet.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00015"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT133", Value = GetValueOrEmpty(report.TRT133) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT535", Value = GetValueOrEmpty(report.TRT535) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT536", Value = GetValueOrEmpty(report.TRT536) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00016
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00016()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00016
            If the company has self-assessed that they are a Significant Global Entity (SGE), please confirm if they are a Country by Country Reporting Entity (CbCRE).
    
            Legacy Rule Format:
            IF [TRT487] = TRUE AND [TRT550] = NULL
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT487] = TRT:RP:pyde.02.43:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
            [TRT550] = TRT:RP:rvctc4.02.17:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator

            Technical Business Rule Format:
            ^TRT487 = TRUE AND ^TRT550 = NULL
    
            Data Elements:
    
            ^TRT487 = RP:OrganisationDetails.SignificantGlobalEntityStatus.Indicator
    
            ^TRT550 = RP:InternationalDealings.CountryByCountryReportingEntityStatus.Indicator
            */
            assertion = (report.TRT487 == true && report.TRT550 == null);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00016", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Country by Country Reporting Entity (CBCRE) indicator should be supplied.",
                    Location = "/xbrli:xbrl/tns:OrganisationDetails.SignificantGlobalEntityStatus.Indicator",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00016"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT487", Value = GetValueOrEmpty(report.TRT487) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT550", Value = GetValueOrEmpty(report.TRT550) });
    
                response.Add(processMessage);
            }
      }
      #endregion 
    
      #region VR.ATO.TRT.W00019
      [MethodImpl(MethodImplOptions.AggressiveInlining)]
      public virtual void VRATOTRTW00019()
      {
        ProcessMessageDocument processMessage;
        bool assertion;
    
            /*  VR.ATO.TRT.W00019
            If 'P' is selected at aggregated turnover range, then the aggregated turnover value should be more than $1 billion
    
            Legacy Rule Format:
            IF [TRT564] < 1000000000 AND [TRT563] = "P"
               RETURN VALIDATION MESSAGE
            ENDIF
            
            [TRT564] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnover.Amount
            [TRT563] = TRT:RP:bafpr1.02.44:Income.AggregatedTurnoverRange.Code

            Technical Business Rule Format:
            ^TRT564 < 1000000000 AND ^TRT563 = 'P'
    
            Data Elements:
    
            ^TRT564 = RP:Income.AggregatedTurnover.Amount
    
            ^TRT563 = RP:Income.AggregatedTurnoverRange.Code
            */
            assertion = (report.TRT564.GetValueOrDefault() < 1000000000 && report.TRT563 == @"P");
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TRT.W00020", Severity = ProcessMessageSeverity.Warning,
                    Description = @"Aggregated turnover value should be more than $1 billion",
                    LongDescription = @"If 'P' is selected at aggregated turnover range, then the aggregated turnover value should be more than $1 billion",
                    Location = "/xbrli:xbrl/tns:Income.AggregatedTurnover.Amount",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() {Name = "RuleIdentifier", Value = "VR.ATO.TRT.W00019"} },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT564", Value = GetValueOrEmpty(report.TRT564) });
    
                processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TRT563", Value = GetValueOrEmpty(report.TRT563) });
    
                response.Add(processMessage);
            }
      }
      #endregion 

    }
} 
